// Any Device - Utilities - Any OS - Any Compiler
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/SDK/common/utils/gsc_util_format.c $
// $Rev: 2809 $
// $Date: 2009-07-29 21:26:35 -0500 (Wed, 29 Jul 2009) $

#include <math.h>
#include <stdio.h>
#include <string.h>

#include "gsc_util.h"



/* #defines	*****************************************************************/

#define	ADD_ITEM(v,t,s)	sprintf(buf + strlen(buf),		\
								"%d %s%s%s",			\
								(int) (v),				\
								(t),					\
								((v) == 1) ? "" : "s",	\
								(s))
#define	ADD_WEEK(v)		ADD_ITEM((v), "Week", ", ")
#define	ADD_DAY(v)		ADD_ITEM((v), "Day", ", ")
#define	ADD_HOUR(v)		ADD_ITEM((v), "Hr", ", ")
#define	ADD_MINUTE(v)	ADD_ITEM((v), "Mn", ", ")
#define	ADD_SECOND(v)	ADD_ITEM((v), "Sec", "")



/*****************************************************************************
*
*	Function:	gsc_format_float
*
*	Purpose:
*
*		Format a floating point number using K, M, G and T multipliers.
*
*	Arguments:
*
*		value	The value to format. This is rounded to the nearest whole
*				number.
*
*		k		The size of a 1K unit.
*
*		buf		We put for mormatted output here.
*
*		size	The size of the buffer.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void gsc_format_float(float value, int k, char* dest, size_t size)
{
	char	buf[64];
	float	delta;
	char*	factor	= "";

	value	= (float) floor(value + 0.5);

	// Check for: K
	delta	= (float) fmod(value, k);

	if ((value) && (delta == 0))
	{
		value	/= k;
		factor	= "K";
	}

	// Check for: M
	delta	= (float) fmod(value, k);

	if ((value) && (delta == 0))
	{
		value	/= k;
		factor	= "M";
	}

	// Check for: G
	delta	= (float) fmod(value, k);

	if ((value) && (delta == 0))
	{
		value	/= k;
		factor	= "G";
	}

	// Check for: T
	delta	= (float) fmod(value, k);

	if ((value) && (delta == 0))
	{
		value	/= k;
		factor	= "T";
	}

	sprintf(buf, "%.0f%s", value, factor);

	strncpy(dest, buf, size);
	dest[size - 1]	= 0;
}



/*****************************************************************************
*
*	Function:	gsc_format_float_prec
*
*	Purpose:
*
*		Format a floating point number using limited digits of precision and
*		using K, M, G and T multipliers.
*
*	Arguments:
*
*		value	The value to format. This is rounded to the nearest whole
*				number.
*
*		k		The size of a 1K unit.
*
*		buf		We put for mormatted output here.
*
*		size	The size of the buffer.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void gsc_format_float_prec(float value, int k, char* dest, size_t size)
{
	static const struct
	{
		int			mult;
		double		limit;
		const char*	format;
	} list[]	=
	{
		/* mult	limit	format	*/
		{ 0,	999  ,	"%.0f"	},

		{ 1,	 9.99,	"%.2fK"	},
		{ 0,	99.9 ,	"%.1fK"	},
		{ 0,	999  ,	"%.0fK"	},

		{ 1,	 9.99,	"%.2fM"	},
		{ 0,	99.9 ,	"%.1fM"	},
		{ 0,	999  ,	"%.0fM"	},

		{ 1,	 9.99,	"%.2fG"	},
		{ 0,	99.9 ,	"%.1fG"	},
		{ 0,	999  ,	"%.0fG"	},

		{ 1,	 9.99,	"%.2fT"	},
		{ 0,	99.9 ,	"%.1fT"	},
		{ 0,	999  ,	"%.0fT"	},

		{ -1,		0,	"%.0fT"	}
	};

	char		buf[64];
	const char*	format;
	int			i;
	float		v	= value;

	value	= (float) floor(value + 0.5);

	for (i = 0;; i++)
	{
		if (list[i].mult < 0)
			break;

		if (list[i].mult)
			v	/= k;

		if (v > list[i].limit)
			continue;

		break;
	}

	format	= list[i].format;
	sprintf(buf, format, v);
	strncpy(dest, buf, size);
	dest[size - 1]	= 0;
}



/*****************************************************************************
*
*	Function:	gsc_format_long
*
*	Purpose:
*
*		Format a decimal number using K, M and G multipliers.
*
*	Arguments:
*
*		value	The value to format.
*
*		k		The size of a 1K unit (usually 1024).
*
*		dest	The formatted value goes here.
*
*		size	This is the size of the above buffer.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void gsc_format_long(long value, int k, char* dest, size_t size)
{
	gsc_format_s32((S32) value, k, dest, size);
}



/*****************************************************************************
*
*	Function:	gsc_format_s32
*
*	Purpose:
*
*		Format a decimal number using K, M and G multipliers.
*
*	Arguments:
*
*		value	The value to format.
*
*		k		The size of a 1K unit (usually 1024).
*
*		dest	The formatted value goes here.
*
*		size	This is the size of the above buffer.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void gsc_format_s32(S32 value, int k, char* dest, size_t size)
{
	char	buf[32];
	char*	mult	= "";
	char*	sign	= "";

	if (value == -1L)
	{
		strncpy(dest, "0xFFFFFFFF", size);
		dest[size - 1]	= 0;
	}
	else
	{
		if (value < 0)
		{
			sign	= "-";
			value	= -value;
		}

		if ((value) && ((value % k) == 0))
		{
			mult	= "K";
			value	/= k;
		}

		if ((value) && ((value % k) == 0))
		{
			mult	= "M";
			value	/= k;
		}

		if ((value) && ((value % k) == 0))
		{
			mult	= "G";
			value	/= k;
		}

		sprintf(buf, "%s%lu%s", sign, (long) value, mult);
		strncpy(dest, buf, size);
		dest[size - 1]	= 0;
	}
}



/*****************************************************************************
*
*	Function:	gsc_format_time_ms
*
*	Purpose:
*
*		Format a period of time given in milliseconds.
*
*	Arguments:
*
*		ms		The number of milliseconds.
*
*		dest	The formatted value goes here.
*
*		size	This is the size of the above buffer.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void gsc_format_time_ms(long ms, char* dest, size_t size)
{
	char	buf[128];
	int		day;
	int		hrs;
	int		min;
	int		sec;
	int		wks;

	sec	= ms / 1000;
	ms	= ms % 1000;

	min	= sec / 60;
	sec	= sec % 60;

	hrs	= min / 60;
	min	= min % 60;

	day	= hrs / 24;
	hrs	= hrs % 24;

	wks	= day / 7;
	day	= day % 7;

	buf[0]	= 0;

	if (wks)
	{
		ADD_WEEK(wks);
		ADD_DAY(day);
		ADD_HOUR(hrs);
		ADD_MINUTE(min);
		sprintf(buf + strlen(buf), "%d.%03d Secs", sec, (int) ms);
	}
	else if (day)
	{
		ADD_DAY(day);
		ADD_HOUR(hrs);
		ADD_MINUTE(min);
		sprintf(buf + strlen(buf), "%d.%03d Secs", sec, (int) ms);
	}
	else if (hrs)
	{
		ADD_HOUR(hrs);
		ADD_MINUTE(min);
		sprintf(buf + strlen(buf), "%d.%03d Secs", sec, (int) ms);
	}
	else if (min)
	{
		ADD_MINUTE(min);
		sprintf(buf + strlen(buf), "%d.%03d Secs", sec, (int) ms);
	}
	else if (sec)
	{
		sprintf(buf + strlen(buf), "%d.%03d Secs", sec, (int) ms);
	}
	else
	{
		sprintf(buf + strlen(buf), "%d ms", (int) ms);
	}

	strncpy(dest, buf, size);
	dest[size - 1]	= 0;
}



/*****************************************************************************
*
*	Function:	gsc_format_time_sec
*
*	Purpose:
*
*		Format a period of time given in seconds.
*
*	Arguments:
*
*		sec		The number of seconds.
*
*		dest	The formatted value goes here.
*
*		size	This is the size of the above buffer.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void gsc_format_time_sec(long sec, char* dest, size_t size)
{
	char	buf[128];
	int		day;
	int		hrs;
	int		min;
	int		wks;

	min	= sec / 60;
	sec	= sec % 60;

	hrs	= min / 60;
	min	= min % 60;

	day	= hrs / 24;
	hrs	= hrs % 24;

	wks	= day / 7;
	day	= day % 7;

	buf[0]	= 0;

	if (wks)
	{
		ADD_WEEK(wks);
		ADD_DAY(day);
		ADD_HOUR(hrs);
		ADD_MINUTE(min);
		ADD_SECOND(sec);
	}
	else if (day)
	{
		ADD_DAY(day);
		ADD_HOUR(hrs);
		ADD_MINUTE(min);
		ADD_SECOND(sec);
	}
	else if (hrs)
	{
		ADD_HOUR(hrs);
		ADD_MINUTE(min);
		ADD_SECOND(sec);
	}
	else if (min)
	{
		ADD_MINUTE(min);
		ADD_SECOND(sec);
	}
	else
	{
		ADD_SECOND(sec);
	}

	strncpy(dest, buf, size);
	dest[size - 1]	= 0;
}


