// Any Device - Utilities - Win32 - Any Compiler
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/SDK/common/utils/win32/gsc_os_util_host.c $
// $Rev: 3621 $
// $Date: 2009-10-06 09:40:05 -0500 (Tue, 06 Oct 2009) $

#include <windows.h>
#include <stdio.h>
#include <string.h>

#include "gsc_util.h"



// variables	***************************************************************

static const struct
{
	const char*	name;
	long		platform;
	long		major;
	long		minor;
	int			hit;
} _ver_list[]	=
{
	{
		/* name		*/	"Windows 7",
		/* platform	*/	VER_PLATFORM_WIN32_NT,
		/* major	*/	6,
		/* minor	*/	1,
		/* hit		*/	0
	},
	{
		/* name		*/	"Windows Vista",
		/* platform	*/	VER_PLATFORM_WIN32_NT,
		/* major	*/	6,
		/* minor	*/	0,
		/* hit		*/	0
	},
	{
		/* name		*/	"Windows XP",
		/* platform	*/	VER_PLATFORM_WIN32_NT,
		/* major	*/	5,
		/* minor	*/	2,
		/* hit		*/	0
	},
	{
		/* name		*/	"Windows XP",
		/* platform	*/	VER_PLATFORM_WIN32_NT,
		/* major	*/	5,
		/* minor	*/	1,
		/* hit		*/	0
	},
	{
		/* name		*/	"Windows 2000",
		/* platform	*/	VER_PLATFORM_WIN32_NT,
		/* major	*/	5,
		/* minor	*/	0,
		/* hit		*/	0
	},
	{
		/* name		*/	"Windows NT",
		/* platform	*/	VER_PLATFORM_WIN32_NT,
		/* major	*/	4,
		/* minor	*/	0,
		/* hit		*/	1
	},
	{
		/* name		*/	"Windows NT",
		/* platform	*/	VER_PLATFORM_WIN32_NT,
		/* major	*/	3,
		/* minor	*/	51,
		/* hit		*/	1
	},
	{
		/* name		*/	"Windows Me",
		/* platform	*/	VER_PLATFORM_WIN32_WINDOWS,
		/* major	*/	4,
		/* minor	*/	90,
		/* hit		*/	1
	},
	{
		/* name		*/	"Windows 98",
		/* platform	*/	VER_PLATFORM_WIN32_WINDOWS,
		/* major	*/	4,
		/* minor	*/	10,
		/* hit		*/	1
	},
	{
		/* name		*/	"Windows 95",
		/* platform	*/	VER_PLATFORM_WIN32_WINDOWS,
		/* major	*/	4,
		/* minor	*/	0,
		/* hit		*/	1
	},
	{
		/* name		*/	"Win32s on Windows 3.1",
		/* platform	*/	VER_PLATFORM_WIN32s,
		/* major	*/	-1,
		/* minor	*/	-1,
		/* hit		*/	1
	},
	{
		/* name		*/	NULL,
		/* platform	*/	0,
		/* major	*/	0,
		/* minor	*/	0,
		/* hit		*/	1
	}
};



/*****************************************************************************
*
*	Function:	_id_hardware
*
*	Purpose:
*
*		Report information on the hardware
*
*	Arguments:
*
*		None.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _id_hardware(void)
{
	const char*	arch;
	SYSTEM_INFO	si;

	GetSystemInfo(&si);

	gsc_label_text("Architecture");

	switch(si.wProcessorArchitecture)
	{
		default:									arch	= "not known";	break;
		case PROCESSOR_ARCHITECTURE_UNKNOWN:		arch	= "UNKNOWN";	break;
		case PROCESSOR_ARCHITECTURE_INTEL:			arch	= "INTEL";		break;
		case PROCESSOR_ARCHITECTURE_MIPS:			arch	= "MIPS";		break;
		case PROCESSOR_ARCHITECTURE_ALPHA:			arch	= "ALPHA";		break;
		case PROCESSOR_ARCHITECTURE_PPC:			arch	= "PPC";		break;
		case PROCESSOR_ARCHITECTURE_IA64:			arch	= "IA64";		break;
//		case PROCESSOR_ARCHITECTURE_IA32_ON_WIN64:	arch	= "IA32";		break;
//		case PROCESSOR_ARCHITECTURE_AMD64:			arch	= "AMD64";		break;
	}

	printf("%s\n", arch);
	gsc_label_text("Processors");
	printf("%d\n", (int) si.dwNumberOfProcessors);
}



/*****************************************************************************
*
*	Function:	_id_os
*
*	Purpose:
*
*		Report the OS.
*
*	Arguments:
*
*		hit		Report if there is a performance hit for this OS.
*
*	Returned:
*
*		!NULL	A pointer to the string.
*
*****************************************************************************/

static const char* _id_os(int* hit)
{
	static char	buf[128];

	int				i;
	OSVERSIONINFO	os;
	const char*		ptr;

	memset(&os, 0, sizeof(os));
	os.dwOSVersionInfoSize	= sizeof(os);
	GetVersionEx(&os);

	for (i = 0;; i++)
	{
		if (_ver_list[i].name == NULL)
			break;

		if (os.dwPlatformId != (DWORD) _ver_list[i].platform)
			continue;

		if (os.dwMajorVersion < (DWORD) _ver_list[i].major)
			continue;

		if (os.dwMinorVersion < (DWORD) _ver_list[i].minor)
			continue;

		break;
	}

	if (_ver_list[i].name == NULL)
		ptr	= "UNKNOWN";
	else
		ptr	= _ver_list[i].name;

	sprintf(buf,
			"%s, %lu.%lu.%lu%s%s",
			ptr,
			(long) os.dwMajorVersion,
			(long) os.dwMinorVersion,
			(long) os.dwBuildNumber & 0xFFFF,
			os.szCSDVersion[0] ? ", " : "",
			os.szCSDVersion);

	if (hit)
		hit[0]	= _ver_list[i].hit;

	return(buf);
}



/*****************************************************************************
*
*	Function:	gsc_os_id_host
*
*	Purpose:
*
*		Identify the host.
*
*	Arguments:
*
*		None.
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors encountered.
*
*****************************************************************************/

int gsc_os_id_host(void)
{
	const char*		os;

	gsc_label_text("Host");
	printf("\n");
	gsc_label_level_inc();

	gsc_label_text("OS");
	os	= _id_os(NULL);
	printf("%s\n", os);

	_id_hardware();

	gsc_label_level_dec();
	return(0);
}



/*****************************************************************************
*
*	Function:	gsc_os_id_os_issue
*
*	Purpose:
*
*		See if the OS might affect operation.
*
*	Arguments:
*
*		None.
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors encountered.
*
*****************************************************************************/

int gsc_os_id_os_issue(void)
{
	int				hit;
	const char*		os;

	gsc_label_text("Operating System");
	os	= _id_os(&hit);

	if (hit)
		printf("%s  %s\n", os, "PERFORMANCE MAY BE DEGRADED");
	else
		printf("OK  %s\n", os);

	return(0);
}


