// HPDI32 - Application - Win32 - Any Compiler
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/SDK/hpdi32/ringbuf/win32/copy.c $
// $Rev: 31044 $
// $Date: 2014-10-27 17:44:10 -0500 (Mon, 27 Oct 2014) $

#include <windows.h>
#include <stdio.h>

#include "copy.h"



/* #defines	*****************************************************************/

#define	_1M	(1024L * 1024L)



/* variables	*************************************************************/

static char	_buffer[_1M];	// For copy_list_to_clipboard().



/*****************************************************************************
*
*	Function:	_copy_list_box
*
*	Purpose:
*
*		Append the given list box content to the given buffer.
*
*	Arguments:
*
*		control		The control to access.
*
*		dest		The text is copied here. It is assumed to be large
*					enough for the request.
*
*		flags		Flags that control processing.
*
*	Returned:
*
*		A pointer to the destination buffer's NULL terminator.
*
*****************************************************************************/

static char* _copy_list_box(HWND control, char* dest, unsigned int flags)
{
	LRESULT	count;
	int		i;
	LRESULT	sel;

	if ((control) && (flags & COPY_PREFEED))
	{
		strcat(dest, "\r\n");
		dest	+= 2;
	}

	count	= SendMessage(control, LB_GETCOUNT, 0, 0);

	for (i = 0; i < count; i++)
	{
		if (flags & COPY_INDENT)
		{
			strcat(dest, "  ");
			dest	+= 2;
		}

		SendMessage(control, LB_GETTEXT, (WPARAM) i, (LPARAM) dest);
		dest	+= strlen(dest);

		if (flags & COPY_LB_GETSEL)
		{
			sel	= SendMessage(control, LB_GETSEL, (WPARAM) i, 0);

			if (sel)
			{
				strcat(dest, " *");
				dest	+= 2;
			}
		}

		strcat(dest, "\r\n");
		dest	+= strlen(dest);
	}

	return(dest);
}



/*****************************************************************************
*
*	Function:	_copy_single
*
*	Purpose:
*
*		Copy the control/window title to a buffer. If specified, add an end
*		of line sequence. If specified, underline the title text.
*
*	Arguments:
*
*		control		The control/window to access.
*
*		dest		The text is copied here. It is assumed to be large
*					enough for the request.
*
*		flags		Flags that control processing.
*
*	Returned:
*
*		A pointer to the destination buffer's NULL terminator.
*
*****************************************************************************/

static char* _copy_single(HWND control, char* dest, unsigned int flags)
{
	char	buf[10240];
	int		enabled;
	LRESULT	i;
	int		index;
	int		len		= 0;
	char*	nl;
	int		post;
	int		pre;
	HWND	parent;
	char*	psz;
	RECT	rect;
	char*	space;
	int		visible	= TRUE;
	int		width;

	dest[0]	= 0;

	if ((control) && (flags & COPY_VISIBLE))
	{
		parent	= GetParent(control);
		GetWindowRect(control, &rect);
		ScreenToClient(parent, (POINT*) &rect.left);
		ScreenToClient(parent, (POINT*) &rect.right);

		if ((rect.left < 0) ||
			(rect.right < 0) ||
			(rect.top < 0) ||
			(rect.bottom < 0))
		{
			control	= NULL;
			visible	= FALSE;
		}
	}

	if ((control) && (flags & COPY_PREFEED))
	{
		strcat(dest, "\r\n");
		dest	+= 2;
	}

	if ((flags & COPY_INDENT) && (visible))
	{
		strcat(dest, "  ");
		dest	+= 2;
	}

	if ((control) && (flags & COPY_BM_GETCHECK))
	{
		i	= SendMessage(control, BM_GETCHECK, 0, 0);

		if (i == BST_CHECKED)
			strcpy(dest, "Y ");
		else
			strcpy(dest, "n ");

		dest	+= 2;
	}

	if (control)
	{
		if (flags & COPY_CB_GETCURSEL)
		{
			index	= (int) SendMessage(control, CB_GETCURSEL, 0, 0);
			SendMessage(control, CB_GETLBTEXT, (WPARAM) index, (LPARAM) dest);
		}
		else
		{
			GetWindowText(control, buf, sizeof(buf));

			if (flags & COPY_WRAP)
			{
				width	= flags & 0xFF;
				psz		= buf;

				for (;;)
				{
					len	= (int) strlen(psz);

					if (len <= width)
					{
						// What remains won't wrap.
						strcat(dest, psz);
						break;
					}

					nl		= strchr(psz, '\n');
					space	= strchr(psz, ' ');

					if ((nl) && ((int) (len - (int) strlen(nl)) <= width))
					{
						// Break at the new line.
						nl[0]	= 0;
						strcat(dest, psz);
						strcat(dest, "\r\n");
						psz	= nl + 1;
						continue;
					}

					if (space == NULL)
					{
						strcat(dest, psz);
						break;
					}

					post	= (int) strlen(space);
					pre		= len - post;

					if (pre > width)
					{
						// The wrap point is to the right of the width.
					}
					else
					{
						// The wrap point at or to the left of the width.
						space	= psz + width;

						for (; space[0] != ' ';)	// Backup to a space.
							space--;

						for (; space[-1] == ' ';)	// Backup over the space.
							space--;
					}

					space[0]	= 0;
					strcat(dest, psz);
					strcat(dest, "\r\n");
					psz	= space + 1;

					for (; psz[0] == ' ';)	// Skip white space.
						psz++;
				}	/*lint !e796 */
			}
			else
			{
				strcat(dest, buf);
			}
		}

		len		= (int) strlen(dest);

		if (flags & COPY_WIDTH)
		{
			width	= flags & 0xFF;

			for (; len < width; len++)
				strcat(dest, " ");
		}

		dest	+= len;
	}
	else if (visible)
	{
		len		= (int) strlen(dest);

		if (flags & COPY_WIDTH)
		{
			width	= flags & 0xFF;

			for (; len < width; len++)
				strcat(dest, " ");
		}

		if (len < 20)
			len		= 20;
	}

	if ((control) && (flags & COPY_ENABLED))
	{
		enabled	= IsWindowEnabled(control);

		if (enabled)
			strcpy(dest, " (enabled)");
		else
			strcpy(dest, " (disabled)");

		dest	+= strlen(dest);
	}
	else if ((control) && (flags & COPY_DISABLED))
	{
		enabled	= IsWindowEnabled(control);

		if (!enabled)
		{
			strcpy(dest, " (disabled)");
			dest	+= strlen(dest);
		}
	}

	if ((flags & COPY_EOL) && (visible))
	{
		strcpy(dest, "\r\n");
		dest	+= 2;
	}

	if ((flags & COPY_UNDERLINE) && (visible))
	{
		for (i = 0; i < len; i++)
			dest[i]	= '=';

		dest[i]	= 0;

		if (flags & COPY_EOL)
			strcat(dest, "\r\n");

		dest	+= strlen(dest);
	}

	return(dest);
}



/*****************************************************************************
*
*	Function:	_copy_item
*
*	Purpose:
*
*		Copy the control/window title to a buffer. If specified, add an end
*		of line sequence. If specified, underline the title text.
*
*	Arguments:
*
*		control		The control/window to access.
*
*		dest		The text is copied here. It is assumed to be large
*					enough for the request.
*
*		flags		Flags that control processing.
*
*	Returned:
*
*		A pointer to the destination buffer's NULL terminator.
*
*****************************************************************************/

static char* _copy_item(HWND control, char* dest, unsigned int flags)
{
	dest[0]	= 0;

	if ((flags & COPY_IF_EXISTS) && (control == NULL))
		;
	else if (flags & COPY_LIST_BOX)
		dest	= _copy_list_box(control, dest, flags);
	else
		dest	= _copy_single(control, dest, flags);

	return(dest);
}



/*****************************************************************************
*
*	Function:	copy_alloc
*
*	Purpose:
*
*		Acquire the resource for a COPY to clipboard operation.
*
*	Arguments:
*
*		wnd			The window handle.
*
*		size		The size of the buffer to use. This MUST be large enough
*					to hold all the text involved.
*
*		gh			The global memory handle allocated is stored here.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

char* copy_alloc(HWND wnd, size_t size, GLOBALHANDLE* gh)
{
	char	buf[256];
	DWORD	dw;
	char*	psz	= NULL;

	for (;;)
	{
		gh[0]	= GlobalAlloc(GMEM_FIXED | GMEM_ZEROINIT, size);

		if (gh[0] == NULL)
		{
			dw	= GetLastError();
			sprintf(buf,
					"GlobalAlloc(, %ld) failure: 0x%lX",
					(long) size,
					(long) dw);
			MessageBox(wnd, buf, "copy_alloc()", MB_ICONEXCLAMATION);
			break;
		}

		psz	= (char*) GlobalLock(gh[0]);

		if (psz == NULL)
		{
			dw	= GetLastError();
			sprintf(buf, "GlobalLock() failure: 0x%lX", (long) dw);
			MessageBox(wnd, buf, "copy_alloc()", MB_ICONEXCLAMATION);
			GlobalFree(gh[0]);
			break;
		}

		break;
	}

	return(psz);
}



/*****************************************************************************
*
*	Function:	copy_list
*
*	Purpose:
*
*		Perform "copy control to text buffer" services on the given list.
*
*	Arguments:
*
*		wnd			The window handle.
*
*		dest		The destination buffer where the text will go. This is
*					presumed to be large enough.
*
*		list		The data to process.
*
*		size		The size of the above list.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

char* copy_list(HWND wnd, char* dest, const copy_t* list, size_t size)
{
	HWND	control;
	int		i;

	for (i = 0; i < (int) size; i++)
	{
		if (list[i].id < 0)
			control	= wnd;
		else
			control	= GetDlgItem(wnd, list[i].id);

		dest	= _copy_item(control, dest, list[i].flags);
	}

	return(dest);
}



/*****************************************************************************
*
*	Function:	copy_list_get_size
*
*	Purpose:
*
*		Compute the size of the buffer needed for the given data list.
*
*	Arguments:
*
*		wnd			The window handle.
*
*		list		The structure list for the controls and such to copy.
*					It is assumed that the referenced controls and such
*					remain unchanged while this is performed.
*
*		size		The size of the list.
*
*	Returned:
*
*		The size in bytes, plus a little fudging.
*
*****************************************************************************/

size_t copy_list_get_size(HWND wnd, const copy_t* list, size_t size)
{
	int		i;
	size_t	len;
	size_t	length	= 0;

	for (i = 0; i < (int) size; i++)
	{
		copy_list(wnd, _buffer, &list[i], 1);
		len		= strlen(_buffer) + 2;
		length	+= len;
	}

	return(length);
}



/*****************************************************************************
*
*	Function:	copy_list_to_clipboard
*
*	Purpose:
*
*		Per the give list, copy the corresponding text to the clipboard.
*
*	Arguments:
*
*		wnd			The window handle.
*
*		list		The structure list for the controls and such to copy.
*					It is assumed that the referenced controls and such
*					remain unchanged while this is performed.
*
*		size		The size of the list.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void copy_list_to_clipboard(HWND wnd, const copy_t* list, size_t size)
{
	GLOBALHANDLE	gh;
	size_t			length	= 0;
	char*			psz;

	length	= copy_list_get_size(wnd, list, size);
	psz		= copy_alloc(wnd, length, &gh);

	if (psz)
	{
		copy_list(wnd, psz, list, size);
		copy_perform(wnd, gh);
	}
}



/*****************************************************************************
*
*	Function:	copy_perform
*
*	Purpose:
*
*		Perform the actula COPY to the clipboard.
*
*	Arguments:
*
*		wnd			The window handle.
*
*		gh			The global memory handle.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void copy_perform(HWND wnd, GLOBALHANDLE gh)
{
	GlobalUnlock(gh);
	OpenClipboard(wnd);
	EmptyClipboard();
	SetClipboardData(CF_TEXT, gh);
	CloseClipboard();
}


