// HPDI32 - Application - Win32 - Any Compiler
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/SDK/hpdi32/ringbuf/win32/gui.c $
// $Rev: 2902 $
// $Date: 2009-08-09 17:48:49 -0500 (Sun, 09 Aug 2009) $

#include <windows.h>
#include <process.h>
#include <stdio.h>
#include <time.h>

#include "gui.h"
#include "proc.h"
#include "ringbuf.h"
#include "resource.h"
#include "hpdi32_util.h"
#include "gsc_util.h"



/* variables	*************************************************************/

		gui_t		gui;

static	time_t		_time_last;
static	time_t		_time_start;



/*****************************************************************************
*
*	Function:	_display_refresh
*
*	Purpose:
*
*		Refresh the window display content.
*
*	Arguments:
*
*		None.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _display_refresh(void)
{
	device_list_refresh();
	buffer_type_refresh();
	buffer_qty_refresh();
	buffer_size_refresh();
	start_refresh();
	stop_refresh();
	copy_refresh();
	close_refresh();

	// The TX menu.
	tx_operation_refresh();

	tx_io_mode_refresh();
	tx_io_dma_ch_sel_refresh();
	tx_io_dma_pri_refresh();
	tx_io_dma_ctrl_refresh();
	tx_io_data_size_refresh();
	tx_io_sc_refresh();
	tx_io_pio_threshold_refresh();
	tx_io_timeout_refresh();

	tx_fifo_ae_refresh();
	tx_fifo_af_refresh();

	tx_auto_start_refresh();
	tx_auto_stop_refresh();
	tx_remote_throttle_refresh();
	tx_clock_divider_refresh();
	tx_status_valid_count_refresh();
	tx_status_valid_mirror_refresh();
	tx_line_valid_off_count_refresh();
	tx_line_valid_on_count_refresh();
	tx_enable_refresh();

	// The RX menu.
	rx_operation_refresh();

	rx_io_mode_refresh();
	rx_io_dma_ch_sel_refresh();
	rx_io_dma_pri_refresh();
	rx_io_dma_ctrl_refresh();
	rx_io_data_size_refresh();
	rx_io_sc_refresh();
	rx_io_pio_threshold_refresh();
	rx_io_timeout_refresh();

	rx_fifo_ae_refresh();
	rx_fifo_af_refresh();

	rx_enable_refresh();

	// The CABLE menu.
	cable_cmd_0_refresh();
	cable_cmd_1_refresh();
	cable_cmd_2_refresh();
	cable_cmd_3_refresh();
	cable_cmd_4_refresh();
	cable_cmd_5_refresh();
	cable_cmd_6_refresh();

	// The MISC menu.
	favor_tx_refresh();
	tri_state_refresh();
}



/*****************************************************************************
*
*	Function:	_idc_start
*
*	Purpose:
*
*		Process WM_COMMAND messages for the START button.
*
*	Arguments:
*
*		None.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idc_start(void)
{
	int	errs;

	status_reset();
	gui.stop_req	= FALSE;
	gui.working		= TRUE;
	_display_refresh();
	errs			= process_start((U8) gui.dev_index,
									gui.api_buf,
									gui.buf_qty,
									gui.buf_size,
									gui.tx_op,
									gui.rx_op,
									gui.rx_file);

	if (errs)
	{
		gui.working	= FALSE;
		_time_start	= 0;
		_display_refresh();
	}
	else
	{
		_time_start	= time(NULL);
	}
}



/*****************************************************************************
*
*	Function:	_wm_command
*
*	Purpose:
*
*		Process a WM_COMMAND message for the primary window.
*
*	Arguments:
*
*		wnd		The window handle.
*
*		wParam	The message's word sized parameter.
*
*		lParam	The message's long sized parameter.
*
*	Returned:
*
*		0		We handled the message.
*		else	Windows should process this message as we didn't.
*
*****************************************************************************/

static LRESULT _wm_command(HWND wnd, WPARAM wParam, LPARAM lParam)
{
	int		id		= LOWORD(wParam);
	LRESULT	ret		= 0;

	switch (id)
	{
		default:						ret	= 1;
										break;

		case IDC_BUF_TYPE_API:
		case IDC_BUF_TYPE_APP:			buffer_type_wm_command(id);
										_display_refresh();
										break;

		case IDC_CABLE_CMD_0_FC:
		case IDC_CABLE_CMD_0_HI:
		case IDC_CABLE_CMD_0_IN:
		case IDC_CABLE_CMD_0_LO:		cable_cmd_0_wm_command(id);
										break;

		case IDC_CABLE_CMD_1_FC:
		case IDC_CABLE_CMD_1_HI:
		case IDC_CABLE_CMD_1_IN:
		case IDC_CABLE_CMD_1_LO:		cable_cmd_1_wm_command(id);
										break;

		case IDC_CABLE_CMD_2_FC:
		case IDC_CABLE_CMD_2_HI:
		case IDC_CABLE_CMD_2_IN:
		case IDC_CABLE_CMD_2_LO:		cable_cmd_2_wm_command(id);
										break;

		case IDC_CABLE_CMD_3_FC:
		case IDC_CABLE_CMD_3_HI:
		case IDC_CABLE_CMD_3_IN:
		case IDC_CABLE_CMD_3_LO:		cable_cmd_3_wm_command(id);
										break;

		case IDC_CABLE_CMD_4_FC:
		case IDC_CABLE_CMD_4_HI:
		case IDC_CABLE_CMD_4_IN:
		case IDC_CABLE_CMD_4_LO:		cable_cmd_4_wm_command(id);
										break;

		case IDC_CABLE_CMD_5_FC:
		case IDC_CABLE_CMD_5_HI:
		case IDC_CABLE_CMD_5_IN:
		case IDC_CABLE_CMD_5_LO:		cable_cmd_5_wm_command(id);
										break;

		case IDC_CABLE_CMD_6_FC:
		case IDC_CABLE_CMD_6_HI:
		case IDC_CABLE_CMD_6_IN:
		case IDC_CABLE_CMD_6_LO:		cable_cmd_6_wm_command(id);
										break;

		case IDC_COPY:					copy_wm_command();
										break;

		case IDC_DEVICE_LIST:			device_list_wm_command(wParam);
										break;

		case IDC_FAVOR_TX_NO:
		case IDC_FAVOR_TX_YES:			favor_tx_wm_command(id);
										break;

		case IDC_QTY_DEC:
		case IDC_QTY_INC:				buffer_qty_wm_command(id);
										break;

		case IDC_RX_ENABLE_NO:
		case IDC_RX_ENABLE_YES:			rx_enable_wm_command(id);
										break;

		case IDC_RX_FIFO_AE:			rx_fifo_ae_wm_command();
										break;

		case IDC_RX_FIFO_AF:			rx_fifo_af_wm_command();
										break;

		case IDC_RX_IO_DATA_SIZE_08:
		case IDC_RX_IO_DATA_SIZE_16:
		case IDC_RX_IO_DATA_SIZE_32:	rx_io_data_size_wm_command(id);
										break;

		case IDC_RX_IO_DMA_CNTL_MAN:
		case IDC_RX_IO_DMA_CNTL_AUTO:	rx_io_dma_ctrl_wm_command(id);
										break;

		case IDC_RX_IO_DMA_PRI_DIS:
		case IDC_RX_IO_DMA_PRI_ENA:		rx_io_dma_pri_wm_command(id);
										break;

		case IDC_RX_IO_DMA_SEL_DYN:
		case IDC_RX_IO_DMA_SEL_STA:		rx_io_dma_ch_sel_wm_command(id);
										break;

		case IDC_RX_IO_MODE_PIO:
		case IDC_RX_IO_MODE_DMA:
		case IDC_RX_IO_MODE_DMDMA:		rx_io_mode_wm_command(id);
										break;

		case IDC_RX_IO_PIO_THRESH:		rx_io_pio_threshold_wm_command();
										break;

		case IDC_RX_IO_SC_ABS:
		case IDC_RX_IO_SC_PRE:			rx_io_sc_wm_command(id);
										break;

		case IDC_RX_IO_TIMEOUT:			rx_io_timeout_wm_command();
										break;

		case IDC_RX_OP_INCREMENT:
		case IDC_RX_OP_LOAD:
		case IDC_RX_OP_RECEIVE:			rx_operation_wm_command(id);
										break;

		case IDC_SIZE_EDIT:				buffer_size_wm_command(wParam);
										break;

		case IDC_START:					_idc_start();
										break;

		case IDC_STOP:					stop_wm_command();
										_display_refresh();
										break;

		case IDC_TRI_STATE_NO:
		case IDC_TRI_STATE_YES:			tri_state_wm_command(id);
										break;

		case IDC_TX_AUTO_START_NO:
		case IDC_TX_AUTO_START_YES:		tx_auto_start_wm_command(id);
										break;

		case IDC_TX_AUTO_STOP_NO:
		case IDC_TX_AUTO_STOP_YES:		tx_auto_stop_wm_command(id);
										break;

		case IDC_TX_CLOCK_DIVIDER:		tx_clock_divider_wm_command();
										break;

		case IDC_TX_ENABLE_NO:
		case IDC_TX_ENABLE_YES:			tx_enable_wm_command(id);
										break;

		case IDC_TX_FIFO_AE:			tx_fifo_ae_wm_command();
										break;

		case IDC_TX_FIFO_AF:			tx_fifo_af_wm_command();
										break;

		case IDC_TX_IO_DATA_SIZE_08:
		case IDC_TX_IO_DATA_SIZE_16:
		case IDC_TX_IO_DATA_SIZE_32:	tx_io_data_size_wm_command(id);
										break;

		case IDC_TX_IO_DMA_CNTL_MAN:
		case IDC_TX_IO_DMA_CNTL_AUTO:	tx_io_dma_ctrl_wm_command(id);
										break;

		case IDC_TX_IO_DMA_SEL_DYN:
		case IDC_TX_IO_DMA_SEL_STA:		tx_io_dma_ch_sel_wm_command(id);
										break;

		case IDC_TX_IO_DMA_PRI_DIS:
		case IDC_TX_IO_DMA_PRI_ENA:		tx_io_dma_pri_wm_command(id);
										break;

		case IDC_TX_IO_MODE_PIO:
		case IDC_TX_IO_MODE_DMA:
		case IDC_TX_IO_MODE_DMDMA:		tx_io_mode_wm_command(id);
										break;

		case IDC_TX_IO_PIO_THRESH:		tx_io_pio_threshold_wm_command();
										break;

		case IDC_TX_IO_SC_ABS:
		case IDC_TX_IO_SC_PRE:			tx_io_sc_wm_command(id);
										break;

		case IDC_TX_IO_TIMEOUT:			tx_io_timeout_wm_command();
										break;

		case IDC_TX_LV_OFF_CNT:			tx_line_valid_off_count_wm_command();
										break;

		case IDC_TX_LV_ON_CNT:			tx_line_valid_on_count_wm_command();
										break;

		case IDC_TX_OP_DISCARD:
		case IDC_TX_OP_SAVE:
		case IDC_TX_OP_XMIT:			tx_operation_wm_command(id);
										break;

		case IDC_TX_REMOTE_DIS:
		case IDC_TX_REMOTE_ENA:			tx_remote_throttle_wm_command(id);
										break;

		case IDC_TX_STS_VAL_CNT:		tx_status_valid_count_wm_command();
										break;

		case IDC_TX_STS_VAL_MIR_DIS:
		case IDC_TX_STS_VAL_MIR_ENA:	tx_status_valid_mirror_wm_command(id);
										break;

		case IDCANCEL:					PostQuitMessage(0);
										break;
	}

	return(ret);
}



/*****************************************************************************
*
*	Function:	_wm_getminmaxinfo
*
*	Purpose:
*
*		Process a WM_GETMINMAXINFO message.
*
*	Arguments:
*
*		wnd		The window handle.
*
*		lParam	The long sized parameter.
*
*	Returned:
*
*		0		All went well.
*		-1		There was a problem.
*
*****************************************************************************/

static LRESULT _wm_getminmaxinfo(HWND wnd, LPARAM lParam)
{
	HWND		control;
	RECT		client;
	MINMAXINFO*	mmi		= (MINMAXINFO*) lParam;
	RECT		outter;
	RECT		rect;
	HWND		status;

	GetWindowRect(wnd, &outter);
	GetClientRect(wnd, &client);
	control	= GetDlgItem(wnd, IDC_COPY);
	status	= GetDlgItem(wnd, IDC_STATUS);

	// Minimim Width
	mmi->ptMinTrackSize.x	= RECT_WIDTH(outter) - RECT_WIDTH(client);
	GetWindowRect(status, &rect);
	ScreenToClient(wnd, (LPPOINT) &rect.left);
	mmi->ptMinTrackSize.x	+= rect.left;
	GetWindowRect(control, &rect);
	ScreenToClient(wnd, (LPPOINT) &rect.right);
	mmi->ptMinTrackSize.x	+= rect.right;

	// Minimum Height
	mmi->ptMinTrackSize.y	= RECT_HEIGHT(outter) - RECT_HEIGHT(client);
	GetWindowRect(status, &rect);
	ScreenToClient(wnd, (LPPOINT) &rect.left);
	mmi->ptMinTrackSize.y	+= rect.top;
	GetWindowRect(control, &rect);
	ScreenToClient(wnd, (LPPOINT) &rect.left);
	ScreenToClient(wnd, (LPPOINT) &rect.right);
	mmi->ptMinTrackSize.y	+= RECT_HEIGHT(rect) * 2;
	return(0);
}



/*****************************************************************************
*
*	Function:	_wm_initdialog
*
*	Purpose:
*
*		Process a WM_INITDIALOG message.
*
*	Arguments:
*
*		wnd		The window handle.
*
*	Returned:
*
*		0		All went well.
*		-1		There was a problem.
*
*****************************************************************************/

static LRESULT _wm_initdialog(HWND wnd)
{
	int		h;
	RECT	rect;
	int		sh;
	int		sw;
	int		x;
	int		y;
	int		w;

	gui.dlg		= wnd;
	gui.menu	= GetMenu(wnd);
	gui.icon_sm	= LoadIcon(gui.instance, MAKEINTRESOURCE(IDI_RB_1));
	gui.icon_lg	= LoadIcon(gui.instance, MAKEINTRESOURCE(IDI_RB_2));

	SetWindowText(wnd, APP_TITLE);
	SendMessage(wnd, WM_SETICON, ICON_SMALL, (LPARAM) gui.icon_sm);
	SendMessage(wnd, WM_SETICON, ICON_BIG, (LPARAM) gui.icon_lg);

	gsc_label_init(34);
	gsc_os_ini_init("ringbuffer");

	// Initialize the various controls.
	device_list_init();
	buffer_type_init();
	buffer_qty_init();
	buffer_size_init();
	start_init();
	stop_init();
	copy_init();
	status_init();

	// The TX menu.
	tx_operation_init();

	tx_io_mode_init();
	tx_io_dma_ch_sel_init();
	tx_io_dma_pri_init();
	tx_io_dma_ctrl_init();
	tx_io_data_size_init();
	tx_io_sc_init();
	tx_io_pio_threshold_init();
	tx_io_timeout_init();

	tx_fifo_ae_init();
	tx_fifo_af_init();

	tx_auto_start_init();
	tx_auto_stop_init();
	tx_remote_throttle_init();
	tx_clock_divider_init();
	tx_status_valid_count_init();
	tx_status_valid_mirror_init();
	tx_line_valid_off_count_init();
	tx_line_valid_on_count_init();
	tx_enable_init();

	// The RX menu.
	rx_operation_init();

	rx_io_mode_init();
	rx_io_dma_ch_sel_init();
	rx_io_dma_pri_init();
	rx_io_dma_ctrl_init();
	rx_io_data_size_init();
	rx_io_sc_init();
	rx_io_pio_threshold_init();
	rx_io_timeout_init();

	rx_fifo_ae_init();
	rx_fifo_af_init();

	rx_enable_init();

	// The CABLE menu.
	cable_cmd_0_init();
	cable_cmd_1_init();
	cable_cmd_2_init();
	cable_cmd_3_init();
	cable_cmd_4_init();
	cable_cmd_5_init();
	cable_cmd_6_init();

	// The MISC menu.
	favor_tx_init();
	tri_state_init();

	// Start the timer.
	SetTimer(wnd, 1, 250, NULL);

	/* Center the window on the screen. */
	GetWindowRect(wnd, &rect);
	sw	= GetSystemMetrics(SM_CXSCREEN);
	sh	= GetSystemMetrics(SM_CYSCREEN);
	w	= RECT_WIDTH(rect);;
	h	= RECT_HEIGHT(rect);;
	x	= (sw - w) / 2;
	y	= (sh - h) / 2;
	MoveWindow(wnd, x, y, w, h, FALSE);

	// Initialize the display.
	_display_refresh();
	return(0);
}



/*****************************************************************************
*
*	Function:	_wm_size
*
*	Purpose:
*
*		Process a WM_SIZE message.
*
*	Arguments:
*
*		wnd		The window handle.
*
*		wParam	The word sized parameter.
*
*		lParam	The long sized parameter.
*
*	Returned:
*
*		0		We processed the message.
*		else	We didn't process the message.
*
*****************************************************************************/

static LRESULT _wm_size(HWND wnd, WPARAM wParam, LPARAM lParam)
{
	HWND	control;
	int		h;
	RECT	rect;
	int		w;
	int		x;
	int		y;
	int		wh;
	int		ww;

	if (wParam == SIZE_MINIMIZED)
	{
		// Do nothing.
	}
	else
	{
		ww	= LOWORD(lParam);
		wh	= HIWORD(lParam);

		// Resize the status list box.
		control	= GetDlgItem(wnd, IDC_STATUS);
		GetWindowRect(control, &rect);
		ScreenToClient(wnd, (LPPOINT) &rect.left);
		ScreenToClient(wnd, (LPPOINT) &rect.right);
		x	= rect.left;
		y	= rect.top;
		w	= ww - (2 * rect.left);
		h	= wh - rect.top - rect.left;
		MoveWindow(control, x, y, w, h, FALSE);

		// Redraw the window.
		InvalidateRect(wnd, NULL, TRUE);
	}

	return(0);
}



/*****************************************************************************
*
*	Function:	_wm_timer
*
*	Purpose:
*
*		Process a WM_TIMER message.
*
*	Arguments:
*
*		wnd		The window handle.
*
*		wParam	The word sized parameter.
*
*	Returned:
*
*		0		We processed the message.
*		else	We didn't process the message.
*
*****************************************************************************/

static LRESULT _wm_timer(HWND wnd, WPARAM wParam)
{
	char	buf[128];
	int		done;
	size_t	len;
	time_t	now;
	char*	psz;

	if (gui.working)
	{
		done	= process_done();

		if (done)
		{
			gui.working	= FALSE;
			_display_refresh();
		}
		else
		{
			now	= time(NULL);

			if (now != _time_last)
			{
				_time_last	= now;
				now			-= _time_start;
				GetWindowText(wnd, buf, sizeof(buf));
				psz	= strchr(buf, '(');

				if (psz)
					psz	= strchr(psz + 1, '(');

				if (psz)
					psz[-1]	= 0;

				strcat(buf, " (");
				len	= strlen(buf);
				psz	= buf + len;
				len	= sizeof(buf) - len;
				gsc_format_time_sec((long) now, psz, len);
				strcat(buf, ")");
				SetWindowText(wnd, buf);
			}
		}
	}

	return(0);
}



/*****************************************************************************
*
*	Function:	_proc_dlg
*
*	Purpose:
*
*		Processes messages for the About window.
*
*	Arguments:
*
*		wnd			The window handle.
*
*		msg			The message to process.
*
*		wParam		The message's word sized parameter.
*
*		lParam		The message's long sized parameter.
*
*	Returned:
*
*		0			We failed.
*		else		All went well.
*
*****************************************************************************/

static LRESULT CALLBACK _proc_dlg(	HWND	wnd,
									UINT	msg,
									WPARAM	wParam,
									LPARAM	lParam)
{
	LRESULT	ret;

	switch (msg)
	{
		default:				ret	= FALSE;
								break;

		case WM_COMMAND:		ret	= _wm_command(wnd, wParam, lParam);
								break;

		case WM_GETMINMAXINFO:	ret	= _wm_getminmaxinfo(wnd, lParam);
								break;

		case WM_INITDIALOG:		ret	= _wm_initdialog(wnd);
								break;

		case WM_SIZE:			ret	= _wm_size(wnd, wParam, lParam);
								break;

		case WM_TIMER:			ret	= _wm_timer(wnd, wParam);
								break;
	}

	return(ret);
}



/*****************************************************************************
*
*	Function:	gui_init
*
*	Purpose:
*
*		Perform a one-time initialization of the primary window code.
*
*	Arguments:
*
*		instance	The application instance handle.
*
*	Returned:
*
*		TRUE		All went well.
*		FALSE		We failed.
*
*****************************************************************************/

BOOL gui_init(HINSTANCE instance)
{
	gui.instance	= instance;
	return(TRUE);
}



/*****************************************************************************
*
*	Function:	gui_open
*
*	Purpose:
*
*		Open the primary window.
*
*	Arguments:
*
*		None.
*
*	Returned:
*
*		N0one.
*
*****************************************************************************/

void gui_open(void)
{
	int	errs;

	errs	= gsc_buf_man_init();

	if (errs)
		MessageBox(NULL, "FAIL: gsc_buf_man_init()", APP_TITLE, MB_ICONERROR);
	else
		DialogBox(gui.instance, (LPCTSTR) IDD_GUI, NULL, (DLGPROC) _proc_dlg);
}


