// HPDI32 - Application - Win32 - Any Compiler
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/SDK/hpdi32/ringbuf/win32/gui_cbl_cmd_2.c $
// $Rev: 2809 $
// $Date: 2009-07-29 21:26:35 -0500 (Wed, 29 Jul 2009) $

#include <windows.h>
#include <stdio.h>

#include "gui.h"
#include "resource.h"
#include "hpdi32_util.h"
#include "gsc_util.h"



/* #defines	*****************************************************************/

#define	KEY	"HPDI32_CABLE_COMMAND_MODE__2"



/* variables	*************************************************************/

static	HMENU		_menu;
static	const char*	_name;
static	int			_setting;



/*****************************************************************************
*
*	Function:	_update_and_save
*
*	Purpose:
*
*		Update the menu check state and save the setting.
*
*	Arguments:
*
*		value	The current setting.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _update_and_save(int value)
{
	switch (value)
	{
		default:
		case HPDI32_CABLE_COMMAND_MODE_FLOW_CONTROL:

			_name		= "Status Valid: HPDI32_CABLE_COMMAND_MODE_FLOW_CONTROL";
			_setting	= HPDI32_CABLE_COMMAND_MODE_FLOW_CONTROL;
			gui_menu_add_item_text(_menu, IDC_CABLE_CMD_2_FC);
			CheckMenuItem(_menu, IDC_CABLE_CMD_2_FC, MF_BYCOMMAND | MF_CHECKED);
			CheckMenuItem(_menu, IDC_CABLE_CMD_2_IN, MF_BYCOMMAND | MF_UNCHECKED);
			CheckMenuItem(_menu, IDC_CABLE_CMD_2_LO, MF_BYCOMMAND | MF_UNCHECKED);
			CheckMenuItem(_menu, IDC_CABLE_CMD_2_HI, MF_BYCOMMAND | MF_UNCHECKED);
			break;

		case HPDI32_CABLE_COMMAND_MODE_GPIO_IN:

			_name		= "GPIO 1 Input: HPDI32_CABLE_COMMAND_MODE_GPIO_IN";
			_setting	= HPDI32_CABLE_COMMAND_MODE_GPIO_IN;
			gui_menu_add_item_text(_menu, IDC_CABLE_CMD_2_IN);
			CheckMenuItem(_menu, IDC_CABLE_CMD_2_FC, MF_BYCOMMAND | MF_UNCHECKED);
			CheckMenuItem(_menu, IDC_CABLE_CMD_2_IN, MF_BYCOMMAND | MF_CHECKED);
			CheckMenuItem(_menu, IDC_CABLE_CMD_2_LO, MF_BYCOMMAND | MF_UNCHECKED);
			CheckMenuItem(_menu, IDC_CABLE_CMD_2_HI, MF_BYCOMMAND | MF_UNCHECKED);
			break;

		case HPDI32_CABLE_COMMAND_MODE_GPIO_OUT_LOW:

			_name		= "GPIO 1 Output Low: HPDI32_CABLE_COMMAND_MODE_GPIO_OUT_LOW";
			_setting	= HPDI32_CABLE_COMMAND_MODE_GPIO_OUT_LOW;
			gui_menu_add_item_text(_menu, IDC_CABLE_CMD_2_LO);
			CheckMenuItem(_menu, IDC_CABLE_CMD_2_FC, MF_BYCOMMAND | MF_UNCHECKED);
			CheckMenuItem(_menu, IDC_CABLE_CMD_2_IN, MF_BYCOMMAND | MF_UNCHECKED);
			CheckMenuItem(_menu, IDC_CABLE_CMD_2_LO, MF_BYCOMMAND | MF_CHECKED);
			CheckMenuItem(_menu, IDC_CABLE_CMD_2_HI, MF_BYCOMMAND | MF_UNCHECKED);
			break;

		case HPDI32_CABLE_COMMAND_MODE_GPIO_OUT_HI:

			_name		= "GPIO 1 Output High: HPDI32_CABLE_COMMAND_MODE_GPIO_OUT_HI";
			_setting	= HPDI32_CABLE_COMMAND_MODE_GPIO_OUT_HI;
			gui_menu_add_item_text(_menu, IDC_CABLE_CMD_2_HI);
			CheckMenuItem(_menu, IDC_CABLE_CMD_2_FC, MF_BYCOMMAND | MF_UNCHECKED);
			CheckMenuItem(_menu, IDC_CABLE_CMD_2_IN, MF_BYCOMMAND | MF_UNCHECKED);
			CheckMenuItem(_menu, IDC_CABLE_CMD_2_LO, MF_BYCOMMAND | MF_UNCHECKED);
			CheckMenuItem(_menu, IDC_CABLE_CMD_2_HI, MF_BYCOMMAND | MF_CHECKED);
			break;
	}

	gsc_os_ini_int_set(NULL, NULL, KEY, _setting);
}



/*****************************************************************************
*
*	Function:	cable_cmd_2_init
*
*	Purpose:
*
*		Initialize things for the Cable Command 2 option.
*
*	Arguments:
*
*		None.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void cable_cmd_2_init(void)
{
	int		i;

	_menu	= GetMenu(gui.dlg);
	_menu	= gui_menu_locate_parent_by_id(_menu, IDC_CABLE_CMD_2_FC);
	gsc_os_ini_int_get(NULL, NULL, KEY, &i, HPDI32_CABLE_COMMAND_MODE_DEFAULT);
	_update_and_save(i);
}



/*****************************************************************************
*
*	Function:	cable_cmd_2_refresh
*
*	Purpose:
*
*		Refresh the Cable Command 2 menu items.
*
*	Arguments:
*
*		None.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void cable_cmd_2_refresh(void)
{
	if (gui.working)
	{
		EnableMenuItem(_menu, IDC_CABLE_CMD_2_FC, MF_BYCOMMAND | MF_GRAYED);
		EnableMenuItem(_menu, IDC_CABLE_CMD_2_IN, MF_BYCOMMAND | MF_GRAYED);
		EnableMenuItem(_menu, IDC_CABLE_CMD_2_LO, MF_BYCOMMAND | MF_GRAYED);
		EnableMenuItem(_menu, IDC_CABLE_CMD_2_HI, MF_BYCOMMAND | MF_GRAYED);
	}
	else
	{
		EnableMenuItem(_menu, IDC_CABLE_CMD_2_FC, MF_BYCOMMAND | MF_ENABLED);
		EnableMenuItem(_menu, IDC_CABLE_CMD_2_IN, MF_BYCOMMAND | MF_ENABLED);
		EnableMenuItem(_menu, IDC_CABLE_CMD_2_LO, MF_BYCOMMAND | MF_ENABLED);
		EnableMenuItem(_menu, IDC_CABLE_CMD_2_HI, MF_BYCOMMAND | MF_ENABLED);
	}
}



/*****************************************************************************
*
*	Function:	cable_cmd_2_wm_command
*
*	Purpose:
*
*		Process a WM_COMMAND message for a Cable Command 2 control or menu
*		items.
*
*	Arguments:
*
*		id		The id of the control or menu item selected.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void cable_cmd_2_wm_command(int id)
{
	switch (id)
	{
		default:
		case IDC_CABLE_CMD_2_FC:

			id	= HPDI32_CABLE_COMMAND_MODE_FLOW_CONTROL;
			break;

		case IDC_CABLE_CMD_2_IN:

			id	= HPDI32_CABLE_COMMAND_MODE_GPIO_IN;
			break;

		case IDC_CABLE_CMD_2_LO:

			id	= HPDI32_CABLE_COMMAND_MODE_GPIO_OUT_LOW;
			break;

		case IDC_CABLE_CMD_2_HI:

			id	= HPDI32_CABLE_COMMAND_MODE_GPIO_OUT_HI;
			break;
	}

	_update_and_save(id);
}



/*****************************************************************************
*
*	Function:	cable_cmd_2_config
*
*	Purpose:
*
*		Configure the given device's Cable Command 2 parameter.
*
*	Arguments:
*
*		dev		The handle for the device to access.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

int cable_cmd_2_config(void* dev)
{
	int	errs	= 0;
	U32	get		= HPDI32_MISC_FEATURES_PRESENT;
	U32	status;

	gsc_label_text(KEY);

	if (_setting != HPDI32_CABLE_COMMAND_MODE_FLOW_CONTROL)
	{
		status	= HPDI32_MISC_FEATURES__GPIO_0_5(dev, &get);
		errs	= label_fail_func(1, "HPDI32_MISC_FEATURES__GPIO_0_5", status);
	}

	if (errs)
	{
	}
	else if (	(_setting != HPDI32_CABLE_COMMAND_MODE_FLOW_CONTROL) &&
				(get == HPDI32_MISC_FEATURES_ABSENT))
	{
		errs++;
		printf("FAIL <---  (selection not supported)\n");
	}
	else
	{
		status	= HPDI32_CABLE_COMMAND_MODE__2_SET(dev, _setting);
		errs	= label_fail_func(1, "HPDI32_CABLE_COMMAND_MODE__2_SET", status);
	}

	if (errs == 0)
		printf("PASS  (%s)\n", _name);

	return(errs);
}


