// HPDI32 - Application - Win32 - Any Compiler
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/SDK/hpdi32/ringbuf/win32/gui_rx_io_dma_pri.c $
// $Rev: 2809 $
// $Date: 2009-07-29 21:26:35 -0500 (Wed, 29 Jul 2009) $

#include <windows.h>
#include <stdio.h>

#include "gui.h"
#include "resource.h"
#include "hpdi32_util.h"
#include "gsc_util.h"



/* #defines	*****************************************************************/

#define	KEY	"HPDI32_IO_DMA_PRIORITY__RX"



/* variables	*************************************************************/

static	HMENU		_menu;
static	const char*	_name;
static	int			_setting;



/*****************************************************************************
*
*	Function:	_update_and_save
*
*	Purpose:
*
*		Update the menu check state and save the setting.
*
*	Arguments:
*
*		value	The current setting.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _update_and_save(int value)
{
	switch (value)
	{
		default:
		case HPDI32_IO_DMA_PRIORITY_DISABLE:

			_name		= "HPDI32_IO_DMA_PRIORITY_DISABLE";
			_setting	= HPDI32_IO_DMA_PRIORITY_DISABLE;
			gui_menu_add_item_text(_menu, IDC_RX_IO_DMA_PRI_DIS);
			CheckMenuItem(_menu, IDC_RX_IO_DMA_PRI_DIS, MF_BYCOMMAND | MF_CHECKED);
			CheckMenuItem(_menu, IDC_RX_IO_DMA_PRI_ENA, MF_BYCOMMAND | MF_UNCHECKED);
			break;

		case HPDI32_IO_DMA_PRIORITY_ENABLE:

			_name		= "HPDI32_IO_DMA_PRIORITY_ENABLE";
			_setting	= HPDI32_IO_DMA_PRIORITY_ENABLE;
			gui_menu_add_item_text(_menu, IDC_RX_IO_DMA_PRI_ENA);
			CheckMenuItem(_menu, IDC_RX_IO_DMA_PRI_DIS, MF_BYCOMMAND | MF_UNCHECKED);
			CheckMenuItem(_menu, IDC_RX_IO_DMA_PRI_ENA, MF_BYCOMMAND | MF_CHECKED);
			break;
	}

	gsc_os_ini_int_set(NULL, NULL, KEY, _setting);
}



/*****************************************************************************
*
*	Function:	rx_io_dma_pri_init
*
*	Purpose:
*
*		Initialize things for the Rx I/O DMA Priority option.
*
*	Arguments:
*
*		None.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void rx_io_dma_pri_init(void)
{
	int		i;

	_menu	= GetMenu(gui.dlg);
	_menu	= gui_menu_locate_parent_by_id(_menu, IDC_RX_IO_DMA_PRI_DIS);
	gsc_os_ini_int_get(NULL, NULL, KEY, &i, HPDI32_IO_DMA_PRIORITY_RX_DEFAULT);
	_update_and_save(i);
}



/*****************************************************************************
*
*	Function:	rx_io_dma_pri_refresh
*
*	Purpose:
*
*		Refresh the Rx I/O DMA Priority menu items.
*
*	Arguments:
*
*		None.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void rx_io_dma_pri_refresh(void)
{
	if (gui.working)
	{
		EnableMenuItem(_menu, IDC_RX_IO_DMA_PRI_DIS, MF_BYCOMMAND | MF_GRAYED);
		EnableMenuItem(_menu, IDC_RX_IO_DMA_PRI_ENA, MF_BYCOMMAND | MF_GRAYED);
	}
	else
	{
		EnableMenuItem(_menu, IDC_RX_IO_DMA_PRI_DIS, MF_BYCOMMAND | MF_ENABLED);
		EnableMenuItem(_menu, IDC_RX_IO_DMA_PRI_ENA, MF_BYCOMMAND | MF_ENABLED);
	}
}



/*****************************************************************************
*
*	Function:	rx_io_dma_pri_wm_command
*
*	Purpose:
*
*		Process a WM_COMMAND message for a Rx I/O DMA Priority control or menu
*		items.
*
*	Arguments:
*
*		id		The id of the control or menu item selected.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void rx_io_dma_pri_wm_command(int id)
{
	switch (id)
	{
		default:
		case IDC_RX_IO_DMA_PRI_DIS:

			id	= HPDI32_IO_DMA_PRIORITY_DISABLE;
			break;

		case IDC_RX_IO_DMA_PRI_ENA:

			id	= HPDI32_IO_DMA_PRIORITY_ENABLE;
			break;
	}

	_update_and_save(id);
}



/*****************************************************************************
*
*	Function:	rx_io_dma_pri_config
*
*	Purpose:
*
*		Configure the given device's Rx I/O DMA Priority parameter.
*
*	Arguments:
*
*		dev		The handle for the device to access.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

int rx_io_dma_pri_config(void* dev)
{
	int	errs	= 0;
	U32	status;

	gsc_label_text(KEY);
	status	= HPDI32_IO_DMA_PRIORITY__RX_SET(dev, _setting);
	errs	= label_fail_func(1, "HPDI32_IO_DMA_PRIORITY__RX_SET", status);

	if (errs == 0)
		printf("PASS  (%s)\n", _name);

	return(errs);
}


