// HPDI32 - Application - Win32 - Any Compiler
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/SDK/hpdi32/ringbuf/win32/gui_tri_state.c $
// $Rev: 2809 $
// $Date: 2009-07-29 21:26:35 -0500 (Wed, 29 Jul 2009) $

#include <windows.h>
#include <stdio.h>

#include "gui.h"
#include "resource.h"
#include "hpdi32_util.h"
#include "gsc_util.h"



/* #defines	*****************************************************************/

#define	KEY	"HPDI32_MISC_TX_RX_TRI_STATE"



/* variables	*************************************************************/

static	HMENU		_menu;
static	const char*	_name;
static	int			_setting;



/*****************************************************************************
*
*	Function:	_update_and_save
*
*	Purpose:
*
*		Update the menu check state and save the setting.
*
*	Arguments:
*
*		value	The current setting.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _update_and_save(int value)
{
	if (value == HPDI32_MISC_TX_RX_TRI_STATE_DISABLE)
	{
		_name		= "HPDI32_MISC_TX_RX_TRI_STATE_DISABLE";
		_setting	= HPDI32_MISC_TX_RX_TRI_STATE_DISABLE;
		gui_menu_add_item_text(_menu, IDC_TRI_STATE_NO);
		CheckMenuItem(_menu, IDC_TRI_STATE_NO, MF_BYCOMMAND | MF_CHECKED);
		CheckMenuItem(_menu, IDC_TRI_STATE_YES, MF_BYCOMMAND | MF_UNCHECKED);
	}
	else
	{
		_name		= "HPDI32_MISC_TX_RX_TRI_STATE_ENABLE";
		_setting	= HPDI32_MISC_TX_RX_TRI_STATE_ENABLE;
		gui_menu_add_item_text(_menu, IDC_TRI_STATE_YES);
		CheckMenuItem(_menu, IDC_TRI_STATE_NO, MF_BYCOMMAND | MF_UNCHECKED);
		CheckMenuItem(_menu, IDC_TRI_STATE_YES, MF_BYCOMMAND | MF_CHECKED);
	}

	gsc_os_ini_int_set(NULL, NULL, KEY, _setting);
}



/*****************************************************************************
*
*	Function:	tri_state_init
*
*	Purpose:
*
*		Initialize things for the Tx/Rx Tri-State option.
*
*	Arguments:
*
*		None.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void tri_state_init(void)
{
	int		i;

	_menu	= GetMenu(gui.dlg);
	_menu	= gui_menu_locate_parent_by_id(_menu, IDC_TRI_STATE_NO);
	gsc_os_ini_int_get(NULL, NULL, KEY, &i, HPDI32_MISC_TX_RX_TRI_STATE_DEFAULT);
	_update_and_save(i);
}



/*****************************************************************************
*
*	Function:	tri_state_refresh
*
*	Purpose:
*
*		Refresh the Tx/Rx Tri-State menu items.
*
*	Arguments:
*
*		None.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void tri_state_refresh(void)
{
	if (gui.working)
	{
		EnableMenuItem(_menu, IDC_TRI_STATE_NO, MF_BYCOMMAND | MF_GRAYED);
		EnableMenuItem(_menu, IDC_TRI_STATE_YES, MF_BYCOMMAND | MF_GRAYED);
	}
	else
	{
		EnableMenuItem(_menu, IDC_TRI_STATE_NO, MF_BYCOMMAND | MF_ENABLED);
		EnableMenuItem(_menu, IDC_TRI_STATE_YES, MF_BYCOMMAND | MF_ENABLED);
	}
}



/*****************************************************************************
*
*	Function:	tri_state_wm_command
*
*	Purpose:
*
*		Process a WM_COMMAND message for a Tx/Rx Tri-State control or menu
*		items.
*
*	Arguments:
*
*		id		The id of the control or menu item selected.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void tri_state_wm_command(int id)
{
	switch (id)
	{
		default:
		case IDC_TRI_STATE_NO:

			id	= HPDI32_MISC_TX_RX_TRI_STATE_DISABLE;
			break;

		case IDC_TRI_STATE_YES:

			id	= HPDI32_MISC_TX_RX_TRI_STATE_ENABLE;
			break;
	}

	_update_and_save(id);
}



/*****************************************************************************
*
*	Function:	tri_state_config
*
*	Purpose:
*
*		Configure the given device's Tx/Rx Tri-State parameter.
*
*	Arguments:
*
*		dev		The handle for the device to access.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

int tri_state_config(void* dev)
{
	int	errs;
	U32	status;

	gsc_label_text(KEY);
	status	= HPDI32_MISC_TX_RX_TRI_STATE__SET(dev, _setting);
	errs	= label_fail_func(1, "HPDI32_MISC_TX_RX_TRI_STATE__SET", status);

	if (errs == 0)
		printf("PASS  (%s)\n", _name);

	return(errs);
}
