// HPDI32 - Application - Win32 - Any Compiler
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/SDK/hpdi32/ringbuf/win32/gui_tx_clock.c $
// $Rev: 2809 $
// $Date: 2009-07-29 21:26:35 -0500 (Wed, 29 Jul 2009) $

#include <windows.h>
#include <stdio.h>

#include "gui.h"
#include "resource.h"
#include "hpdi32_util.h"
#include "gsc_util.h"



/* #defines	*****************************************************************/

#define	KEY	"HPDI32_TX_CLOCK_DIVIDER"



/* variables	*************************************************************/

static	HMENU	_menu;
static	int		_setting;



/*****************************************************************************
*
*	Function:	_update_and_save
*
*	Purpose:
*
*		Update the menu check state and save the setting.
*
*	Arguments:
*
*		value	The current setting.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _update_and_save(int value)
{
	if (value < 0)
		value	= 0;
	else if (value > HPDI32_TX_CLOCK_DIVIDER_MAX)
		value	= HPDI32_TX_CLOCK_DIVIDER_MAX;

	_setting	= value;
	gsc_os_ini_int_set(NULL, NULL, KEY, _setting);
	gui_menu_item_add_int(_menu, IDC_TX_CLOCK_DIVIDER, _setting);
}



/*****************************************************************************
*
*	Function:	tx_clock_divider_init
*
*	Purpose:
*
*		Initialize things for the Tx Clock Divider option.
*
*	Arguments:
*
*		None.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void tx_clock_divider_init(void)
{
	int		i;

	_menu	= GetMenu(gui.dlg);
	gsc_os_ini_int_get(NULL, NULL, KEY, &i, HPDI32_TX_CLOCK_DIVIDER_DEFAULT);
	_update_and_save(i);
}



/*****************************************************************************
*
*	Function:	tx_clock_divider_refresh
*
*	Purpose:
*
*		Refresh the Tx Clock Divider menu item.
*
*	Arguments:
*
*		None.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void tx_clock_divider_refresh(void)
{
	if (gui.working)
		EnableMenuItem(_menu, IDC_TX_CLOCK_DIVIDER, MF_BYCOMMAND | MF_GRAYED);
	else
		EnableMenuItem(_menu, IDC_TX_CLOCK_DIVIDER, MF_BYCOMMAND | MF_ENABLED);
}



/*****************************************************************************
*
*	Function:	tx_clock_divider_wm_command
*
*	Purpose:
*
*		Process a WM_COMMAND message for a Tx Clock Divider control or
*		menu items.
*
*	Arguments:
*
*		None.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void tx_clock_divider_wm_command(void)
{
	gui_edit_int(	"Tx Clock Divider",
					&_setting,
					0,
					HPDI32_TX_CLOCK_DIVIDER_MAX,
					"",
					"0 = /1, else = /2x");
	_update_and_save(_setting);
}



/*****************************************************************************
*
*	Function:	tx_clock_divider_config
*
*	Purpose:
*
*		Configure the given device's Tx Clock Divider parameter.
*
*	Arguments:
*
*		dev		The handle for the device to access.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

int tx_clock_divider_config(void* dev)
{
	int	errs	= 0;
	U32	status;

	gsc_label_text(KEY);
	status	= HPDI32_TX_CLOCK_DIVIDER__SET(dev, _setting);
	errs	= label_fail_func(1, "HPDI32_TX_CLOCK_DIVIDER__SET", status);

	if (errs == 0)
		printf("PASS  (%d)\n", _setting);

	return(errs);
}


