// HPDI32 - Application - Win32 - Any Compiler
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/SDK/hpdi32/ringbuf/win32/gui_tx_io_pio_thresh.c $
// $Rev: 2809 $
// $Date: 2009-07-29 21:26:35 -0500 (Wed, 29 Jul 2009) $

#include <windows.h>
#include <stdio.h>

#include "gui.h"
#include "resource.h"
#include "hpdi32_util.h"
#include "gsc_util.h"



/* #defines	*****************************************************************/

#define	KEY	"HPDI32_IO_PIO_THRESHOLD__TX"



/* variables	*************************************************************/

static	HMENU	_menu;
static	int		_setting;



/*****************************************************************************
*
*	Function:	_update_and_save
*
*	Purpose:
*
*		Update the menu check state and save the setting.
*
*	Arguments:
*
*		value	The current setting.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _update_and_save(int value)
{
	if (value < 0)
		value	= 0;

	_setting	= value;
	gsc_os_ini_int_set(NULL, NULL, KEY, _setting);
	gui_menu_item_add_int(_menu, IDC_TX_IO_PIO_THRESH, _setting);
}



/*****************************************************************************
*
*	Function:	tx_io_pio_threshold_init
*
*	Purpose:
*
*		Initialize things for the Tx I/O PIO Threshold option.
*
*	Arguments:
*
*		None.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void tx_io_pio_threshold_init(void)
{
	int		i;

	_menu	= GetMenu(gui.dlg);
	gsc_os_ini_int_get(NULL, NULL, KEY, &i, HPDI32_IO_PIO_THRESHOLD_DEFAULT);
	_update_and_save(i);
}



/*****************************************************************************
*
*	Function:	tx_io_pio_threshold_refresh
*
*	Purpose:
*
*		Refresh the Tx I/O PIO Threshold menu item.
*
*	Arguments:
*
*		None.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void tx_io_pio_threshold_refresh(void)
{
	if (gui.working)
		EnableMenuItem(_menu, IDC_TX_IO_PIO_THRESH, MF_BYCOMMAND | MF_GRAYED);
	else
		EnableMenuItem(_menu, IDC_TX_IO_PIO_THRESH, MF_BYCOMMAND | MF_ENABLED);
}



/*****************************************************************************
*
*	Function:	tx_io_pio_threshold_wm_command
*
*	Purpose:
*
*		Process a WM_COMMAND message for a Tx I/O PIO Threshold control or
*		menu items.
*
*	Arguments:
*
*		None.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void tx_io_pio_threshold_wm_command(void)
{
	gui_edit_int(	"Tx I/O PIO Threshold",
					&_setting,
					0,
					32767,
					"Samples",
					NULL);
	_update_and_save(_setting);
}



/*****************************************************************************
*
*	Function:	tx_io_pio_threshold_config
*
*	Purpose:
*
*		Configure the given device's Tx I/O PIO Threshold parameter.
*
*	Arguments:
*
*		dev		The handle for the device to access.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

int tx_io_pio_threshold_config(void* dev)
{
	int	errs	= 0;
	U32	status;

	gsc_label_text(KEY);
	status	= HPDI32_IO_PIO_THRESHOLD__TX_SET(dev, _setting);
	errs	= label_fail_func(1, "HPDI32_IO_PIO_THRESHOLD__TX_SET", status);

	if (errs == 0)
		printf("PASS  (%d)\n", _setting);

	return(errs);
}


