// HPDI32 - Application - Win32 - Any Compiler
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/SDK/hpdi32/ringbuf/win32/proc.c $
// $Rev: 31440 $
// $Date: 2014-11-13 09:44:06 -0600 (Thu, 13 Nov 2014) $

#include <windows.h>
#include <limits.h>
#include <stdio.h>
#include <time.h>

#include "hpdi32.h"

#include "ringbuf.h"
#include "proc.h"
#include "rx_proc.h"
#include "tx_proc.h"
#include "hpdi32_util.h"
#include "gsc_util.h"



/* #defines	*****************************************************************/

#define	TX_FILE_NAME	"datafile.bin"



/* variables	*************************************************************/

static	void*			_dev_handle;
static	int				_dev_index;
static	proc_data_t		_rx	= { TRUE };
static	gsc_os_sem_t		_sem;
static	int				_sem_init;
static	proc_data_t		_tx	= { TRUE };



//*****************************************************************************
static int _rx_overflow_clear(void* dev)
{
	int	errs	= 0;
	U32	status;

	gsc_label_text("HPDI32_RX_OVERRUN");
	status	= HPDI32_RX_OVERRUN__CLEAR(dev);
	errs	= label_fail_func(1, "HPDI32_RX_OVERRUN__CLEAR", status);

	if (errs == 0)
		printf("PASS  (%s)\n", "HPDI32_RX_OVERRUN_CLEAR");

	return(errs);
}



//*****************************************************************************
static int _rx_overflow_check(void* dev)
{
	int	errs	= 0;
	S32	get;
	U32	status;

	gsc_label_text("HPDI32_RX_OVERRUN");
	status	= HPDI32_RX_OVERRUN__GET(dev, &get);
	errs	= label_fail_func(1, "HPDI32_RX_OVERRUN__GET", status);

	if (errs == 0)
		printf("PASS  (%s)\n", get ? "Yes" : "No");

	return(errs);
}



/*****************************************************************************
*
*	Function:	_access_device
*
*	Purpose:
*
*		Do everything that is needed to begin using an HPDI32.
*
*	Arguments:
*
*		index	The index of the board to access.
*
*		dev		We record the device handle here.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

static int _access_device(U8 index, void** dev)
{
	U8	count;
	int	errs;
	int	indx	= index;
	S32	rx_size;
	S32	tx_size;

	// Initial access to the API.
	errs	= dev_api_status();
	count	= errs ? 0 : dev_board_count();
	errs	= errs ? errs : gsc_select_1_board(count, &indx);

	if (errs == 0)
	{
		gsc_label_text("HPDI32 Being Accessed");
		printf("%d\n", indx);
	}

	errs	= errs ? errs : dev_open(index, dev);
	errs	= errs ? errs : id_software(dev[0], VERSION);
	errs	= errs ? errs : id_hardware(dev[0], index);
	errs	= errs ? errs : id_features(dev[0]);
	errs	= errs ? errs : operation_issues(dev[0], &tx_size, &rx_size);

	if (errs == 0)
	{
		gsc_label_text("Configure Device");
		printf("\n");
		gsc_label_level_inc();

		errs	+= favor_tx_config(dev[0]);
		errs	+= tri_state_config(dev[0]);

		errs	+= cable_cmd_0_config(dev[0]);
		errs	+= cable_cmd_1_config(dev[0]);
		errs	+= cable_cmd_2_config(dev[0]);
		errs	+= cable_cmd_3_config(dev[0]);
		errs	+= cable_cmd_4_config(dev[0]);
		errs	+= cable_cmd_5_config(dev[0]);
		errs	+= cable_cmd_6_config(dev[0]);

		errs	+= tx_io_mode_config(dev[0]);
		errs	+= tx_io_dma_ch_sel_config(dev[0]);
		errs	+= tx_io_dma_pri_config(dev[0]);
		errs	+= tx_io_dma_ctrl_config(dev[0]);
		errs	+= tx_io_data_size_config(dev[0]);
		errs	+= tx_io_sc_config(dev[0]);
		errs	+= tx_io_pio_threshold_config(dev[0]);
		errs	+= tx_io_timeout_config(dev[0]);

		errs	+= tx_fifo_ae_config(dev[0]);
		errs	+= tx_fifo_af_config(dev[0]);

		errs	+= tx_auto_start_config(dev[0]);
		errs	+= tx_auto_stop_config(dev[0]);
		errs	+= tx_remote_throttle_config(dev[0]);
		errs	+= tx_clock_divider_config(dev[0]);
		errs	+= tx_status_valid_count_config(dev[0]);
		errs	+= tx_status_valid_mirror_config(dev[0]);
		errs	+= tx_line_valid_off_count_config(dev[0]);
		errs	+= tx_line_valid_on_count_config(dev[0]);
		errs	+= tx_enable_config(dev[0]);

		errs	+= rx_io_mode_config(dev[0]);
		errs	+= rx_io_dma_ch_sel_config(dev[0]);
		errs	+= rx_io_dma_pri_config(dev[0]);
		errs	+= rx_io_dma_ctrl_config(dev[0]);
		errs	+= rx_io_data_size_config(dev[0]);
		errs	+= rx_io_sc_config(dev[0]);
		errs	+= rx_io_pio_threshold_config(dev[0]);
		errs	+= rx_io_timeout_config(dev[0]);

		errs	+= rx_fifo_ae_config(dev[0]);
		errs	+= rx_fifo_af_config(dev[0]);

		gsc_label_level_dec();
	}

	return(errs);
}



/*****************************************************************************
*
*	Function:	_alloc_buffers
*
*	Purpose:
*
*		Allocate the data buffers.
*
*	Arguments:
*
*		dev		The handle for the device to access. This may be NULL.
*
*		api		Use API buffers?
*
*		qty		The number of buffers to allocate.
*
*		size	The size of the buffers to allocate.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

static int _alloc_buffers(void* dev, int api, int qty, size_t size)
{
	char	buf[128];
	int		errs;

	gsc_label_text("Allocating Buffers");
	errs	= gsc_buf_man_stop(FALSE, NULL);

	if (errs)
	{
		gsc_label_fail_text(1, "gsc_buf_man_stop() failed");
	}
	else if ((qty <= 0) || (qty > 255))
	{
		sprintf(buf, "invalid quantity: %d", qty);
		errs	= gsc_label_fail_text(1, buf);
	}
	else
	{
		dev		= api ? dev : NULL;
		errs	= gsc_buf_man_setup(dev, (U8) qty, size);

		if (errs)
			gsc_label_fail_text(1, "gsc_buf_man_setup() failed");
		else
			printf("PASS\n");
	}

	return(errs);
}



/*****************************************************************************
*
*	Function:	_check_for_previous_done
*
*	Purpose:
*
*		Make sure any previous work has come to a halt.
*
*	Arguments:
*
*		None.
*
*	Returned:
*
*		0		Previous work is done.
*		> 0		Previous work is still active.
*
*****************************************************************************/

static int _check_for_previous_done(void)
{
	char	buf[128];
	int		errs	= 0;

	if (_rx.done == FALSE)
	{
		sprintf(buf, "%s not yet done.", _rx.name);
		errs	= gsc_label_fail_text(1, buf);
	}
	else if (_tx.done == FALSE)
	{
		sprintf(buf, "%s not yet done.", _tx.name);
		errs	= gsc_label_fail_text(1, buf);
	}
	else
	{
		errs	= 0;
	}

	return(errs);
}



/*****************************************************************************
*
*	Function:	_rx_start
*
*	Purpose:
*
*		Start the Rx process and check for errors.
*
*	Arguments:
*
*		dev		The device handle to use.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

static int _rx_start(void* dev)
{
	int	errs;

	_rx.dev		= dev;
	_rx.done	= FALSE;
	_rx.err		= 0;
	_rx.name	= "Rx Process";
	_rx.text[0]	= 0;
	_rx.stop	= FALSE;
	errs		= rx_process_start(&_rx);

	if (errs)
	{
		_rx.done	= TRUE;
		_rx.stop	= TRUE;
		_rx.waiting	= TRUE;
	}

	return(errs);
}



/*****************************************************************************
*
*	Function:	_start_status
*
*	Purpose:
*
*		Report startup information.
*
*	Arguments:
*
*		index	The index of the device to access.
*
*		api		Use API Buffers?
*
*		qty		The number of buffers to utilize.
*
*		size	The size of the buffers to use.
*
*	Returned:
*
*		>= 0	The number of errors seen. Any errors means it was not fully
*				started.
*
*****************************************************************************/

static void _start_status(U8 index, int api, int qty, size_t size)
{
	const char*	psz;
	char		sz[32];
	float		total;
	char		ttl[32];
	time_t		tt;
	struct tm*	stm;

	gsc_format_long((long) size, 1024, sz, sizeof(sz));
	total	= (float) qty * size;

	if (total < (float) LONG_MAX)
		gsc_format_long((long) total, 1024, ttl, sizeof(ttl));
	else
		gsc_format_float(total, 1024, ttl, sizeof(ttl));

	gsc_label_text("Starting");
	time(&tt);
	stm	= localtime(&tt);
	psz	= asctime(stm);
	printf("%s", psz);

	gsc_label_level_inc();

	gsc_label_text("Device Index");
	printf("HPDI32 #%d\n", index);

	gsc_label_text("Buffer Type");
	printf("%s Buffers\n", api ? "API" : "Application");

	gsc_label_text("Buffer Quantity");
	printf("%d Buffer%s\n", qty, (qty == 1) ? "" : "s");

	gsc_label_text("Buffer Size");
	printf("%s Byte\n", sz);

	gsc_label_text("Total Storage");
	printf("%s Byte\n", ttl);

	gsc_label_level_dec();
}



/*****************************************************************************
*
*	Function:	_sync_semaphore_init
*
*	Purpose:
*
*		Make sure we've got the sychronization semaphore we use to get the
*		Rx and Tx processign threads going.
*
*	Arguments:
*
*		None.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

static int _sync_semaphore_init(void)
{
	int	errs;

	if (_sem_init)
	{
		errs	= 0;
	}
	else
	{
		errs	= gsc_os_sem_create_qty(&_sem, 2, 0);

		if (errs)
			gsc_label_fail_text(1, "gsc_os_sem_create_qty() failed");
		else
			_sem_init	= TRUE;
	}

	return(errs);
}



/*****************************************************************************
*
*	Function:	_tx_start
*
*	Purpose:
*
*		Start the Tx process and check for errors.
*
*	Arguments:
*
*		dev		The device handle to use.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

static int _tx_start(void* dev)
{
	int	errs;

	_tx.dev		= dev;
	_tx.done	= FALSE;
	_tx.err		= 0;
	_tx.name	= "Tx Process";
	_tx.text[0]	= 0;
	_tx.stop	= FALSE;
	errs		= tx_process_start(&_tx);

	if (errs)
	{
		_tx.done	= TRUE;
		_tx.stop	= TRUE;
		_tx.waiting	= TRUE;
	}

	return(errs);
}



/*****************************************************************************
*
*	Function:	_start_threads
*
*	Purpose:
*
*		Start and synchronize the Rx and Tx threads.
*
*	Arguments:
*
*		dev		The device handle to use.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

static int _start_threads(void* dev)
{
	int	errs;
	int	qty;
	int	rx;
	int	tx;

	// Start the threads.
	rx		= _rx_start(dev);
	tx		= _tx_start(dev);
	errs	= rx + tx;

	if (errs)
	{
		_rx.stop	= TRUE;
		_tx.stop	= TRUE;
	}

	// Synchronize the Rx and Tx threads.
	if ((rx) && (tx))
		qty	= 0;
	else if ((rx) || (tx))
		qty	= 1;
	else
		qty	= 2;

	if (errs == 0)
	{
		gsc_label_text("Processing Started");
		printf("\n");
	}

	for (;;)
	{
		if ((_rx.waiting == TRUE) && (_tx.waiting == TRUE))
		{
			gsc_os_sem_put_qty(&_sem, qty);
			break;
		}

		Sleep(10);
	}

	return(errs);
}



/*****************************************************************************
*
*	Function:	_xfer_stats
*
*	Purpose:
*
*		Report on data transfer statistics.
*
*	Arguments:
*
*		None.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _xfer_stats(proc_data_t* data)
{
	float	bps;
	char	buf[64];
	long	ms;

	if (data == NULL)
	{
		gsc_label_text("Transfer Statistics");
		printf("\n");
		gsc_label_level_inc();
		_xfer_stats(&_rx);
		_xfer_stats(&_tx);
		gsc_label_level_dec();
	}
	else
	{
		gsc_label_text(data->name);
		printf("%s\n", data->op_title);
		gsc_label_level_inc();

		gsc_label_text("Elapsed Time");
		ms	= data->ms_end - data->ms_begin;
		gsc_format_time_ms(ms, buf, sizeof(buf));
		printf("%s\n", buf);

		gsc_label_text("Total Data");
		gsc_format_float_prec((float) data->bytes, 1024, buf, sizeof(buf));


		printf("%s Bytes (1K = 1024, %.0f bytes)\n", buf, (float) data->bytes);

		if (ms)
			bps	= (float) ((data->bytes / ms) * 1000);
		else
			bps	= 0;

		gsc_label_text("Transfer Rate");
		gsc_format_float_prec(bps, 1000, buf, sizeof(buf));
		printf("%s Bytes/Second (1K = 1000)\n", buf);

		gsc_label_level_dec();
	}
}


/*****************************************************************************
*
*	Function:	process_start
*
*	Purpose:
*
*		Start the process/thread that gets the ball rolling.
*
*	Arguments:
*
*		index	The index of the device to access.
*
*		api		Use API Buffers?
*
*		qty		The number of buffers to utilize.
*
*		size	The size of the buffers to use.
*
*		tx_op	The Tx process operation to be performed. This is one of the
*				TX_OP_XXX macro values.
*
*		rx_op	The Rx process operation to be performed. This is one of the
*				RX_OP_XXX macro values.
*
*		rx_file	The name of the file to read data from loading receive data
*				from a file.
*
*	Returned:
*
*		>= 0	The number of errors seen. Any errors means it was not fully
*				started.
*
*****************************************************************************/

int process_start(
	U8			index,
	int			api,
	int			qty,
	size_t		size,
	int			tx_op,
	int			rx_op,
	const char*	rx_file)
{
	void*	dev		= NULL;
	int		errs;

	_tx.operation	= tx_op;
	strcpy(_tx.file_name, TX_FILE_NAME);

	_rx.operation	= rx_op;
	strcpy(_rx.file_name, rx_file);

	_start_status(index, api, qty, size);
	errs	= _sync_semaphore_init();

	if (errs == 0)
		errs	= _check_for_previous_done();

	if (errs == 0)
		errs	= _access_device(index, &dev);

	if (errs == 0)
		errs	= _alloc_buffers(dev, api, qty, size);

	if (errs == 0)
		errs	= _start_threads(dev);

	if (errs == 0)
	{
		errs		+= rx_enable_config(dev);
		errs		+= _rx_overflow_clear(dev);
		_dev_index	= index;
		_dev_handle	= dev;
	}
	else
	{
		printf("TESTING ABORTED DUE TO ERRORS\n");
		gsc_buf_man_free_all();

		if (dev)
			errs	+= dev_close((U8) index, dev);
	}

	return(errs);
}



/*****************************************************************************
*
*	Function:	process_stop
*
*	Purpose:
*
*		Request that all processing be stopped.
*
*	Arguments:
*
*		None.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void process_stop(void)
{
	_rx_overflow_check(_dev_handle);
	_rx.stop	= TRUE;
	_tx.stop	= TRUE;
	gsc_buf_man_stop(TRUE, NULL);
}



/*****************************************************************************
*
*	Function:	process_done
*
*	Purpose:
*
*		Tell the caller if processing is done.
*
*	Arguments:
*
*		None.
*
*	Returned:
*
*		FALSE	Processing is not done.
*		TRUE	Processing is done.
*
*****************************************************************************/

int process_done(void)
{
	void*		dev	= _dev_handle;
	int			done;
	const char*	psz;
	struct tm*	stm;
	time_t		tt;

	done	= ((_rx.done) && (_tx.done)) ? TRUE : FALSE;

	if ((done) && (dev))
	{
		_dev_handle	= NULL;
		gsc_buf_man_free_all();
		dev_close(_dev_index, dev);

		gsc_label_text(_rx.name);

		if (_rx.err)
		{
			gsc_label_fail_text(1, "processing error");
			gsc_label_level_inc();
			gsc_label_text("Error");
			printf("%s\n", _rx.text);
			gsc_label_level_dec();
		}
		else
		{
			printf("%s\n", _rx.text);
		}

		gsc_label_text(_tx.name);

		if (_tx.err)
		{
			gsc_label_fail_text(1, "processing error");
			gsc_label_level_inc();
			gsc_label_text("Error");
			printf("%s\n", _tx.text);
			gsc_label_level_dec();
		}
		else
		{
			printf("%s\n", _tx.text);
		}

		gsc_buf_man_stats();
		_xfer_stats(NULL);

		gsc_label_text("Processing Done");
		time(&tt);
		stm	= localtime(&tt);
		psz	= asctime(stm);
		printf("%s", psz);
	}

	return(done);
}



/*****************************************************************************
*
*	Function:	dev_api_buf_alloc
*
*	Purpose:
*
*		Allocate an API Buffer.
*		THIS FUNCTION IS REQUIRED FROM UTIL_BM.H.
*
*	Arguments:
*
*		dev		The device handle to utilize.
*
*		index	The index of the allocation. This must be only zero or one.
*				Any other value produces a failure.
*
*		size	The size of the memory to allocate.
*
*	Returned:
*
*		NULL	The allocation failed.
*		else	The allocation succeeded.
*
*****************************************************************************/

void* dev_api_buf_alloc(void* dev, unsigned char index, size_t size)
{
	void*			buf_ptr;
	U32				status;
	unsigned long	sz;
	void*			vp;

	if (index == 0)
	{
		for (;;)	// A convenience loop.
		{
			// Allocate the memory.
			HPDI32_IO_BUFFER_SIZE__TX_SET(dev, 0, NULL);
			status	= HPDI32_IO_BUFFER_SIZE__TX_SET(dev, (unsigned long) size, &sz);

			if (status != GSC_SUCCESS)
			{
				HPDI32_IO_BUFFER_SIZE__TX_SET(dev, 0, NULL);
				vp	= NULL;
				break;
			}

			if (sz != size)
			{
				HPDI32_IO_BUFFER_SIZE__TX_SET(dev, 0, NULL);
				vp	= NULL;
				break;
			}

			// Get the pointer.
			status	= HPDI32_IO_BUFFER_POINTER__TX_GET(dev, (unsigned long*) (void*) &buf_ptr);

			if (status != GSC_SUCCESS)
			{
				HPDI32_IO_BUFFER_SIZE__TX_SET(dev, 0, NULL);
				vp	= NULL;
				break;
			}

			if (buf_ptr == NULL)
			{
				HPDI32_IO_BUFFER_SIZE__TX_SET(dev, 0, NULL);
				vp	= NULL;
				break;
			}

			vp	= buf_ptr;
			break;
		}
	}
	else if (index == 1)
	{
		for (;;)	// A convenience loop.
		{
			// Allocate the memory.
			HPDI32_IO_BUFFER_SIZE__RX_SET(dev, 0, NULL);
			status	= HPDI32_IO_BUFFER_SIZE__RX_SET(dev, (unsigned long) size, &sz);

			if (status != GSC_SUCCESS)
			{
				HPDI32_IO_BUFFER_SIZE__RX_SET(dev, 0, NULL);
				vp	= NULL;
				break;
			}

			if (sz != size)
			{
				HPDI32_IO_BUFFER_SIZE__RX_SET(dev, 0, NULL);
				vp	= NULL;
				break;
			}

			// Get the pointer.
			status	= HPDI32_IO_BUFFER_POINTER__RX_GET(dev, (unsigned long*) (void*) &buf_ptr);

			if (status != GSC_SUCCESS)
			{
				HPDI32_IO_BUFFER_SIZE__RX_SET(dev, 0, NULL);
				vp	= NULL;
				break;
			}

			if (buf_ptr == NULL)
			{
				HPDI32_IO_BUFFER_SIZE__RX_SET(dev, 0, NULL);
				vp	= NULL;
				break;
			}

			vp	= buf_ptr;
			break;
		}
	}
	else
	{
		vp	= NULL;
	}

	return(vp);
}



/*****************************************************************************
*
*	Function:	dev_api_buf_free
*
*	Purpose:
*
*		Free an API Buffer.
*		THIS FUNCTION IS REQUIRED FROM UTIL_BM.H.
*
*	Arguments:
*
*		dev		The device handle to utilize.
*
*		index	The index of the allocation. This must be only zero or one.
*				Any other value produces a failure.
*
*		vp		The pointer to the buffer to free.
*
*	Returned:
*
*		NULL	The allocation failed.
*		else	The allocation succeeded.
*
*****************************************************************************/

int dev_api_buf_free(void* dev, unsigned char index, void* vp)
{
	int		errs;
	U32		status;

	if (index == 0)
	{
		status	= HPDI32_IO_BUFFER_SIZE__TX_SET(dev, 0, NULL);
		errs	=  (status == GSC_SUCCESS) ? 0 : 1;
	}
	else if (index == 1)
	{
		status	= HPDI32_IO_BUFFER_SIZE__RX_SET(dev, 0, NULL);
		errs	=  (status == GSC_SUCCESS) ? 0 : 1;
	}
	else
	{
		errs	= 1;
	}

	return(errs);
}



/*****************************************************************************
*
*	Function:	process_thread_sync
*
*	Purpose:
*
*		Provide a synchronization point for the Rx and Tx threads. This
*		function should be called only by two threads as each starts up.
*
*	Arguments:
*
*		data	This is the data passed to the process starting routine.
*
*	Returned:
*
*		>= 0	The number of errors seen. Stop processing on an error.
*
*****************************************************************************/

int process_thread_sync(proc_data_t* data)
{
	int					errs;
	gsc_os_sem_get_t	sg;

	if (data == NULL)
	{
		errs	= 1;
	}
	else
	{
		errs			= 0;
		data->waiting	= TRUE;

		for (;;)
		{
			sg	= gsc_os_sem_get_to(&_sem, 1000);

			switch (sg)
			{
				default:
				case GSC_OS_SEM_GET_ER:

					errs	= 1;
					break;

				case GSC_OS_SEM_GET_TO:

					if (data->stop)
						break;
					else
						continue;

				case GSC_OS_SEM_GET_OK:

					break;
			}

			break;
		}

		data->waiting	= FALSE;
	}

	return(errs);
}


