// HPDI32 - Application - Any OS - Any Compiler
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/SDK/hpdi32/rx_simple/main.c $
// $Rev: 39022 $
// $Date: 2017-06-13 17:03:48 -0500 (Tue, 13 Jun 2017) $

#include "main.h"



/* variables	*************************************************************/

static	int	_def_board		= 0;
static	int	_tx_sim			= 0;



/*****************************************************************************
*
*	Function:	_parse_args
*
*	Purpose:
*
*		Parse the command line arguments.
*
*	Arguments:
*
*		dev		The handle of the board to access.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

static int _parse_args(int argc, char** argv)
{
	char	c;
	int		errs	= 0;
	int		i;
	int		j;
	int		k;

	printf("rx_simple: A simple 'rx' sample application\n");
	printf("USAGE: rx_simple <-sim> <#>\n");
	printf("  -sim  Simulate a Tx device providing data. DETACH CABLE!\n");
	printf("  #     The index of the board to test.\n");

	printf("Arguments: ");
	printf("(%d argument%s)\n", argc - 1, ((argc - 1) == 1) ? "" : "s");

	for (i = 0; i < argc; i++)
	{
		printf("  Argument %d: %s\n", i, argv[i]);
	}

	for (i = 1; i < argc; i++)
	{
		if (strcmp(argv[i], "-sim") == 0)
		{
			_tx_sim	= 1;
			continue;
		}

		j	= sscanf(argv[i], "%d%c", &k, &c);

		if ((j == 1) && (k >= 0))
		{
			_def_board	= k;
			continue;
		}

		errs	= 1;
		printf("ERROR: invalid argument: %s\n", argv[i]);
		break;
	}

	return(errs);
}



/*****************************************************************************
*
*	Function:	main
*
*	Purpose:
*
*		Program entry point.
*
*	Arguments:
*
*		argc			The number of command line arguments.
*
*		argv			The list of command line arguments.
*
*	Returned:
*
*		EXIT_SUCCESS	No errors were seen.
*		EXIT_FAILURE	Errors were seen.
*
*****************************************************************************/

int main(int argc, char** argv)
{
	U32		arg;
	void*	dev;
	int		errs;
	int		ret;
	U32		stat;
	U32		status;

	errs	= _parse_args(argc, argv);
	printf("\n");

	if (errs == 0)
	{
		// Verify the status of the API.
		printf("HPDI32 API Library status: ");
		status	= hpdi32_api_status(&stat, &arg, HPDI32_API_VERSION);

		if (status != GSC_SUCCESS)
		{
			printf("FAIL <---  (status request: status = 0x%lX)\n", (long) status);
			errs++;
		}
		else if (stat != GSC_SUCCESS)
		{
			printf("FAIL <---  (API status: stat = 0x%lX, arg = 0x%lX)\n", (long) stat, (long) arg);
			errs++;
		}
		else
		{
			printf("PASS\n");
		}
	}

	if (errs == 0)
	{
		// Open access to the device.
		printf("Opening HPDI32 %d: ", (int) _def_board), _tx_sim;
		status	= hpdi32_open(_def_board, &dev);

		if (status == GSC_SUCCESS)
		{
			printf("PASS\n");
		}
		else
		{
			errs++;
			printf("FAIL <---\n");
		}
	}

	if (errs == 0)
	{
		// Perform the test.
		errs	+= rx_data(dev, _tx_sim);

		// Close access to the device.
		printf("Closing HPDI32 %d: ", _def_board);
		status	= hpdi32_close(dev);

		if (status == GSC_SUCCESS)
		{
			printf("PASS\n");
		}
		else
		{
			errs++;
			printf("FAIL <---\n");
		}
	}

	// Report the results.
	printf("\nRESULTS: %s\n", errs ? "FAIL <---" : "PASS");

	// Exit the applicaton.
	ret	= errs ? EXIT_FAILURE : EXIT_SUCCESS;
	return(ret);
}


