// HPDI32 - Application - Any OS - Any Compiler
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/SDK/hpdi32/tx_simple/tx_simple.c $
// $Rev: 39023 $
// $Date: 2017-06-13 17:04:53 -0500 (Tue, 13 Jun 2017) $

#include "main.h"



// variables ******************************************************************

static	U32	buffer[256L * 1024];	// 1MB of data.



/******************************************************************************
*
*	Function:	_config
*
*	Purpose:
*
*		Configure things in preparation for the test.
*
*	Arguments:
*
*		handle	The handle of the board to access.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
******************************************************************************/

static int _config(void* handle)
{
	int	errs	= 0;
	U32	status;

	printf("Configuring:\n");

	printf("  Initialize: ");
	status	= hpdi32_init(handle);
	printf("%s\n", (status == GSC_SUCCESS) ? "PASS" : "FAIL <---");
	errs	+= (status == GSC_SUCCESS) ? 0 : 1;

	printf("  Tx I/O Mode: DMDMA: ");
	status	= HPDI32_IO_MODE__TX_DMDMA(handle);
	printf("%s\n", (status == GSC_SUCCESS) ? "PASS" : "FAIL <---");
	errs	+= (status == GSC_SUCCESS) ? 0 : 1;

	// This is done in case we are connected to another HPDI32.
	printf("  Tx/Rx Tri-State Mode: Enable: ");
	status	= HPDI32_MISC_TX_RX_TRI_STATE__YES(handle);
	printf("%s\n", (status == GSC_SUCCESS) ? "PASS" : "FAIL <---");
	errs	+= (status == GSC_SUCCESS) ? 0 : 1;

	printf("  Tx Enable: Yes: ");
	status	= HPDI32_TX_ENABLE__YES(handle);
	printf("%s\n", (status == GSC_SUCCESS) ? "PASS" : "FAIL <---");
	errs	+= (status == GSC_SUCCESS) ? 0 : 1;

	return(errs);
}



/******************************************************************************
*
*	Function:	_tx_data
*
*	Purpose:
*
*		Transmit the data.
*
*	Arguments:
*
*		handle	The handle of the board to access.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
******************************************************************************/

static int _tx_data(void* handle)
{
	size_t		bytes	= sizeof(buffer);
	int			errs	= 0;
	const char*	factor	= "";
	int			i;
	U32			sent	= 0;
	U32			status;
	long long	total	= 0;

	printf("Transmitting Data:\n");

	for (i = 0; i < 1024; i++)
	{
		if ((i % 64) == 0)
			printf("\n  ");

		printf(".");
		sent	= 0;
		status	= hpdi32_write(handle, buffer, sizeof(buffer), &sent);

		if (sent > 0)
			total	+= sent;

		if (status != GSC_SUCCESS)
		{
			printf(" FAIL <---\n");
			errs++;
			break;
		}

		if (sent != sizeof(buffer))
		{
			printf(" FAIL <--- (wrote only %ld bytes)", (long) sent);
			errs++;
			break;
		}
	}

	printf("\n\n");

	if ((total ) && ((total % 1024) == 0))
	{
		total	/= 1024;
		factor	= "K";
	}

	if ((total ) && ((total % 1024) == 0))
	{
		total	/= 1024;
		factor	= "M";
	}

	if ((total ) && ((total % 1024) == 0))
	{
		total	/= 1024;
		factor	= "G";
	}

	printf("Wrote %lld%s Bytes\n", total, factor);
	return(errs);
}



/******************************************************************************
*
*	Function:	tx_data
*
*	Purpose:
*
*		Perform a simple data transmit operation.
*
*	Arguments:
*
*		handle	The handle of the board to access.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
******************************************************************************/

int	tx_data(void* handle)
{
	int		errs		= 0;
	int		fifos		= 1;

	memset(buffer, 0, sizeof(buffer));

	errs	+= _config(handle);

	if (errs == 0)
		errs	+= _tx_data(handle);

	return(errs);
}


