// HPDI32 - Application - Win32 - Any Compiler
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/SDK/hpdi32/visualhpdi32/win32/cable.c $
// $Rev: 2902 $
// $Date: 2009-08-09 17:48:49 -0500 (Sun, 09 Aug 2009) $

/*****************************************************************************
*
*	Implement support for the Cable services.
*
*****************************************************************************/

#include <windows.h>

#include "cable.h"
#include "align.h"
#include "io.h"
#include "resource.h"
#include "util_ctrl.h"



/*****************************************************************************
*
*	Function:	_service_init
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_INIT service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_init(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	static const align_t list[]	=
	{
		{
		/* id		*/	IDC_CABLE,
		/* ref		*/	IDC_CABLE_CLOCK,
		/* flags	*/	ALIGN_L_L | ALIGN_R_R | ALIGN_B_T | ALIGN_H,
		/* value	*/	0
		},
		{ 0, 0, 0, 0 }
	};

	HWND	control;

	align_control_list(dlg, list);
	control	= GetDlgItem(dlg, IDC_CABLE);
	SendMessage(control, WM_SETFONT, (WPARAM) device->font.defalt_bl, 0);
	return(0);
}



/*****************************************************************************
*
*	Function:	_service_refresh
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_REFRESH service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_refresh(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	static const int	list[]	=
	{
		IDC_CABLE_TX_ENABLED_VERT,
		IDC_CABLE_RX_CLOCK_VERT,
		IDC_CABLE_C0_EXT,
		IDC_CABLE_C1_EXT,
		IDC_CABLE_C2_EXT,
		IDC_CABLE_TX_RR_VERT,
		IDC_CABLE_TX_DR_VERT,
		IDC_CABLE_RX_FV_VERT,
		IDC_CABLE_RX_LV_VERT,
		IDC_CABLE_TX_SV_VERT,
		IDC_CABLE_RX_ENABLED_VERT,
		IDC_CABLE_C2_JUNC,
		IDC_CABLE_C3_JUNC
	};

	HWND	control;
	int		i;

	if (device->handle)
	{
		ctrl_menu_set(dlg, IDC_CABLE, CTRL_ENABLE);

		// Make sure everything that crosses the data bus gets repainted.

		for (i = ARRAY_ELEMENTS(list); i--;)
		{
			control	= GetDlgItem(dlg, list[i]);
			InvalidateRect(control, NULL, TRUE);
		}
	}
	else
	{
		ctrl_menu_set(dlg, IDC_CABLE, CTRL_DISABLE);
	}

	return(0);
}



/*****************************************************************************
*
*	Function:	_service_wm_ctlcolorstatic
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_WM_CTLCOLORSTATIC
*		service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0		Use the default brush.
*		else	Use this brush handle for the background.
*
*****************************************************************************/

static LRESULT _service_wm_ctlcolorstatic(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	HDC		dc	= (HDC) wParam;
	LRESULT	ret	= (LRESULT) device->paint.btnhighlt.brush;

	SetBkColor(dc, device->paint.btnhighlt.color);
	return(ret);
}



/*****************************************************************************
*
*	Function:	cable_service
*
*	Purpose:
*
*		Implement the common service entry point for the Cable services.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		service	The service to perform.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0		No service was called.
*		else	The value from the called service, which may be zero.
*
*****************************************************************************/

LRESULT cable_service(
	HWND			dlg,
	device_t*		device,
	service_type_t	service,
	WPARAM			wParam,
	LPARAM			lParam)
{
	static const service_call_t	list[]	=
	{
		/* service						function					*/
		{ SERVICE_INIT,					_service_init				},
		{ SERVICE_REFRESH,				_service_refresh			},
		{ SERVICE_WM_CTLCOLORSTATIC,	_service_wm_ctlcolorstatic	},
		{ SERVICE_END_LIST,				NULL						}
	};

	LRESULT	ret;

	ret	= service_call(dlg, device, list, service, wParam, lParam);
	return(ret);
}


