// HPDI32 - Application - Win32 - Any Compiler
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/SDK/hpdi32/visualhpdi32/win32/cable_cmd_6.c $
// $Rev: 31050 $
// $Date: 2014-10-27 17:51:19 -0500 (Mon, 27 Oct 2014) $

/*****************************************************************************
*
*	Implement support for the cable's Command 6 (Rx Enabled) option.
*
*****************************************************************************/

#include <windows.h>

#include "cable.h"
#include "align.h"
#include "arrow.h"
#include "button.h"
#include "resource.h"
#include "util_ctrl.h"



/* #defines	*****************************************************************/

#define	TITLE	"Rx Enabled"



/* prototypes	*************************************************************/

static U32 _cfg_wiz(HWND				dlg,
					device_t*			device,
					const parm_entry_t*	entry,
					U32					value);



/* variables	*************************************************************/

const parm_entry_t	cable_command_6_mode	=
{
	/* title			*/	"Command 6 Mode (Rx Enabled/GPIO 5",
	/* parm + parm_text	*/	VAR_STR_MACRO(HPDI32_CABLE_COMMAND_MODE),
	/* suffix			*/	"_6",
	/* which			*/	HPDI32_WHICH_COMMAND_6_,
	/* set_arg			*/	"",
	/* get_arg			*/	"",
	/* values			*/	&cable_command_value,
	/* cfg_wiz			*/	_cfg_wiz,
	/* desc				*/
	"This parameter controls how the Command 6 cable signal functions. When"
	" configured as Flow Control the signal functions as Rx Enabled. When"
	" configured as General Purpose I/O the signal functions as GPIO 5, which"
	" is a discrete input/output."
};



/*****************************************************************************
*
*	Function:	_cable_data_init
*
*	Purpose:
*
*		Initialize the structure and return it to the caller.
*
*	Arguments:
*
*		device		The device structure to access.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static cable_data_t* _cable_data_init(device_t* device)
{
	static	cable_data_t	data;

	data.cmd_index		= 6;
	data.device			= device;
	data.feature		= HPDI32_MISC_FEATURES_GPIO_0_5;
	data.gpio_index		= 5;
	data.mode.defalt	= HPDI32_CABLE_COMMAND_MODE_DEFAULT;
	data.section_name	= "cable command 6";
	data.title			= TITLE;
	data.which			= HPDI32_WHICH_COMMAND_6_;
	return(&data);
}



/*****************************************************************************
*
*	Function:	_cfg_wiz
*
*	Purpose:
*
*		Provide customized processing for the Configuration Wizards processing
*		of this cable command signal's configuration option.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		device		The device structure to access.
*
*		entry		The structure for the parameter to process.
*
*		value		The current argument value.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _cfg_wiz(HWND				dlg,
					device_t*			device,
					const parm_entry_t*	entry,
					U32					value)
{
	cable_data_t*	data;
	U32				status;

	data	= _cable_data_init(device);
	status	= cable_command_cfg_wiz(dlg, entry, data);
	return(status);
}



/*****************************************************************************
*
*	Function:	_service_init
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_INIT service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_init(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	static	align_t list[]	=
	{
		// The Cable Rx Enabled arrow.
		{// 0
		/* id		*/	IDC_CABLE_RX_ENABLED_ARROW,
		/* ref		*/	IDC_RX_ENABLED_ARROW,
		/* flags	*/	ALIGN_T_T | ALIGN_B_B,
		/* value	*/	0
		},
		{// 1
		/* id		*/	IDC_CABLE_RX_ENABLED_ARROW,
		/* ref		*/	IDC_RX_ENABLED_ARROW,
		/* flags	*/	ALIGN_L_R,
		/* value	*/	MARGIN
		},
		{// 2
		/* id		*/	IDC_CABLE_RX_ENABLED_ARROW,
		/* ref		*/	0,
		/* flags	*/	ALIGN_W,
		/* value	*/	-999	// see below
		},

		// The Cable Rx Enabled extension.
		{// 3
		/* id		*/	IDC_CABLE_RX_ENABLED_EXT,
		/* ref		*/	IDC_CABLE_RX_ENABLED_ARROW,
		/* flags	*/	ALIGN_L_R | ALIGN_T_C,
		/* value	*/	0
		},
		{// 4
		/* id		*/	IDC_CABLE_RX_ENABLED_EXT,
		/* ref		*/	0,
		/* flags	*/	ALIGN_W,
		/* value	*/	-999	// see below
		},
		{// 5
		/* id		*/	IDC_CABLE_RX_ENABLED_EXT,
		/* ref		*/	0,
		/* flags	*/	ALIGN_H,
		/* value	*/	1
		},

		// The Cable GPIO 5 arrow.
		{// 6
		/* id		*/	IDC_CABLE_GPIO_5_ARROW,
		/* ref		*/	IDC_GPIO_5_ARROW,
		/* flags	*/	ALIGN_T_T | ALIGN_B_B,
		/* value	*/	0
		},
		{// 7
		/* id		*/	IDC_CABLE_GPIO_5_ARROW,
		/* ref		*/	IDC_CABLE_RX_ENABLED_ARROW,
		/* flags	*/	ALIGN_L_L | ALIGN_R_R,
		/* value	*/	0
		},

		// The Cable GPIO 5 extension.
		{// 8
		/* id		*/	IDC_CABLE_GPIO_5_EXT,
		/* ref		*/	IDC_CABLE_GPIO_5_ARROW,
		/* flags	*/	ALIGN_T_C,
		/* value	*/	0
		},
		{// 9
		/* id		*/	IDC_CABLE_GPIO_5_EXT,
		/* ref		*/	IDC_CABLE_RX_ENABLED_EXT,
		/* flags	*/	ALIGN_L_L | ALIGN_R_R,
		/* value	*/	0
		},
		{// 10
		/* id		*/	IDC_CABLE_GPIO_5_EXT,
		/* ref		*/	0,
		/* flags	*/	ALIGN_H,
		/* value	*/	1
		},

		// The Cable Command 6 button.
		{// 11
		/* id		*/	IDC_CABLE_C6,
		/* ref		*/	IDC_CABLE_C5,
		/* flags	*/	ALIGN_COPY,
		/* value	*/	0
		},
		{// 12
		/* id		*/	IDC_CABLE_C6,
		/* ref		*/	IDC_CABLE_C5,
		/* flags	*/	ALIGN_T_B | ALIGN_H,
		/* value	*/	MARGIN
		},

		// The Cable Command 6 arrow.
		{// 13
		/* id		*/	IDC_CABLE_C6_ARROW,
		/* ref		*/	IDC_CABLE_C5_ARROW,
		/* flags	*/	ALIGN_COPY,
		/* value	*/	0
		},
		{// 14
		/* id		*/	IDC_CABLE_C6_ARROW,
		/* ref		*/	IDC_CABLE_C5_ARROW,
		/* flags	*/	ALIGN_T_B | ALIGN_H,
		/* value	*/	MARGIN
		},

		// The Cable Command 6 extension.
		{// 15
		/* id		*/	IDC_CABLE_C6_EXT,
		/* ref		*/	IDC_CABLE_RX_ENABLED_EXT,
		/* flags	*/	ALIGN_L_R,
		/* value	*/	0
		},
		{// 16
		/* id		*/	IDC_CABLE_C6_EXT,
		/* ref		*/	IDC_CABLE_C6_ARROW,
		/* flags	*/	ALIGN_T_C | ALIGN_R_L,
		/* value	*/	0
		},
		{// 17
		/* id		*/	IDC_CABLE_C6_EXT,
		/* ref		*/	0,
		/* flags	*/	ALIGN_H,
		/* value	*/	1
		},

		// The Cable Rx Enabled vertical bar.
		{// 18
		/* id		*/	IDC_CABLE_RX_ENABLED_VERT,
		/* ref		*/	IDC_CABLE_RX_ENABLED_EXT,
		/* flags	*/	ALIGN_L_R | ALIGN_T_T,
		/* value	*/	0
		},
		{// 19
		/* id		*/	IDC_CABLE_RX_ENABLED_VERT,
		/* ref		*/	0,
		/* flags	*/	ALIGN_W,
		/* value	*/	1
		},
		{// 20
		/* id		*/	IDC_CABLE_RX_ENABLED_VERT,
		/* ref		*/	IDC_CABLE_C6_EXT,
		/* flags	*/	ALIGN_B_T,
		/* value	*/	-1
		},

		// The Cable GPIO 5 vertical bar.
		{// 21
		/* id		*/	IDC_CABLE_GPIO_5_VERT,
		/* ref		*/	IDC_CABLE_RX_ENABLED_VERT,
		/* flags	*/	ALIGN_L_L | ALIGN_R_R,
		/* value	*/	0
		},
		{// 22
		/* id		*/	IDC_CABLE_GPIO_5_VERT,
		/* ref		*/	IDC_CABLE_C6_EXT,
		/* flags	*/	ALIGN_T_B,
		/* value	*/	1
		},
		{// 23
		/* id		*/	IDC_CABLE_GPIO_5_VERT,
		/* ref		*/	IDC_CABLE_GPIO_5_EXT,
		/* flags	*/	ALIGN_B_B,
		/* value	*/	0
		},

		// The Cable Command 6 arrow junction.
		{// 24
		/* id		*/	IDC_CABLE_C6_JUNC,
		/* ref		*/	IDC_CABLE_C6_EXT,
		/* flags	*/	ALIGN_L_L | ALIGN_T_T,
		/* value	*/	-1
		},
		{// 25
		/* id		*/	IDC_CABLE_C6_JUNC,
		/* ref		*/	IDC_CABLE_C6_JUNC,
		/* flags	*/	ALIGN_R_L | ALIGN_B_T,
		/* value	*/	3
		},

		{ 0, 0, 0, 0 }
	};

	list[2].value	= device->btn_height;
	list[4].value	= (device->btn_height * 1) / 2;

	align_control_list(dlg, list);
	button_init(dlg, device, IDC_CABLE_C6);

	arrow_init(dlg, device, IDC_CABLE_C6_ARROW);
	arrow_type_set(dlg, IDC_CABLE_C6_ARROW, ARROW_HORZ);
	arrow_init(dlg, device, IDC_CABLE_C6_EXT);
	arrow_type_set(dlg, IDC_CABLE_C6_EXT, ARROW_HORZ);
	arrow_init(dlg, device, IDC_CABLE_C6_JUNC);
	arrow_type_set(dlg, IDC_CABLE_C6_JUNC, ARROW_RECT_SOLID);

	arrow_init(dlg, device, IDC_CABLE_RX_ENABLED_ARROW);
	arrow_type_set(dlg, IDC_CABLE_RX_ENABLED_ARROW, ARROW_HORZ);
	arrow_init(dlg, device, IDC_CABLE_RX_ENABLED_EXT);
	arrow_type_set(dlg, IDC_CABLE_RX_ENABLED_EXT, ARROW_HORZ);
	arrow_init(dlg, device, IDC_CABLE_RX_ENABLED_VERT);
	arrow_type_set(dlg, IDC_CABLE_RX_ENABLED_VERT, ARROW_VERT);

	arrow_init(dlg, device, IDC_CABLE_GPIO_5_ARROW);
	arrow_type_set(dlg, IDC_CABLE_GPIO_5_ARROW, ARROW_HORZ);
	arrow_init(dlg, device, IDC_CABLE_GPIO_5_EXT);
	arrow_type_set(dlg, IDC_CABLE_GPIO_5_EXT, ARROW_HORZ);
	arrow_init(dlg, device, IDC_CABLE_GPIO_5_VERT);
	arrow_type_set(dlg, IDC_CABLE_GPIO_5_VERT, ARROW_VERT);
	return(0);
}



/*****************************************************************************
*
*	Function:	_service_refresh
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_REFRESH service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_refresh(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	HWND	control;
	U32		mode;
	U32		rx;
	U32		status;
	U32		tri_state;

	for (;;)	// A convenience loop.
	{
		if (device->handle == NULL)
		{
			// Cable Command 6 is inactive as the device is closed.
			ctrl_menu_set(dlg, IDC_CABLE_C6, CTRL_DISABLE);
			ctrl_menu_set(dlg, IDC_CABLE_C6_ARROW, CTRL_DISABLE);
			ctrl_menu_set(dlg, IDC_CABLE_C6_EXT, CTRL_DISABLE);
			ctrl_menu_set(dlg, IDC_CABLE_C6_JUNC, CTRL_DISABLE);

			ctrl_menu_set(dlg, IDC_CABLE_RX_ENABLED_ARROW, CTRL_DISABLE);
			ctrl_menu_set(dlg, IDC_CABLE_RX_ENABLED_EXT, CTRL_DISABLE);
			ctrl_menu_set(dlg, IDC_CABLE_RX_ENABLED_VERT, CTRL_DISABLE);

			ctrl_menu_set(dlg, IDC_CABLE_GPIO_5_ARROW, CTRL_DISABLE);
			ctrl_menu_set(dlg, IDC_CABLE_GPIO_5_EXT, CTRL_DISABLE);
			ctrl_menu_set(dlg, IDC_CABLE_GPIO_5_VERT, CTRL_DISABLE);

			arrow_type_set(dlg, IDC_CABLE_C6_ARROW, ARROW_HORZ);
			arrow_type_set(dlg, IDC_CABLE_RX_ENABLED_ARROW, ARROW_HORZ);
			arrow_type_set(dlg, IDC_CABLE_GPIO_5_ARROW, ARROW_HORZ);
			break;
		}

		// The device is open.
		status	= HPDI32_CABLE_COMMAND_MODE__6_GET(device->handle, &mode);
		error_status(dlg, IDC_INFO_TEXT, status);

		if (mode == HPDI32_CABLE_COMMAND_MODE_GPIO_IN)
		{
			// Cable Command 6 is GPIO Input.
			ctrl_menu_set(dlg, IDC_CABLE_C6, CTRL_ENABLE);
			ctrl_menu_set(dlg, IDC_CABLE_C6_ARROW, CTRL_ENABLE);
			ctrl_menu_set(dlg, IDC_CABLE_C6_EXT, CTRL_ENABLE);
			ctrl_menu_set(dlg, IDC_CABLE_C6_JUNC, CTRL_ENABLE);

			ctrl_menu_set(dlg, IDC_CABLE_RX_ENABLED_ARROW, CTRL_DISABLE);
			ctrl_menu_set(dlg, IDC_CABLE_RX_ENABLED_EXT, CTRL_DISABLE);
			ctrl_menu_set(dlg, IDC_CABLE_RX_ENABLED_VERT, CTRL_DISABLE);

			ctrl_menu_set(dlg, IDC_CABLE_GPIO_5_ARROW, CTRL_ENABLE);
			ctrl_menu_set(dlg, IDC_CABLE_GPIO_5_EXT, CTRL_ENABLE);
			ctrl_menu_set(dlg, IDC_CABLE_GPIO_5_VERT, CTRL_ENABLE);

			arrow_type_set(dlg, IDC_CABLE_C6_ARROW, ARROW_LEFT);
			arrow_type_set(dlg, IDC_CABLE_RX_ENABLED_ARROW, ARROW_HORZ);
			arrow_type_set(dlg, IDC_CABLE_GPIO_5_ARROW, ARROW_LEFT);
			break;
		}

		if ((mode == HPDI32_CABLE_COMMAND_MODE_GPIO_OUT_LOW) ||
			(mode == HPDI32_CABLE_COMMAND_MODE_GPIO_OUT_HI))
		{
			// Cable Command 6 is GPIO Output.
			ctrl_menu_set(dlg, IDC_CABLE_C6, CTRL_ENABLE);
			ctrl_menu_set(dlg, IDC_CABLE_C6_ARROW, CTRL_ENABLE);
			ctrl_menu_set(dlg, IDC_CABLE_C6_EXT, CTRL_ENABLE);
			ctrl_menu_set(dlg, IDC_CABLE_C6_JUNC, CTRL_ENABLE);

			ctrl_menu_set(dlg, IDC_CABLE_RX_ENABLED_ARROW, CTRL_DISABLE);
			ctrl_menu_set(dlg, IDC_CABLE_RX_ENABLED_EXT, CTRL_DISABLE);
			ctrl_menu_set(dlg, IDC_CABLE_RX_ENABLED_VERT, CTRL_DISABLE);

			ctrl_menu_set(dlg, IDC_CABLE_GPIO_5_ARROW, CTRL_ENABLE);
			ctrl_menu_set(dlg, IDC_CABLE_GPIO_5_EXT, CTRL_ENABLE);
			ctrl_menu_set(dlg, IDC_CABLE_GPIO_5_VERT, CTRL_ENABLE);

			arrow_type_set(dlg, IDC_CABLE_C6_ARROW, ARROW_RIGHT);
			arrow_type_set(dlg, IDC_CABLE_RX_ENABLED_ARROW, ARROW_HORZ);
			arrow_type_set(dlg, IDC_CABLE_GPIO_5_ARROW, ARROW_RIGHT);
			break;
		}

		// The signal is configured as Flow Control.
		status	= HPDI32_RX_ENABLE__GET(device->handle, &rx);
		error_status(dlg, IDC_INFO_TEXT, status);

		if (rx == HPDI32_RX_ENABLE_YES)
		{
			// Cable Command 6 is Rx Enabled (output).
			ctrl_menu_set(dlg, IDC_CABLE_C6, CTRL_ENABLE);
			ctrl_menu_set(dlg, IDC_CABLE_C6_ARROW, CTRL_ENABLE);
			ctrl_menu_set(dlg, IDC_CABLE_C6_EXT, CTRL_ENABLE);
			ctrl_menu_set(dlg, IDC_CABLE_C6_JUNC, CTRL_ENABLE);

			ctrl_menu_set(dlg, IDC_CABLE_RX_ENABLED_ARROW, CTRL_ENABLE);
			ctrl_menu_set(dlg, IDC_CABLE_RX_ENABLED_EXT, CTRL_ENABLE);
			ctrl_menu_set(dlg, IDC_CABLE_RX_ENABLED_VERT, CTRL_ENABLE);

			ctrl_menu_set(dlg, IDC_CABLE_GPIO_5_ARROW, CTRL_DISABLE);
			ctrl_menu_set(dlg, IDC_CABLE_GPIO_5_EXT, CTRL_DISABLE);
			ctrl_menu_set(dlg, IDC_CABLE_GPIO_5_VERT, CTRL_DISABLE);

			arrow_type_set(dlg, IDC_CABLE_C6_ARROW, ARROW_RIGHT);
			arrow_type_set(dlg, IDC_CABLE_RX_ENABLED_ARROW, ARROW_RIGHT);
			arrow_type_set(dlg, IDC_CABLE_GPIO_5_ARROW, ARROW_HORZ);
			break;
		}

		// See if The Tx/Rx  Enable feature is enabled.
		status	= HPDI32_MISC_TX_RX_TRI_STATE__GET(device->handle, &tri_state);
		error_status(dlg, IDC_INFO_TEXT, status);

		if (tri_state == HPDI32_MISC_TX_RX_TRI_STATE_DISABLE)
		{
			// Cable Command 6 is Rx Enabled (input).
			ctrl_menu_set(dlg, IDC_CABLE_C6, CTRL_ENABLE);
			ctrl_menu_set(dlg, IDC_CABLE_C6_ARROW, CTRL_ENABLE);
			ctrl_menu_set(dlg, IDC_CABLE_C6_EXT, CTRL_ENABLE);
			ctrl_menu_set(dlg, IDC_CABLE_C6_JUNC, CTRL_ENABLE);

			ctrl_menu_set(dlg, IDC_CABLE_RX_ENABLED_ARROW, CTRL_ENABLE);
			ctrl_menu_set(dlg, IDC_CABLE_RX_ENABLED_EXT, CTRL_ENABLE);
			ctrl_menu_set(dlg, IDC_CABLE_RX_ENABLED_VERT, CTRL_ENABLE);

			ctrl_menu_set(dlg, IDC_CABLE_GPIO_5_ARROW, CTRL_DISABLE);
			ctrl_menu_set(dlg, IDC_CABLE_GPIO_5_EXT, CTRL_DISABLE);
			ctrl_menu_set(dlg, IDC_CABLE_GPIO_5_VERT, CTRL_DISABLE);

			arrow_type_set(dlg, IDC_CABLE_C6_ARROW, ARROW_RIGHT);
			arrow_type_set(dlg, IDC_CABLE_RX_ENABLED_ARROW, ARROW_RIGHT);
			arrow_type_set(dlg, IDC_CABLE_GPIO_5_ARROW, ARROW_HORZ);
			break;
		}

		// Cable Command 6 is tri-stated.
		ctrl_menu_set(dlg, IDC_CABLE_C6, CTRL_ENABLE);
		ctrl_menu_set(dlg, IDC_CABLE_C6_ARROW, CTRL_DISABLE);
		ctrl_menu_set(dlg, IDC_CABLE_C6_EXT, CTRL_DISABLE);
		ctrl_menu_set(dlg, IDC_CABLE_C6_JUNC, CTRL_DISABLE);

		ctrl_menu_set(dlg, IDC_CABLE_RX_ENABLED_ARROW, CTRL_DISABLE);
		ctrl_menu_set(dlg, IDC_CABLE_RX_ENABLED_EXT, CTRL_DISABLE);
		ctrl_menu_set(dlg, IDC_CABLE_RX_ENABLED_VERT, CTRL_DISABLE);

		ctrl_menu_set(dlg, IDC_CABLE_GPIO_5_ARROW, CTRL_DISABLE);
		ctrl_menu_set(dlg, IDC_CABLE_GPIO_5_EXT, CTRL_DISABLE);
		ctrl_menu_set(dlg, IDC_CABLE_GPIO_5_VERT, CTRL_DISABLE);

		arrow_type_set(dlg, IDC_CABLE_C6_ARROW, ARROW_HORZ);
		arrow_type_set(dlg, IDC_CABLE_RX_ENABLED_ARROW, ARROW_HORZ);
		arrow_type_set(dlg, IDC_CABLE_GPIO_5_ARROW, ARROW_HORZ);
		break;
	}

	control	= GetDlgItem(dlg, IDC_CABLE_C6_ARROW);
	InvalidateRect(control, NULL, TRUE);
	return(0);
}



/*****************************************************************************
*
*	Function:	_service_wm_command
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_WM_COMMAND service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_wm_command(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	cable_data_t*	data;
	LRESULT			ret;

	data	= _cable_data_init(device);
	ret		= cable_command_service(dlg, device, SERVICE_WM_COMMAND, (ULONG) data, lParam);
	return(ret);
}



/*****************************************************************************
*
*	Function:	cable_command_6_service
*
*	Purpose:
*
*		Implement the common service entry point for the Cable Command 6
*		services.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		service	The service to perform.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0		No service was called.
*		else	The value from the called service, which may be zero.
*
*****************************************************************************/

LRESULT cable_command_6_service(
	HWND			dlg,
	device_t*		device,
	service_type_t	service,
	WPARAM			wParam,
	LPARAM			lParam)
{
	static const service_call_t	list[]	=
	{
		/* service				function			*/
		{ SERVICE_INIT,			_service_init		},
		{ SERVICE_REFRESH,		_service_refresh	},
		{ SERVICE_WM_COMMAND,	_service_wm_command	},
		{ SERVICE_END_LIST,		NULL				}
	};

	LRESULT	ret;

	ret	= service_call(dlg, device, list, service, wParam, lParam);
	return(ret);
}


