// HPDI32 - Application - Win32 - Any Compiler
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/SDK/hpdi32/visualhpdi32/win32/close.c $
// $Rev: 2902 $
// $Date: 2009-08-09 17:48:49 -0500 (Sun, 09 Aug 2009) $

/*****************************************************************************
*
*	Implement services for closing a device.
*
*****************************************************************************/

#include <windows.h>
#include <stdio.h>

#include "close.h"
#include "resource.h"
#include "rx.h"
#include "tx.h"
#include "util_ctrl.h"



/*****************************************************************************
*
*	Function:	_service_refresh
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_REFRESH service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_refresh(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	HWND	control;

	for (;;)	// A convenience loop.
	{
		if (device->handle == NULL)
		{
			// The device is closed.
			ctrl_menu_set(dlg, IDC_CLOSE, CTRL_DISABLE);
			break;
		}

		// The device is open.
		// Force the open/close button to the CLOSE function.
		control	= GetDlgItem(dlg, IDC_OPEN);
		SetWindowLong(control, GWL_ID, IDC_CLOSE);
		SetWindowText(control, "Close");

		ctrl_menu_set(dlg, IDC_CLOSE, CTRL_ENABLE);
		break;
	}

	return(0);
}



/*****************************************************************************
*
*	Function:	_service_wm_command
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_WM_COMMAND service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_wm_command(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	char	buf[64];
	HWND	control;
	HCURSOR	cursor;
	U32		status;

	cursor	= SetCursor(device->cursor.wait);
	sprintf(buf, "Closing device #%d ...", (int) device->index);
	main_refresh(device, buf);
	status	= hpdi32_close(device->handle);
	device->handle	= NULL;
	sprintf(buf, "Closed device #%d.", (int) device->index);
	tx_oscillator_service(dlg, device, SERVICE_DEVICE_CLOSE, 0, 0);
	tx_io_service(dlg, device, SERVICE_DEVICE_CLOSE, 0, 0);
	rx_io_service(dlg, device, SERVICE_DEVICE_CLOSE, 0, 0);
	main_refresh(device, buf);
	error_status(dlg, 0, status);
	control	= GetDlgItem(dlg, IDC_OPEN);
	SetFocus(control);
	SetCursor(cursor);
	return(0);
}



/*****************************************************************************
*
*	Function:	close_service
*
*	Purpose:
*
*		Implement the common service entry point for the Device Close services.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		service	The service to perform.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0		No service was called.
*		else	The value from the called service, which may be zero.
*
*****************************************************************************/

LRESULT close_service(
	HWND			dlg,
	device_t*		device,
	service_type_t	service,
	WPARAM			wParam,
	LPARAM			lParam)
{
	static const service_call_t	list[]	=
	{
		/* service				function			*/
		{ SERVICE_REFRESH,		_service_refresh	},
		{ SERVICE_WM_COMMAND,	_service_wm_command	},
		{ SERVICE_END_LIST,		NULL				}
	};

	LRESULT	ret;

	ret	= service_call(dlg, device, list, service, wParam, lParam);
	return(ret);
}


