// HPDI32 - Application - Win32 - Any Compiler
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/SDK/hpdi32/visualhpdi32/win32/config.c $
// $Rev: 2809 $
// $Date: 2009-07-29 21:26:35 -0500 (Wed, 29 Jul 2009) $

/*****************************************************************************
*
*	Implement services for parameter Configuration.
*
*****************************************************************************/

#include <windows.h>

#include "config.h"
#include "c_code.h"
#include "cable.h"
#include "fifo.h"
#include "gui.h"
#include "misc.h"
#include "resource.h"
#include "rx.h"
#include "tx.h"
#include "util_ctrl.h"



/* variables *****************************************************************/

static const char	_intro_text[]	=
	"This Configuration Wizard will take you step by step through the process"
	" of configuring an HPDI32. This includes parameters from the four basic"
	" categories listed below. Within the categories each applicable"
	" parameter is presented for modification. All parameters, with their"
	" current settings and options, are presented with descriptions,"
	" suggestions and input guidelines to aid in the configuration process.\n"
	"\n"
	"    1. Some Miscellaneous Parameters\n"
	"    2. Several Cable Parameters\n"
	"    3. Several Transmit related parameters\n"
	"    4. Several Receive related parameters\n"
	"\n"
	"When completed, use the Generate C Source Code menu option to view"
	" sample C code that will configure an HPDI32 according to the settings"
	" you've made with Visual HPDI32. The C code may be saved and the sample"
	" code included in your own project. The sample configuration function"
	" may then be called by your own code to perform the requested"
	" confiuration.";

static const char	_done_text[]	=
	"The configuration process is now complete. From here, you may click the"
	" Back button as needed to go back and review the settings you've made"
	" and/or to make alternate selections. You may instead click the Finish"
	" button to close the wizard and begin using the HPDI32 with the settings"
	" you've made.\n"
	"\n"
	"When the wizard is closed, use the Generate C Source Code menu option to"
	" view and save sample C code that will configure an HPDI32 according to"
	" the settings made. As you confinue to access and modify parameters from"
	" within Visual HPDI32, those settings will automatically be reflected in"
	" the sample C code.";

static const config_progress_t	_progress_misc	=
{
	/* misc		*/	IDI_ARROW_RIGHT,
	/* cable	*/	0,
	/* tx		*/	0,
	/* rx		*/	0
};

static const config_progress_t	_progress_cable	=
{
	/* misc		*/	IDI_CHECK_MARK,
	/* cable	*/	IDI_ARROW_RIGHT,
	/* tx		*/	0,
	/* rx		*/	0
};

static const config_progress_t	_progress_tx	=
{
	/* misc		*/	IDI_CHECK_MARK,
	/* cable	*/	IDI_CHECK_MARK,
	/* tx		*/	IDI_ARROW_RIGHT,
	/* rx		*/	0
};

static const config_progress_t	_progress_rx	=
{
	/* misc		*/	IDI_CHECK_MARK,
	/* cable	*/	IDI_CHECK_MARK,
	/* tx		*/	IDI_CHECK_MARK,
	/* rx		*/	IDI_ARROW_RIGHT
};

static const config_progress_t	_progress_done	=
{
	/* misc		*/	IDI_CHECK_MARK,
	/* cable	*/	IDI_CHECK_MARK,
	/* tx		*/	IDI_CHECK_MARK,
	/* rx		*/	IDI_CHECK_MARK
};

static const config_parm_list_t	_misc_list	=
{
	/* title	*/	"Miscellaneous Parameter",
	/* prefix	*/	NULL,
	/* list		*/	&misc_parm_list
};

static const config_parm_list_t	_cable_list	=
{
	/* title	*/	"Cable Parameter",
	/* prefix	*/	NULL,
	/* list		*/	&cable_parm_list
};

static const config_parm_list_t	_tx_io_list	=
{
	/* title	*/	"I/O Parameter",
	/* prefix	*/	"Tx",
	/* list		*/	&tx_io_parm_list
};

static const config_parm_list_t	_tx_fifo_list	=
{
	/* title	*/	"FIFO Parameter",
	/* prefix	*/	"Tx",
	/* list		*/	&fifo_tx_parm_list
};

static const config_parm_list_t	_tx_list	=
{
	/* title	*/	"Transmitter Parameter",
	/* prefix	*/	NULL,
	/* list		*/	&tx_parm_list
};

static const config_parm_list_t	_rx_io_list	=
{
	/* title	*/	"I/O Parameter",
	/* prefix	*/	"Rx",
	/* list		*/	&rx_io_parm_list
};

static const config_parm_list_t	_rx_fifo_list	=
{
	/* title	*/	"FIFO Parameter",
	/* prefix	*/	"Rx",
	/* list		*/	&fifo_rx_parm_list
};

static const config_parm_list_t	_rx_list	=
{
	/* title	*/	"Receiver Parameter",
	/* prefix	*/	NULL,
	/* list		*/	&rx_parm_list
};

static	config_item_t	_config_list[]	=
{
	{
		/* title	*/	"Introduction",
		/* desc		*/	"Welcome to the Configuration Wizard.",
		/* type		*/	TYPE_TEXT,
		/* vp		*/	(void*) _intro_text
	},
	{
		/* title	*/	"Miscellaneous Parameters",
		/* desc		*/	"We are now ready to configure Miscellaneous Parameters.",
		/* type		*/	TYPE_PROGRESS,
		/* vp		*/	(void*) &_progress_misc
	},
	{
		/* title	*/	"Miscellaneous Parameters",
		/* desc		*/	"",
		/* type		*/	TYPE_PARM_LIST,
		/* vp		*/	(void*) &_misc_list
	},
	{
		/* title	*/	"Cable Parameters",
		/* desc		*/	"We are now ready to configure Cable Parameters.",
		/* type		*/	TYPE_PROGRESS,
		/* vp		*/	(void*) &_progress_cable
	},
	{
		/* title	*/	"Cable Parameters",
		/* desc		*/	"",
		/* type		*/	TYPE_PARM_LIST,
		/* vp		*/	(void*) &_cable_list
	},
	{
		/* title	*/	"Transmit Related Parameters",
		/* desc		*/	"We are now ready to configure Transmit related Parameters.",
		/* type		*/	TYPE_PROGRESS,
		/* vp		*/	(void*) &_progress_tx
	},
	{
		/* title	*/	"Transmit Related Parameters",
		/* desc		*/	"",
		/* type		*/	TYPE_PARM_LIST,
		/* vp		*/	(void*) &_tx_io_list
	},
	{
		/* title	*/	"Transmit Related Parameters",
		/* desc		*/	"",
		/* type		*/	TYPE_PARM_LIST,
		/* vp		*/	(void*) &_tx_fifo_list
	},
	{
		/* title	*/	"Transmit Related Parameters",
		/* desc		*/	"",
		/* type		*/	TYPE_PARM_LIST,
		/* vp		*/	(void*) &_tx_list
	},
	{
		/* title	*/	"Receive Related Parameters",
		/* desc		*/	"We are now ready to configure Receive related Parameters.",
		/* type		*/	TYPE_PROGRESS,
		/* vp		*/	(void*) &_progress_rx
	},
	{
		/* title	*/	"Receive Related Parameters",
		/* desc		*/	"",
		/* type		*/	TYPE_PARM_LIST,
		/* vp		*/	(void*) &_rx_io_list
	},
	{
		/* title	*/	"Receive Related Parameters",
		/* desc		*/	"",
		/* type		*/	TYPE_PARM_LIST,
		/* vp		*/	(void*) &_rx_fifo_list
	},
	{
		/* title	*/	"Receive Related Parameters",
		/* desc		*/	"",
		/* type		*/	TYPE_PARM_LIST,
		/* vp		*/	(void*) &_rx_list
	},
	{
		/* title	*/	"Configuration Complete",
		/* desc		*/	"All appropriate parameters have been configured.",
		/* type		*/	TYPE_PROGRESS,
		/* vp		*/	(void*) &_progress_done
	},
	{
		/* title	*/	"Finished",
		/* desc		*/	"Configuration is now complete.",
		/* type		*/	TYPE_TEXT,
		/* vp		*/	(void*) _done_text
	}
};



/*****************************************************************************
*
*	Function:	_scan_config_item_t
*
*	Purpose:
*
*		Locate the parameter structure coresponding to the given index.
*
*	Arguments:
*
*		item		The structure to access.
*
*		index		The index of the parameter to access relative to the
*					item being accessed.
*
*	Returned:
*
*		NULL		It wasn't found.
*		else		A pointer to the located item.
*
*****************************************************************************/

static const parm_entry_t* _scan_config_item_t(
	const config_item_t*	item,
	int*					index)
{
	const parm_entry_t*			entry	= NULL;
	const parm_list_t*			list;
	const config_parm_list_t*	parm;

	if (item->type == TYPE_PARM_LIST)
	{
		parm	= (const config_parm_list_t*) item->vp;
		list	= parm->list;

		if (index[0] < (int) list->size)
			entry		= list->list[index[0]];
		else
			index[0]	-= (int) list->size;
	}

	return(entry);
}



/*****************************************************************************
*
*	Function:	_parm_text_to_value
*
*	Purpose:
*
*		Convert a parameters text value to a numeric value.
*
*	Arguments:
*
*		entry		The structure to process.
*
*		src			The text representation to convert.
*
*		value		The value is recorded here.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

static U32 _parm_text_to_value(
	const parm_entry_t*	entry,
	const char*			src,
	U32*				value)
{
	size_t						i;
	const value_list_entry_t*	list;
	U32							status;

	if (entry->values->type == VALUE_TYPE_LIST)
	{
		status	= numeric_text_2_u32(src, 1024, value);
		list	= entry->values->u.list->list;
		i		= entry->values->u.list->size;

		for (; i--;)
		{
			if (stricmp(src, list[i].value_text) == 0)
			{
				value[0]	= list[i].value;
				status		= GSC_SUCCESS;
				break;
			}
		}
	}
	else if (entry->values->type == VALUE_TYPE_RANGE)
	{
		status	= numeric_text_2_u32(src, entry->values->u.range->k, value);
	}
	else
	{
		status	= GSC_FAILED;
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	config_list_retrieve
*
*	Purpose:
*
*		Give the caller access to the configuration parameter list.
*
*	Arguments:
*
*		list	The list pointer goes here.
*
*		size	The size of the list goes here.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void config_list_retrieve(const config_item_t** list, size_t* size)
{
	list[0]	= _config_list;
	size[0]	= ARRAY_ELEMENTS(_config_list);
}



/*****************************************************************************
*
*	Function:	config_show_entry_c_code
*
*	Purpose:
*
*		Generate C code for the current list box selection.
*
*	Arguments:
*
*		dlg			The dialog box to access.
*
*		device		The device being accessed.
*
*		list		The data list to access.
*
*		size		The size of the list.
*
*		dest		Format the C code into this buffer.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void config_show_entry_c_code(
	HWND					dlg,
	device_t*				device,
	const config_item_t*	list,
	size_t					size,
	char*					dest)
{
	char						buf[128];
	HWND						control;
	const parm_entry_t*			entry;
	int							i;
	int							index;
	int							j;
	const config_parm_list_t*	parm;
	char*						psz;
	U32							status;
	U32							u32;

	control	= GetDlgItem(dlg, IDC_CFG_FILE_LIST);
	index	= (int) SendMessage(control, LB_GETCURSEL, 0, 0);

	for (;;)	// A convenience loop.
	{
		if (index == LB_ERR)
			break;

		SendMessage(control, LB_GETTEXT, (WPARAM) index, (LPARAM) buf);
		psz	= strchr(buf, '=');

		if ((buf[0] == 0) || (strchr(buf, ' ')) || (psz == NULL))
			break;

		for (i = 0; i < (int) size; i++)
		{
			j		= index;
			entry	= _scan_config_item_t(&list[i], &index);

			if (entry)
			{
				status	= _parm_text_to_value(entry, psz + 1, &u32);

				if (status == GSC_SUCCESS)
				{
					parm	= (const config_parm_list_t*) list[i].vp;
					strcpy(dest, "status = ");
					c_code_format_entry(device,
										parm->list,
										j,
										TRUE,
										u32,
										dest + strlen(dest));
				}
				else
				{
					strcpy(dest, "VALUE ERROR");
				}

				break;
			}
		}

		break;
	}
}


