// HPDI32 - Application - Win32 - Any Compiler
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/SDK/hpdi32/visualhpdi32/win32/data_xfer.c $
// $Rev: 2902 $
// $Date: 2009-08-09 17:48:49 -0500 (Sun, 09 Aug 2009) $

/*****************************************************************************
*
*	Implement common support services for the Data Read and Data Write
*	operations.
*
*****************************************************************************/

#include <windows.h>
#include <process.h>
#include <stdio.h>

#include "data_xfer.h"
#include "align.h"
#include "ini.h"
#include "gui.h"
#include "resource.h"
#include "util_ctrl.h"



/* #defines	*****************************************************************/

#define	DMA_MAN_UNSUPPORTED	"DMA with Manual Control is unsupported at this time."



/* prototypes	*************************************************************/

static LRESULT	_wm_paint(HWND dlg, int invalidate);



/*****************************************************************************
*
*	Function:	_show_window_status
*
*	Purpose:
*
*		Update the window to show the current status;
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		msg		The message to display.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _show_window_status(HWND dlg, data_xfer_t* data)
{
	if (data->status != GSC_SUCCESS)
	{
		error_status(dlg, IDC_DATA_XFER_STATUS, data->status);
	}
	else if (data->status_local[0])
	{
		data->status_global[0]	= 0;
		SetDlgItemText(dlg, IDC_DATA_XFER_STATUS, data->status_local);
	}
	else if (data->status_global[0])
	{
		SetDlgItemText(dlg, IDC_DATA_XFER_STATUS, data->status_global);
	}
	else
	{
		SetDlgItemText(dlg, IDC_DATA_XFER_STATUS, NULL);
	}
}



/*****************************************************************************
*
*	Function:	_align_controls
*
*	Purpose:
*
*		Align the controls within the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		data		The dialog specific data we track.
*
*		resize		Resize the window after moving the controls?
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _align_controls(HWND dlg, data_xfer_t* data, int resize)
{
	static	align_t list1[]	=
	{
		// The "Start" button.
		{// 0
		/* id		*/	IDC_DATA_XFER_START,
		/* ref		*/	0,
		/* flags	*/	ALIGN_L | ALIGN_W | ALIGN_T | ALIGN_H,
		/* value	*/	-999	// see below
		},

		// The "Stop" button.
		{// 1
		/* id		*/	IDC_DATA_XFER_STOP,
		/* ref		*/	IDC_DATA_XFER_START,
		/* flags	*/	ALIGN_COPY,
		/* value	*/	0
		},
		{// 2
		/* id		*/	IDC_DATA_XFER_STOP,
		/* ref		*/	IDC_DATA_XFER_START,
		/* flags	*/	ALIGN_L_R | ALIGN_W,
		/* value	*/	-999	// see below
		},

		// The "File" button.
		{// 3
		/* id		*/	IDC_DATA_XFER_FILE,
		/* ref		*/	IDC_DATA_XFER_STOP,
		/* flags	*/	ALIGN_COPY,
		/* value	*/	0
		},
		{// 4
		/* id		*/	IDC_DATA_XFER_FILE,
		/* ref		*/	IDC_DATA_XFER_STOP,
		/* flags	*/	ALIGN_L_R | ALIGN_W,
		/* value	*/	-999	// see below
		},

		// The "Close" button.
		{// 5
		/* id		*/	IDCANCEL,
		/* ref		*/	IDC_DATA_XFER_FILE,
		/* flags	*/	ALIGN_COPY,
		/* value	*/	0
		},
		{// 6
		/* id		*/	IDCANCEL,
		/* ref		*/	IDC_DATA_XFER_FILE,
		/* flags	*/	ALIGN_L_R | ALIGN_W,
		/* value	*/	-999	// see below
		},

		// The "Amount" static text.
		{// 7
		/* id		*/	IDC_DATA_XFER_AMOUNT_TEXT,
		/* ref		*/	IDCANCEL,
		/* flags	*/	ALIGN_L_R | ALIGN_W,
		/* value	*/	-999	// see below
		},
		{// 8
		/* id		*/	IDC_DATA_XFER_AMOUNT_TEXT,
		/* ref		*/	IDCANCEL,
		/* flags	*/	ALIGN_VC_C | ALIGN_H,
		/* value	*/	0
		},

		// The "Amount" edit box.
		{// 9
		/* id		*/	IDC_DATA_XFER_AMOUNT_EDIT,
		/* ref		*/	IDC_DATA_XFER_AMOUNT_TEXT,
		/* flags	*/	ALIGN_L_R | ALIGN_W,
		/* value	*/	MARGIN
		},
		{// 10
		/* id		*/	IDC_DATA_XFER_AMOUNT_EDIT,
		/* ref		*/	IDCANCEL,
		/* flags	*/	ALIGN_T_T | ALIGN_B_B,
		/* value	*/	0
		},

		// The "Go To" button.
		{// 11
		/* id		*/	IDC_DATA_XFER_GOTO,
		/* ref		*/	IDC_DATA_XFER_AMOUNT_EDIT,
		/* flags	*/	ALIGN_L_R | ALIGN_W,
		/* value	*/	-999	// see below
		},

		{// 12
		/* id		*/	IDC_DATA_XFER_GOTO,
		/* ref		*/	IDCANCEL,
		/* flags	*/	ALIGN_T_T | ALIGN_B_B,
		/* value	*/	0
		},

		// The "Go To" edit box.
		{// 13
		/* id		*/	IDC_DATA_XFER_GOTO_EDIT,
		/* ref		*/	IDC_DATA_XFER_GOTO,
		/* flags	*/	ALIGN_L_R | ALIGN_W,
		/* value	*/	MARGIN
		},
		{// 14
		/* id		*/	IDC_DATA_XFER_GOTO_EDIT,
		/* ref		*/	IDCANCEL,
		/* flags	*/	ALIGN_T_T | ALIGN_B_B,
		/* value	*/	0
		},

		{ 0, 0, 0, 0 }
	};

	static	align_t list2[]	=
	{
		// The status text.
		{// 0
		/* id		*/	IDC_DATA_XFER_STATUS,
		/* ref		*/	0,
		/* flags	*/	ALIGN_L,
		/* value	*/	-999	// see below
		},
		{// 1
		/* id		*/	IDC_DATA_XFER_STATUS,
		/* ref		*/	0,
		/* flags	*/	ALIGN_B_B | ALIGN_H,
		/* value	*/	-MARGIN
		},
		{// 2
		/* id		*/	IDC_DATA_XFER_STATUS,
		/* ref		*/	0,
		/* flags	*/	ALIGN_R_R,
		/* value	*/	-999	// see below
		},

		// The status frame.
		{// 3
		/* id		*/	IDC_DATA_XFER_FRAME,
		/* ref		*/	0,
		/* flags	*/	ALIGN_L,
		/* value	*/	-1
		},
		{// 4
		/* id		*/	IDC_DATA_XFER_FRAME,
		/* ref		*/	IDC_DATA_XFER_STATUS,
		/* flags	*/	ALIGN_T_T,
		/* value	*/	- MARGIN
		},
		{// 5
		/* id		*/	IDC_DATA_XFER_FRAME,
		/* ref		*/	0,
		/* flags	*/	ALIGN_B_B | ALIGN_R_R,
		/* value	*/	1
		},

		// The scroll bar.
		{// 6
		/* id		*/	IDC_DATA_XFER_POSITION,
		/* ref		*/	0,
		/* flags	*/	ALIGN_L,
		/* value	*/	-999	// see below
		},
		{// 7
		/* id		*/	IDC_DATA_XFER_POSITION,
		/* ref		*/	0,
		/* flags	*/	ALIGN_R_R,
		/* value	*/	-999	// see below
		},
		{// 8
		/* id		*/	IDC_DATA_XFER_POSITION,
		/* ref		*/	IDC_DATA_XFER_FRAME,
		/* flags	*/	ALIGN_B_T | ALIGN_H,
		/* value	*/	-999	// see below
		},

		// The data frame.
		{// 9
		/* id		*/	IDC_DATA_XFER_DATA,
		/* ref		*/	IDC_DATA_XFER_POSITION,
		/* flags	*/	ALIGN_L_L | ALIGN_B_T | ALIGN_R_R,
		/* value	*/	0
		},
		{// 10
		/* id		*/	IDC_DATA_XFER_DATA,
		/* ref		*/	IDC_DATA_XFER_START,
		/* flags	*/	ALIGN_T_B,
		/* value	*/	-999	// see below
		},

		{ 0, 0, 0, 0 }
	};

	list1[0].value	= data->device->margin;
	list1[2].value	= data->device->margin;
	list1[4].value	= data->device->margin;
	list1[6].value	= data->device->margin;
	list1[7].value	= data->device->margin;
	list1[11].value	= data->device->margin;

	list2[0].value	= data->device->margin;
	list2[2].value	= - data->device->margin;
	list2[6].value	= data->device->margin;
	list2[7].value	= - data->device->margin;
	list2[8].value	= - data->device->margin;
	list2[10].value	= data->device->margin;

	if (resize)
		align_control_list(dlg, list1);

	align_control_list(dlg, list2);

	GetWindowRect(data->data.control, &data->data.rect);
	ScreenToClient(data->data.control, (POINT*) &data->data.rect.left);
	ScreenToClient(data->data.control, (POINT*) &data->data.rect.right);
	data->data.rect.left++;
	data->data.rect.right--;
	data->data.rect.top++;
	data->data.rect.bottom--;
}



/*****************************************************************************
*
*	Function:	_refresh_window_local
*
*	Purpose:
*
*		Update the controls based on the current state without reseting status.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		data		The dialog specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _refresh_window_local(HWND dlg, data_xfer_t* data)
{
	HWND	control;

	set_win_title(dlg, data->device, FALSE, TRUE, data->title);

	control	= GetDlgItem(dlg, IDC_DATA_XFER_STOP);
	EnableWindow(control, data->active);
	control	= GetDlgItem(dlg, IDCANCEL);
	EnableWindow(control, (BOOL) !data->active);
	control	= GetDlgItem(dlg, IDC_DATA_XFER_GOTO);
	EnableWindow(control, (BOOL) ((!data->active) && (data->data.count)));
	control	= GetDlgItem(dlg, IDC_DATA_XFER_GOTO_EDIT);
	EnableWindow(control, (BOOL) ((!data->active) && (data->data.count)));

	(data->refresh_window)(dlg, data);
	_show_window_status(dlg, data);
	_wm_paint(dlg, TRUE);

	if (data->device->handle == NULL)
		SendMessage(dlg, IDC_GPIO_XFER_STOP, 0, 0);
}



/*****************************************************************************
*
*	Function:	_refresh_window
*
*	Purpose:
*
*		Update the controls based on the current state.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		device		The device to access.
*
*		msg			A status message to apply.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _refresh_window(HWND dlg, device_t* device, const char* msg)
{
	data_xfer_t*	data		= (data_xfer_t*) GET_WINDOW_USER_DATA(dlg);

	if (!data->active)
	{
		data->status_local[0]	= 0;

		if (msg)
			strcpy(data->status_global, msg);
		else
			data->status_global[0]	= 0;

		_refresh_window_local(dlg, data);
	}
}



/*****************************************************************************
*
*	Function:	_idc_data_xfer_amount_edit
*
*	Purpose:
*
*		Read and validate the value in the Amount edit control.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		data		The dialog specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idc_data_xfer_amount_edit(HWND dlg, data_xfer_t* data)
{
	char	buf[32];
	U32		value;

	data->status_local[0]	= 0;
	data->status_global[0]	= 0;
	data->status	= ctrl_text_2_u32(dlg, IDC_DATA_XFER_AMOUNT_EDIT, 1024, &value);

	if ((value == 0) || (value > _1G))
		data->status	= GSC_INVALID_DATA;

	if (data->status == GSC_SUCCESS)
	{
		data->amount	= value;
		format_u32(value, 1024, buf);
		ctrl_menu_set_text(dlg, IDC_DATA_XFER_AMOUNT_EDIT, buf);
	}
	else
	{
		ctrl_menu_set(dlg, IDC_DATA_XFER_AMOUNT_EDIT, CTRL_FOCUS);
	}

	_show_window_status(dlg, data);
}



/*****************************************************************************
*
*	Function:	_thread_progress
*
*	Purpose:
*
*		Update the progress of the read operation.
*
*	Arguments:
*
*		pvArg	The thread's entry arguemtn.
*
*	Returned:
*
*		0		We always return 0.
*
*****************************************************************************/

static unsigned WINAPI _thread_progress(LPVOID pvArg)
{
	data_xfer_t*	data	= (data_xfer_t*) pvArg;
	int				stop;
	DWORD			thread_id;

	thread_id	= GetCurrentThreadId();
	AttachThreadInput(thread_id, data->thread.id, TRUE);

	for (;;)
	{
		if (data->time_start == 0)
		{
			Sleep(1);
		}
		else
		{
			stop	= (data->xfer_progress)(data);
			_refresh_window_local(data->dlg, data);

			if (stop)
				break;
			else
				Sleep(250);
		}
	}

	return(0);
}



/*****************************************************************************
*
*	Function:	_thread_xfer
*
*	Purpose:
*
*		Implement the Data Transfer service.
*
*	Arguments:
*
*		pvArg	The thread's entry arguemtn.
*
*	Returned:
*
*		0		We always return 0.
*
*****************************************************************************/

static unsigned WINAPI _thread_xfer(LPVOID pvArg)
{
	data_xfer_t*	data		= (data_xfer_t*) pvArg;
	unsigned		id;
	U32				repeat;
	U32				status;
	DWORD			thread_id;
	U32				xfer		= 0;
	U32				xfer_tot	= data->xfer_tot;
	U32				xfered;

	data->active		= TRUE;
	data->abort			= FALSE;
	data->abort_request	= FALSE;
	data->api_buf_ptr	= (U8*) data->api_buf;
	data->app_buf_ptr	= data->app_buf;
	data->status		= GSC_SUCCESS;
	repeat				= data->repeat;
	data->repeat		= 0;

	thread_id	= GetCurrentThreadId();
	AttachThreadInput(thread_id, data->thread.id, TRUE);

	_refresh_window_local(data->dlg, data);
	ctrl_menu_set(data->dlg, IDC_DATA_XFER_STOP, CTRL_ENABLE | CTRL_FOCUS);
	_beginthreadex(NULL, 0, _thread_progress, data, 0, &id);
	data->time_start	= timeGetTime();

	for (;;)
	{
		if (xfer)
		{
		}
		else if (data->api_buf)
		{
			// Data is being transferred through the API buffer.
			// The Tx/Rx code will tend to the trailing fragments.
			data->api_buf_ptr	= (U8*) data->api_buf;

			if (xfer_tot >= data->api_buf_size)
				xfer	= data->api_buf_size;
			else
				xfer	= xfer_tot;
		}
		else
		{
			// Data is being transferred directly via the Application buffer.
			xfer	= data->xfer_tot;
		}

		status	= (data->xfer_data)(data, xfer, &xfered);

		if (status != GSC_SUCCESS)
		{
			data->time_stop	= timeGetTime();
			data->status	= status;
			(data->xfer_end)(data);
			main_refresh(data->device, NULL);
			data->active	= FALSE;
			break;
		}

		if (data->abort_request)
		{
			data->time_stop	= timeGetTime();
			data->abort		= TRUE;
			(data->xfer_end)(data);
			main_refresh(data->device, NULL);
			data->active	= FALSE;
			break;
		}

		xfer				-= xfered;
		xfer_tot			-= xfered;
		data->xfered_tot	+= xfered;
		data->app_buf_ptr	+= xfered;

		if (data->api_buf)
			data->api_buf_ptr	+= xfered;

		if ((xfer) || (xfer_tot))
			continue;

		if (repeat)
		{
			data->xfered_tot	= 0;
			data->repeat++;

			if (data->repeat < repeat)
			{
				xfer_tot			= data->xfer_tot;
				data->app_buf_ptr	= data->app_buf;
				continue;
			}
		}

		data->time_stop	= timeGetTime();
		(data->xfer_end)(data);
		main_refresh(data->device, NULL);
		data->active	= FALSE;
		break;
	}

	ctrl_menu_set(data->dlg, IDC_DATA_XFER_START, CTRL_ENABLE | CTRL_FOCUS);
	_refresh_window_local(data->dlg, data);
	_wm_paint(data->dlg, TRUE);
	return(0);
}



/*****************************************************************************
*
*	Function:	_idc_data_xfer_file
*
*	Purpose:
*
*		Prompt for a file name and perform an appropriate action.
*
*	Arguments:
*
*		data		The dialog specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idc_data_xfer_file(HWND dlg, data_xfer_t* data)
{
	data->status			= GSC_SUCCESS;
	data->status_global[0]	= 0;
	data->status_local[0]	= 0;

	(data->xfer_file)(data);
	_show_window_status(dlg, data);
	_refresh_window_local(dlg, data);
	_wm_paint(dlg, TRUE);
}



/*****************************************************************************
*
*	Function:	_idc_data_xfer_goto
*
*	Purpose:
*
*		Process a Go To request.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		device		The device to access.
*
*		data		The data we're to display.
*
*		force		Force an update of the display data.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _idc_data_xfer_goto(HWND dlg, data_xfer_t* data, int force)
{
	char	buf[32];
	U32		ds;
	U32		offset;
	U32		stat;
	U32		status	= GSC_SUCCESS;

	data->status	= GSC_SUCCESS;

	switch (data->data.size)
	{
		default:
		case	HPDI32_IO_DATA_SIZE_8_BITS:		ds	= 1;	break;
		case	HPDI32_IO_DATA_SIZE_16_BITS:	ds	= 2;	break;
		case	HPDI32_IO_DATA_SIZE_32_BITS:	ds	= 4;	break;
	}

	stat		= ctrl_text_2_u32(dlg, IDC_DATA_XFER_GOTO_EDIT, 1024, &offset);
	status		= (status == GSC_SUCCESS) ? stat : status;

	if (stat != GSC_SUCCESS)
	{
		offset	= 0;
		ctrl_menu_set(dlg, IDC_DATA_XFER_GOTO_EDIT, CTRL_FOCUS);
	}
	else
	{
		if (data->data.count == 0)
		{
			offset	= 0;
		}
		else
		{
			if (data->data.page_delta >= data->data.count)
				offset	= 0;
			else if (offset > (data->data.count - data->data.page_delta))
				offset	= data->data.count - data->data.page_delta;

			if ((offset) && (offset >= data->data.count))
				offset	= data->data.count - 1;

			offset	-= offset % ds;
		}

		sprintf(buf, "0x%lX", (long) offset);
		ctrl_menu_set_text(dlg, IDC_DATA_XFER_GOTO_EDIT, buf);

		if (offset != data->offset)
		{
			data->offset	= offset;
			force			= TRUE;
		}
	}

	if (force)
		_wm_paint(dlg, TRUE);

	if (status != GSC_SUCCESS)
		data->status	= status;

	_show_window_status(dlg, data);
	return(0);
}



/*****************************************************************************
*
*	Function:	_idc_data_xfer_start
*
*	Purpose:
*
*		Respond to a START button click.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		data		The dialog specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idc_data_xfer_start(HWND dlg, data_xfer_t* data)
{
	U32			dma_mode;
	int			ds;
	unsigned	id;
	U32			io_mode;
	int			procede;
	U32			stat;
	U32			status		= GSC_SUCCESS;
	THREAD_T	thread;

	for (;;)	// A convenience loop.
	{
		// Make sure the I/O setup is supported.
		stat	= HPDI32_IO_MODE__GET(data->device->handle, data->which, &io_mode);
		status	= (status == GSC_SUCCESS) ? stat : status;

		stat	= HPDI32_IO_DMA_CONTROL_MODE__GET(	data->device->handle,
													data->which,
													&dma_mode);
		status	= (status == GSC_SUCCESS) ? stat : status;

		if (status != GSC_SUCCESS)
		{
			error_status(dlg, IDC_DATA_XFER_STATUS, status);
			break;
		}

		if ((io_mode == HPDI32_IO_MODE_DMA) &&
			(dma_mode == HPDI32_IO_DMA_CONTROL_MODE_MANUAL))
		{
			ctrl_menu_set_text(dlg, IDC_DATA_XFER_STATUS, DMA_MAN_UNSUPPORTED);
			break;
		}

		// Get the API Buffer info, in case we're to use it.
		stat	= HPDI32_IO_BUFFER_POINTER__GET(	data->device->handle,
													data->which,
													(void*) &data->api_buf);
		status	= (status == GSC_SUCCESS) ? stat : status;

		stat	= HPDI32_IO_BUFFER_SIZE__GET(	data->device->handle,
												data->which,
												&data->api_buf_size);
		status	= (status == GSC_SUCCESS) ? stat : status;

		if (status != GSC_SUCCESS)
		{
			error_status(dlg, IDC_DATA_XFER_STATUS, status);
			break;
		}

		// See if the Tx/Rx code is ready.
		procede	= (data->xfer_begin)(data);
		_wm_paint(dlg, FALSE);

		if (!procede)
			break;

		// Make sure we can still procede.

		switch (data->data.size)
		{
			default:
			case	HPDI32_IO_DATA_SIZE_8_BITS:		ds	= 1;	break;
			case	HPDI32_IO_DATA_SIZE_16_BITS:	ds	= 2;	break;
			case	HPDI32_IO_DATA_SIZE_32_BITS:	ds	= 4;	break;
		}

		data->api_buf_size	-= data->api_buf_size % (U32) ds;
		data->xfer_tot		-= data->xfer_tot % (U32) ds;
		data->xfered_tot	= 0;

		if ((data->api_buf) && (data->api_buf_size < (U32) ds))
		{
			ctrl_menu_set_text(	dlg,
								IDC_DATA_XFER_STATUS,
								"Invalid API Buffer size.");
			(data->xfer_end)(data);
			break;
		}

		if (data->app_buf_size < (U32) ds)
		{
			ctrl_menu_set_text(	dlg,
								IDC_DATA_XFER_STATUS,
								"Invalid Application Buffer size.");
			(data->xfer_end)(data);
			break;
		}

		data->thread.id		= GetCurrentThreadId();
		thread				= _beginthreadex(NULL, 0, _thread_xfer, data, 0, &id);

		if (thread == (DWORD) -1L)
		{
			ctrl_menu_set_text(dlg, IDC_DATA_XFER_STATUS, "Unable to initiate operation.");
			(data->xfer_end)(data);
		}

		break;
	}
}



/*****************************************************************************
*
*	Function:	_idcancel
*
*	Purpose:
*
*		Process a request to close the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idcancel(HWND dlg)
{
	data_xfer_t*	data	= (data_xfer_t*) GET_WINDOW_USER_DATA(dlg);

	if (data)
	{
		ini_u32_set(NULL, data->section, data->amount_key, data->amount);

		if (data->active)
			SendMessage(dlg, WM_COMMAND, IDC_DATA_XFER_STOP, 0);

		data->dlg			= NULL;
		data->data.count	= 0;
		main_refresh_add(dlg, NULL);

		if (data->app_buf)
		{
			free(data->app_buf);
			data->app_buf_size	= 0;
			data->app_buf		= NULL;
		}

		win_pos_save(dlg, data->section);
	}

	DestroyWindow(dlg);
}



/*****************************************************************************
*
*	Function:	_wm_command
*
*	Purpose:
*
*		Process WM_COMMAND messages.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		wParam		The message's WORD parameter.
*
*	Returned:
*
*		TRUE		We processed the message.
*		FALSE		We did not process the message.
*
*****************************************************************************/

static LRESULT _wm_command(HWND dlg, WPARAM wParam)
{
	data_xfer_t*	data	= (data_xfer_t*) GET_WINDOW_USER_DATA(dlg);
	int				id		= LOWORD(wParam);
	LRESULT	ret		= TRUE;

	switch (id)
	{
		default:

			ret	= FALSE;
			break;

		case IDC_DATA_XFER_AMOUNT_EDIT:

			if (HIWORD(wParam) == EN_KILLFOCUS)
				_idc_data_xfer_amount_edit(dlg, data);

			ret	= FALSE;
			break;

		case IDC_DATA_XFER_FILE:

			_idc_data_xfer_file(dlg, data);
			break;

		case IDC_DATA_XFER_GOTO:

			_idc_data_xfer_goto(dlg, data, FALSE);
			break;

		case IDC_DATA_XFER_START:

			_idc_data_xfer_start(dlg, data);
			break;

		case IDC_DATA_XFER_STOP:

			data->abort_request	= TRUE;
			break;

		case IDCANCEL:

			_idcancel(dlg);
			break;
	}

	return(ret);
}



/*****************************************************************************
*
*	Function:	_wm_destroy
*
*	Purpose:
*
*		Process WM_DESTROY messages.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*	Returned:
*
*		0			We processed the message.
*
*****************************************************************************/

static LRESULT _wm_destroy(HWND dlg)
{
	_idcancel(dlg);
	DestroyWindow(dlg);
	return(0);
}



/*****************************************************************************
*
*	Function:	_wm_getminmaxinfo
*
*	Purpose:
*
*		Process a WM_GETMINMAXINFO message for the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		lParam		The messages's long size parameter.
*
*	Returned:
*
*		0			We processed the message.
*
*****************************************************************************/

static LRESULT _wm_getminmaxinfo(HWND dlg, LPARAM lParam)
{
	HWND		control;
	MINMAXINFO*	mmi		= (MINMAXINFO*) lParam;
	RECT		rect;

	control	= GetDlgItem(dlg, IDC_DATA_XFER_FILE);
	GetWindowRect(control, &rect);
	mmi->ptMinTrackSize.y	= 6 * (rect.bottom - rect.top);
	mmi->ptMinTrackSize.x	= rect.left;

	control	= GetDlgItem(dlg, IDC_DATA_XFER_START);
	GetWindowRect(control, &rect);

	if (mmi->ptMinTrackSize.x > rect.left)
		mmi->ptMinTrackSize.x	= rect.left;

	control	= GetDlgItem(dlg, IDC_DATA_XFER_STOP);
	GetWindowRect(control, &rect);

	if (mmi->ptMinTrackSize.x > rect.left)
		mmi->ptMinTrackSize.x	= rect.left;

	control	= GetDlgItem(dlg, IDCANCEL);
	GetWindowRect(control, &rect);
	mmi->ptMinTrackSize.x	+= rect.right;

	GetWindowRect(dlg, &rect);
	mmi->ptMinTrackSize.x	-= 2 * rect.left;
	return(0);
}



/*****************************************************************************
*
*	Function:	_wm_hscroll
*
*	Purpose:
*
*		Process a WM_HSCROLL message for the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		wParam		The message's WORD sized argument.
*
*	Returned:
*
*		FALSE		We processed the message.
*		TRUE		We did not processed the message.
*
*****************************************************************************/

static LRESULT _wm_hscroll(HWND dlg, WPARAM wParam)
{
	char			buf[32];
	int				code	= LOWORD(wParam);
	HWND			control;
	data_xfer_t*	data	= (data_xfer_t*) GET_WINDOW_USER_DATA(dlg);
	long			offset;
	SCROLLINFO		si;

	offset	= (long) data->offset;

	switch (code)
	{
		default:														break;
		case SB_BOTTOM:		offset	= (long) data->data.count;			break;
		case SB_LINEDOWN:	offset	+= (long) data->data.col_delta;		break;
		case SB_LINEUP:		offset	-= (long) data->data.col_delta;		break;
		case SB_PAGEDOWN:	offset	+= (long) data->data.page_delta;	break;
		case SB_PAGEUP:		offset	-= (long) data->data.page_delta;	break;
		case SB_TOP:		offset	= 0;								break;

		case SB_THUMBPOSITION:
		case SB_THUMBTRACK:

			memset(&si, 0, sizeof(si));
			si.cbSize	= sizeof(si);
			si.fMask	= SIF_TRACKPOS;
			control		= GetDlgItem(dlg, IDC_DATA_XFER_POSITION);
			GetScrollInfo(control, SB_CTL, &si);
			offset		= si.nTrackPos;
			break;
	}

	if (offset < 0)
		offset	= 0;

	sprintf(buf, "0x%lX", offset);
	ctrl_menu_set_text(dlg, IDC_DATA_XFER_GOTO_EDIT, buf);
	_idc_data_xfer_goto(dlg, data, FALSE);
	return(TRUE);
}



/*****************************************************************************
*
*	Function:	_wm_initdialog
*
*	Purpose:
*
*		Process a WM_INITDIALOG message for the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		lParam		The message's long sized argument.
*
*	Returned:
*
*		TRUE		Windows should set the initial control focus.
*		FALSE		Windows should NOT set the initial control focus.
*
*****************************************************************************/

static LRESULT _wm_initdialog(HWND dlg, LPARAM lParam)
{
	char			buf[32];
	HWND			control;
	data_xfer_t*	data	= (data_xfer_t*) lParam;

	SET_WINDOW_USER_DATA(dlg, lParam);

	SendMessage(dlg, WM_SETICON, ICON_SMALL, (LPARAM) data->device->icon.little);
	SendMessage(dlg, WM_SETICON, ICON_BIG, (LPARAM) data->device->icon.big);

	data->active	= FALSE;
	data->offset	= 0;

	data->data.control	= GetDlgItem(dlg, IDC_DATA_XFER_DATA);
	data->data.size		= HPDI32_IO_DATA_SIZE_32_BITS;

	_align_controls(dlg, data, TRUE);
	control	= GetDlgItem(dlg, IDC_DATA_XFER_GOTO_EDIT);
	SetWindowText(control, "0x0");
	(data->wm_initdialog)(dlg, data);

	ini_u32_get(NULL, data->section, data->amount_key, &data->amount, 1UL * 1024 * 1024);
	sprintf(buf, "%lu", (long) data->amount);
	ctrl_menu_set_text(dlg, IDC_DATA_XFER_AMOUNT_EDIT, buf);
	_idc_data_xfer_amount_edit(dlg, data);

	win_pos_restore(dlg, data->section, TRUE);
	_refresh_window_local(dlg, data);
	main_refresh_add(dlg, _refresh_window);
	return(TRUE);
}



/*****************************************************************************
*
*	Function:	_wm_paint
*
*	Purpose:
*
*		Process a WM_PAINT message for the dialogs.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		invalidate	Invalidate the entire data area?
*
*	Returned:
*
*		0			We processed the message.
*
*****************************************************************************/

static LRESULT _wm_paint(HWND dlg, int invalidate)
{
	unsigned long	address;
	HBRUSH			brush;
	char			buf[128];
	COLORREF		color_bk;
	COLORREF		color_txt;
	int				columns;
	U32				count;
	HWND			control;
	data_xfer_t*	data	= (data_xfer_t*) GET_WINDOW_USER_DATA(dlg);
	HDC				dc;
	U32				ds;
	int				enable;
	HFONT			font;
	int				h;
	int				i;
	int				j;
	int				len;
	HPEN			pen;
	int				rows;
	U32				src;
	TEXTMETRIC		tm;
	unsigned long	ul;
	int				w;
	int				x;
	int				y;

	switch (data->data.size)
	{
		default:
		case	HPDI32_IO_DATA_SIZE_8_BITS:		ds	= 1;	break;
		case	HPDI32_IO_DATA_SIZE_16_BITS:	ds	= 2;	break;
		case	HPDI32_IO_DATA_SIZE_32_BITS:	ds	= 4;	break;
	}

	count	= data->data.count - (data->data.count % ds);

	dc		= GetDC(data->data.control);
	font	= SelectObject(dc, data->device->font.fixed);
	GetTextMetrics(dc, &tm);
	w		= tm.tmAveCharWidth * 10;
	columns	= (data->data.rect.right - data->data.rect.left+ 1) / w;
	columns	= (columns <= 1) ? 1 : columns;
	h		= tm.tmHeight;
	rows	= ((data->data.rect.bottom - data->data.rect.top + 1) / h) - 1;
	rows	= (rows <= 1) ? 1 : rows;

	data->data.col_delta	= (U32) columns * ds;
	data->data.page_delta	= (U32) rows * (U32) columns * ds;

	if (invalidate)
	{
		InvalidateRect(data->data.control, NULL, TRUE);
		UpdateWindow(data->data.control);
	}

	// Paint the background.
	color_bk	= SetBkColor(dc, data->device->paint.window.color);
	pen			= SelectObject(dc, data->device->paint.window.pen);
	brush		= SelectObject(dc, data->device->paint.window.brush);
	Rectangle(	dc,
				data->data.rect.left,
				data->data.rect.top,
				data->data.rect.right,
				data->data.rect.bottom);

	// Display the column headings.
	address		= data->offset;
	x			= data->data.rect.left + tm.tmAveCharWidth;
	color_txt	= SetTextColor(dc, data->device->paint.window_text.color);

	for (i = 0; i < columns; i++)
	{
		if (address >= count)
			break;

		sprintf(buf, "%08lX", address);
		len	= (int) strlen(buf);
		ExtTextOut(	dc,
					x,
					data->data.rect.top,
					ETO_CLIPPED,
					&data->data.rect,
					buf,
					(UINT) len,
					NULL);
		address	+= (unsigned long) rows * ds;
		x		+= w;
	}

	// Display the data.
	address	= data->offset;
	src		= (U32) data->offset + (U32) data->app_buf;
	x		= data->data.rect.left + tm.tmAveCharWidth;

	for (i = 0; i < columns; i++)
	{
		y	= data->data.rect.top + h + 1;

		for (j = 0; j < rows; j++)
		{
			if ((address + (ds - 1)) >= count)
				break;

			switch (ds)
			{
				default:
				case	1:	ul	= (unsigned long) (((U8*) (src))[0]);
							sprintf(buf, "      %02lX", ul);
							break;

				case	2:	ul	= (unsigned long) (((U16*) (src))[0]);
							sprintf(buf, "    %04lX", ul);
							break;

				case	4:	ul	= (unsigned long) (((U32*) (src))[0]);
							sprintf(buf, "%08lX", ul);
							break;
			}

			len	= (int) strlen(buf);
			ExtTextOut(	dc,
						x,
						y,
						ETO_CLIPPED,
						&data->data.rect,
						buf,
						(UINT) len,
						NULL);
			address	+= ds;
			src		+= ds;
			y		+= h;
		}

		x	+= w;
	}

	// Paint the grid lines.
	SelectObject(dc, data->device->paint.black.pen);

	for (x = data->data.rect.left;;)
	{
		x	+= w;

		if (x >= data->data.rect.right)
			break;

		MoveToEx(dc, x, data->data.rect.top, NULL);
		LineTo(dc, x, data->data.rect.bottom);
	}

	MoveToEx(dc, data->data.rect.left, h, NULL);
	LineTo(dc, data->data.rect.right, h);

	// Update the scroll bar.
	control	= GetDlgItem(dlg, IDC_DATA_XFER_POSITION);

	if (data->data.page_delta >= count)
	{
		SetScrollRange(control, SB_CTL, 0, 0, FALSE);
		enable	= FALSE;
	}
	else
	{
		SetScrollRange(	control,
						SB_CTL,
						0,
						(int) (count - data->data.page_delta),
						FALSE);
		enable	= (count) && (data->active == FALSE);
	}

	SetScrollPos(control, SB_CTL, (int) data->offset, TRUE);
	EnableWindow(control, enable);

	// Clean up.
	SelectObject(dc, brush);
	SelectObject(dc, font);
	SelectObject(dc, pen);
	SetBkColor(dc, color_bk);
	SetTextColor(dc, color_txt);
	ReleaseDC(data->data.control, dc);
	return(0);
}



/*****************************************************************************
*
*	Function:	_wm_size
*
*	Purpose:
*
*		Process a WM_SIZE message for the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*	Returned:
*
*		0			We processed the message.
*
*****************************************************************************/

static LRESULT _wm_size(HWND dlg)
{
	data_xfer_t*	data	= (data_xfer_t*) GET_WINDOW_USER_DATA(dlg);

	_align_controls(dlg, data, TRUE);
	InvalidateRect(dlg, NULL, TRUE);
	UpdateWindow(dlg);
	_wm_paint(dlg, FALSE);
	_idc_data_xfer_goto(dlg, data, TRUE);
	return(0);
}



/*****************************************************************************
*
*	Function:	_wm_syscommand
*
*	Purpose:
*
*		Process a WM_SYSCOMMAND message for the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		wParam		The message's word sized parameter.
*
*	Returned:
*
*		TRUE		We processed the message.
*		FALSE		We did not process the message.
*
*****************************************************************************/

static LRESULT _wm_syscommand(HWND dlg, WPARAM wParam)
{
	LRESULT	ret;

	switch (wParam)
	{
		default:

			ret	= FALSE;
			break;

		case SC_CLOSE:

			_idcancel(dlg);
			ret	= TRUE;
			break;
	}

	return(ret);
}



/*****************************************************************************
*
*	Function:	_dlg_proc
*
*	Purpose:
*
*		Processes messages for the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		msg			The message to process.
*
*		wParam		The message's word sized parameter.
*
*		lParam		The message's long sized parameter.
*
*	Returned:
*
*		A message specific value.
*
*****************************************************************************/

static INT_PTR CALLBACK _dlg_proc(	HWND	dlg,
									UINT	msg,
									WPARAM	wParam,
									LPARAM	lParam)
{
	LRESULT	ret;

	switch (msg)
	{
		default:

			ret	= FALSE;
			break;

		case WM_COMMAND:

			ret	= _wm_command(dlg, wParam);
			break;

		case WM_DESTROY:

			ret	= _wm_destroy(dlg);
			break;

		case WM_GETMINMAXINFO:

			ret	= _wm_getminmaxinfo(dlg, lParam);
			break;

		case WM_HSCROLL:

			ret	= _wm_hscroll(dlg, wParam);
			break;

		case WM_INITDIALOG:

			ret	= _wm_initdialog(dlg, lParam);
			break;

		case WM_PAINT:

			ret	= _wm_paint(dlg, FALSE);
			break;

		case WM_SIZE:

			ret	= _wm_size(dlg);
			break;

		case WM_SYSCOMMAND:

			ret	= _wm_syscommand(dlg, wParam);
			break;
	}

	return(ret);
}



/*****************************************************************************
*
*	Function:	_service_wm_command
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_WM_COMMAND service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_wm_command(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	data_xfer_t*	data	= (data_xfer_t*) wParam;

	if (data->dlg)	/*lint !e727 */
	{
		SetFocus(data->dlg);
	}
	else
	{
		data->device	= device;
		data->dlg		= CreateDialogParam(device->instance,
											(LPCTSTR) IDD_DATA_XFER,
											dlg,
											(DLGPROC) _dlg_proc,
											(LPARAM) data);
		ShowWindow(data->dlg, SW_SHOWNORMAL);
		_refresh_window_local(data->dlg, data);
	}

	return(0);
}



/*****************************************************************************
*
*	Function:	data_xfer_service
*
*	Purpose:
*
*		Implement the common service entry point for the Data Transfer
*		services.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		service	The service to perform.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0		No service was called.
*		else	The value from the called service, which may be zero.
*
*****************************************************************************/

LRESULT data_xfer_service(
	HWND			dlg,
	device_t*		device,
	service_type_t	service,
	WPARAM			wParam,
	LPARAM			lParam)
{
	static const service_call_t	list[]	=
	{
		/* service				function			*/
		{ SERVICE_WM_COMMAND,	_service_wm_command	},
		{ SERVICE_END_LIST,		NULL				}
	};

	LRESULT	ret;

	ret	= service_call(dlg, device, list, service, wParam, lParam);
	return(ret);
}


