// HPDI32 - Application - Win32 - Any Compiler
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/SDK/hpdi32/visualhpdi32/win32/gpio.h $
// $Rev: 2902 $
// $Date: 2009-08-09 17:48:49 -0500 (Sun, 09 Aug 2009) $

/*****************************************************************************
*
*	Export items for the GPIO services.
*
*****************************************************************************/

#ifndef __GPIO_H__
#define	__GPIO_H__

#include "service.h"		/*lint !e537 */
#include "static.h"		/*lint !e537 */



/* typedefs	*****************************************************************/

typedef struct
{
	const char*	fc_name;
	int			id_arrow;
	int			id_static;
	U8			mask;
	U32			which;
} gpio_t;

typedef struct _gpio_xfer_t	gpio_xfer_t;

struct _gpio_xfer_t
{
	// These are filled in up front by the direction specific code.
	U8*			buffer;			// GPIO data is stored here.
	int			capacity;		// The size of the buffer.
	int			idc_interval;	// Process these events?
	void		(*refresh_window)(HWND dlg, gpio_xfer_t* data);
	const char*	section;		// The .ini section name.
	const char*	title;			// The dialog title portion.
	void		(*wm_initdialog)(HWND dlg, gpio_xfer_t* data);
	int			(*xfer_begin)(gpio_xfer_t* data);
	U32			(*xfer_data)(gpio_xfer_t* data);
	void		(*xfer_end)(gpio_xfer_t* data);
	int			(*xfer_file)(HWND dlg, gpio_xfer_t* data);	// Save/load a file.

	// These are filled in by the common code or after the dialog is opened.
	int			abort;		// Abort the current transfer.
	int			active;		// A transfer is in progress.
	device_t*	device;
	HWND		dlg;
	int			interval;	// ms increments.
	int			offset;		// Begin data display at this data item.
	float		scale;		// Pixels per data item.
	int			transfer;	// The number of values to transfer.
	int			values;		// Number of values in buffer.

	struct
	{
		int		values_last;
		int		x_right;
		int		x;
	} cursor;

	struct
	{
		HWND	control;
		HCURSOR	cursor_last;
		int		wait_til_up;	// Ignore activity till mouse is up.
		int		inside;			// Is the cursor inside the area?
		RECT	rect;
		int		trackable;		// Can we track left or right?
		int		width;
	} graph;

	struct
	{
		int		captured;		// Is the mouse captured?
		HWND	control;
		HCURSOR	cursor_last;
		int		index;			// Current tracking index.
		int		wait_til_up;	// Ignore activity till mouse is up.
		int		inside;			// Is the cursor inside the area?
		RECT	rect;
		int		width;
		int		x_min;			// Minimim X tracking pixel movement.
		int		x_max;			// Maximum X tracking pixel movement.
		int		y;
	} ruler;

	struct
	{
		int	y_top;
		int	y_bot;
	} signals[7];

	struct
	{
		DWORD	id;
	} thread;

	struct
	{
		int	enabled;
		int	index;
		int	selected;
	} x;

	struct
	{
		int	enabled;
		int	index;
		int	selected;
	} y;
};



/* prototypes	*************************************************************/

void	gpio_gui_paint(HWND control, static_t* data, const gpio_t* gpio);

LRESULT	gpio_service(		HWND			dlg,
							device_t*		device,
							service_type_t	service,
							WPARAM			wParam,
							LPARAM			lParam);

LRESULT	gpio_x_service(		HWND			dlg,
							device_t*		device,
							service_type_t	service,
							WPARAM			wParam,
							LPARAM			lParam);

LRESULT	gpio_xfer_service(	HWND			dlg,
							device_t*		device,
							service_type_t	service,
							WPARAM			wParam,
							LPARAM			lParam);

LRESULT	gpio_0_service(		HWND			dlg,
							device_t*		device,
							service_type_t	service,
							WPARAM			wParam,
							LPARAM			lParam);

LRESULT	gpio_1_service(		HWND			dlg,
							device_t*		device,
							service_type_t	service,
							WPARAM			wParam,
							LPARAM			lParam);

LRESULT	gpio_2_service(		HWND			dlg,
							device_t*		device,
							service_type_t	service,
							WPARAM			wParam,
							LPARAM			lParam);

LRESULT	gpio_3_service(		HWND			dlg,
							device_t*		device,
							service_type_t	service,
							WPARAM			wParam,
							LPARAM			lParam);

LRESULT	gpio_4_service(		HWND			dlg,
							device_t*		device,
							service_type_t	service,
							WPARAM			wParam,
							LPARAM			lParam);

LRESULT	gpio_5_service(		HWND			dlg,
							device_t*		device,
							service_type_t	service,
							WPARAM			wParam,
							LPARAM			lParam);

LRESULT	gpio_6_service(		HWND			dlg,
							device_t*		device,
							service_type_t	service,
							WPARAM			wParam,
							LPARAM			lParam);

LRESULT	gpio_read_service(	HWND			dlg,
							device_t*		device,
							service_type_t	service,
							WPARAM			wParam,
							LPARAM			lParam);

LRESULT	gpio_write_service(	HWND			dlg,
							device_t*		device,
							service_type_t	service,
							WPARAM			wParam,
							LPARAM			lParam);



#endif
