// HPDI32 - Application - Win32 - Any Compiler
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/SDK/hpdi32/visualhpdi32/win32/gpio_3.c $
// $Rev: 2902 $
// $Date: 2009-08-09 17:48:49 -0500 (Sun, 09 Aug 2009) $

/*****************************************************************************
*
*	Implement support for GPIO 3 services.
*
*****************************************************************************/

#include <windows.h>

#include "gpio.h"
#include "align.h"
#include "arrow.h"
#include "cable.h"
#include "resource.h"
#include "util_ctrl.h"



/* variables *****************************************************************/

static const gpio_t	_gpio	=
{
	/* fc_name		*/	"Tx Data Ready",
	/* id_arrow		*/	IDC_GPIO_3_ARROW,
	/* id_static	*/	IDC_GPIO_3,
	/* mask			*/	0x08,
	/* which		*/	HPDI32_WHICH_GPIO_3_
};



/*****************************************************************************
*
*	Function:	_gui_paint
*
*	Purpose:
*
*		Provide additional painting support for the GUI Rx Clock control.
*
*	Arguments:
*
*		control		The handle to the control.
*
*		data		The control specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _gui_paint(HWND control, static_t* data)
{
	gpio_gui_paint(control, data, &_gpio);
}



/*****************************************************************************
*
*	Function:	_service_init
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_INIT service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_init(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	static const align_t list[]	=
	{
		{
		/* id		*/	IDC_GPIO_3,
		/* ref		*/	IDC_GPIO_2,
		/* flags	*/	ALIGN_COPY,
		/* value	*/	0
		},
		{
		/* id		*/	IDC_GPIO_3,
		/* ref		*/	IDC_GPIO_2,
		/* flags	*/	ALIGN_T_B | ALIGN_H,
		/* value	*/	MARGIN
		},
		{
		/* id		*/	IDC_GPIO_3_ARROW,
		/* ref		*/	IDC_GPIO_2_ARROW,
		/* flags	*/	ALIGN_COPY,
		/* value	*/	0
		},
		{
		/* id		*/	IDC_GPIO_3_ARROW,
		/* ref		*/	IDC_GPIO_2_ARROW,
		/* flags	*/	ALIGN_T_B | ALIGN_H,
		/* value	*/	MARGIN
		},
		{ 0, 0, 0, 0 }
	};

	align_control_list(dlg, list);
	static_init(dlg, device, IDC_GPIO_3);
	static_paint_function_set(dlg, IDC_GPIO_3, _gui_paint);
	arrow_init(dlg, device, IDC_GPIO_3_ARROW);
	return(0);
}



/*****************************************************************************
*
*	Function:	_service_refresh
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_REFRESH service for
*		the GUI window.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_refresh(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	LRESULT	ret;

	ret	= gpio_x_service(dlg, device, SERVICE_REFRESH, (ULONG) &_gpio, lParam);
	return(ret);
}



/*****************************************************************************
*
*	Function:	gpio_3_service
*
*	Purpose:
*
*		Implement the common service entry point for the GPIO 3 services.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		service	The service to perform.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0		No service was called.
*		else	The value from the called service, which may be zero.
*
*****************************************************************************/

LRESULT gpio_3_service(
	HWND			dlg,
	device_t*		device,
	service_type_t	service,
	WPARAM			wParam,
	LPARAM			lParam)
{
	static const service_call_t	list[]	=
	{
		/* service			function			*/
		{ SERVICE_INIT,		_service_init		},
		{ SERVICE_REFRESH,	_service_refresh	},
		{ SERVICE_END_LIST,	NULL				}
	};

	LRESULT	ret;

	ret	= service_call(dlg, device, list, service, wParam, lParam);
	return(ret);
}


