// HPDI32 - Application - Win32 - Any Compiler
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/SDK/hpdi32/visualhpdi32/win32/gpio_write.c $
// $Rev: 31452 $
// $Date: 2014-11-13 09:58:59 -0600 (Thu, 13 Nov 2014) $

/*****************************************************************************
*
*	Implement support for GPIO Write services.
*
*****************************************************************************/

#include <windows.h>
#include <stdio.h>

#include "gui.h"
#include "gpio.h"
#include "align.h"
#include "button.h"
#include "resource.h"
#include "util_ctrl.h"



/* #defines	*****************************************************************/

#define	SECTION_NAME		"gpio write"



/*****************************************************************************
*
*	Function:	_refresh_window
*
*	Purpose:
*
*		Update the controls based on the current state.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		device		The device to access.
*
*		msg			A status message to apply.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _refresh_window(HWND dlg, gpio_xfer_t* data)
{
	BOOL	access;
	HWND	control;
	BOOL	inactive	= data->active == FALSE;
	BOOL	idle_data	= (data->values) && inactive;

	if (data->device->handle)
		access	= TRUE;
	else
		access	= FALSE;

	control	= GetDlgItem(dlg, IDC_GPIO_XFER_FILE);
	EnableWindow(control, inactive);
	control	= GetDlgItem(dlg, IDC_GPIO_XFER_START);
	EnableWindow(control, (BOOL) ((access) && (idle_data)));
	control	= GetDlgItem(dlg, IDC_GPIO_XFER_INTERVAL_TEXT);
	EnableWindow(control, (BOOL) ((access) && (idle_data)));
	control	= GetDlgItem(dlg, IDC_GPIO_XFER_INTERVAL_LIST);
	EnableWindow(control, (BOOL) ((access) && (idle_data)));
	control	= GetDlgItem(dlg, IDC_GPIO_XFER_SCALE_TEXT);
	EnableWindow(control, idle_data);
	control	= GetDlgItem(dlg, IDC_GPIO_XFER_SCALE_SCROLL);
	EnableWindow(control, idle_data);
}



/*****************************************************************************
*
*	Function:	_wm_initdialog
*
*	Purpose:
*
*		Process a WM_INITDIALOG message for the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		data		The dialog specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _wm_initdialog(HWND dlg, gpio_xfer_t* data)
{
	HWND	file;
	HWND	stop;
	HWND	strt;

	// Start -> File Load
	// Stop  -> Start Write
	// File  -> Stop

	strt	= GetDlgItem(dlg, IDC_GPIO_XFER_START);
	stop	= GetDlgItem(dlg, IDC_GPIO_XFER_STOP);
	file	= GetDlgItem(dlg, IDC_GPIO_XFER_FILE);

	SetWindowLong(strt,  GWL_ID, IDC_GPIO_XFER_FILE);
	SetWindowLong(stop,  GWL_ID, IDC_GPIO_XFER_START);
	SetWindowLong(file,  GWL_ID, IDC_GPIO_XFER_STOP);

	SetWindowText(strt, "Load");
	SetWindowText(stop, "Write");
	SetWindowText(file, "Stop");
}



/*****************************************************************************
*
*	Function:	_xfer_begin
*
*	Purpose:
*
*		Prepare for data transfer. This is called before the transfer thread
*		is started.
*
*	Arguments:
*
*		data	The dialog specific data we track.
*
*	Returned:
*
*		TRUE	Continue with data transfer.
*		FALSE	Don't continue with data transfer.
*
*****************************************************************************/

static int _xfer_begin(gpio_xfer_t* data)
{
	static const U32	list[]	=
	{
		HPDI32_WHICH_GPIO_0_,
		HPDI32_WHICH_GPIO_1_,
		HPDI32_WHICH_GPIO_2_,
		HPDI32_WHICH_GPIO_3_,
		HPDI32_WHICH_GPIO_4_,
		HPDI32_WHICH_GPIO_5_,
		HPDI32_WHICH_GPIO_6_
	};

	int	gpio	= FALSE;
	int	i;
	U32	mode;
	int	ret;
	U32	stat;
	U32	status	= GSC_SUCCESS;

	for (i = 0; i < (int) ARRAY_ELEMENTS(list); i++)
	{
		stat	= HPDI32_CABLE_COMMAND_MODE__GET(	data->device->handle,
													list[i],
													&mode);
		status	= (status == GSC_SUCCESS) ? stat : status;

		if ((mode == HPDI32_CABLE_COMMAND_MODE_GPIO_OUT_LOW) ||
			(mode == HPDI32_CABLE_COMMAND_MODE_GPIO_OUT_HI))
		{
			gpio	= TRUE;
			break;
		}
	}

	data->transfer	= data->values;

	if (status != GSC_SUCCESS)
	{
		ret	= FALSE;
		error_status(data->dlg, IDC_GPIO_XFER_STATUS, status);
	}
	else if (gpio)
	{
		ret				= TRUE;
		data->values	= 0;
		ctrl_menu_set(data->dlg, IDC_GPIO_XFER_STOP, CTRL_ENABLE | CTRL_FOCUS);
	}
	else
	{
		ret	= FALSE;
		ctrl_menu_set_text(	data->dlg,
							IDC_GPIO_XFER_STATUS,
							"No signals are GPIO output.");
	}

	return(ret);
}



/*****************************************************************************
*
*	Function:	_xfer_data
*
*	Purpose:
*
*		Perform a single data value transfer request.
*
*	Arguments:
*
*		data	The dialog specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well. Continue with data transfer.
*		else		An appropriate error status. Terminate transfer.
*
*****************************************************************************/

static U32 _xfer_data(gpio_xfer_t* data)
{
	U32	status;

	status	= hpdi32_gpio_write(data->device->handle, data->buffer[data->values]);
	return(status);
}



/*****************************************************************************
*
*	Function:	_xfer_end
*
*	Purpose:
*
*		Cleanup after data transfer. This is called from the transfer thread
*		if the thread is started. Otherwise it is called from the main thread.
*		This is called only if xfer_begin() indicates that the transfer should
*		procede.
*
*	Arguments:
*
*		data	The dialog specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _xfer_end(gpio_xfer_t* data)
{
	data->values	= data->transfer;
	ctrl_menu_set(data->dlg, IDC_GPIO_XFER_START, CTRL_ENABLE | CTRL_FOCUS);
}



/*****************************************************************************
*
*	Function:	_xfer_file
*
*	Purpose:
*
*		Prompt for a file name and save the data.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		data		The window specific data we track.
*
*	Returned:
*
*		TRUE		Apply the INTERVAL value that is currently selected?
*		FALSE		Use the existing interval value.
*
*****************************************************************************/

static int _xfer_file(HWND dlg, gpio_xfer_t* data)
{
	TCHAR			buf[128 + _MAX_PATH];
	char			dir[_MAX_PATH];
	FILE*			file;
	TCHAR			name[_MAX_PATH];
	int				i;
	OPENFILENAME	ofn;

	strcpy(name, "gpio.bin");
	GetCurrentDirectory(sizeof(dir), dir);

	// Initialize OPENFILENAME
	memset(&ofn, 0, sizeof(OPENFILENAME));
	ofn.lStructSize		= sizeof(OPENFILENAME);
	ofn.hwndOwner		= dlg;
	ofn.lpstrFilter		= "Binary Files (*.bin)\0*.bin\0"
							"All (*.*)\0*.*\0";
	ofn.nFilterIndex	= 1;
	ofn.lpstrFile		= name;
	ofn.nMaxFile		= sizeof(name);
	ofn.lpstrInitialDir	= dir;
	ofn.lpstrTitle		= "Load GPIO Data";
	ofn.Flags			= OFN_PATHMUSTEXIST
						| OFN_FILEMUSTEXIST
						| OFN_HIDEREADONLY;
	i					= GetOpenFileName(&ofn);

	for (;i;)
	{
		file	= fopen(name, "rb");

		if (file == NULL)
		{
			sprintf(buf, "Unable to open '%s'.", name);
			ctrl_menu_set_text(dlg, IDC_GPIO_XFER_STATUS, buf);
			break;
		}

		i	= (int) fread(data->buffer, 1, (size_t) data->capacity, file);

		if (i <= 0)
		{
			data->values	= 0;
			sprintf(buf, "Read failure from '%s'.", name);
		}
		else
		{
			data->values	= i;
			sprintf(buf, "Read %d bytes from '%s'.", data->values, name);
		}

		fclose(file);
		ctrl_menu_set_text(dlg, IDC_GPIO_XFER_STATUS, buf);
		break;
	}

	return(TRUE);
}



/*****************************************************************************
*
*	Function:	_service_init
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_INIT service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_init(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	static const align_t list[]	=
	{
		{
		/* id		*/	IDC_GPIO_WRITE,
		/* ref		*/	IDC_RX_ENABLE,
		/* flags	*/	ALIGN_COPY,
		/* value	*/	0
		},
		{
		/* id		*/	IDC_GPIO_WRITE,
		/* ref		*/	IDC_GPIO_1,
		/* flags	*/	ALIGN_T_T | ALIGN_H,
		/* value	*/	0
		},
		{ 0, 0, 0, 0 }
	};

	align_control_list(dlg, list);
	button_init(dlg, device, IDC_GPIO_WRITE);
	return(0);
}



/*****************************************************************************
*
*	Function:	_service_refresh
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_REFRESH service for
*		the GUI window.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_refresh(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	if (device->handle)
		ctrl_menu_set(dlg, IDC_GPIO_WRITE, CTRL_ENABLE);
	else
		ctrl_menu_set(dlg, IDC_GPIO_WRITE, CTRL_DISABLE);

	return(0);
}



/*****************************************************************************
*
*	Function:	_service_wm_command
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_WM_COMMAND service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_wm_command(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	static	U8			buffer[3000];
	static	gpio_xfer_t	data;
	LRESULT				ret;

	data.buffer			= buffer;
	data.capacity		= (int) sizeof(buffer);
	data.idc_interval	= TRUE;
	data.refresh_window	= _refresh_window;
	data.section		= SECTION_NAME;
	data.title			= "GPIO Write";
	data.wm_initdialog	= _wm_initdialog;
	data.xfer_begin		= _xfer_begin;
	data.xfer_data		= _xfer_data;
	data.xfer_end		= _xfer_end;
	data.xfer_file		= _xfer_file;

	ret	= gpio_xfer_service(dlg, device, SERVICE_WM_COMMAND, (ULONG) &data, lParam);
	return(ret);
}



/*****************************************************************************
*
*	Function:	gpio_write_service
*
*	Purpose:
*
*		Implement the common service entry point for the GPIO Write services.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		service	The service to perform.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0		No service was called.
*		else	The value from the called service, which may be zero.
*
*****************************************************************************/

LRESULT gpio_write_service(
	HWND			dlg,
	device_t*		device,
	service_type_t	service,
	WPARAM			wParam,
	LPARAM			lParam)
{
	static const service_call_t	list[]	=
	{
		/* service				function			*/
		{ SERVICE_INIT,			_service_init		},
		{ SERVICE_REFRESH,		_service_refresh	},
		{ SERVICE_WM_COMMAND,	_service_wm_command	},
		{ SERVICE_END_LIST,		NULL				}
	};

	LRESULT	ret;

	ret	= service_call(dlg, device, list, service, wParam, lParam);
	return(ret);
}


