// HPDI32 - Application - Win32 - Any Compiler
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/SDK/hpdi32/visualhpdi32/win32/main.c $
// $Rev: 2902 $
// $Date: 2009-08-09 17:48:49 -0500 (Sun, 09 Aug 2009) $

/*****************************************************************************
*
*	Implement the Visual HPDI32 application.
*
*****************************************************************************/

#include <windows.h>
#include <stdio.h>
#include <string.h>

#include "gui.h"
#include "ini.h"
#include "preference.h"
#include "regs.h"
#include "resource.h"
#include "tx.h"



/* #defines	*****************************************************************/

#define	RGB_BLACK			RGB(0,0,0)
#define	RGB_BLUE			RGB(0,0,255)
#define	RGB_GREEN			RGB(0,255,0)
#define	RGB_GRAY_DK			RGB(128,128,128)
#define	RGB_GRAY_LT			RGB(212,208,200)
#define	RGB_WHITE			RGB(255,255,255)



/* typedefs	*****************************************************************/

typedef struct
{
	HWND	dlg;
	void	(*func)(HWND dlg, device_t* device, const char* msg);
} refresh_t;



/* variables	*************************************************************/

static	BOOL		_location_relative;
static	refresh_t	_refresh_list[40];

#define	_MACRO(m)	m, #m
static const struct
{
	U32			status;
	const char*	text;
} _err_list[]	=
{
    { _MACRO(GSC_FAILED)						},
    { _MACRO(GSC_ACCESS_DENIED)					},
    { _MACRO(GSC_DMA_CHANNEL_UNAVAILABLE)		},
    { _MACRO(GSC_DMA_CHANNEL_INVALID)			},
    { _MACRO(GSC_DMA_CHANNEL_TYPE_ERROR)		},
    { _MACRO(GSC_DMA_IN_PROGRESS)				},
    { _MACRO(GSC_DMA_DONE)						},
    { _MACRO(GSC_DMA_PAUSED)					},
    { _MACRO(GSC_DMA_NOT_PAUSED)				},
    { _MACRO(GSC_DMA_CHANNEL_COMMAND_INVALID)	},
    { _MACRO(GSC_DMA_MAN_READY)					},
    { _MACRO(GSC_DMA_MAN_NOT_READY)				},
    { _MACRO(GSC_DMA_INVALID_CHANNEL_PRIORITY)	},
    { _MACRO(GSC_DMA_MAN_CORRUPTED)				},
    { _MACRO(GSC_DMA_INVALID_ELEMENT_INDEX)		},
    { _MACRO(GSC_DMA_NO_MORE_ELEMENTS)			},
    { _MACRO(GSC_DMA_SGL_INVALID)				},
    { _MACRO(GSC_DMA_SGL_QUEUE_FULL)			},
    { _MACRO(GSC_NULL_PARAM)					},
    { _MACRO(GSC_INVALID_BUS_INDEX)				},
    { _MACRO(GSC_UNSUPPORTED_FUNCTION)			},
    { _MACRO(GSC_INVALID_PCI_SPACE)				},
    { _MACRO(GSC_INVALID_IOP_SPACE)				},
    { _MACRO(GSC_INVALID_SIZE)					},
    { _MACRO(GSC_INVALID_ADDRESS)				},
    { _MACRO(GSC_INVALID_ADDRESS_TYPE)			},
    { _MACRO(GSC_INVALID_INDEX)					},
    { _MACRO(GSC_MU_NOT_READY)					},
    { _MACRO(GSC_MU_FIFO_EMPTY)					},
    { _MACRO(GSC_MU_FIFO_FULL)					},
    { _MACRO(GSC_INVALID_REGISTER)				},
    { _MACRO(GSC_DOORBELL_CLEAR_FAILED)			},
    { _MACRO(GSC_INVALID_USER_PIN)				},
    { _MACRO(GSC_INVALID_USER_STATE)			},
    { _MACRO(GSC_EEPROM_NOT_PRESENT)			},
    { _MACRO(GSC_EEPROMP_TYPE_NOT_SUPPORTED)	},
    { _MACRO(GSC_EEPROM_BLANK)					},
    { _MACRO(GSC_CONFIG_ACCESS_FAILED)			},
    { _MACRO(GSC_INVALID_DEVICE_INFO)			},
    { _MACRO(GSC_NO_ACTIVE_DRIVER)				},
    { _MACRO(GSC_INSUFFICIENT_RESOURCES)		},
    { _MACRO(GSC_OBJECT_ALREADY_ALLOCATED)		},
    { _MACRO(GSC_ALREADY_INITIALIZED)			},
    { _MACRO(GSC_NOT_INITIALIZED)				},
    { _MACRO(GSC_BAD_CONFIG_REG_ENDIAN_MODE)	},
    { _MACRO(GSC_INVALID_POWER_STATE)			},
    { _MACRO(GSC_POWER_DOWN)					},
    { _MACRO(GSC_FLYBY_NOT_SUPPORTED)			},
    { _MACRO(GSC_CHANNEL_NOT_SUPPORTED)			},
    { _MACRO(GSC_NO_ACTION)						},
    { _MACRO(GSC_HS_NOT_SUPPORTED)				},
    { _MACRO(GSC_VP_NOT_SUPPORTED)				},
    { _MACRO(GSC_VPD_NOT_ENABLED)				},
    { _MACRO(GSC_NO_MORE_CAP)					},
    { _MACRO(GSC_INVALID_OFFSET)				},
    { _MACRO(GSC_BAD_PIN_DIRECTION)				},
    { _MACRO(GSC_PCI_TIMEOUT)					},
    { _MACRO(GSC_DMA_CHANNEL_CLOSED)			},
    { _MACRO(GSC_DMA_CHANNEL_ERROR)				},
    { _MACRO(GSC_INVALID_HANDLE)				},
    { _MACRO(GSC_BUFFER_NOT_READY)				},
    { _MACRO(GSC_INVALID_DATA)					},
    { _MACRO(GSC_DO_NOTHING)					},
    { _MACRO(GSC_DMA_SGL_BUILD_FAILED)			},
    { _MACRO(GSC_PM_NOT_SUPPORTED)				},
    { _MACRO(GSC_INVALID_VERSION_DRIVER)		},
    { _MACRO(GSC_WAIT_TIMEOUT)					},
    { _MACRO(GSC_WAIT_CANCELED)					},

	{ _MACRO(GSC_LIB_LOAD_FAILURE)				},
    { _MACRO(GSC_INIT_FAILURE)					},
    { _MACRO(GSC_INVALID_VERSION_API)			},
	{ _MACRO(GSC_INTERNAL_ERROR)				},
	{ _MACRO(GSC_TOO_MANY_OPEN_HANDLES)			},
	{ _MACRO(GSC_INVALID_API_HANDLE)			},
	{ _MACRO(GSC_THREAD_FAILURE)				},
	{ _MACRO(GSC_ABORTED)						}
};



/*****************************************************************************
*
*	Function:	win_pos_restore
*
*	Purpose:
*
*		Retrieve the window's position and restore the window to the position
*		recorded.
*
*	Arguments:
*
*		win			The window of interest.
*
*		name		The name to use.
*
*		sizeable	Can the window's size be changed?
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void win_pos_restore(HWND win, const char* name, int sizeable)
{
	int		bottom;
	char	buf[128];
	int		height;
	int		i;
	int		left;
	int		margin;
	HWND	parent;
	RECT	rect;
	int		relative	= _location_relative;
	int		right;
	int		sh			= GetSystemMetrics(SM_CYSCREEN);
	int		sw			= GetSystemMetrics(SM_CXSCREEN);
	int		title;
	int		top;
	int		x;
	int		y;
	int		width;

	ini_str_get(NULL, name, "location", buf, sizeof(buf));
	i	= sscanf(buf, "%d,%d,%d,%d,%d", &relative, &left, &top, &right, &bottom);

	if (i < 5)
	{
		GetWindowRect(win, &rect);
		width		= rect.right - rect.left;
		height		= rect.bottom - rect.top;
		x			= (sw - width) / 2;
		y			= (sh - height) / 2;
	}
	else
	{
		if (sizeable)
		{
			rect.left	= left;
			rect.top	= top;
			rect.right	= right;
			rect.bottom	= bottom;
		}
		else
		{
			GetWindowRect(win, &rect);
			width		= rect.right - rect.left;
			height		= rect.bottom - rect.top;
			rect.left	= left;
			rect.top	= top;
			rect.right	= left + width;
			rect.bottom	= top + height;
		}

		parent	= GetParent(win);

		if ((parent) && (relative))
		{
			ClientToScreen(parent, (LPPOINT) &rect.left);
			ClientToScreen(parent, (LPPOINT) &rect.right);
		}

		x		= rect.left;
		y		= rect.top;
		width	= rect.right - rect.left;
		height	= rect.bottom - rect.top;
	}

	// Make sure the title bar is visible.
	GetWindowRect(win, &rect);
	ScreenToClient(win, (LPPOINT) &rect.left);
	margin	= (rect.left < 0) ? -rect.left : rect.left;
	title	= (rect.top < 0) ? -rect.top : rect.top;
	title	= (title - margin) / 2;

	if (y < -margin)
		y	= -margin;
	else if (y > (sh - (margin + title)))
		y	= sh - (margin + title);

	if (x > (sw - (margin + (2 * title))))
		x	= sw - (margin + (2 * title));
	else if (x < -(width - ((3 * title) + margin)))
		x	= -(width - ((3 * title) + margin));

	MoveWindow(win, x, y, width, height, FALSE);
}



/*****************************************************************************
*
*	Function:	win_pos_save
*
*	Purpose:
*
*		Save the window's position so it can be restored later. The position
*		saved is relative to its parent.
*
*	Arguments:
*
*		win		The window of interest.
*
*		name	The name to use.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void win_pos_save(HWND win, const char* name)
{
	char	buf[128];
	HWND	parent;
	RECT	rect;
	int		relative;
	BOOL	test;

	for (;;)	// A convenience loop.
	{
		test	= IsIconic(win);

		if (test)
			break;

		test	= IsZoomed(win);

		if (test)
			break;

		test	= GetWindowRect(win, &rect);

		if (test == FALSE)
			break;

		parent	= GetParent(win);

		if (parent)
		{
			relative	= _location_relative;

			if (relative)
			{
				ScreenToClient(parent, (LPPOINT) &rect.left);
				ScreenToClient(parent, (LPPOINT) &rect.right);
			}
		}
		else
		{
			relative	= FALSE;
		}

		sprintf(buf,
				"%d,%d,%d,%d,%d",
				(int) relative,
				(int) rect.left, (int) rect.top, (int) rect.right, (int) rect.bottom);
		ini_str_set(NULL, name, "location", buf);
		break;
	}
}



/*****************************************************************************
*
*	Function:	error_status
*
*	Purpose:
*
*		Report that the API returned an error status.
*
*	Arguments:
*
*		dlg		The current window.
*
*		id		The control to which the error should go.
*
*		status	The status returned.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void error_status(HWND dlg, int id, U32 status)
{
	char	buf[256];
	HWND	control;
	int		i;

	if (status != GSC_SUCCESS)
	{
		control	= GetDlgItem(dlg, id);

		if (control == NULL)
			control	= GetDlgItem(dlg, IDC_INFO_TEXT);

		sprintf(buf, "Error: 0x%lX", (long) status);

		for (i = ARRAY_ELEMENTS(_err_list); i--;)
		{
			if (status == _err_list[i].status)
			{
				strcat(buf, " (");
				strcat(buf, _err_list[i].text);
				strcat(buf, ")");
				break;
			}
		}

		if (control)
			SetWindowText(control, buf);
	}
}



/*****************************************************************************
*
*	Function:	main_close_all
*
*	Purpose:
*
*		Close all windows except the one given as a parameter.
*
*	Arguments:
*
*		dlg		The handle of the window to ignore.
*
*		device	The device data to access.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void main_close_all(HWND dlg, device_t* device)
{
	HCURSOR	cursor;
	int		i;

	cursor	= SetCursor(device->cursor.wait);

	for (i = 0; i < (int) ARRAY_ELEMENTS(_refresh_list); i++)
	{
		if (dlg == _refresh_list[i].dlg)
			continue;

		if ((_refresh_list[i].dlg) && (_refresh_list[i].func))
		{
			SetFocus(_refresh_list[i].dlg);
			SendMessage(_refresh_list[i].dlg, WM_COMMAND, IDCANCEL, 0);
		}
	}

	SetCursor(cursor);
	SetFocus(dlg);
}



/*****************************************************************************
*
*	Function:	main_refresh
*
*	Purpose:
*
*		Update the state of the main window(s) to reflect the state of the
*		device.
*
*	Arguments:
*
*		device	The device structure to access.
*
*		msg		A status message for the window.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void main_refresh(device_t* device, const char* msg)
{
	char	buf[128];
	HWND	focus	= GetFocus();
	int		i;

	if (device->usable == FALSE)
	{
		sprintf(buf,
				"HPDI32 API unusable, status 0x%lX",
				(long) device->api_status);
		msg	= buf;
	}

	for (i = 0; i < (int) ARRAY_ELEMENTS(_refresh_list); i++)
	{
		if ((_refresh_list[i].dlg) && (_refresh_list[i].func))
			(_refresh_list[i].func)(_refresh_list[i].dlg, device, msg);
	}

	SetFocus(focus);
}



/*****************************************************************************
*
*	Function:	main_refresh_add
*
*	Purpose:
*
*		Add a callback to the refresh call list.
*
*	Arguments:
*
*		dlg		The handle of the dialog to be refreshed.
*
*		func	The function to call. If this is NULL, then the dialog is
*				removed from the list.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void main_refresh_add(
	HWND	dlg,
	void	(*func)(HWND dlg, device_t* device, const char* msg))
{
	refresh_t*	entry	= NULL;
	int			i;

	// See if the entry is already in the list.

	for (i = 0; i < (int) ARRAY_ELEMENTS(_refresh_list); i++)
	{

		if ((dlg) && (dlg == _refresh_list[i].dlg))
		{
			entry	= &_refresh_list[i];
			break;
		}
	}

	if (entry)
	{
		if (func)
		{
			// Update the function pointer.
			entry->func	= func;
		}
		else
		{
			// Remote the entry from the callback list.
			entry->dlg	= NULL;
			entry->func	= NULL;
		}
	}
	else if ((dlg) && (func))
	{
		// All the entry to list.

		for (i = 0; i < (int) ARRAY_ELEMENTS(_refresh_list); i++)
		{

			if (_refresh_list[i].dlg == NULL)
			{
				entry		= &_refresh_list[i];
				entry->dlg	= dlg;
				entry->func	= func;
				break;
			}
		}
	}
}



/*****************************************************************************
*
*	Function:	set_win_title
*
*	Purpose:
*
*		Construct a windows title string for the caller and apply it to the
*		given window.
*
*	Arguments:
*
*		win		The window to access.
*
*		device	The device structure to use.
*
*		version	Incorporate the version number in the title
*
*		index	Incorporate the current device index in the title?
*
*		suffix	Any text to appear after the title.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void set_win_title(	HWND			win,
					const device_t*	device,
					int				version,
					int				index,
					const char*		suffix)
{
	char	buf[2048];	/* Plenty large enough. */
	char	tmp[32];

	strcpy(buf, APP_TITLE);

	if (version)
	{
		strcat(buf, " (");
		strcat(buf, VERSION_STRING);
		strcat(buf, ")");
	}

	if ((index) && (device->handle))
	{
		sprintf(tmp, ": #%d", (int) device->index);
		strcat(buf, tmp);
	}

	if ((suffix) && (suffix[0]))
	{
		strcat(buf, ": ");
		strcat(buf, suffix);
	}

	SetWindowText(win, buf);
}



/*****************************************************************************
*
*	Function:	format_float
*
*	Purpose:
*
*		Format a value into a string with a factor suffix.
*
*	Arguments:
*
*		value	The value to format.
*
*		dest	The destination buffer.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void format_float(float value, char* dest)
{
	static const struct
	{
		double		limit;
		const char*	format;
		long		div;
	} rates[]	=
	{
		/* limit		format		div			*/
		{        1000.,	"%.0f ",	1			},
		{       99900.,	"%.1f K",	1000		},
		{     1000000.,	"%.0f K",	1000		},
		{    99900000.,	"%.1f M",	1000000		},
		{  1000000000.,	"%.0f M",	1000000		},
		{ 99900000000.,	"%.1f G",	1000000000	},
		{          -1,	"%.0f G",	1000000000	}
	};

	int	i;

	for (i = 0; i < (int) ARRAY_ELEMENTS(rates); i++)
	{
		if ((value < rates[i].limit) || (rates[i].limit == -1))
		{
			sprintf(dest, rates[i].format, (double) value / rates[i].div);
			break;
		}
	}
}



/*****************************************************************************
*
*	Function:	format_u32
*
*	Purpose:
*
*		Format a value into a string with a factor suffix.
*
*	Arguments:
*
*		value	The value to format.
*
*		k		The size of a 1K unit.
*
*		dest	The destination buffer.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void format_u32(U32 value, int k, char* dest)
{
	const char*	factor	= "";

	if ((value) && ((value % (U32) k) == 0))
	{
		value	/= (U32) k;
		factor	= "K";
	}

	if ((value) && ((value % (U32) k) == 0))
	{
		value	/= (U32) k;
		factor	= "M";
	}

	if ((value) && ((value % (U32) k) == 0))
	{
		value	/= (U32) k;
		factor	= "G";
	}

	sprintf(dest, "%lu%s", (unsigned long) value, factor);
}



/*****************************************************************************
*
*	Function:	_paint_object_init
*
*	Purpose:
*
*		Initialize the structure per the given arguments.
*
*	Arguments:
*
*		obj			The structure to initialize.
*
*		sys_color	Is it a system color?
*
*		color		The system or RGB color to use.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _paint_object_init(paint_object_t* obj, int sys_color, ULONG color)
{
	paint_object_t	tmp	= obj[0];

	if (sys_color)
		obj->color	= GetSysColor((int) color);
	else
		obj->color	= color;

	obj->brush	= CreateSolidBrush(obj->color);
	obj->pen	= CreatePen(PS_SOLID, 0, obj->color);

	if (tmp.brush)
		DeleteObject(tmp.brush);

	if (tmp.pen)
		DeleteObject(tmp.pen);
}



/*****************************************************************************
*
*	Function:	_paint_object_reset
*
*	Purpose:
*
*		Release the content of the given structure.
*
*	Arguments:
*
*		obj			The structure to reset.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _paint_object_reset(paint_object_t* obj)
{
	DeleteObject(obj->brush);
	DeleteObject(obj->pen);
	obj->color	= 0;
	obj->brush	= NULL;
	obj->pen	= NULL;
}



/*****************************************************************************
*
*	Function:	_paint_objects_init
*
*	Purpose:
*
*		Initialize the given paint object structures.
*
*	Arguments:
*
*		device		The structure contains the paint objects to reset.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _paint_objects_init(device_t* device)
{
	DWORD		dw;
	LOGBRUSH	lb;

	_paint_object_init(&device->paint.background, TRUE, COLOR_BACKGROUND);
	_paint_object_init(&device->paint.black, FALSE, RGB_BLACK);
	_paint_object_init(&device->paint.btnface, TRUE, COLOR_BTNFACE);
	_paint_object_init(&device->paint.btnhighlt, TRUE, COLOR_BTNHIGHLIGHT);
	_paint_object_init(&device->paint.btntext, TRUE, COLOR_BTNTEXT);
	_paint_object_init(&device->paint.btnshadow, TRUE, COLOR_BTNSHADOW);
	_paint_object_init(&device->paint.gray_dk, FALSE, RGB_GRAY_DK);
	_paint_object_init(&device->paint.gray_lt, FALSE, RGB_GRAY_LT);
	_paint_object_init(&device->paint.green, FALSE, RGB_GREEN);
	_paint_object_init(&device->paint.red, FALSE, RGB_RED);
	_paint_object_init(&device->paint.white, FALSE, RGB_WHITE);
	_paint_object_init(&device->paint.window, TRUE, COLOR_WINDOW);
	_paint_object_init(&device->paint.window_text, TRUE, COLOR_WINDOWTEXT);
	_paint_object_init(&device->paint.x_cursor, FALSE, RGB_RED);
	_paint_object_init(&device->paint.y_cursor, FALSE, RGB_BLUE);

	if (device->paint.btntext_dot.pen == NULL)
	{
		_paint_object_init(&device->paint.btntext_dot, TRUE, COLOR_BTNTEXT);

		DeleteObject(device->paint.btntext_dot.pen);

		dw	= GetVersion();

		if (dw & 0x8000)
		{
			// This is Windows 95/98/Me.
			device->paint.btntext_dot.pen	= CreatePen(
												PS_DOT,
												0,
												device->paint.btntext_dot.color);
		}
		else
		{
			// This is Windows NT/2000/XP.
			lb.lbColor	= device->paint.btntext_dot.color;
			lb.lbStyle	= BS_SOLID;
			device->paint.btntext_dot.pen	= ExtCreatePen(
												PS_COSMETIC | PS_ALTERNATE,
												1,
												&lb,
												0,
												NULL);
		}
	}
}



/*****************************************************************************
*
*	Function:	_paint_objects_reset
*
*	Purpose:
*
*		Release the given paint object structures.
*
*	Arguments:
*
*		device		The structure contains the paint objects to reset.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _paint_objects_reset(device_t* device)
{
	_paint_object_reset(&device->paint.background);
	_paint_object_reset(&device->paint.black);
	_paint_object_reset(&device->paint.btnface);
	_paint_object_reset(&device->paint.btnhighlt);
	_paint_object_reset(&device->paint.btntext);
	_paint_object_reset(&device->paint.btntext_dot);
	_paint_object_reset(&device->paint.btnshadow);
	_paint_object_reset(&device->paint.gray_dk);
	_paint_object_reset(&device->paint.gray_lt);
	_paint_object_reset(&device->paint.green);
	_paint_object_reset(&device->paint.red);
	_paint_object_reset(&device->paint.white);
	_paint_object_reset(&device->paint.window);
	_paint_object_reset(&device->paint.window_text);
	_paint_object_reset(&device->paint.x_cursor);
	_paint_object_reset(&device->paint.y_cursor);
}



/*****************************************************************************
*
*	Function:	WinMain
*
*	Purpose:
*
*		Implement the application entry point.
*
*	Arguments:
*
*		hInstance		The application instance handle.
*
*		hPrevInstance	The instance handle for the previous actively
*						running instance of the application.
*
*		lpCmdLine		The command line arguments.
*
*		nCmdShow		How we are to display the window.
*
*	Returned:
*
*		0				We always return 0.
*
*****************************************************************************/

int APIENTRY WinMain(	HINSTANCE	hInstance,
						HINSTANCE	hPrevInstance,
						LPSTR		lpCmdLine,
						int			nCmdShow)
{
	device_t	device;
	int			i;
	MSG			msg;
	LRESULT		ret;
	int			used;

	memset(&device, 0, sizeof(device));
	ini_init();
	_location_relative		= PREF_LOCATION_RELATIVE__GET();
	device.cursor.arrows	= LoadCursor(NULL, IDC_SIZEWE);
	device.cursor.cross		= LoadCursor(hInstance, MAKEINTRESOURCE(IDC_CURSOR_CROSS));
	device.cursor.cross_x	= LoadCursor(hInstance, MAKEINTRESOURCE(IDC_CURSOR_CROSS_X));
	device.cursor.cross_y	= LoadCursor(hInstance, MAKEINTRESOURCE(IDC_CURSOR_CROSS_Y));
	device.cursor.wait		= LoadCursor(NULL, IDC_WAIT);
	device.icon.big			= LoadIcon(hInstance, MAKEINTRESOURCE(IDI_VISUAL2));
	device.icon.little		= LoadIcon(hInstance, MAKEINTRESOURCE(IDI_VISUAL1));
	device.instance			= hInstance;
	_paint_objects_init(&device);

	ret	= gui_service(NULL, &device, SERVICE_WM_COMMAND, 0, 0);

	while ((ret == 0) && (GetMessage(&msg, NULL, 0, 0)))
	{
		used	= TRUE;

		for (i = 0; i < (int) ARRAY_ELEMENTS(_refresh_list); i++)
		{
			if (_refresh_list[i].dlg)
			{
				used	= IsDialogMessage(_refresh_list[i].dlg, &msg);

				if (used)
					break;
			}
		}

		if (!used)
		{
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
	}

	_paint_objects_reset(&device);
	hpdi32_close(device.handle);

	return((int) msg.wParam);
}



/*****************************************************************************
*
*	Function:	_decimal_text_2_u32
*
*	Purpose:
*
*		Convert a decimal number in text form to a U32. The number  may
*		include a K, M or G suffix.
*
*	Arguments:
*
*		src			The source string to parse.
*
*		k			The size of a single 1K unit. If 0, it is ignored.
*
*		u32			The value is recorded here.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

static U32 _decimal_text_2_u32(const char* src, int k, U32* u32)
{
	char			c;
	char			factor	= 0;
	int				i;
	unsigned long	limit;
	U32				status	= GSC_SUCCESS;
	unsigned long	ul;

	i	= sscanf(src, "%lu%c%c", &ul, &factor, &c);

	for (;;)	// A convenience loop.
	{
		if (i == 1)
		{
			u32[0]	= (U32) ul;
			break;
		}

		if ((i < 1) || (i > 2))
		{
			status	= GSC_FAILED;
			break;
		}

		if ((k) && ((factor == 'k') || (factor == 'K')))
		{
			limit	= 0xFFFFFFFFUL / (U32) k;

			if (ul > limit)
				status	= GSC_FAILED;
			else
				u32[0]	= (U32) ul * (U32) k;

			break;
		}

		if ((k) && ((factor == 'm') || (factor == 'M')))
		{
			limit	= 0xFFFFFFFFUL / (U32) k / (U32) k;

			if (ul > limit)
				status	= GSC_FAILED;
			else
				u32[0]	= (U32) ul * (U32) k * (U32) k;

			break;
		}

		if ((k) && ((factor == 'g') || (factor == 'g')))
		{
			limit	= 0xFFFFFFFFUL / (U32) k / (U32) k / (U32) k;

			if (ul > limit)
				status	= GSC_FAILED;
			else
				u32[0]	= (U32) ul * (U32) k * (U32) k * (U32) k;

			break;
		}

		status	= GSC_FAILED;
		break;
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	_hex_text_2_u32
*
*	Purpose:
*
*		Convert hex number in text form to a U32.
*
*	Arguments:
*
*		src			The source string to parse, it may include a prefix of
*					"0x".
*
*		u32			The value is recorded here.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

static U32 _hex_text_2_u32(const char* src, U32* u32)
{
	char			c;
	int				i;
	U32				status	= GSC_SUCCESS;
	unsigned long	ul;

	if ((src[0] == '0') && (src[1] == 'x'))
		i	= sscanf(src, "0x%lX%c", &ul, &c);
	else
		i	= sscanf(src, "%lX%c", &ul, &c);

	if (i == 1)
		u32[0]	= (U32) ul;
	else
		status	= GSC_FAILED;

	return(status);
}



/*****************************************************************************
*
*	Function:	hex_text_2_u32
*
*	Purpose:
*
*		Convert a text formatted number to a U32. The number must be in hex
*		form.
*
*	Arguments:
*
*		src			The text to convert.
*
*		u32			The value is recorded here.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

U32 hex_text_2_u32(const char* src, U32* u32)
{
	U32		status	= GSC_SUCCESS;

	if ((src == NULL) || (src[0] == 0))
		u32[0]	= 0;
	else
		status	= _hex_text_2_u32(src, u32);

	return(status);
}



/*****************************************************************************
*
*	Function:	numeric_text_2_u32
*
*	Purpose:
*
*		Convert a text formatted number to a U32. The number may be in hex
*		form or decimal form, which may include a K, M or G suffix.
*
*	Arguments:
*
*		src			The text to convert.
*
*		k			The size of a single 1K unit. If 0, it is ignored.
*
*		u32			The value is recorded here.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

U32 numeric_text_2_u32(const char* src, int k, U32* u32)
{
	U32		status	= GSC_SUCCESS;

	if ((src == NULL) || (src[0] == 0))
		u32[0]	= 0;
	else if ((src[0] == '0') && (src[1] == 'x'))
		status	= _hex_text_2_u32(src, u32);
	else
		status	= _decimal_text_2_u32(src, k, u32);

	return(status);
}



