// HPDI32 - Application - Win32 - Any Compiler
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/SDK/hpdi32/visualhpdi32/win32/misc_favor_tx.c $
// $Rev: 2902 $
// $Date: 2009-08-09 17:48:49 -0500 (Sun, 09 Aug 2009) $

/*****************************************************************************
*
*	Implement support for the Miscellaneous Favor Tx items.
*
*****************************************************************************/

#include <windows.h>

#include "misc.h"
#include "resource.h"
#include "util_ctrl.h"



/* variables	*************************************************************/

static const value_list_entry_t	_favor_tx_value_list[]	=
{
	{
	/* user_text			*/	"Yes (Favor data transmission.)",
	/* value + value_text	*/	VAR_STR_MACRO(HPDI32_MISC_FAVOR_TX_ENABLE),
	/* suffix				*/	"_YES"
	},
	{
	/* user_text			*/	"No (Favor data reception.)",
	/* value + value_text	*/	VAR_STR_MACRO(HPDI32_MISC_FAVOR_TX_DISABLE),
	/* suffix				*/	"_NO"
	}
};

static const value_list_t	_favor_tx_values[]	=
{
	{
	/* list	*/	_favor_tx_value_list,
	/* size	*/	ARRAY_ELEMENTS(_favor_tx_value_list)
	}
};

static const value_t	_favor_tx_value	=
{
	/* type	*/	VALUE_TYPE_LIST,
	{
	/* u.vp	*/	_favor_tx_values
	}
};

const parm_entry_t	misc_parm_favor_tx	=
{
	/* title			*/	"Favor data Transmission over data Reception",
	/* parm + parm_text	*/	VAR_STR_MACRO(HPDI32_MISC_FAVOR_TX),
	/* suffix			*/	"",
	/* which			*/	0,
	/* set_arg			*/	"",
	/* get_arg			*/	"",
	/* values			*/	&_favor_tx_value,
	/* cfg_wiz			*/	NULL,
	/* desc				*/
	"This tells the API how to proceed when setting a parameter that must"
	" give preference either to data transmission or to data reception. This"
	" applies only to those very few parameters that include both a transmit"
	" specific default (xxx_TX_DEFAULT) and a receive specific default"
	" (xxx_RX_DEFAULT). This preference is applied only when an application"
	" sets such a parameter to the value GSC_DEFAULT. The preference selected"
	" is relevant only when performing simultaneous DMA reads from and DMA"
	" writes to the same HPDI32."
	" NOTE: Applications can generally ignore this parameter if performing"
	" transmit only or receive only operations."
};



/*****************************************************************************
*
*	Function:	_service_refresh
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_REFRESH service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_refresh(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	U32		favor;
	U32		status;

	if (device->handle)
	{
		status	= HPDI32_MISC_FAVOR_TX__GET(device->handle, &favor);
		error_status(dlg, 0, status);

		if (favor == HPDI32_MISC_FAVOR_TX_ENABLE)
			ctrl_menu_set(dlg, IDC_MISC_FAVOR_TX, CTRL_ENABLE | CTRL_BM_CHECK);
		else
			ctrl_menu_set(dlg, IDC_MISC_FAVOR_TX, CTRL_ENABLE | CTRL_BM_UNCHECK);
	}
	else
	{
		ctrl_menu_set(dlg, IDC_MISC_FAVOR_TX, CTRL_DISABLE | CTRL_BM_UNCHECK);
	}

	return(0);
}



/*****************************************************************************
*
*	Function:	_service_wm_command
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_WM_COMMAND service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_wm_command(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	U32		favor;
	U32		status;

	status	= HPDI32_MISC_FAVOR_TX__GET(device->handle, &favor);
	error_status(dlg, 0, status);

	if (favor == HPDI32_MISC_FAVOR_TX_DISABLE)
		status	= HPDI32_MISC_FAVOR_TX__YES(device->handle);
	else
		status	= HPDI32_MISC_FAVOR_TX__NO(device->handle);

	error_status(dlg, 0, status);
	main_refresh(device, NULL);
	return(0);
}



/*****************************************************************************
*
*	Function:	misc_favor_tx_service
*
*	Purpose:
*
*		Implement the common service entry point for the Miscellaneous Favor
*		Tx parameter services.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		service	The service to perform.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0		No service was called.
*		else	The value from the called service, which may be zero.
*
*****************************************************************************/

LRESULT misc_favor_tx_service(
	HWND			dlg,
	device_t*		device,
	service_type_t	service,
	WPARAM			wParam,
	LPARAM			lParam)
{
	static const service_call_t	list[]	=
	{
		/* service				function			*/
		{ SERVICE_REFRESH,		_service_refresh	},
		{ SERVICE_WM_COMMAND,	_service_wm_command	},
		{ SERVICE_END_LIST,		NULL				}
	};

	LRESULT	ret;

	ret	= service_call(dlg, device, list, service, wParam, lParam);
	return(ret);
}


