// HPDI32 - Application - Win32 - Any Compiler
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/SDK/hpdi32/visualhpdi32/win32/preference.c $
// $Rev: 31050 $
// $Date: 2014-10-27 17:51:19 -0500 (Mon, 27 Oct 2014) $

/*****************************************************************************
*
*	Implement support for preferences.
*
*****************************************************************************/

#include <windows.h>

#include "preference.h"
#include "ini.h"



/* #defines	*****************************************************************/

#define	SECTION_NAME	"preference"



/*****************************************************************************
*
*	Function:	pref_bool_get
*
*	Purpose:
*
*		Retrieve the boolean value of a preference setting.
*
*	Arguments:
*
*		preference	The preference to access.
*
*	Returned:
*
*		TRUE		The preference is set.
*		FALSE		The preference is clear.
*
*****************************************************************************/

BOOL pref_bool_get(pref_t preference)
{
	BOOL	value;

	switch (preference)
	{
		default:
		case PREF_API_BUFFER_SIZE:
		case PREF_OSCILLATOR_RATE:

			value	= FALSE;
			break;

		case PREF_LOCATION_RELATIVE:

			ini_bool_get(NULL, SECTION_NAME, "location.relative", &value, TRUE);
			break;
	}

	return(value);
}



/*****************************************************************************
*
*	Function:	pref_u32_get
*
*	Purpose:
*
*		Retrieve the U32 value of a preference setting.
*
*	Arguments:
*
*		preference	The preference to access.
*
*	Returned:
*
*		0			The preference wasn't found.
*		0			The preference setting was non-numeric or invalid.
*		else		The preference setting.
*
*****************************************************************************/

U32 pref_u32_get(pref_t preference)
{
	U32	value;

	switch (preference)
	{
		default:
		case PREF_LOCATION_RELATIVE:

			value	= 0;
			break;

		case PREF_API_BUFFER_SIZE:

			ini_u32_get(NULL, SECTION_NAME, "api buffer size", &value, 0);

			if (value == 0)
			{
				value	= 2UL * 1024 * 1024;
				ini_u32_set(NULL, SECTION_NAME, "api buffer size", value);
			}

			break;

		case PREF_OSCILLATOR_RATE:

			ini_u32_get(NULL, SECTION_NAME, "oscillator", &value, 0);

			if (value == 0)
			{
				value	= 20000000UL;
				ini_u32_set(NULL, SECTION_NAME, "oscillator", value);
			}

			break;
	}

	return(value);
}


