// HPDI32 - Application - Win32 - Any Compiler
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/SDK/hpdi32/visualhpdi32/win32/reg.c $
// $Rev: 2902 $
// $Date: 2009-08-09 17:48:49 -0500 (Sun, 09 Aug 2009) $

/*****************************************************************************
*
*	Implement a dialog providing access to individual registers.
*
*****************************************************************************/

#include <windows.h>
#include <stdio.h>

#include "regs.h"
#include "align.h"
#include "gui.h"
#include "resource.h"
#include "util_ctrl.h"



/* #defines	*****************************************************************/

#define	SECTION_NAME		"register: "



/* prototypes	*************************************************************/

static	void	_idc_reg_field_edit(HWND dlg, reg_t* data, WPARAM wParam, int field);
static	void	_idc_reg_value_edit(HWND dlg, reg_t* data, WPARAM wParam);



/*****************************************************************************
*
*	Function:	_align_complete
*
*	Purpose:
*
*		Align the controls within the window.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		data		The dialog specific data we track.
*
*		bottom		The index of the field at the bottom of the first column.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _align_complete(HWND dlg, reg_t* data, int bottom)
{
	HWND	control;
	int		id;
	RECT	rect;
	RECT	tmp;

	GetWindowRect(dlg, &rect);

	control	= GetDlgItem(dlg, IDC_REG_BIT_BUTTON_00);
	GetWindowRect(control, &tmp);
	rect.right	= tmp.right;

	if (data->def->fields)
	{
		id		= (int) (IDC_REG_FIELD_LIST_00 + data->def->fields - 1);
		control	= GetDlgItem(dlg, id);
		GetWindowRect(control, &tmp);

		if (rect.right < tmp.right)
			rect.right	= tmp.right;
	}

	rect.right	+= data->device->margin + GetSystemMetrics(SM_CXDLGFRAME);
	MoveWindow(	dlg,
				rect.left,
				rect.top,
				RECT_WIDTH(rect),
				RECT_HEIGHT(rect),
				FALSE);

	if (data->def->fields)
		id	= IDC_REG_FIELD_BIT_00 + bottom;
	else
		id	= IDC_REG_BIT_BUTTON_00;

	{
		#define	MACRO(i,_i,_r,_f,_v)	ALIGN_T_(list[i],_i,_r,_f,_v);

		align_t	list[16];

		// The frame under the menu (which we don't have yet).
		MACRO(0, IDC_REG_FRAME_1, 0, ALIGN_L | ALIGN_T | ALIGN_R | ALIGN_B, -1);

		// The APPLY button.
		MACRO(1, IDC_APPLY, id, ALIGN_T_B | ALIGN_H, data->device->margin);
		MACRO(2, IDC_APPLY, 0, ALIGN_R_C | ALIGN_W, - data->device->margin / 2);

		// The REFRESH button.
		MACRO(3, IDC_REFRESH, IDC_APPLY, ALIGN_COPY, 0);
		MACRO(4, IDC_REFRESH, IDC_APPLY, ALIGN_R_L | ALIGN_W, - data->device->margin);

		// The OK button.
		MACRO(5, IDOK, IDC_APPLY, ALIGN_COPY, 0);
		MACRO(6, IDOK, IDC_APPLY, ALIGN_L_R | ALIGN_W, data->device->margin);

		// The CLOSE button.
		MACRO(7, IDCANCEL, IDOK, ALIGN_COPY, 0);
		MACRO(8, IDCANCEL, IDOK, ALIGN_L_R | ALIGN_W, data->device->margin);

		// The status frame.
		MACRO( 9, IDC_REG_FRAME_2, 0, ALIGN_L, -1);
		MACRO(10, IDC_REG_FRAME_2, IDOK, ALIGN_T_B, data->device->margin);
		MACRO(11, IDC_REG_FRAME_2, 0, ALIGN_R_R | ALIGN_B_B, +1);

		// The status text
		MACRO(12, IDC_REG_STATUS, 0, ALIGN_L_L, data->device->margin);
		MACRO(13, IDC_REG_STATUS, IDC_REG_FRAME_2, ALIGN_T_T | ALIGN_H, MARGIN);
		MACRO(14, IDC_REG_STATUS, 0, ALIGN_R_R, - data->device->margin);

		MACRO(15, 0, 0, 0, 0);

		align_control_list(dlg, list);
	}

	control	= GetDlgItem(dlg, IDC_REG_STATUS);
	GetWindowRect(control, &tmp);
	rect.bottom	= tmp.bottom;
	rect.bottom	+= MARGIN + GetSystemMetrics(SM_CYDLGFRAME);
	MoveWindow(	dlg,
				rect.left,
				rect.top,
				RECT_WIDTH(rect),
				RECT_HEIGHT(rect),
				FALSE);
}



/*****************************************************************************
*
*	Function:	_init_byte_controls
*
*	Purpose:
*
*		Initialize the text and button controls for a byte's worth of bits.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		data		The dialog specific data we track.
*
*		index		The value of the most significant bit.
*
*		x			The relative X location.
*
*		y			The relative Y location.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _init_byte_controls(HWND dlg, reg_t* data, int index, int x, int y)
{
	int				btn_h;
	HWND			control;
	int				i;
	RECT			rect;
	int				top;
	int				txt_h;
	int				width;
	int				y1;
	int				y2;

	control		= GetDlgItem(dlg, IDC_REG_DESCRIPTION);
	GetWindowRect(control, &rect);
	ScreenToClient(dlg, (LPPOINT) &rect.left);
	ScreenToClient(dlg, (LPPOINT) &rect.right);
	top	= rect.bottom;

	control	= GetDlgItem(dlg, IDC_REG_BIT_TEXT_00);
	GetWindowRect(control, &rect);
	width	= RECT_WIDTH(rect);
	txt_h	= RECT_HEIGHT(rect);

	control	= GetDlgItem(dlg, IDC_REG_BIT_BUTTON_00);
	GetWindowRect(control, &rect);
	btn_h	= RECT_HEIGHT(rect);

	x	= data->device->margin
		+ (x * data->device->margin)
		+ (x * 8 * width)
		+ (x * 8 * MARGIN);

	y1	= top
		+ data->device->margin
		+ (y * txt_h)
		+ (y * btn_h)
		+ (y * data->device->margin);
	y2	= y1 + txt_h;

	for (i = 8; i; i--, index--)
	{
		// Update the text lable.
		align_control(dlg, IDC_REG_BIT_TEXT_00 + index, 0, ALIGN_L, x);
		align_control(dlg, IDC_REG_BIT_TEXT_00 + index, 0, ALIGN_W, width);
		align_control(dlg, IDC_REG_BIT_TEXT_00 + index, 0, ALIGN_T, y1);
		align_control(dlg, IDC_REG_BIT_TEXT_00 + index, 0, ALIGN_H, txt_h);
		SendDlgItemMessage(	dlg,
							IDC_REG_BIT_TEXT_00 + index,
							WM_SETFONT,
							(WPARAM) data->device->font.defalt,
							0);

		// Update the value button.
		align_control(dlg, IDC_REG_BIT_BUTTON_00 + index, 0, ALIGN_L, x);
		align_control(dlg, IDC_REG_BIT_BUTTON_00 + index, 0, ALIGN_W, width);
		align_control(dlg, IDC_REG_BIT_BUTTON_00 + index, 0, ALIGN_T, y2);
		align_control(dlg, IDC_REG_BIT_BUTTON_00 + index, 0, ALIGN_H, btn_h);
		SendDlgItemMessage(	dlg,
							IDC_REG_BIT_BUTTON_00 + index,
							WM_SETFONT,
							(WPARAM) data->device->font.defalt,
							0);

		x	+= width + MARGIN;

		if (i == 5)
			x	+= MARGIN;
	}
}



/*****************************************************************************
*
*	Function:	_init_bit_controls
*
*	Purpose:
*
*		Initialize the BIT controls needed for the referenced register.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		data		The dialog specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _init_bit_controls(HWND dlg, reg_t* data)
{
	HWND	control;
	int		i;

	switch (data->bits)
	{
		default:
		case 8:		i	= 8;
					_init_byte_controls(dlg, data, 7, 0, 0);
					break;

		case 16:	i	= 16;
					_init_byte_controls(dlg, data, 15, 0, 0);
					_init_byte_controls(dlg, data,  7, 1, 0);
					break;

		case 32:	i	= 32;
					_init_byte_controls(dlg, data, 31, 0, 0);
					_init_byte_controls(dlg, data, 23, 1, 0);
					_init_byte_controls(dlg, data, 15, 0, 1);
					_init_byte_controls(dlg, data,  7, 1, 1);
					break;
	}

	for (; i <= 31; i++)
	{
		control		= GetDlgItem(dlg, IDC_REG_BIT_TEXT_00 + i);
		DestroyWindow(control);
		control		= GetDlgItem(dlg, IDC_REG_BIT_BUTTON_00 + i);
		DestroyWindow(control);
	}
}



/*****************************************************************************
*
*	Function:	_init_field_bit_align
*
*	Purpose:
*
*		Align the given list of bit text fields.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		data		The dialog specific data we track.
*
*		begin		The first field to look at.
*
*		end			The last field to look at.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _init_field_bit_align(HWND dlg, reg_t* data, int begin, int end)
{
	UINT	align;
	char	buf[64];
	HWND	control;
	HDC		dc;
	HFONT	font;
	int		i;
	int		left;
	int		len;
	int		margin;
	SIZE	size;
	int		top;
	int		width;

	if (begin == 0)
	{
		align	= ALIGN_L | ALIGN_W;
		left	= 0;
		margin	= data->device->margin;
		top		= IDC_REG_FIELD_BIT_TITLE_1;
	}
	else
	{
		align	= ALIGN_L_R | ALIGN_W;
		left	= IDC_REG_FIELD_LIST_00;
		margin	= 2 * data->device->margin;
		top		= IDC_REG_FIELD_BIT_TITLE_2;
	}

	// Get the width of the column title text.

	control	= GetDlgItem(dlg, top);
	dc		= GetDC(control);

	font	= SelectObject(dc, data->device->font.defalt);
	len	= GetWindowText(control, buf, sizeof(buf));
	GetTextExtentPoint32(dc, buf, len, &size);
	width	= size.cx;

	// Get the wisth of each control's text.
	SelectObject(dc, data->device->font.defalt_b);

	for (i = begin; i <= end; i++)
	{
		len	= (int) strlen(data->field[i].bits);
		GetTextExtentPoint32(dc, data->field[i].bits, len, &size);

		if (width < size.cx)
			width	= size.cx;
	}

	SelectObject(dc, font);
	ReleaseDC(control, dc);

	// Align the fields.
	align_control(dlg, top, left, align, margin);
	align_control(dlg, top, 0, ALIGN_W, width);
	align_control(	dlg,
					top,
					IDC_REG_BIT_BUTTON_00,
					ALIGN_T_B | ALIGN_H,
					data->device->margin);

	for (i = begin; i <= end; i++)
	{
		align_control(dlg, IDC_REG_FIELD_BIT_00 + i, left, align, margin);
		align_control(dlg, IDC_REG_FIELD_BIT_00 + i, 0, ALIGN_W, width);
		align_control(	dlg,
						IDC_REG_FIELD_BIT_00 + i,
						top,
						ALIGN_T_B | ALIGN_H,
						MARGIN);
		top	= IDC_REG_FIELD_BIT_00 + i;
	}
}



/*****************************************************************************
*
*	Function:	_init_field_bit_text
*
*	Purpose:
*
*		Generate the text that gives the field bits.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		data		The dialog specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _init_field_bit_text(HWND dlg, reg_t* data)
{
	U32		bit;
	HWND	control;
	int		first;
	int		i;
	int		last;
	U32		mask;
	char*	psz;

	for (i = 0; i < (int) data->def->fields; i++)
	{
		bit		= 0x80000000;
		first	= 31;
		mask	= data->def->field[i].mask;
		psz		= data->field[i].bits;
		psz[0]	= 0;

		for (; mask;)
		{
			// Find the first bit set.

			for (;;)
			{
				if (bit & mask)
				{
					mask	&= ~bit;
					break;
				}
				else
				{
					bit	/= 2;
					first--;
				}
			}

			// Find the ending bit in the sequence.

			for (last = first, bit /= 2;;)
			{
				if (bit & mask)
				{
					mask	&= ~bit;
					bit		/= 2;
					last--;
					continue;
				}
				else
				{
					break;
				}
			}

			if (psz[0])
			{
				strcat(psz, ",");
				psz	+= strlen(psz);
			}

			if (first == last)
				sprintf(psz, "%d", first);
			else
				sprintf(psz, "%d-%d", first, last);

			first	= last - 1;
		}

		control	= GetDlgItem(dlg, IDC_REG_FIELD_BIT_00 + i);
		SetWindowText(control, data->field[i].bits);
	}
}



/*****************************************************************************
*
*	Function:	_init_field_edit
*
*	Purpose:
*
*		Initialize the referenced edit controls.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		data		The dialog specific data we track.
*
*		begin		The first field to look at.
*
*		end			The last field to look at.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _init_field_edit(HWND dlg, reg_t* data, int begin, int end)
{
	int		bits;
	char	buf[32];
	HWND	control;
	HDC		dc;
	HFONT	font;
	int		i;
	int		j;
	int		left;
	int		len;
	U32		mask;
	SIZE	size;
	int		top;
	int		width;

	if (begin == 0)
	{
		left	= IDC_REG_FIELD_NAME_TITLE_1;
		top		= IDC_REG_FIELD_VALUE_TITLE_1;
	}
	else
	{
		left	= IDC_REG_FIELD_NAME_TITLE_2;
		top		= IDC_REG_FIELD_VALUE_TITLE_2;
	}

	// Get the width of the column title text.
	control	= GetDlgItem(dlg, top);
	dc		= GetDC(control);
	font	= SelectObject(dc, data->device->font.defalt);
	len		= GetWindowText(control, buf, sizeof(buf));
	GetTextExtentPoint32(dc, buf, len, &size);
	width	= size.cx;

	// Get the width of each control's text.
	SelectObject(dc, data->device->font.fixed);

	for (i = begin; i <= end; i++)
	{
		// Initialize the control.
		SendDlgItemMessage(	dlg,
							IDC_REG_FIELD_EDIT_00 + i,
							WM_SETFONT,
							(WPARAM) data->device->font.fixed,
							0);

		// See how many characters it should accommodate.

		for (bits = 0, mask = 0x1; mask; mask *= 2)
		{
			if (mask & data->def->field[i].mask)
				bits++;
		}

		j	= (bits + 3) / 4;
		sprintf(buf, "%*s ", j, " ");

		// Get its width.
		len	= (int) strlen(buf);
		GetTextExtentPoint32(dc, buf, len, &size);

		if (width < size.cx)
			width	= size.cx;
	}

	SelectObject(dc, font);
	ReleaseDC(control, dc);

	// Align the fields.
	align_control(	dlg,
					top,
					left,
					ALIGN_L_R | ALIGN_W,
					data->device->margin);
	align_control(dlg, top, 0, ALIGN_W, width);
	align_control(dlg, top, left, ALIGN_T_T | ALIGN_B_B, 0);

	for (i = begin; i <= end; i++)
	{
		align_control(	dlg,
						IDC_REG_FIELD_EDIT_00 + i,
						top,
						ALIGN_L_L | ALIGN_R_R,
						0);
		align_control(	dlg,
						IDC_REG_FIELD_EDIT_00 + i,
						IDC_REG_FIELD_NAME_00 + i,
						ALIGN_T_T | ALIGN_B_B,
						0);
		SendDlgItemMessage(	dlg,
							IDC_REG_FIELD_EDIT_00 + i,
							EM_SETLIMITTEXT,
							12,
							0);
	}
}



/*****************************************************************************
*
*	Function:	_init_field_name_align
*
*	Purpose:
*
*		Align the given list of name fields.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		data		The dialog specific data we track.
*
*		begin		The first field to look at.
*
*		end			The last field to look at.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _init_field_name_align(HWND dlg, reg_t* data, int begin, int end)
{
	char	buf[64];
	HWND	control;
	HDC		dc;
	HFONT	font;
	int		i;
	int		left;
	int		len;
	SIZE	size;
	int		top;
	int		width;

	if (begin == 0)
	{
		left	= IDC_REG_FIELD_BIT_TITLE_1;
		top		= IDC_REG_FIELD_NAME_TITLE_1;
	}
	else
	{
		left	= IDC_REG_FIELD_BIT_TITLE_2;
		top		= IDC_REG_FIELD_NAME_TITLE_2;
	}

	// Get the width of the column title text.

	control	= GetDlgItem(dlg, top);
	dc		= GetDC(control);

	SelectObject(dc, data->device->font.defalt);
	len	= GetWindowText(control, buf, sizeof(buf));
	GetTextExtentPoint32(dc, buf, len, &size);
	width	= size.cx;

	// Get the wisth of each control's text.
	font	= SelectObject(dc, data->device->font.defalt_b);

	for (i = begin; i <= end; i++)
	{
		len	= (int) strlen(data->def->field[i].name);
		GetTextExtentPoint32(dc, data->def->field[i].name, len, &size);

		if (width < size.cx)
			width	= size.cx;
	}

	SelectObject(dc, font);
	ReleaseDC(control, dc);

	// Align the fields.
	align_control(	dlg,
					top,
					left,
					ALIGN_L_R | ALIGN_W,
					data->device->margin);
	align_control(dlg, top, 0, ALIGN_W, width);
	align_control(dlg, top, left, ALIGN_T_T | ALIGN_B_B, 0);

	for (i = begin; i <= end; i++)
	{
		align_control(	dlg,
						IDC_REG_FIELD_NAME_00 + i,
						IDC_REG_FIELD_BIT_00 + i,
						ALIGN_L_R | ALIGN_W,
						data->device->margin);
		align_control(dlg, IDC_REG_FIELD_NAME_00 + i, 0, ALIGN_W, width);
		align_control(	dlg,
						IDC_REG_FIELD_NAME_00 + i,
						IDC_REG_FIELD_BIT_00 + i,
						ALIGN_T_T | ALIGN_B_B,
						0);
	}
}



/*****************************************************************************
*
*	Function:	_init_field_name_text
*
*	Purpose:
*
*		Set the text for the field name controls.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		data		The dialog specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _init_field_name_text(HWND dlg, reg_t* data)
{
	int	i;

	for (i = 0; i < (int) data->def->fields; i++)
	{
		ctrl_menu_set_text(	dlg,
							IDC_REG_FIELD_NAME_00 + i,
							data->def->field[i].name);
	}
}



/*****************************************************************************
*
*	Function:	_init_field_value_list
*
*	Purpose:
*
*		Compute the width needed for the referenced field's value list, and
*		fill the control with the appropriate data..
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		data		The dialog specific data we track.
*
*		index		The index of the field to look at.
*
*	Returned:
*
*		0			Do not display a value list.
*		> 0			The needed width in pixels.
*
*****************************************************************************/

static int _init_field_value_list(HWND dlg, reg_t* data, int index)
{
	HWND				control;
	HDC					dc;
	const reg_field_t*	field	= &data->def->field[index];
	HFONT				font;
	int					i;
	int					len;
	SIZE				size;
	const reg_value_t*	value;
	int					width	= 0;

	if (field->values)
	{
		value	= field->value;
		control	= GetDlgItem(dlg, IDC_REG_FIELD_LIST_00 + index);
		dc		= GetDC(control);
		font	= data->device->font.defalt;
		SendMessage(control, WM_SETFONT, (WPARAM) font, 0);
		font	= SelectObject(dc, font);

		for (i = 0; i < (int) field->values; i++, value++)
		{
			SendMessage(control, CB_ADDSTRING, 0, (LPARAM) value->name);
			len	= (int) strlen(value->name);
			GetTextExtentPoint32(dc, value->name, len, &size);

			if (width < size.cx)
				width	= size.cx;
		}

		SendMessage(control, CB_SETTOPINDEX, 0, 0);
		SendMessage(control, CB_SETCURSEL, (WPARAM) -1, 0);

		GetTextExtentPoint32(dc, " ", 1, &size);
		width	+= size.cx;
		width	+= GetSystemMetrics(SM_CXVSCROLL);
		width	+= GetSystemMetrics(SM_CXDLGFRAME) * 2;

		SelectObject(dc, font);
		ReleaseDC(control, dc);
	}

	return(width);
}



/*****************************************************************************
*
*	Function:	_init_field_values
*
*	Purpose:
*
*		Initialize the referenced list controls.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		data		The dialog specific data we track.
*
*		begin		The first field to look at.
*
*		end			The last field to look at.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _init_field_values(HWND dlg, reg_t* data, int begin, int end)
{
	int	i;
	int	j;
	int	margin;
	int	width	= 0;

	// Get the width of each control's text.

	for (i = begin; i <= end; i++)
	{
		j	= _init_field_value_list(dlg, data, i);

		if (width < j)
			width	= j;
	}

	// Align the fields.

	if (width)
		margin	= MARGIN;
	else
		margin	= 0;

	for (i = begin; i <= end; i++)
	{
		align_control(	dlg,
						IDC_REG_FIELD_LIST_00 + i,
						IDC_REG_FIELD_EDIT_00 + i,
						ALIGN_L_R | ALIGN_W,
						margin);
		align_control(	dlg,
						IDC_REG_FIELD_LIST_00 + i,
						0,
						ALIGN_W,
						width);
		align_control(	dlg,
						IDC_REG_FIELD_LIST_00 + i,
						IDC_REG_FIELD_EDIT_00 + i,
						ALIGN_T_T | ALIGN_H,
						0);
	}
}



/*****************************************************************************
*
*	Function:	_init_field_controls
*
*	Purpose:
*
*		Initialize the controls that apply to the register fields.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		data		The dialog specific data we track.
*
*		bottom		We put here the index of the field at the bottom of the
*					first column.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _init_field_controls(HWND dlg, reg_t* data, int* bottom)
{
	int		bot;
	HWND	control;
	int		i;
	int		top;

	if (data->def->fields == 0)
	{
		bottom[0]	= 0;

		control	= GetDlgItem(dlg, IDC_REG_FIELD_BIT_TITLE_1);
		DestroyWindow(control);
		control	= GetDlgItem(dlg, IDC_REG_FIELD_NAME_TITLE_1);
		DestroyWindow(control);
		control	= GetDlgItem(dlg, IDC_REG_FIELD_VALUE_TITLE_1);
		DestroyWindow(control);

		control	= GetDlgItem(dlg, IDC_REG_FIELD_BIT_TITLE_2);
		DestroyWindow(control);
		control	= GetDlgItem(dlg, IDC_REG_FIELD_NAME_TITLE_2);
		DestroyWindow(control);
		control	= GetDlgItem(dlg, IDC_REG_FIELD_VALUE_TITLE_2);
		DestroyWindow(control);
	}
	else
	{
		if (data->def->fields <= 1)
		{
			control	= GetDlgItem(dlg, IDC_REG_FIELD_BIT_TITLE_2);
			DestroyWindow(control);
			control	= GetDlgItem(dlg, IDC_REG_FIELD_NAME_TITLE_2);
			DestroyWindow(control);
			control	= GetDlgItem(dlg, IDC_REG_FIELD_VALUE_TITLE_2);
			DestroyWindow(control);
		}

		bot			= (int) ((data->def->fields + 1) / 2) - 1;
		bottom[0]	= bot;
		_init_field_bit_text(dlg, data);
		_init_field_name_text(dlg, data);

		_init_field_bit_align(dlg, data, 0, bot);
		_init_field_name_align(dlg, data, 0, bot);
		_init_field_edit(dlg, data, 0, bot);
		_init_field_values(dlg, data, 0, bot);

		if (data->def->fields > (size_t) bot)
		{
			top	= bot + 1;
			bot	= (int) data->def->fields - 1;
			_init_field_bit_align(dlg, data, top, bot);
			_init_field_name_align(dlg, data, top, bot);
			_init_field_edit(dlg, data, top, bot);
			_init_field_values(dlg, data, top, bot);
		}
	}

	for (i = 0; i < (int) data->def->fields; i++)
	{
		if (data->def->field[i].values)
			continue;

		control	= GetDlgItem(dlg, IDC_REG_FIELD_LIST_00 + i);
		ShowWindow(control, SW_HIDE);
	}

	for (	i = (int) data->def->fields;
			i < (int) ARRAY_ELEMENTS(data->field);
			i++)
	{
		control	= GetDlgItem(dlg, IDC_REG_FIELD_BIT_00 + i);
		DestroyWindow(control);
		control	= GetDlgItem(dlg, IDC_REG_FIELD_NAME_00 + i);
		DestroyWindow(control);
		control	= GetDlgItem(dlg, IDC_REG_FIELD_EDIT_00 + i);
		DestroyWindow(control);
		control	= GetDlgItem(dlg, IDC_REG_FIELD_LIST_00 + i);
		DestroyWindow(control);
	}
}



/*****************************************************************************
*
*	Function:	_init_reg_controls
*
*	Purpose:
*
*		Initialize the controls that apply to the register as a whole.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		data		The dialog specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _init_reg_controls(HWND dlg, reg_t* data)
{
	#define	MACRO(i,_i,_r,_f,_v)	ALIGN_T_(list[i],_i,_r,_f,_v);

	char	buf[256];
	HWND	control;
	align_t list[6];

	// The register name text.
	MACRO(0, IDC_REG_NAME, 0, ALIGN_L | ALIGN_W | ALIGN_T | ALIGN_H, data->device->margin);

	// The register value edit control.
	MACRO(1, IDC_REG_VALUE_EDIT, IDC_REG_NAME, ALIGN_L_R | ALIGN_W, MARGIN);
	MACRO(2, IDC_REG_VALUE_EDIT, IDC_REG_NAME, ALIGN_T_T | ALIGN_B_B, 0);

	// The register description text (preliminary).
	MACRO(3, IDC_REG_DESCRIPTION, IDC_REG_VALUE_EDIT, ALIGN_L_R, data->device->margin);
	MACRO(4, IDC_REG_DESCRIPTION, IDC_REG_VALUE_EDIT, ALIGN_T_T | ALIGN_B_B, 0);

	MACRO(5, 0, 0, 0, 0 );

	strcpy(buf, data->def->name);
	strcat(buf, ":");
	ctrl_menu_set_text(dlg, IDC_REG_NAME, buf);
	align_control_width(dlg, IDC_REG_NAME, data->device->font.defalt);

	control	= GetDlgItem(dlg, IDC_REG_VALUE_EDIT);
	data->value_changing	= TRUE;
	SetWindowText(control, "12345678 ");
	align_control_width(dlg, IDC_REG_VALUE_EDIT, data->device->font.fixed);
	SetWindowText(control, "");
	data->value_changing	= FALSE;
	SendDlgItemMessage(	dlg,
						IDC_REG_VALUE_EDIT,
						WM_SETFONT,
						(WPARAM) data->device->font.fixed,
						0);

	ctrl_menu_set_text(dlg, IDC_REG_DESCRIPTION, data->def->desc);
	align_control_list(dlg, list);
}



/*****************************************************************************
*
*	Function:	_refresh_bit_controls_highlight
*
*	Purpose:
*
*		Refresh the bit highlighting state per the current field selection.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		data		The dialog specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _refresh_bit_controls_highlight(HWND dlg, reg_t* data)
{
	HWND	control;
	HFONT	font;
	int		i;
	U32		mask;

	for (i = 0, mask = 0x1; i < (int) data->bits; i++, mask *= 2)
	{
		if (data->field_sel < 0)
			font	= data->device->font.defalt;
		else if (mask & data->def->field[data->field_sel].mask)
			font	= data->device->font.defalt_b;
		else
			font	= data->device->font.defalt;

		control	= GetDlgItem(dlg, IDC_REG_BIT_BUTTON_00 + i);
		SendMessage(control, WM_SETFONT, (WPARAM) font, 0);
		InvalidateRect(control, NULL, FALSE);

		control	= GetDlgItem(dlg, IDC_REG_BIT_TEXT_00 + i);
		SendMessage(control, WM_SETFONT, (WPARAM) font, 0);
		InvalidateRect(control, NULL, FALSE);
	}
}



/*****************************************************************************
*
*	Function:	_refresh_bit_controls_state
*
*	Purpose:
*
*		Update the bit controls per the state of the system.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		data		The dialog specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _refresh_bit_controls_state(HWND dlg, reg_t* data)
{
	UINT	flags;
	int		i;
	U32		mask;
	char*	psz;

	for (i = 0, mask = 0x1; i < (int) data->bits; i++, mask *= 2)
	{
		if (data->device->handle == NULL)
			psz		= "";
		else if (mask & data->value)
			psz		= "1";
		else
			psz		= "0";

		if (data->device->handle == NULL)
			flags	= CTRL_DISABLE;
		else if (mask & data->def->read_only)
			flags	= CTRL_DISABLE;
		else
			flags	= CTRL_ENABLE;

		ctrl_menu_set(dlg, IDC_REG_BIT_BUTTON_00 + i, flags);
		ctrl_menu_set_text(dlg, IDC_REG_BIT_BUTTON_00 + i, psz);
	}
}



/*****************************************************************************
*
*	Function:	_refresh_field_controls_highlight
*
*	Purpose:
*
*		Update the highlight state per the current field selection.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		data		The dialog specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _refresh_field_controls_highlight(HWND dlg, reg_t* data)
{
	HWND	control;
	HFONT	font;
	int		i;

	for (i = 0; i < (int) data->def->fields; i++)
	{
		if (i == data->field_sel)
			font	= data->device->font.defalt_b;
		else
			font	= data->device->font.defalt;

		control	= GetDlgItem(dlg, IDC_REG_FIELD_BIT_00 + i);
		SendMessage(control, WM_SETFONT, (WPARAM) font, 0);
		InvalidateRect(control, NULL, FALSE);

		control	= GetDlgItem(dlg, IDC_REG_FIELD_NAME_00 + i);
		SendMessage(control, WM_SETFONT, (WPARAM) font, 0);
		InvalidateRect(control, NULL, FALSE);
	}
}



/*****************************************************************************
*
*	Function:	_refresh_field_controls_state
*
*	Purpose:
*
*		Update the field controls per the state of the system.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		data		The dialog specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _refresh_field_controls_state(HWND dlg, reg_t* data)
{
	char	buf[16];
	int		edit_id	= IDC_REG_FIELD_EDIT_00;
	int		i;
	int		j;
	int		list_id	= IDC_REG_FIELD_LIST_00;
	U32		mask;
	U32		value;
	U32		u32;

	for (i = 0; i < (int) data->def->fields; i++, edit_id++, list_id++)
	{
		if (data->device->handle == NULL)
		{
			ctrl_menu_set(dlg, edit_id, CTRL_DISABLE | CTRL_NULL);
			ctrl_menu_set(dlg, list_id, CTRL_DISABLE);
			SendDlgItemMessage(dlg, list_id, CB_SETCURSEL, (WPARAM) -1, 0);
			continue;
		}

		if (data->def->field[i].mask & data->def->read_only)
		{
			ctrl_menu_set(dlg, edit_id, CTRL_DISABLE);
			ctrl_menu_set(dlg, list_id, CTRL_DISABLE);
		}
		else
		{
			ctrl_menu_set(dlg, edit_id, CTRL_ENABLE);
			ctrl_menu_set(dlg, list_id, CTRL_ENABLE);
		}

		value	= data->value & data->def->field[i].mask;

		// Update the list selection.

		for (j = (int) data->def->field[i].values; j--;)
		{
			if (value == data->def->field[i].value[j].value)
				break;
		}

		SendDlgItemMessage(dlg, list_id, CB_SETCURSEL, (WPARAM) j, 0);

		// Update the edit text.
		mask	= data->def->field[i].mask;

		for (u32 = 0; mask; mask *= 2, value *= 2)
		{
			if (mask & 0x80000000)
			{
				u32	*= 2;

				if (value & 0x80000000)
					u32++;
			}
		}

		sprintf(buf, "%lX", (long) u32);

		if (!data->field[i].changing)
			ctrl_menu_set_text(dlg, edit_id, buf);
	}
}



/*****************************************************************************
*
*	Function:	_refresh_reg_controls
*
*	Purpose:
*
*		Update the register level controls per the state of the system.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		data		The dialog specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _refresh_reg_controls(HWND dlg, reg_t* data)
{
	char	buf[128];
	U32		mask;

	strcpy(buf, data->def->name);

	if (data->changes)
		strcat(buf, " *");

	set_win_title(dlg, data->device, FALSE, TRUE, buf);

	if (data->device->handle == NULL)
	{
		ctrl_menu_set(dlg, IDCANCEL, CTRL_ENABLE | CTRL_FOCUS);
		ctrl_menu_set(dlg, IDC_REG_VALUE_EDIT, CTRL_DISABLE | CTRL_NULL);
		ctrl_menu_set(dlg, IDC_REFRESH, CTRL_DISABLE);
		ctrl_menu_set(dlg, IDC_APPLY, CTRL_DISABLE);
		ctrl_menu_set(dlg, IDOK, CTRL_DISABLE);
	}
	else
	{
		switch (data->bits)
		{
			default:
			case 8:		mask	= 0x000000FF;	break;
			case 16:	mask	= 0x0000FFFF;	break;
			case 32:	mask	= 0xFFFFFFFF;	break;
		}

		if ((mask & data->def->read_only) == mask)
		{
			ctrl_menu_set(dlg, IDC_REG_VALUE_EDIT, CTRL_DISABLE);
			ctrl_menu_set(dlg, IDC_REFRESH, CTRL_ENABLE);
			ctrl_menu_set(dlg, IDC_APPLY, CTRL_DISABLE);
			ctrl_menu_set(dlg, IDOK, CTRL_DISABLE);
		}
		else
		{
			ctrl_menu_set(dlg, IDC_REG_VALUE_EDIT, CTRL_ENABLE);
			ctrl_menu_set(dlg, IDC_REFRESH, CTRL_ENABLE);
			ctrl_menu_set(dlg, IDC_APPLY, CTRL_ENABLE);
			ctrl_menu_set(dlg, IDOK, CTRL_ENABLE);
		}

		if (!data->value_changing)
		{
			sprintf(buf, "%0*lX", data->bits / 4, (long) data->value);
			ctrl_menu_set_text(dlg, IDC_REG_VALUE_EDIT, buf);
		}
	}
}



/*****************************************************************************
*
*	Function:	_refresh_status
*
*	Purpose:
*
*		Update the displayed status per the current state.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		data		The dialog specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _refresh_status(HWND dlg, reg_t* data)
{
	if (data->status != GSC_SUCCESS)
	{
		error_status(dlg, IDC_REG_STATUS, data->status);
	}
	else if (data->status_global[0])
	{
		SetDlgItemText(dlg, IDC_REG_STATUS, data->status_global);
	}
	else if (data->status_local[0])
	{
		data->status_global[0]	= 0;
		SetDlgItemText(dlg, IDC_REG_STATUS, data->status_local);
	}
	else
	{
		SetDlgItemText(dlg, IDC_REG_STATUS, NULL);
	}
}



/*****************************************************************************
*
*	Function:	_refresh_field_selection
*
*	Purpose:
*
*		Update the displayed field selection state.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		data		The dialog specific data we track.
*
*		int			field
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _refresh_field_selection(HWND dlg, reg_t* data, int field)
{
	if (field >= 0)
	{
		strcpy(data->status_local, "Field: ");
		strcat(data->status_local, data->def->field[field].desc);
	}
	else
	{
		data->status_local[0]	= 0;
	}

	data->field_sel	= field;
	data->status_global[0]	= 0;
	_refresh_bit_controls_highlight(dlg, data);
	_refresh_field_controls_highlight(dlg, data);
	_refresh_status(dlg, data);
}



/*****************************************************************************
*
*	Function:	_refresh_local
*
*	Purpose:
*
*		Update the content of the window per locally made changes.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		data		The dialog specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _refresh_local(HWND dlg, reg_t* data)
{
	data->ignore_nm	= TRUE;
	_refresh_reg_controls(dlg, data);
	_refresh_bit_controls_state(dlg, data);
	_refresh_field_controls_state(dlg, data);
	data->ignore_nm	= FALSE;
	_refresh_status(dlg, data);
}



/*****************************************************************************
*
*	Function:	_refresh_window
*
*	Purpose:
*
*		Update the content of the window per the state of the system.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		device		The device to access.
*
*		msg			A status message to display.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _refresh_window(HWND dlg, device_t* device, const char* msg)
{
	reg_t*	data	= (reg_t*) GET_WINDOW_USER_DATA(dlg);

	data->changes			= FALSE;
	data->status			= GSC_SUCCESS;
	data->status_global[0]	= 0;

	if ((msg == NULL) || (msg[0] == 0))
		msg	= "Click on labels for field information.";

	strncpy(data->status_global, msg, sizeof(data->status_global));

	if (data->device->handle)
	{
		data->status	= hpdi32_reg_read(	data->device->handle,
											data->def->reg_id,
											&data->value);
	}

	_refresh_bit_controls_highlight(dlg, data);
	_refresh_field_controls_highlight(dlg, data);
	_refresh_local(dlg, data);

	if (data->def->refresh)
		(data->def->refresh)(data);
}



/*****************************************************************************
*
*	Function:	_idc_apply
*
*	Purpose:
*
*		Respond to the APPLY button.
*
*	Arguments:
*
*		data	The dialog specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idc_apply(reg_t* data)
{
	data->changes			= FALSE;
	data->status_global[0]	= 0;
	data->status_local[0]	= 0;
	data->status			= hpdi32_reg_write(	data->device->handle,
												data->def->reg_id,
												data->value);
	main_refresh(data->device, NULL);
}



/*****************************************************************************
*
*	Function:	_idc_refresh
*
*	Purpose:
*
*		Respond to the REFRESH button.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		data	The dialog specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idc_refresh(HWND dlg, reg_t* data)
{
	_refresh_window(dlg, data->device, NULL);
}



/*****************************************************************************
*
*	Function:	_idc_reg_bit_button
*
*	Purpose:
*
*		Respond to bit button activity.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		data	The dialog specific data we track.
*
*		wParam	The message's WORD sized parameter.
*
*		bit		The bit button index clicked.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idc_reg_bit_button(HWND dlg, reg_t* data, WPARAM wParam, int bit)
{
	int	code	= HIWORD(wParam);
	int	i;
	U32	mask;

	if (data->ignore_nm)
	{
	}
	else if (code == BN_SETFOCUS)
	{
		mask	= 0x1UL << bit;

		for (i = (int) data->def->fields; i--;)
		{
			if (mask & data->def->field[i].mask)
				break;
		}

		_refresh_field_selection(dlg, data, i);
	}
	else if (code == BN_CLICKED)
	{
		mask					= 0x1UL << bit;
		data->changes			= TRUE;
		data->value				^= mask;
		data->field_sel			= -1;
		data->status_global[0]	= 0;
		data->status_local[0]	= 0;

		for (i = 0; i < (int) data->def->fields; i++)
		{
			if (mask & data->def->field[i].mask)
			{
				data->field_sel	= i;
				strcpy(data->status_local, "Field: ");
				strcat(data->status_local, data->def->field[i].desc);
				break;
			}
		}

		_refresh_local(dlg, data);
	}
}



/*****************************************************************************
*
*	Function:	_idc_reg_bit_text
*
*	Purpose:
*
*		Respond to bit text control activity.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		data	The dialog specific data we track.
*
*		wParam	The message's WORD sized argument.
*
*		bit		The bit button index clicked.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idc_reg_bit_text(HWND dlg, reg_t* data, WPARAM wParam, int bit)
{
	int	code	= HIWORD(wParam);
	U32	mask;

	if (data->ignore_nm)
	{
	}
	else if ((code == STN_CLICKED) || (code == STN_DBLCLK))
	{
		mask	= 0x1UL << bit;

		if ((data->device->handle == NULL) || (mask & data->def->read_only))
		{
			wParam	= MAKELONG(IDC_REG_BIT_BUTTON_00 + bit, BN_SETFOCUS);
			_idc_reg_bit_button(dlg, data, wParam, bit);
			ctrl_menu_set(dlg, IDCANCEL, CTRL_FOCUS);
		}
		else
		{
			ctrl_menu_set(dlg, IDC_REG_BIT_BUTTON_00 + bit, CTRL_FOCUS);
		}
	}
}



/*****************************************************************************
*
*	Function:	_idc_reg_description
*
*	Purpose:
*
*		Respond to register description text control activity.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		data	The dialog specific data we track.
*
*		wParam	The message's WORD sized argument.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idc_reg_description(HWND dlg, reg_t* data, WPARAM wParam)
{
	int	code	= HIWORD(wParam);
	U32	mask;

	if (data->ignore_nm)
	{
	}
	else if ((code == STN_CLICKED) || (code == STN_DBLCLK))
	{
		switch (data->bits)
		{
			default:
			case 8:		mask	= 0x000000FF;	break;
			case 16:	mask	= 0x0000FFFF;	break;
			case 32:	mask	= 0xFFFFFFFF;	break;
		}

		if ((data->device->handle == NULL) ||
			((mask & data->def->read_only) == mask))
		{
			wParam	= MAKELONG(IDC_REG_VALUE_EDIT, EN_SETFOCUS);
			_idc_reg_value_edit(dlg, data, wParam);
			ctrl_menu_set(dlg, IDCANCEL, CTRL_FOCUS);
		}
		else
		{
			ctrl_menu_set(dlg, IDC_REG_VALUE_EDIT, CTRL_FOCUS);
		}
	}
}



/*****************************************************************************
*
*	Function:	_idc_reg_field_bit
*
*	Purpose:
*
*		Respond to field bit text control activity.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		data	The dialog specific data we track.
*
*		wParam	The message's WORD sized argument.
*
*		field	The infex of the field accessed.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idc_reg_field_bit(HWND dlg, reg_t* data, WPARAM wParam, int field)
{
	int	code	= HIWORD(wParam);

	if (data->ignore_nm)
	{
	}
	else if ((code == STN_CLICKED) || (code == STN_DBLCLK))
	{
		if ((data->device->handle == NULL) ||
			(data->def->field[field].mask & data->def->read_only))
		{
			wParam	= MAKELONG(IDC_REG_FIELD_EDIT_00 + field, EN_SETFOCUS);
			_idc_reg_field_edit(dlg, data, wParam, field);
			ctrl_menu_set(dlg, IDCANCEL, CTRL_FOCUS);
		}
		else
		{
			ctrl_menu_set(dlg, IDC_REG_FIELD_EDIT_00 + field, CTRL_FOCUS);
		}
	}
}



/*****************************************************************************
*
*	Function:	_idc_reg_field_name
*
*	Purpose:
*
*		Respond to field name text control activity.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		data	The dialog specific data we track.
*
*		wParam	The message's WORD sized argument.
*
*		field	The infex of the field accessed.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idc_reg_field_name(HWND dlg, reg_t* data, WPARAM wParam, int field)
{
	int	code	= HIWORD(wParam);

	if (data->ignore_nm)
	{
	}
	else if ((code == STN_CLICKED) || (code == STN_DBLCLK))
	{
		if ((data->device->handle == NULL) ||
			(data->def->field[field].mask & data->def->read_only))
		{
			wParam	= MAKELONG(IDC_REG_FIELD_EDIT_00 + field, EN_SETFOCUS);
			_idc_reg_field_edit(dlg, data, wParam, field);
			ctrl_menu_set(dlg, IDCANCEL, CTRL_FOCUS);
		}
		else
		{
			ctrl_menu_set(dlg, IDC_REG_FIELD_EDIT_00 + field, CTRL_FOCUS);
		}
	}
}



/*****************************************************************************
*
*	Function:	_idc_reg_field_edit
*
*	Purpose:
*
*		Respond to edit control activity.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		data	The dialog specific data we track.
*
*		wParam	The message's WORD sized parameter.
*
*		field	The field index being accessed.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idc_reg_field_edit(HWND dlg, reg_t* data, WPARAM wParam, int field)
{
	char	buf[32];
	int		code	= HIWORD(wParam);
	U32		mask;
	U32		status;
	U32		u32;
	U32		value;

	if (data->ignore_nm)
	{
	}
	else if (code == EN_SETFOCUS)
	{
		_refresh_field_selection(dlg, data, field);
	}
	else if ((code == EN_CHANGE) || (code == EN_KILLFOCUS))
	{
		if (code == EN_CHANGE)
			data->changes	= TRUE;

		data->field_sel			= field;
		data->status_global[0]	= 0;
		strcpy(data->status_local, "Field: ");
		strcat(data->status_local, data->def->field[field].desc);
		status	= ctrl_hex_2_u32(dlg, IDC_REG_FIELD_EDIT_00 + field, &u32);
		value	= 0;

		if (status == GSC_SUCCESS)
		{
			for (mask = 0x1; mask; mask *= 2)
			{
				if (mask & data->def->field[field].mask)
				{
					if (u32 & 0x1)
						value	|= mask;

					u32	/= 2;
				}
			}

			data->value	&= ~data->def->field[field].mask;
			data->value	|= value;
		}
		else
		{
			SendDlgItemMessage(	dlg,
								IDC_REG_FIELD_EDIT_00 + field,
								WM_GETTEXT,
								(WPARAM) sizeof(buf),
								(LPARAM) buf);
			sprintf(data->status_local, "Invalid hex value: '%s'", buf);
			ctrl_menu_set(dlg, IDC_REG_FIELD_EDIT_00 + field, CTRL_FOCUS);
		}

		data->field[field].changing	= TRUE;
		_refresh_local(dlg, data);
		data->field[field].changing	= FALSE;
	}
}



/*****************************************************************************
*
*	Function:	_idc_reg_field_list
*
*	Purpose:
*
*		Respond to list selection changes.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		data	The dialog specific data we track.
*
*		wParam	The message's WORD sized parameter.
*
*		field	The field index being accessed.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idc_reg_field_list(HWND dlg, reg_t* data, WPARAM wParam, int field)
{
	int		code	= HIWORD(wParam);
	LRESULT	index;

	if (data->ignore_nm)
	{
	}
	else if (code == CBN_SETFOCUS)
	{
		_refresh_field_selection(dlg, data, field);
	}
	else if (code == CBN_SELCHANGE)
	{
		data->changes			= TRUE;
		data->field_sel			= field;
		data->status_global[0]	= 0;
		strcpy(data->status_local, "Field: ");
		strcat(data->status_local, data->def->field[field].desc);
		index		= SendDlgItemMessage(	dlg,
											IDC_REG_FIELD_LIST_00 + field,
											CB_GETCURSEL,
											0,
											0);
		data->value	&= ~data->def->field[field].mask;
		data->value	|= data->def->field[field].value[index].value;
		_refresh_local(dlg, data);
	}
}



/*****************************************************************************
*
*	Function:	_idc_reg_name
*
*	Purpose:
*
*		Respond to register name text control activity.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		data	The dialog specific data we track.
*
*		wParam	The message's WORD sized argument.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idc_reg_name(HWND dlg, reg_t* data, WPARAM wParam)
{
	int	code	= HIWORD(wParam);
	U32	mask;

	if (data->ignore_nm)
	{
	}
	else if ((code == STN_CLICKED) || (code == STN_DBLCLK))
	{
		switch (data->bits)
		{
			default:
			case 8:		mask	= 0x000000FF;	break;
			case 16:	mask	= 0x0000FFFF;	break;
			case 32:	mask	= 0xFFFFFFFF;	break;
		}

		if ((data->device->handle == NULL) ||
			((mask & data->def->read_only) == mask))
		{
			wParam	= MAKELONG(IDC_REG_VALUE_EDIT, EN_SETFOCUS);
			_idc_reg_value_edit(dlg, data, wParam);
			ctrl_menu_set(dlg, IDCANCEL, CTRL_FOCUS);
		}
		else
		{
			ctrl_menu_set(dlg, IDC_REG_VALUE_EDIT, CTRL_FOCUS);
		}
	}
}



/*****************************************************************************
*
*	Function:	_idc_reg_value_edit
*
*	Purpose:
*
*		Respond to the register edit control activity.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		data	The dialog specific data we track.
*
*		wParam	The message's WORD sized parameter.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idc_reg_value_edit(HWND dlg, reg_t* data, WPARAM wParam)
{
	char	buf[32];
	int		code	= HIWORD(wParam);
	U32		status;
	U32		value;

	if (data->ignore_nm)
	{
	}
	else if (code == EN_SETFOCUS)
	{
		_refresh_field_selection(dlg, data, -1);
	}
	else if ((code == EN_CHANGE) || (code == EN_KILLFOCUS))
	{
		if (code == EN_CHANGE)
			data->changes	= TRUE;

		data->field_sel			= -1;
		data->status_global[0]	= 0;
		strcpy(data->status_local, data->def->desc);
		status	= ctrl_hex_2_u32(dlg, IDC_REG_VALUE_EDIT, &value);

		if (status == GSC_SUCCESS)
		{
			data->value	= (data->value & data->def->read_only)
						| (value & ~data->def->read_only);
		}
		else
		{
			SendDlgItemMessage(	dlg,
								IDC_REG_VALUE_EDIT,
								WM_GETTEXT,
								(WPARAM) sizeof(buf),
								(LPARAM) buf);
			sprintf(data->status_local, "Invalid hex value: '%s'", buf);
		}

		data->value_changing	= TRUE;
		_refresh_local(dlg, data);
		data->value_changing	= FALSE;
	}
}



/*****************************************************************************
*
*	Function:	_idcancel
*
*	Purpose:
*
*		Respond to a request to close the dialog.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idcancel(HWND dlg)
{
	reg_t*	data;
	char	section[128];

	main_refresh_add(dlg, NULL);
	data	= (reg_t*) GET_WINDOW_USER_DATA(dlg);

	if (data)
	{
		data->dlg	= NULL;
		strcpy(section, SECTION_NAME);
		strcat(section, data->def->name);
		win_pos_save(dlg, section);
	}

	DestroyWindow(dlg);
}



/*****************************************************************************
*
*	Function:	_wm_command
*
*	Purpose:
*
*		Process WM_COMMAND messages.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		wParam		The message's WORD parameter.
*
*	Returned:
*
*		TRUE		We processed the message.
*		FALSE		We did not process the message.
*
*****************************************************************************/

static LRESULT _wm_command(HWND dlg, WPARAM wParam)
{
	reg_t*	data	= (reg_t*) GET_WINDOW_USER_DATA(dlg);
	int		id		= LOWORD(wParam);
	LRESULT	ret		= TRUE;

	switch (id)
	{
		default:

			ret	= FALSE;
			break;

		case IDC_APPLY:

			_idc_apply(data);
			break;

		case IDC_REFRESH:

			_idc_refresh(dlg, data);
			break;

		case IDCANCEL:

			_idcancel(dlg);
			break;

		case IDOK:

			_idc_apply(data);
			_idcancel(dlg);
			break;

		case IDC_REG_BIT_BUTTON_00:
		case IDC_REG_BIT_BUTTON_01:
		case IDC_REG_BIT_BUTTON_02:
		case IDC_REG_BIT_BUTTON_03:
		case IDC_REG_BIT_BUTTON_04:
		case IDC_REG_BIT_BUTTON_05:
		case IDC_REG_BIT_BUTTON_06:
		case IDC_REG_BIT_BUTTON_07:
		case IDC_REG_BIT_BUTTON_08:
		case IDC_REG_BIT_BUTTON_09:
		case IDC_REG_BIT_BUTTON_10:
		case IDC_REG_BIT_BUTTON_11:
		case IDC_REG_BIT_BUTTON_12:
		case IDC_REG_BIT_BUTTON_13:
		case IDC_REG_BIT_BUTTON_14:
		case IDC_REG_BIT_BUTTON_15:
		case IDC_REG_BIT_BUTTON_16:
		case IDC_REG_BIT_BUTTON_17:
		case IDC_REG_BIT_BUTTON_18:
		case IDC_REG_BIT_BUTTON_19:
		case IDC_REG_BIT_BUTTON_20:
		case IDC_REG_BIT_BUTTON_21:
		case IDC_REG_BIT_BUTTON_22:
		case IDC_REG_BIT_BUTTON_23:
		case IDC_REG_BIT_BUTTON_24:
		case IDC_REG_BIT_BUTTON_25:
		case IDC_REG_BIT_BUTTON_26:
		case IDC_REG_BIT_BUTTON_27:
		case IDC_REG_BIT_BUTTON_28:
		case IDC_REG_BIT_BUTTON_29:
		case IDC_REG_BIT_BUTTON_30:
		case IDC_REG_BIT_BUTTON_31:

			_idc_reg_bit_button(dlg, data, wParam, id - IDC_REG_BIT_BUTTON_00);
			break;

		case IDC_REG_BIT_TEXT_00:
		case IDC_REG_BIT_TEXT_01:
		case IDC_REG_BIT_TEXT_02:
		case IDC_REG_BIT_TEXT_03:
		case IDC_REG_BIT_TEXT_04:
		case IDC_REG_BIT_TEXT_05:
		case IDC_REG_BIT_TEXT_06:
		case IDC_REG_BIT_TEXT_07:
		case IDC_REG_BIT_TEXT_08:
		case IDC_REG_BIT_TEXT_09:
		case IDC_REG_BIT_TEXT_10:
		case IDC_REG_BIT_TEXT_11:
		case IDC_REG_BIT_TEXT_12:
		case IDC_REG_BIT_TEXT_13:
		case IDC_REG_BIT_TEXT_14:
		case IDC_REG_BIT_TEXT_15:
		case IDC_REG_BIT_TEXT_16:
		case IDC_REG_BIT_TEXT_17:
		case IDC_REG_BIT_TEXT_18:
		case IDC_REG_BIT_TEXT_19:
		case IDC_REG_BIT_TEXT_20:
		case IDC_REG_BIT_TEXT_21:
		case IDC_REG_BIT_TEXT_22:
		case IDC_REG_BIT_TEXT_23:
		case IDC_REG_BIT_TEXT_24:
		case IDC_REG_BIT_TEXT_25:
		case IDC_REG_BIT_TEXT_26:
		case IDC_REG_BIT_TEXT_27:
		case IDC_REG_BIT_TEXT_28:
		case IDC_REG_BIT_TEXT_29:
		case IDC_REG_BIT_TEXT_30:
		case IDC_REG_BIT_TEXT_31:

			_idc_reg_bit_text(dlg, data, wParam, id - IDC_REG_BIT_TEXT_00);
			break;

		case IDC_REG_FIELD_BIT_00:
		case IDC_REG_FIELD_BIT_01:
		case IDC_REG_FIELD_BIT_02:
		case IDC_REG_FIELD_BIT_03:
		case IDC_REG_FIELD_BIT_04:
		case IDC_REG_FIELD_BIT_05:
		case IDC_REG_FIELD_BIT_06:
		case IDC_REG_FIELD_BIT_07:
		case IDC_REG_FIELD_BIT_08:
		case IDC_REG_FIELD_BIT_09:
		case IDC_REG_FIELD_BIT_10:
		case IDC_REG_FIELD_BIT_11:
		case IDC_REG_FIELD_BIT_12:
		case IDC_REG_FIELD_BIT_13:
		case IDC_REG_FIELD_BIT_14:
		case IDC_REG_FIELD_BIT_15:
		case IDC_REG_FIELD_BIT_16:
		case IDC_REG_FIELD_BIT_17:
		case IDC_REG_FIELD_BIT_18:
		case IDC_REG_FIELD_BIT_19:
		case IDC_REG_FIELD_BIT_20:
		case IDC_REG_FIELD_BIT_21:
		case IDC_REG_FIELD_BIT_22:
		case IDC_REG_FIELD_BIT_23:
		case IDC_REG_FIELD_BIT_24:
		case IDC_REG_FIELD_BIT_25:
		case IDC_REG_FIELD_BIT_26:
		case IDC_REG_FIELD_BIT_27:
		case IDC_REG_FIELD_BIT_28:
		case IDC_REG_FIELD_BIT_29:
		case IDC_REG_FIELD_BIT_30:
		case IDC_REG_FIELD_BIT_31:

			_idc_reg_field_bit(dlg, data, wParam, id - IDC_REG_FIELD_BIT_00);
			break;

		case IDC_REG_FIELD_EDIT_00:
		case IDC_REG_FIELD_EDIT_01:
		case IDC_REG_FIELD_EDIT_02:
		case IDC_REG_FIELD_EDIT_03:
		case IDC_REG_FIELD_EDIT_04:
		case IDC_REG_FIELD_EDIT_05:
		case IDC_REG_FIELD_EDIT_06:
		case IDC_REG_FIELD_EDIT_07:
		case IDC_REG_FIELD_EDIT_08:
		case IDC_REG_FIELD_EDIT_09:
		case IDC_REG_FIELD_EDIT_10:
		case IDC_REG_FIELD_EDIT_11:
		case IDC_REG_FIELD_EDIT_12:
		case IDC_REG_FIELD_EDIT_13:
		case IDC_REG_FIELD_EDIT_14:
		case IDC_REG_FIELD_EDIT_15:
		case IDC_REG_FIELD_EDIT_16:
		case IDC_REG_FIELD_EDIT_17:
		case IDC_REG_FIELD_EDIT_18:
		case IDC_REG_FIELD_EDIT_19:
		case IDC_REG_FIELD_EDIT_20:
		case IDC_REG_FIELD_EDIT_21:
		case IDC_REG_FIELD_EDIT_22:
		case IDC_REG_FIELD_EDIT_23:
		case IDC_REG_FIELD_EDIT_24:
		case IDC_REG_FIELD_EDIT_25:
		case IDC_REG_FIELD_EDIT_26:
		case IDC_REG_FIELD_EDIT_27:
		case IDC_REG_FIELD_EDIT_28:
		case IDC_REG_FIELD_EDIT_29:
		case IDC_REG_FIELD_EDIT_30:
		case IDC_REG_FIELD_EDIT_31:

			_idc_reg_field_edit(dlg, data, wParam, id - IDC_REG_FIELD_EDIT_00);
			break;

		case IDC_REG_FIELD_LIST_00:
		case IDC_REG_FIELD_LIST_01:
		case IDC_REG_FIELD_LIST_02:
		case IDC_REG_FIELD_LIST_03:
		case IDC_REG_FIELD_LIST_04:
		case IDC_REG_FIELD_LIST_05:
		case IDC_REG_FIELD_LIST_06:
		case IDC_REG_FIELD_LIST_07:
		case IDC_REG_FIELD_LIST_08:
		case IDC_REG_FIELD_LIST_09:
		case IDC_REG_FIELD_LIST_10:
		case IDC_REG_FIELD_LIST_11:
		case IDC_REG_FIELD_LIST_12:
		case IDC_REG_FIELD_LIST_13:
		case IDC_REG_FIELD_LIST_14:
		case IDC_REG_FIELD_LIST_15:
		case IDC_REG_FIELD_LIST_16:
		case IDC_REG_FIELD_LIST_17:
		case IDC_REG_FIELD_LIST_18:
		case IDC_REG_FIELD_LIST_19:
		case IDC_REG_FIELD_LIST_20:
		case IDC_REG_FIELD_LIST_21:
		case IDC_REG_FIELD_LIST_22:
		case IDC_REG_FIELD_LIST_23:
		case IDC_REG_FIELD_LIST_24:
		case IDC_REG_FIELD_LIST_25:
		case IDC_REG_FIELD_LIST_26:
		case IDC_REG_FIELD_LIST_27:
		case IDC_REG_FIELD_LIST_28:
		case IDC_REG_FIELD_LIST_29:
		case IDC_REG_FIELD_LIST_30:
		case IDC_REG_FIELD_LIST_31:

			_idc_reg_field_list(dlg, data, wParam, id - IDC_REG_FIELD_LIST_00);
			break;

		case IDC_REG_FIELD_NAME_00:
		case IDC_REG_FIELD_NAME_01:
		case IDC_REG_FIELD_NAME_02:
		case IDC_REG_FIELD_NAME_03:
		case IDC_REG_FIELD_NAME_04:
		case IDC_REG_FIELD_NAME_05:
		case IDC_REG_FIELD_NAME_06:
		case IDC_REG_FIELD_NAME_07:
		case IDC_REG_FIELD_NAME_08:
		case IDC_REG_FIELD_NAME_09:
		case IDC_REG_FIELD_NAME_10:
		case IDC_REG_FIELD_NAME_11:
		case IDC_REG_FIELD_NAME_12:
		case IDC_REG_FIELD_NAME_13:
		case IDC_REG_FIELD_NAME_14:
		case IDC_REG_FIELD_NAME_15:
		case IDC_REG_FIELD_NAME_16:
		case IDC_REG_FIELD_NAME_17:
		case IDC_REG_FIELD_NAME_18:
		case IDC_REG_FIELD_NAME_19:
		case IDC_REG_FIELD_NAME_20:
		case IDC_REG_FIELD_NAME_21:
		case IDC_REG_FIELD_NAME_22:
		case IDC_REG_FIELD_NAME_23:
		case IDC_REG_FIELD_NAME_24:
		case IDC_REG_FIELD_NAME_25:
		case IDC_REG_FIELD_NAME_26:
		case IDC_REG_FIELD_NAME_27:
		case IDC_REG_FIELD_NAME_28:
		case IDC_REG_FIELD_NAME_29:
		case IDC_REG_FIELD_NAME_30:
		case IDC_REG_FIELD_NAME_31:

			_idc_reg_field_name(dlg, data, wParam, id - IDC_REG_FIELD_NAME_00);
			break;

		case IDC_REG_DESCRIPTION:

			_idc_reg_description(dlg, data, wParam);
			break;

		case IDC_REG_NAME:

			_idc_reg_name(dlg, data, wParam);
			break;

		case IDC_REG_VALUE_EDIT:

			_idc_reg_value_edit(dlg, data, wParam);
			break;
	}

	return(ret);
}



/*****************************************************************************
*
*	Function:	_wm_destroy
*
*	Purpose:
*
*		Process WM_DESTROY messages.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*	Returned:
*
*		0			We processed the message.
*
*****************************************************************************/

static LRESULT _wm_destroy(HWND dlg)
{
	_idcancel(dlg);
	DestroyWindow(dlg);
	return(0);
}



/*****************************************************************************
*
*	Function:	_wm_initdialog
*
*	Purpose:
*
*		Process a WM_INITDIALOG message for the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		lParam		The message's long sized argument.
*
*	Returned:
*
*		TRUE		Windows should set the initial control focus.
*		FALSE		Windows should NOT set the initial control focus.
*
*****************************************************************************/

static LRESULT _wm_initdialog(HWND dlg, LPARAM lParam)
{
	int		bottom;
	reg_t*	data	= (reg_t*) lParam;
	char	section[128];

	SET_WINDOW_USER_DATA(dlg, lParam);

	SendMessage(dlg, WM_SETICON, ICON_SMALL, (LPARAM) data->device->icon.little);
	SendMessage(dlg, WM_SETICON, ICON_BIG, (LPARAM) data->device->icon.big);

	if (data->def->init)
		(data->def->init)(data->device, data->def);

	strcpy(section, SECTION_NAME);
	strcat(section, data->def->name);
	win_pos_restore(dlg, section, FALSE);

	data->changes	= FALSE;
	data->field_sel	= -1;
	_init_reg_controls(dlg, data);
	_init_bit_controls(dlg, data);
	_init_field_controls(dlg, data, &bottom);
	_align_complete(dlg, data, bottom);

	_refresh_window(dlg, data->device, NULL);
	main_refresh_add(dlg, _refresh_window);
	return(TRUE);
}



/*****************************************************************************
*
*	Function:	_wm_syscommand
*
*	Purpose:
*
*		Process a WM_SYSCOMMAND message for the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		wParam		The message's word sized parameter.
*
*	Returned:
*
*		TRUE		We processed the message.
*		FALSE		We did not process the message.
*
*****************************************************************************/

static LRESULT _wm_syscommand(HWND dlg, WPARAM wParam)
{
	LRESULT	ret;

	switch (wParam)
	{
		default:

			ret	= FALSE;
			break;

		case SC_CLOSE:

			_idcancel(dlg);
			ret	= TRUE;
			break;
	}

	return(ret);
}



/*****************************************************************************
*
*	Function:	_dlg_proc
*
*	Purpose:
*
*		Processes messages for the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		msg			The message to process.
*
*		wParam		The message's word sized parameter.
*
*		lParam		The message's long sized parameter.
*
*	Returned:
*
*		A message specific value.
*
*****************************************************************************/

static INT_PTR CALLBACK _dlg_proc(	HWND	dlg,
									UINT	msg,
									WPARAM	wParam,
									LPARAM	lParam)
{
	LRESULT	ret;

	switch (msg)
	{
		default:

			ret	= FALSE;
			break;

		case WM_COMMAND:

			ret	= _wm_command(dlg, wParam);
			break;

		case WM_DESTROY:

			ret	= _wm_destroy(dlg);
			break;

		case WM_INITDIALOG:

			ret	= _wm_initdialog(dlg, lParam);
			break;

		case WM_SYSCOMMAND:

			ret	= _wm_syscommand(dlg, wParam);
			break;
	}

	return(ret);
}



/*****************************************************************************
*
*	Function:	_service_wm_command
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_WM_COMMAND service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_wm_command(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	reg_t*		data	= (reg_t*) lParam;

	if (data->dlg)
	{
		SetFocus(data->dlg);
	}
	else
	{
		switch (GSC_REG_SIZE(data->def->reg_id))
		{
			default:
			case 1:	data->bits	= 8;	break;
			case 2:	data->bits	= 16;	break;
			case 4:	data->bits	= 32;	break;
		}

		data->device		= device;
		data->last_handle	= NULL;
		data->dlg			= CreateDialogParam(device->instance,
												(LPCTSTR) IDD_REG,
												dlg,
												(DLGPROC) _dlg_proc,
												(LPARAM) data);
		ShowWindow(data->dlg, SW_SHOWNORMAL);
	}

	return(0);
}



/*****************************************************************************
*
*	Function:	reg_service
*
*	Purpose:
*
*		Implement the common service entry point for the individual Register
*		access services.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		service	The service to perform.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0		No service was called.
*		else	The value from the called service, which may be zero.
*
*****************************************************************************/

LRESULT reg_service(
	HWND			dlg,
	device_t*		device,
	service_type_t	service,
	WPARAM			wParam,
	LPARAM			lParam)
{
	static const service_call_t	list[]	=
	{
		/* service				function			*/
		{ SERVICE_WM_COMMAND,	_service_wm_command	},
		{ SERVICE_END_LIST,		NULL				}
	};

	LRESULT	ret;

	ret	= service_call(dlg, device, list, service, wParam, lParam);
	return(ret);
}


