// HPDI32 - Application - Win32 - Any Compiler
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/SDK/hpdi32/visualhpdi32/win32/regs.h $
// $Rev: 31050 $
// $Date: 2014-10-27 17:51:19 -0500 (Mon, 27 Oct 2014) $

/*****************************************************************************
*
*	Export items for the register access services.
*
*****************************************************************************/

#ifndef __REGS_H__
#define	__REGS_H__

#include "service.h"	/*lint !e537 */



/* prototypes	*************************************************************/

typedef struct
{
	const char*	name;
	U32			value;
} reg_value_t;		// This is used to describe individual field values.

typedef struct
{
	const char*			name;
	const char*			desc;
	U32					mask;
	const reg_value_t*	value;	// List of named values.
	size_t				values;	// Number of named values.
} reg_field_t;		// This is used to describe individual register fields.

typedef struct	_reg_def_t	reg_def_t;
typedef	struct	_reg_t		reg_t;

struct _reg_def_t
{
	const char*			name;
	const char*			desc;
	U32					reg_id;
	U32					read_only;
	const reg_field_t*	field;		// List of fields.
	size_t				fields;		// Number of fields.
	void				(*init)(device_t* device, reg_def_t* def);
	void				(*refresh)(reg_t* data);
};

struct _reg_t
{
	int				bits;			// Register width in bits.
	int				changes;		// Are changes pending?
	reg_def_t*		def;
	device_t*		device;
	HWND			dlg;
	int				field_sel;		// Highlight this field.
	int				ignore_nm;		// Ignore notification messages?
	void*			last_handle;
	U32				status;
	char			status_global[256];
	char			status_local[256];
	U32				value;
	int				value_changing;	// Is the user making changes?

	struct
	{
		int		changing;	// Is the user making changes?
		char	bits[80];
	} field[32];	// Info on field controls.

};



/* variables	*************************************************************/

extern	reg_def_t	reg_fw_bcr;
extern	reg_def_t	reg_fw_bsr;
extern	reg_def_t	reg_fw_frr;
extern	reg_def_t	reg_fw_fsr;
extern	reg_def_t	reg_fw_icr;
extern	reg_def_t	reg_fw_ielr;
extern	reg_def_t	reg_fw_ihlr;
extern	reg_def_t	reg_fw_isr;
extern	reg_def_t	reg_fw_rar;
extern	reg_def_t	reg_fw_rlcr;
extern	reg_def_t	reg_fw_rfsr;
extern	reg_def_t	reg_fw_rfwr;
extern	reg_def_t	reg_fw_rscr;
extern	reg_def_t	reg_fw_tar;
extern	reg_def_t	reg_fw_tcdr;
extern	reg_def_t	reg_fw_tfsr;
extern	reg_def_t	reg_fw_tfwr;
extern	reg_def_t	reg_fw_tlilcr;
extern	reg_def_t	reg_fw_tlvlcr;
extern	reg_def_t	reg_fw_tsvlcr;



/* prototypes	*************************************************************/

LRESULT	reg_service(		HWND			dlg,
							device_t*		device,
							service_type_t	service,
							WPARAM			wParam,
							LPARAM			lParam);

LRESULT	regs_fw_service(	HWND			dlg,
							device_t*		device,
							service_type_t	service,
							WPARAM			wParam,
							LPARAM			lParam);

LRESULT	regs_pci_service(	HWND			dlg,
							device_t*		device,
							service_type_t	service,
							WPARAM			wParam,
							LPARAM			lParam);

LRESULT	regs_plx_service(	HWND			dlg,
							device_t*		device,
							service_type_t	service,
							WPARAM			wParam,
							LPARAM			lParam);



#endif
