// HPDI32 - Application - Win32 - Any Compiler
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/SDK/hpdi32/visualhpdi32/win32/regs_fw_frr.c $
// $Rev: 2809 $
// $Date: 2009-07-29 21:26:35 -0500 (Wed, 29 Jul 2009) $

/*****************************************************************************
*
*	Provide data describing the Firmware Revision Register.
*
*****************************************************************************/

#include "regs.h"



/* prototypes	*************************************************************/

static	void	_init(device_t* device, reg_def_t* def);
static	void	_refresh(reg_t* data);



/* variables	*************************************************************/

static const reg_value_t	_bs_values[]	=
{
	{
	/* name		*/	"32-bit",
	/* value	*/	0
	},
	{
	/* name		*/	"64-bit",
	/* value	*/	HPDI32_FRR_SPEC_BUS_SIZE
	}
};

static const reg_value_t	_ff_values[]	=
{
	{
	/* name		*/	"PCI",
	/* value	*/	HPDI32_FRR_SPEC_FF_ENCODE(HPDI32_FRR_SPEC_FF_PCI)
	},
	{
	/* name		*/	"PMC",
	/* value	*/	HPDI32_FRR_SPEC_FF_ENCODE(HPDI32_FRR_SPEC_FF_PMC)
	},
	{
	/* name		*/	"cPCI",
	/* value	*/	HPDI32_FRR_SPEC_FF_ENCODE(HPDI32_FRR_SPEC_FF_CPCI)
	},
	{
	/* name		*/	"PC/104+",
	/* value	*/	HPDI32_FRR_SPEC_FF_ENCODE(HPDI32_FRR_SPEC_FF_PC104P)
	}
};

static const reg_value_t	_fsr_values[]	=
{
	{
	/* name		*/	"Absent",
	/* value	*/	0
	},
	{
	/* name		*/	"Present",
	/* value	*/	HPDI32_FRR_FSR
	}
};

static const reg_value_t	_fw64_values[]	=
{
	{
	/* name		*/	"32-bit firmware",
	/* value	*/	0
	},
	{
	/* name		*/	"64-bit firmware",
	/* value	*/	HPDI32_FRR_FW_64
	}
};

static const reg_value_t	_hr_values[]	=
{
	{
	/* name		*/	"(no revision)",
	/* value	*/	HPDI32_FRR_SPEC_HW_REV_ENCODE(0)
	},
	{
	/* name		*/	"Rev. A",
	/* value	*/	HPDI32_FRR_SPEC_HW_REV_ENCODE(1)
	},
	{
	/* name		*/	"Rev. B",
	/* value	*/	HPDI32_FRR_SPEC_HW_REV_ENCODE(2)
	},
	{
	/* name		*/	"Rev. C",
	/* value	*/	HPDI32_FRR_SPEC_HW_REV_ENCODE(3)
	},
	{
	/* name		*/	"Rev. D",
	/* value	*/	HPDI32_FRR_SPEC_HW_REV_ENCODE(4)
	},
	{
	/* name		*/	"Rev. E",
	/* value	*/	HPDI32_FRR_SPEC_HW_REV_ENCODE(5)
	}
};

static const reg_value_t	_new_values[]	=
{
	{
	/* name		*/	"Original",
	/* value	*/	0
	},
	{
	/* name		*/	"New Specification",
	/* value	*/	HPDI32_FRR_CONFORMANT
	}
};

static const reg_value_t	_pcb_original_values[]	=
{
	{
	/* name		*/	"(no revision)",
	/* value	*/	HPDI32_FRR_PCB_ENCODE(0)
	},
	{
	/* name		*/	"Rev. A",
	/* value	*/	HPDI32_FRR_PCB_ENCODE(1)
	},
	{
	/* name		*/	"Rev. B",
	/* value	*/	HPDI32_FRR_PCB_ENCODE(2)
	},
	{
	/* name		*/	"Rev. C",
	/* value	*/	HPDI32_FRR_PCB_ENCODE(3)
	},
	{
	/* name		*/	"Rev. D",
	/* value	*/	HPDI32_FRR_PCB_ENCODE(4)
	}
};

static const reg_value_t	_pmc_values[]	=
{
	{
	/* name		*/	"non-PMC Form Factor",
	/* value	*/	0
	},
	{
	/* name		*/	"PMC Form Factor",
	/* value	*/	HPDI32_FRR_PMC
	}
};

static reg_field_t	_fields_original_32[]	=
{
	{
	/* name		*/	"FSR",
	/* desc		*/	"Feature Set Register",
	/* mask		*/	HPDI32_FRR_FSR,
	/* value	*/	_fsr_values,
	/* values	*/	ARRAY_ELEMENTS(_fsr_values)
	},
	{
	/* name		*/	"NEW",
	/* desc		*/	"Conforms to the new specification.",
	/* mask		*/	HPDI32_FRR_CONFORMANT,
	/* value	*/	_new_values,
	/* values	*/	ARRAY_ELEMENTS(_new_values)
	},
	{
	/* name		*/	"PMC",
	/* desc		*/	"PMC Form Factor",
	/* mask		*/	HPDI32_FRR_PMC,
	/* value	*/	_pmc_values,
	/* values	*/	ARRAY_ELEMENTS(_pmc_values)
	},
	{
	/* name		*/	"SID",
	/* desc		*/	"SUB Id",
	/* mask		*/	HPDI32_FRR_SUB_ID,
	/* value	*/	NULL,
	/* values	*/	0
	},
	{
	/* name		*/	"PCBR",
	/* desc		*/	"PCB Revision",
	/* mask		*/	HPDI32_FRR_PCB,
	/* value	*/	_pcb_original_values,
	/* values	*/	ARRAY_ELEMENTS(_pcb_original_values)
	},
	{
	/* name		*/	"FR",
	/* desc		*/	"Firmware Revision",
	/* mask		*/	HPDI32_FRR_FIRMWARE,
	/* value	*/	NULL,
	/* values	*/	0
	}
} ;

static reg_field_t	_fields_original_64[]	=
{
	{
	/* name		*/	"FSR",
	/* desc		*/	"Feature Set Register",
	/* mask		*/	HPDI32_FRR_FSR,
	/* value	*/	_fsr_values,
	/* values	*/	ARRAY_ELEMENTS(_fsr_values)
	},
	{
	/* name		*/	"NEW",
	/* desc		*/	"Conforms to the new specification.",
	/* mask		*/	HPDI32_FRR_CONFORMANT,
	/* value	*/	_new_values,
	/* values	*/	ARRAY_ELEMENTS(_new_values)
	},
	{
	/* name		*/	"FW64",
	/* desc		*/	"64-bit firmware",
	/* mask		*/	HPDI32_FRR_FW_64,
	/* value	*/	_fw64_values,
	/* values	*/	ARRAY_ELEMENTS(_fw64_values)
	},
	{
	/* name		*/	"SID",
	/* desc		*/	"SUB Id",
	/* mask		*/	HPDI32_FRR_SUB_ID,
	/* value	*/	NULL,
	/* values	*/	0
	},
	{
	/* name		*/	"PCBR",
	/* desc		*/	"PCB Revision",
	/* mask		*/	HPDI32_FRR_PCB,
	/* value	*/	_pcb_original_values,
	/* values	*/	ARRAY_ELEMENTS(_pcb_original_values)
	},
	{
	/* name		*/	"FR",
	/* desc		*/	"Firmware Revision",
	/* mask		*/	HPDI32_FRR_FIRMWARE,
	/* value	*/	NULL,
	/* values	*/	0
	}
} ;

static reg_field_t	_fields_new[]	=
{
	{
	/* name		*/	"FSR",
	/* desc		*/	"Feature Set Register",
	/* mask		*/	HPDI32_FRR_FSR,
	/* value	*/	_fsr_values,
	/* values	*/	ARRAY_ELEMENTS(_fsr_values)
	},
	{
	/* name		*/	"NEW",
	/* desc		*/	"Conforms to the new specification.",
	/* mask		*/	HPDI32_FRR_CONFORMANT,
	/* value	*/	_new_values,
	/* values	*/	ARRAY_ELEMENTS(_new_values)
	},
	{
	/* name		*/	"RES",
	/* desc		*/	"Reserved",
	/* mask		*/	HPDI32_FRR_SPEC_UNUSED,
	/* value	*/	NULL,
	/* values	*/	0
	},
	{
	/* name		*/	"BS",
	/* desc		*/	"Bus Size",
	/* mask		*/	HPDI32_FRR_SPEC_BUS_SIZE,
	/* value	*/	_bs_values,
	/* values	*/	ARRAY_ELEMENTS(_bs_values)
	},
	{
	/* name		*/	"FF",
	/* desc		*/	"Form Factor",
	/* mask		*/	HPDI32_FRR_SPEC_FORM_FACTOR,
	/* value	*/	_ff_values,
	/* values	*/	ARRAY_ELEMENTS(_ff_values)
	},
	{
	/* name		*/	"HR",
	/* desc		*/	"Hardware Revision",
	/* mask		*/	HPDI32_FRR_SPEC_HW_REV,
	/* value	*/	_hr_values,
	/* values	*/	ARRAY_ELEMENTS(_hr_values)
	},
	{
	/* name		*/	"BV",
	/* desc		*/	"Board Version",
	/* mask		*/	HPDI32_FRR_SPEC_BOARD_VER,
	/* value	*/	NULL,
	/* values	*/	0
	},
	{
	/* name		*/	"FR",
	/* desc		*/	"Firmware Revision",
	/* mask		*/	HPDI32_FRR_SPEC_FIRMWARE,
	/* value	*/	NULL,
	/* values	*/	0
	}
} ;

reg_def_t	reg_fw_frr	=
{
	/* name			*/	"FRR",
	/* desc			*/	"Firmware Revision Register",
	/* reg_id		*/	HPDI32_FRR,
	/* read_only	*/	0xFFFFFFFF,
	/* field		*/	NULL,
	/* fields		*/	0,
	/* init			*/	_init,
	/* refresh		*/	_refresh
};



/*****************************************************************************
*
*	Function:	_init
*
*	Purpose:
*
*		Initialize the register structure in preparation for use.
*
*	Arguments:
*
*		device		The device to access.
*
*		ref			The register definition to access.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _init(device_t* device, reg_def_t* def)
{
	U32	bw;
	U32	frr;

	HPDI32_MISC_PCI_BUS_WIDTH__GET(device->handle, &bw);
	hpdi32_reg_read(device->handle, HPDI32_FRR, &frr);

	if (frr & HPDI32_FRR_CONFORMANT)
	{
		def->desc	= "Firmware Revision Register (updated specification)";
		def->field	= _fields_new;
		def->fields	= ARRAY_ELEMENTS(_fields_new);
	}
	else if (bw == HPDI32_MISC_PCI_BUS_WIDTH_32)
	{
		def->desc	= "Firmware Revision Register (original format)";
		def->field	= _fields_original_32;
		def->fields	= ARRAY_ELEMENTS(_fields_original_32);
	}
	else
	{
		def->desc	= "Firmware Revision Register (original format)";
		def->field	= _fields_original_64;
		def->fields	= ARRAY_ELEMENTS(_fields_original_64);
	}
}



/*****************************************************************************
*
*	Function:	_refresh
*
*	Purpose:
*
*		Respond to a request to refresh the register window.
*
*	Arguments:
*
*		data		The dialog specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _refresh(reg_t* data)
{
	static	int	busy	= FALSE;

	HWND	parent;

	if (!busy)
	{
		busy	= TRUE;

		if (data->last_handle != data->device->handle)
		{
			parent	= GetParent(data->dlg);
			SendDlgItemMessage(data->dlg, IDCANCEL, WM_COMMAND, 0, 0);
			reg_service(parent,
						data->device,
						SERVICE_WM_COMMAND,
						0,
						(ULONG) data);
			data->last_handle	= data->device->handle;
		}

		busy	= FALSE;
	}
}


