// HPDI32 - Application - Win32 - Any Compiler
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/SDK/hpdi32/visualhpdi32/win32/regs_plx.c $
// $Rev: 2902 $
// $Date: 2009-08-09 17:48:49 -0500 (Sun, 09 Aug 2009) $

/*****************************************************************************
*
*	Implement the PLX Registers dialog.
*
*****************************************************************************/

#include <windows.h>
#include <stdio.h>

#include "regs.h"
#include "align.h"
#include "copy.h"
#include "gui.h"
#include "resource.h"
#include "util_ctrl.h"



/* #defines	*****************************************************************/

#define	SECTION_NAME		"registers: plx"



/* typedefs	*****************************************************************/

typedef struct
{
	device_t*	device;
	HWND		dlg;
	void*		last_handle;
	char		status_local[256];
} plx_t;



/* prototypes	*************************************************************/

static	LRESULT	_wm_size(HWND dlg);



/* variables	*************************************************************/

static struct
{
	U32			bus_size;
	U32			reg_id;
	const char*	abbrev;		// The abbreviated name.
	const char*	name;
	U32			value;
} _plx_regs[]	=
{
	{
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_64,
		/* reg_id	*/	GSC_PLX_9080_LAS0RR,
		/* abbrev	*/	"LAS0RR",
		/* name		*/	"Local Address Space 0 Range Register for PCI-to-Local Bus",
		/* value	*/	0
	},
	{
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_32,
		/* reg_id	*/	GSC_PLX_9080_LAS0BA,
		/* abbrev	*/	"LAS0BA",
		/* name		*/	"Local Address Space 0 Local Base Address (Remap) Register",
		/* value	*/	0
	},
	{
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_32,
		/* reg_id	*/	GSC_PLX_9080_MARBR,
		/* abbrev	*/	"MARBR",
		/* name		*/	"Mode/Arbitration Register",
		/* value	*/	0
	},
	{
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_32,
		/* reg_id	*/	GSC_PLX_9080_BIGEND,
		/* abbrev	*/	"BIGEND",
		/* name		*/	"Big/Little Endian Descriptor Register",
		/* value	*/	0
	},
	{
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_32,
		/* reg_id	*/	GSC_PLX_9080_EROMRR,
		/* abbrev	*/	"EROMRR",
		/* name		*/	"Expansion ROM Range Register",
		/* value	*/	0
	},
	{
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_32,
		/* reg_id	*/	GSC_PLX_9080_EROMBA,
		/* abbrev	*/	"EROMBA",
		/* name		*/	"Expansion ROM Local Base Address (Remap) Register and BREQ0 Control",
		/* value	*/	0
	},
	{
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_32,
		/* reg_id	*/	GSC_PLX_9080_LBRD0,
		/* abbrev	*/	"LBRD0",
		/* name		*/	"Local Address Space 0/Expansion ROM Bus Region Descriptor Register",
		/* value	*/	0
	},
	{
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_32,
		/* reg_id	*/	GSC_PLX_9080_DMRR,
		/* abbrev	*/	"DMRR",
		/* name		*/	"Local Range for Direct Master to PCI",
		/* value	*/	0
	},
	{
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_32,
		/* reg_id	*/	GSC_PLX_9080_DMLBAM,
		/* abbrev	*/	"DMLBAM",
		/* name		*/	"Local Bus Base Address Register for Direct Master to PCI Memory",
		/* value	*/	0
	},
	{
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_32,
		/* reg_id	*/	GSC_PLX_9080_DMLBAI,
		/* abbrev	*/	"DMLBAI",
		/* name		*/	"Local Base Address for Direct Master to PCI IO/CFG",
		/* value	*/	0
	},
	{
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_32,
		/* reg_id	*/	GSC_PLX_9080_DMPBAM,
		/* abbrev	*/	"DMPBAM",
		/* name		*/	"PCI Base Address (Remap) Register for Direct Master to PCI Memory",
		/* value	*/	0
	},
	{
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_32,
		/* reg_id	*/	GSC_PLX_9080_DMCFGA,
		/* abbrev	*/	"DMCFGA",
		/* name		*/	"PCI Configuration Address Register for Direct Master to PCI IO/CFG",
		/* value	*/	0
	},
	{
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_32,
		/* reg_id	*/	GSC_PLX_9080_LAS1RR,
		/* abbrev	*/	"LAS1RR",
		/* name		*/	"Local Address Space 1 Range Register for PCI-to-Local Bus",
		/* value	*/	0
	},
	{
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_32,
		/* reg_id	*/	GSC_PLX_9080_LAS1BA,
		/* abbrev	*/	"LAS1BA",
		/* name		*/	"Local Address Space 1 Local Base Address (Remap) Register",
		/* value	*/	0
	},
	{
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_32,
		/* reg_id	*/	GSC_PLX_9080_LBRD1,
		/* abbrev	*/	"LBRD1",
		/* name		*/	"Local Address Space 1 Bus Region Descriptor Register",
		/* value	*/	0
	},
	{
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_64,
		/* reg_id	*/	GSC_PLX_9656_LMISC1,
		/* abbrev	*/	"LMISC1",
		/* name		*/	"Local Miscellaneous Control Register 1",
		/* value	*/	0
	},
	{
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_64,
		/* reg_id	*/	GSC_PLX_9656_PROT_AREA,
		/* abbrev	*/	"PROT_AREA",
		/* name		*/	"Serial EEPROM Write-Protected Address Boundary Register",
		/* value	*/	0
	},
	{
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_64,
		/* reg_id	*/	GSC_PLX_9656_LMISC2,
		/* abbrev	*/	"LMISC2",
		/* name		*/	"Local Miscellaneous Control Register 2",
		/* value	*/	0
	},
	{
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_64,
		/* reg_id	*/	GSC_PLX_9656_DMDAC,
		/* abbrev	*/	"DMDAC",
		/* name		*/	"Direct Master PCI Dual Address Cycle Upper Address Register",
		/* value	*/	0
	},
	{
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_64,
		/* reg_id	*/	GSC_PLX_9656_ARB,
		/* abbrev	*/	"ARB",
		/* name		*/	"PCI Arbiter Control Register",
		/* value	*/	0
	},
	{
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_64,
		/* reg_id	*/	GSC_PLX_9656_ABTADR,
		/* abbrev	*/	"ABTADR",
		/* name		*/	"PCI Abort Address Register",
		/* value	*/	0
	},
	{
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_32,
		/* reg_id	*/	GSC_PLX_9080_MBOX0,
		/* abbrev	*/	"MBOX0",
		/* name		*/	"Mailbox Register 0",
		/* value	*/	0
	},
	{
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_32,
		/* reg_id	*/	GSC_PLX_9080_MBOX1,
		/* abbrev	*/	"MBOX1",
		/* name		*/	"Mailbox Register 1",
		/* value	*/	0
	},
	{
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_32,
		/* reg_id	*/	GSC_PLX_9080_MBOX2,
		/* abbrev	*/	"MBOX2",
		/* name		*/	"Mailbox Register 2",
		/* value	*/	0
	},
	{
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_32,
		/* reg_id	*/	GSC_PLX_9080_MBOX3,
		/* abbrev	*/	"MBOX3",
		/* name		*/	"Mailbox Register 3",
		/* value	*/	0
	},
	{
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_32,
		/* reg_id	*/	GSC_PLX_9080_MBOX4,
		/* abbrev	*/	"MBOX4",
		/* name		*/	"Mailbox Register 4",
		/* value	*/	0
	},
	{
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_32,
		/* reg_id	*/	GSC_PLX_9080_MBOX5,
		/* abbrev	*/	"MBOX5",
		/* name		*/	"Mailbox Register 5",
		/* value	*/	0
	},
	{
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_32,
		/* reg_id	*/	GSC_PLX_9080_MBOX6,
		/* abbrev	*/	"MBOX6",
		/* name		*/	"Mailbox Register 6",
		/* value	*/	0
	},
	{
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_32,
		/* reg_id	*/	GSC_PLX_9080_MBOX7,
		/* abbrev	*/	"MBOX7",
		/* name		*/	"Mailbox Register 7",
		/* value	*/	0
	},
	{
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_32,
		/* reg_id	*/	GSC_PLX_9080_P2LDBELL,
		/* abbrev	*/	"P2LDBELL",
		/* name		*/	"PCI-to-Local Doorbell Register",
		/* value	*/	0
	},
	{
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_32,
		/* reg_id	*/	GSC_PLX_9080_L2PDBELL,
		/* abbrev	*/	"L2PDBELL",
		/* name		*/	"Local-to-PCI Doorbell Register",
		/* value	*/	0
	},
	{
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_32,
		/* reg_id	*/	GSC_PLX_9080_INTCSR,
		/* abbrev	*/	"INTCSR",
		/* name		*/	"Interrupt Control/Status Register",
		/* value	*/	0
	},
	{
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_32,
		/* reg_id	*/	GSC_PLX_9080_CNTRL,
		/* abbrev	*/	"CNTRL",
		/* name		*/	"Serial EEPROM Control, PCI Command Codes, User I/O Control, Init Control Register",
		/* value	*/	0
	},
	{
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_32,
		/* reg_id	*/	GSC_PLX_9080_PCIHIDR,
		/* abbrev	*/	"PCIHIDR",
		/* name		*/	"PCI Permanent Configuration ID Register",
		/* value	*/	0
	},
	{
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_32,
		/* reg_id	*/	GSC_PLX_9080_PCIHREV,
		/* abbrev	*/	"PCIHREV",
		/* name		*/	"PCI Permanent Revision ID Register",
		/* value	*/	0
	},
	{
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_32,
		/* reg_id	*/	GSC_PLX_9080_DMAMODE0,
		/* abbrev	*/	"DMAMODE0",
		/* name		*/	"DMA Channel 0 Mode Register",
		/* value	*/	0
	},
	{
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_32,
		/* reg_id	*/	GSC_PLX_9080_DMAPADR0,
		/* abbrev	*/	"DMAPADR0",
		/* name		*/	"DMA Channel 0 PCI Address Register",
		/* value	*/	0
	},
	{
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_32,
		/* reg_id	*/	GSC_PLX_9080_DMALADR0,
		/* abbrev	*/	"DMALADR0",
		/* name		*/	"DMA Channel 0 Local Address Register",
		/* value	*/	0
	},
	{
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_32,
		/* reg_id	*/	GSC_PLX_9080_DMASIZ0,
		/* abbrev	*/	"DMASIZ0",
		/* name		*/	"DMA Channel 0 Transfer Size (Bytes) Register",
		/* value	*/	0
	},
	{
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_32,
		/* reg_id	*/	GSC_PLX_9080_DMADPR0,
		/* abbrev	*/	"DMADPR0",
		/* name		*/	"DMA Channel 0 Descriptor Pointer Register",
		/* value	*/	0
	},
	{
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_32,
		/* reg_id	*/	GSC_PLX_9080_DMACSR0,
		/* abbrev	*/	"DMACSR0",
		/* name		*/	"DMA Channel 0 Command/Status Register",
		/* value	*/	0
	},
	{
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_64,
		/* reg_id	*/	GSC_PLX_9656_DMADAC0,
		/* abbrev	*/	"DMADAC0",
		/* name		*/	"DMA Channel 0 PCI Dual Address Cycle Upper Address Register",
		/* value	*/	0
	},
	{
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_32,
		/* reg_id	*/	GSC_PLX_9080_DMAMODE1,
		/* abbrev	*/	"DMAMODE1",
		/* name		*/	"DMA Channel 1 Mode Register",
		/* value	*/	0
	},
	{
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_32,
		/* reg_id	*/	GSC_PLX_9080_DMAPADR1,
		/* abbrev	*/	"DMAPADR1",
		/* name		*/	"DMA Channel 1 PCI Address Register",
		/* value	*/	0
	},
	{
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_32,
		/* reg_id	*/	GSC_PLX_9080_DMALADR1,
		/* abbrev	*/	"DMALADR1",
		/* name		*/	"DMA Channel 1 Local Address Register",
		/* value	*/	0
	},
	{
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_32,
		/* reg_id	*/	GSC_PLX_9080_DMASIZ1,
		/* abbrev	*/	"DMASIZ1",
		/* name		*/	"DMA Channel 1 Transfer Size (Bytes) Register",
		/* value	*/	0
	},
	{
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_32,
		/* reg_id	*/	GSC_PLX_9080_DMADPR1,
		/* abbrev	*/	"DMADPR1",
		/* name		*/	"DMA Channel 1 Descriptor Pointer Register",
		/* value	*/	0
	},
	{
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_32,
		/* reg_id	*/	GSC_PLX_9080_DMACSR1,
		/* abbrev	*/	"DMACSR1",
		/* name		*/	"DMA Channel 1 Command/Status Register",
		/* value	*/	0
	},
	{
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_64,
		/* reg_id	*/	GSC_PLX_9656_DMADAC1,
		/* abbrev	*/	"DMADAC1",
		/* name		*/	"DMA Channel 1 PCI Dual Address Cycle Upper Address Register",
		/* value	*/	0
	},
	{
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_32,
		/* reg_id	*/	GSC_PLX_9080_DMAARB,
		/* abbrev	*/	"DMAARB",
		/* name		*/	"DMA Arbitration Register (same as MARBR)",
		/* value	*/	0
	},
	{
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_32,
		/* reg_id	*/	GSC_PLX_9080_DMATHR,
		/* abbrev	*/	"DMATHR",
		/* name		*/	"DMA Threshold Register",
		/* value	*/	0
	},
	{
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_32,
		/* reg_id	*/	GSC_PLX_9080_OPLFIS,
		/* abbrev	*/	"OPLFIS",
		/* name		*/	"Outbound Post List FIFO Interrupt Status Register",
		/* value	*/	0
	},
	{
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_32,
		/* reg_id	*/	GSC_PLX_9080_OPLFIM,
		/* abbrev	*/	"OPLFIM",
		/* name		*/	"Outbound Post List FIFO Interrupt Mask Register",
		/* value	*/	0
	},
	{
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_32,
		/* reg_id	*/	GSC_PLX_9080_IQP,
		/* abbrev	*/	"IQP",
		/* name		*/	"Inbound Queue Port Register",
		/* value	*/	0
	},
	{
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_32,
		/* reg_id	*/	GSC_PLX_9080_OQP,
		/* abbrev	*/	"OQP",
		/* name		*/	"Outbound Queue Port Register",
		/* value	*/	0
	},
	{
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_32,
		/* reg_id	*/	GSC_PLX_9080_MQCR,
		/* abbrev	*/	"MQCR",
		/* name		*/	"Messaging Queue Configuration Register",
		/* value	*/	0
	},
	{
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_32,
		/* reg_id	*/	GSC_PLX_9080_QBAR,
		/* abbrev	*/	"QBAR",
		/* name		*/	"Queue Base Address Register",
		/* value	*/	0
	},
	{
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_32,
		/* reg_id	*/	GSC_PLX_9080_IFHPR,
		/* abbrev	*/	"IFHPR",
		/* name		*/	"Inbound Free Head Pointer Register",
		/* value	*/	0
	},
	{
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_32,
		/* reg_id	*/	GSC_PLX_9080_IFTPR,
		/* abbrev	*/	"IFTPR",
		/* name		*/	"Inbound Free Tail Pointer Register",
		/* value	*/	0
	},
	{
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_32,
		/* reg_id	*/	GSC_PLX_9080_IPHPR,
		/* abbrev	*/	"IPHPR",
		/* name		*/	"Inbound Post Head Pointer Register",
		/* value	*/	0
	},
	{
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_32,
		/* reg_id	*/	GSC_PLX_9080_IPTPR,
		/* abbrev	*/	"IPTPR",
		/* name		*/	"Inbound Post Tail Pointer Register",
		/* value	*/	0
	},
	{
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_32,
		/* reg_id	*/	GSC_PLX_9080_OFHPR,
		/* abbrev	*/	"OFHPR",
		/* name		*/	"Outbound Free Head Pointer Register",
		/* value	*/	0
	},
	{
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_32,
		/* reg_id	*/	GSC_PLX_9080_OFTPR,
		/* abbrev	*/	"OFTPR",
		/* name		*/	"Outbound Free Tail Pointer Register",
		/* value	*/	0
	},
	{
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_32,
		/* reg_id	*/	GSC_PLX_9080_OPHPR,
		/* abbrev	*/	"OPHPR",
		/* name		*/	"Outbound Post Head Pointer Register",
		/* value	*/	0
	},
	{
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_32,
		/* reg_id	*/	GSC_PLX_9080_OPTPR,
		/* abbrev	*/	"OPTPR",
		/* name		*/	"Outbound Post tail Pointer Register",
		/* value	*/	0
	},
	{
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_32,
		/* reg_id	*/	GSC_PLX_9080_QSR,
		/* abbrev	*/	"QSR",
		/* name		*/	"Queue Status/Control Register",
		/* value	*/	0
	}
};



/*****************************************************************************
*
*	Function:	_get_reg_info
*
*	Purpose:
*
*		Retrieve the info for the given register index and put it in the
*		given buffer.
*
*	Arguments:
*
*		data		The window specific data we track.
*
*		index		The index of the desired register.
*
*		buf			Put the text here.
*
*		bw			The device bus width.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _get_reg_info(plx_t* data, int index, char* buf, U32 bw)
{
	int		access;
	int		offset	= GSC_REG_OFFSET(_plx_regs[index].reg_id);
	int		size	= GSC_REG_SIZE(_plx_regs[index].reg_id);

	sprintf(buf,
			"%s: offset 0x%lX, size %d-bits",
			_plx_regs[index].name,
			(long) offset,
			size * 8);

	if (_plx_regs[index].bus_size <= bw)
		access	= TRUE;
	else
		access	= FALSE;

	if ((data->device->handle) && (access == FALSE))
		strcat(buf, " (not available on 32-bit boards)");
}



/*****************************************************************************
*
*	Function:	_refresh_window
*
*	Purpose:
*
*		Update the content of the window per the state of the system.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		device		The device to access.
*
*		msg			A status message to display.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _refresh_window(HWND dlg, device_t* device, const char* msg)
{
	char		buf[256];
	U32			bw		= 0;
	int			bytes;
	HWND		control;
	LRESULT		count;
	plx_t*		data	= (plx_t*) GET_WINDOW_USER_DATA(dlg);
	HDC			dc;
	int			enable;
	int			i;
	int			j;
	char*		psz;
	WPARAM		sel;
	U32			stat;
	U32			status	= GSC_SUCCESS;
	TEXTMETRIC	tm;
	WPARAM		top;
	U32			u32;
	int			width	= 0;

	set_win_title(dlg, device, FALSE, TRUE, "PLX Registers");
	enable	= (device->handle) ? 1 : 0;
	control	= GetDlgItem(dlg, IDC_REGS_PLX_LIST);
	count	= SendMessage(control, LB_GETCOUNT, 0, 0);
	sel		= (WPARAM) SendMessage(control, LB_GETCURSEL, 0, 0);
	top		= (WPARAM) SendMessage(control, LB_GETTOPINDEX, 0, 0);
	SendMessage(control, WM_SETREDRAW, FALSE, 0);
	dc		= GetDC(control);
	GetTextMetrics(dc, &tm);
	ReleaseDC(control, dc);

	if (enable)
	{
		stat	= HPDI32_MISC_PCI_BUS_WIDTH__GET(device->handle, &bw);
		status	= (status == GSC_SUCCESS) ? stat : status;
		ctrl_menu_set(dlg, IDCANCEL, CTRL_ENABLE);
		ctrl_menu_set(dlg, IDC_REFRESH, CTRL_ENABLE);
	}
	else
	{
		SendMessage(control, LB_RESETCONTENT, FALSE, 0);
		ctrl_menu_set(dlg, IDCANCEL, CTRL_ENABLE | CTRL_FOCUS);
		ctrl_menu_set(dlg, IDC_REFRESH, CTRL_DISABLE);
	}

	for (i = 0; i < (int) ARRAY_ELEMENTS(_plx_regs); i++)
	{
		strcpy(buf, _plx_regs[i].abbrev);
		strcat(buf, "                    ");
		buf[11]	= 0;
		psz		= buf + strlen(buf);

		if (enable == FALSE)
		{
			psz[0]	= 0;
		}
		else if (_plx_regs[i].bus_size <= bw)
		{
			stat	= hpdi32_reg_read(	device->handle,
										_plx_regs[i].reg_id,
										&u32);
			status	= (status == GSC_SUCCESS) ? stat : status;
			_plx_regs[i].value	= u32;
			bytes	= GSC_REG_SIZE(_plx_regs[i].reg_id);

			if (bytes == 4)
				sprintf(psz, "%08lX", (long) u32);
			else if (bytes == 2)
				sprintf(psz, "%04lX", (long) u32);
			else
				sprintf(psz, "%02lX", (long) u32);
		}
		else
		{
			strcpy(psz, "N/A");			/*lint !e669 */
		}

		strcat(psz, "                    ");	/*lint !e669 */
		psz[10]	= 0;							/*lint !e661 !e662 */
		psz		+= strlen(psz);

		_get_reg_info(data, i, psz, bw);

		if (count == 0)
		{
			SendMessage(control, LB_ADDSTRING, 0, (LPARAM) buf);
		}
		else
		{
			SendMessage(control, LB_DELETESTRING, (WPARAM) i, 0);
			SendMessage(control, LB_INSERTSTRING, (WPARAM) i, (LPARAM) buf);
		}

		j	= (int) strlen(buf);

		if (width < j)
			width	= j;
	}

	width	= (width + 2) * tm.tmAveCharWidth;
	SendMessage(control, LB_SETHORIZONTALEXTENT, (WPARAM) width + 20, 0);
	SendMessage(control, LB_SETCURSEL, sel, 0);
	SendMessage(control, LB_SETTOPINDEX, top, 0);
	SendMessage(control, WM_SETREDRAW, TRUE, 0);

	if (status != GSC_SUCCESS)
		error_status(dlg, IDC_REGS_PLX_STATUS, status);
	else if ((msg) && (msg[0]))
		ctrl_menu_set_text(dlg, IDC_REGS_PLX_STATUS, msg);
	else
		ctrl_menu_set_text(dlg, IDC_REGS_PLX_STATUS, data->status_local);
}



/*****************************************************************************
*
*	Function:	_idc_copy
*
*	Purpose:
*
*		Implement a COPY function for the dialog content.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idc_copy(HWND dlg)
{
	static const copy_t	list[]	=
	{
		/* id					flags						*/
		{ -1,					COPY_EOL | COPY_UNDERLINE	},

		{ IDC_REGS_PLX_LIST,	COPY_LIST_BOX				},

		{ 0,					COPY_UNDERLINE				},
		{ 0,					COPY_EOL					},

		{ IDC_REFRESH,			COPY_EOL					},
		{ IDCANCEL,				COPY_EOL					},

		{ 0,					COPY_UNDERLINE				},
		{ 0,					COPY_EOL					},

		{ IDC_REGS_PLX_STATUS,	COPY_EOL					}
	};

	copy_list_to_clipboard(dlg, list, ARRAY_ELEMENTS(list));
}



/*****************************************************************************
*
*	Function:	_idc_regs_plx_list
*
*	Purpose:
*
*		Process IDC_REGS_PLX_LIST list box control input.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		wParam		The message's word sized parameter.
*
*		data		The window specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idc_regs_plx_list(HWND dlg, WPARAM wParam, plx_t* data)
{
	char	buf[1024];
	U32		bw		= 0;
	HWND	control;
	int		i;
	int		notify	= HIWORD(wParam);

	if (notify == LBN_SELCHANGE)
	{
		if (data->device->handle)
			HPDI32_MISC_PCI_BUS_WIDTH__GET(data->device->handle, &bw);

		control	= GetDlgItem(dlg, IDC_REGS_PLX_LIST);
		i		= (int) SendMessage(control, LB_GETCURSEL, 0, 0);
		_get_reg_info(data, i, buf, bw);
		strcpy(data->status_local, buf);
		ctrl_menu_set_text(dlg,  IDC_REGS_PLX_STATUS, buf);
	}
}



/*****************************************************************************
*
*	Function:	_idcancel
*
*	Purpose:
*
*		Respond to a request to close the dialog.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idcancel(HWND dlg)
{
	plx_t*	data;

	main_refresh_add(dlg, NULL);
	data	= (plx_t*) GET_WINDOW_USER_DATA(dlg);

	if (data)
		data->dlg	= NULL;

	win_pos_save(dlg, SECTION_NAME);
	DestroyWindow(dlg);
}



/*****************************************************************************
*
*	Function:	_wm_command
*
*	Purpose:
*
*		Process WM_COMMAND messages.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		wParam		The message's WORD parameter.
*
*	Returned:
*
*		TRUE		We processed the message.
*		FALSE		We did not process the message.
*
*****************************************************************************/

static LRESULT _wm_command(HWND dlg, WPARAM wParam)
{
	plx_t*	data	= (plx_t*) GET_WINDOW_USER_DATA(dlg);
	int		id		= LOWORD(wParam);
	LRESULT	ret		= TRUE;

	switch (id)
	{
		default:

			ret	= FALSE;
			break;

		case IDC_REFRESH:

			_refresh_window(dlg, data->device, NULL);
			break;

		case IDCANCEL:

			_idcancel(dlg);
			break;

		case IDC_COPY:

			_idc_copy(dlg);
			break;

		case IDC_REGS_PLX_LIST:

			_idc_regs_plx_list(dlg, wParam, data);
			break;
	}

	return(ret);
}



/*****************************************************************************
*
*	Function:	_wm_destroy
*
*	Purpose:
*
*		Process WM_DESTROY messages.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*	Returned:
*
*		0			We processed the message.
*
*****************************************************************************/

static LRESULT _wm_destroy(HWND dlg)
{
	_idcancel(dlg);
	DestroyWindow(dlg);
	return(0);
}



/*****************************************************************************
*
*	Function:	_wm_getminmaxinfo
*
*	Purpose:
*
*		Process a WM_GETMINMAXINFO message for the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		lParam		The messages's long size parameter.
*
*	Returned:
*
*		0			We processed the message.
*
*****************************************************************************/

static LRESULT _wm_getminmaxinfo(HWND dlg, LPARAM lParam)
{
	HWND		control;
	MINMAXINFO*	mmi		= (MINMAXINFO*) lParam;
	RECT		rect;

	// Width
	control	= GetDlgItem(dlg, IDC_REGS_PLX_LIST);
	GetWindowRect(control, &rect);
	mmi->ptMinTrackSize.x	= rect.left;

	GetWindowRect(dlg, &rect);
	mmi->ptMinTrackSize.x	-= rect.left;
	mmi->ptMinTrackSize.x	*= 2;

	control	= GetDlgItem(dlg, IDCANCEL);
	GetWindowRect(control, &rect);
	mmi->ptMinTrackSize.x	+= rect.right;

	control	= GetDlgItem(dlg, IDC_REFRESH);
	GetWindowRect(control, &rect);
	mmi->ptMinTrackSize.x	-= rect.left;

	// Height
	control	= GetDlgItem(dlg, IDC_REGS_PLX_STATUS);
	GetWindowRect(control, &rect);
	mmi->ptMinTrackSize.y	= rect.bottom;

	control	= GetDlgItem(dlg, IDC_REGS_PLX_LIST);
	GetWindowRect(control, &rect);
	mmi->ptMinTrackSize.y	-= rect.bottom;
	mmi->ptMinTrackSize.y	+= rect.top;

	GetWindowRect(dlg, &rect);
	mmi->ptMinTrackSize.y	-= rect.top;

	control	= GetDlgItem(dlg, IDCANCEL);
	GetWindowRect(control, &rect);
	mmi->ptMinTrackSize.y	+= 3 * (rect.bottom - rect.top);

	return(0);
}



/*****************************************************************************
*
*	Function:	_wm_initdialog
*
*	Purpose:
*
*		Process a WM_INITDIALOG message for the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		lParam		The message's long sized argument.
*
*	Returned:
*
*		TRUE		Windows should set the initial control focus.
*		FALSE		Windows should NOT set the initial control focus.
*
*****************************************************************************/

static LRESULT _wm_initdialog(HWND dlg, LPARAM lParam)
{
	HWND	control;
	plx_t*	data;

	SET_WINDOW_USER_DATA(dlg, lParam);
	data	= (plx_t*) GET_WINDOW_USER_DATA(dlg);
	data->status_local[0]	= 0;

	SendMessage(dlg, WM_SETICON, ICON_SMALL, (LPARAM) data->device->icon.little);
	SendMessage(dlg, WM_SETICON, ICON_BIG, (LPARAM) data->device->icon.big);

	control		= GetDlgItem(dlg, IDC_REGS_PLX_LIST);
	SendMessage(control, WM_SETFONT, (WPARAM) data->device->font.fixed, 0);

	_wm_size(dlg);
	win_pos_restore(dlg, SECTION_NAME, TRUE);
	main_refresh_add(dlg, _refresh_window);
	_refresh_window(dlg, data->device, NULL);
	return(TRUE);
}



/*****************************************************************************
*
*	Function:	_wm_size
*
*	Purpose:
*
*		Process a WM_SIZE message for the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*	Returned:
*
*		0			We processed the message.
*
*****************************************************************************/

static LRESULT _wm_size(HWND dlg)
{
	#define	MACRO(i,_i,_r,_f,_v)	ALIGN_T_(list[i],_i,_r,_f,_v);

	plx_t*	data	= (plx_t*) GET_WINDOW_USER_DATA(dlg);
	align_t	list[17];

	// The outter frame.
	MACRO(0, IDC_REGS_PLX_FRAME_1, 0, ALIGN_L, -1);
	MACRO(1, IDC_REGS_PLX_FRAME_1, 0, ALIGN_T, 0);
	MACRO(2, IDC_REGS_PLX_FRAME_1, 0, ALIGN_R_R | ALIGN_B_B, 1);

	// The status text.
	MACRO(3, IDC_REGS_PLX_STATUS, 0, ALIGN_L, data->device->margin);
	MACRO(4, IDC_REGS_PLX_STATUS, 0, ALIGN_R_R, - data->device->margin);
	MACRO(5, IDC_REGS_PLX_STATUS, 0, ALIGN_B_B | ALIGN_H, - MARGIN);

	// The status frame.
	MACRO(6, IDC_REGS_PLX_FRAME_2, IDC_REGS_PLX_FRAME_1, ALIGN_L_L | ALIGN_R_R | ALIGN_B_B, 0);
	MACRO(7, IDC_REGS_PLX_FRAME_2, IDC_REGS_PLX_STATUS, ALIGN_T_T, - MARGIN);

	// The REFRESH button.
	MACRO(8, IDC_REFRESH, 0, ALIGN_R_C | ALIGN_W, - data->device->margin / 2);
	MACRO(9, IDC_REFRESH, IDC_REGS_PLX_FRAME_2, ALIGN_B_T | ALIGN_H, - data->device->margin);

	// The CLOSE button.
	MACRO(10, IDCANCEL, IDC_REFRESH, ALIGN_COPY, 0);
	MACRO(11, IDCANCEL, IDC_REFRESH, ALIGN_L_R | ALIGN_W, data->device->margin);

	// The list box.
	MACRO(12, IDC_REGS_PLX_LIST, 0, ALIGN_L, data->device->margin);
	MACRO(13, IDC_REGS_PLX_LIST, 0, ALIGN_T, 1 + data->device->margin);
	MACRO(14, IDC_REGS_PLX_LIST, 0, ALIGN_R_R, - data->device->margin);
	MACRO(15, IDC_REGS_PLX_LIST, IDCANCEL, ALIGN_B_T, - data->device->margin);

	MACRO(16, 0, 0, 0, 0);

	align_control_list(dlg, list);
	InvalidateRect(dlg, NULL, TRUE);
	return(0);
}



/*****************************************************************************
*
*	Function:	_wm_syscommand
*
*	Purpose:
*
*		Process a WM_SYSCOMMAND message for the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		wParam		The message's word sized parameter.
*
*	Returned:
*
*		TRUE		We processed the message.
*		FALSE		We did not process the message.
*
*****************************************************************************/

static LRESULT _wm_syscommand(HWND dlg, WPARAM wParam)
{
	LRESULT	ret;

	switch (wParam)
	{
		default:

			ret	= FALSE;
			break;

		case SC_CLOSE:

			_idcancel(dlg);
			ret	= TRUE;
			break;
	}

	return(ret);
}



/*****************************************************************************
*
*	Function:	_dlg_proc
*
*	Purpose:
*
*		Processes messages for the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		msg			The message to process.
*
*		wParam		The message's word sized parameter.
*
*		lParam		The message's long sized parameter.
*
*	Returned:
*
*		A message specific value.
*
*****************************************************************************/

static INT_PTR CALLBACK _dlg_proc(	HWND	dlg,
									UINT	msg,
									WPARAM	wParam,
									LPARAM	lParam)
{
	LRESULT	ret;

	switch (msg)
	{
		default:

			ret	= FALSE;
			break;

		case WM_COMMAND:

			ret	= _wm_command(dlg, wParam);
			break;

		case WM_DESTROY:

			ret	= _wm_destroy(dlg);
			break;

		case WM_GETMINMAXINFO:

			ret	= _wm_getminmaxinfo(dlg, lParam);
			break;

		case WM_INITDIALOG:

			ret	= _wm_initdialog(dlg, lParam);
			break;

		case WM_SIZE:

			ret	= _wm_size(dlg);
			break;

		case WM_SYSCOMMAND:

			ret	= _wm_syscommand(dlg, wParam);
			break;
	}

	return(ret);
}



/*****************************************************************************
*
*	Function:	_service_refresh
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_REFRESH service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_refresh(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	if (device->handle)
		ctrl_menu_set(dlg, IDC_REGS_PLX, CTRL_ENABLE);
	else
		ctrl_menu_set(dlg, IDC_REGS_PLX, CTRL_DISABLE);

	return(0);
}



/*****************************************************************************
*
*	Function:	_service_wm_command
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_WM_COMMAND service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_wm_command(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	static	plx_t	data;

	if (data.dlg)		/*lint !e727 */
	{
		SetFocus(data.dlg);
	}
	else
	{
		data.device			= device;
		data.last_handle	= NULL;
		data.dlg			= CreateDialogParam(device->instance,
												(LPCTSTR) IDD_REGS_PLX,
												dlg,
												(DLGPROC) _dlg_proc,
												(LPARAM) &data);
		ShowWindow(data.dlg, SW_SHOWNORMAL);
	}

	return(0);
}



/*****************************************************************************
*
*	Function:	regs_plx_service
*
*	Purpose:
*
*		Implement the common service entry point for the PLX Register services.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		service	The service to perform.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0		No service was called.
*		else	The value from the called service, which may be zero.
*
*****************************************************************************/

LRESULT regs_plx_service(
	HWND			dlg,
	device_t*		device,
	service_type_t	service,
	WPARAM			wParam,
	LPARAM			lParam)
{
	static const service_call_t	list[]	=
	{
		/* service				function			*/
		{ SERVICE_REFRESH,		_service_refresh	},
		{ SERVICE_WM_COMMAND,	_service_wm_command	},
		{ SERVICE_END_LIST,		NULL				}
	};

	LRESULT	ret;

	ret	= service_call(dlg, device, list, service, wParam, lParam);
	return(ret);
}


