// HPDI32 - Application - Win32 - Any Compiler
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/SDK/hpdi32/visualhpdi32/win32/select.c $
// $Rev: 2902 $
// $Date: 2009-08-09 17:48:49 -0500 (Sun, 09 Aug 2009) $

/*****************************************************************************
*
*	Implement Device Select services.
*
*****************************************************************************/

#include <windows.h>
#include <stdio.h>

#include "select.h"
#include "ini.h"
#include "resource.h"
#include "util_ctrl.h"



/* #defines	*****************************************************************/

#define	SECTION_NAME		"gui"
#define	KEY_NAME			"index"



/*****************************************************************************
*
*	Function:	_service_init
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_INIT service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_init(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	char	buf[128];
	HWND	control;
	int		i;
	HMENU	menu;
	U32		status;

	if (device->usable)
	{
		menu	= GetMenu(dlg);
		menu	= GetSubMenu(menu, 0);
		menu	= GetSubMenu(menu, 0);
		status	= hpdi32_board_count(&device->count);
		error_status(dlg, IDC_INFO_TEXT, status);

		if (status == GSC_SUCCESS)
		{
			if (device->count)
				DeleteMenu(menu, 0, MF_BYPOSITION);

			control	= GetDlgItem(dlg, IDC_SELECT);

			for (i = 0; i < (int) device->count; i++)
			{
				sprintf(buf, "HPDI32 #%d", i);
				SendMessage(control, CB_ADDSTRING, 0, (LPARAM) buf);
				AppendMenu(menu, MF_STRING, (UINT) i + IDC_SELECT_DEV_0, buf);
			}

			ini_int_get(NULL, SECTION_NAME, KEY_NAME, &i, 0);

			if (i < 0)
				i	= 0;
			else if (i >= (int) device->count)
				i	= (int) device->count - 1;

			device->index	= (U8) i;
			SendMessage(control, CB_SETCURSEL, (UINT) i, 0);
			CheckMenuItem(menu, (UINT) i, MF_CHECKED | MF_BYPOSITION);
		}
	}

	return(0);
}



/*****************************************************************************
*
*	Function:	_service_refresh
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_REFRESH service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_refresh(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	if (device->handle)
		ctrl_menu_set(dlg, IDC_SELECT, CTRL_DISABLE);
	else if (device->count)
		ctrl_menu_set(dlg, IDC_SELECT, CTRL_ENABLE);
	else
		ctrl_menu_set(dlg, IDC_SELECT, CTRL_DISABLE);

	return(0);
}



/*****************************************************************************
*
*	Function:	_service_wm_command
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_WM_COMMAND service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_wm_command(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	HWND	control;
	int		id		= LOWORD(wParam);
	HMENU	menu;
	int		notify;

	if (id == IDC_SELECT)
	{
		// This is a combobox action.
		notify	= HIWORD(wParam);

		if (notify == LBN_SELCHANGE)
		{
			control	= (HWND) lParam;
			menu	= GetMenu(dlg);
			menu	= GetSubMenu(menu, 0);
			menu	= GetSubMenu(menu, 0);
			CheckMenuItem(	menu,
							device->index,
							MF_UNCHECKED | MF_BYPOSITION);
			device->index	= (U8) SendMessage(control, CB_GETCURSEL, 0, 0);
			CheckMenuItem(	menu,
							device->index,
							MF_CHECKED | MF_BYPOSITION);
			ini_int_set(NULL, SECTION_NAME, KEY_NAME, device->index);
			DrawMenuBar(dlg);
		}
	}
	else
	{
		// This is a menu item action.
		menu	= GetMenu(dlg);
		menu	= GetSubMenu(menu, 0);
		menu	= GetSubMenu(menu, 0);
		CheckMenuItem(menu, device->index, MF_UNCHECKED | MF_BYPOSITION);
		device->index	= (U8) (id - IDC_SELECT_DEV_0);
		ini_int_set(NULL, SECTION_NAME, KEY_NAME, device->index);
		CheckMenuItem(menu, device->index, MF_CHECKED | MF_BYPOSITION);
		DrawMenuBar(dlg);
		control	= GetDlgItem(dlg, IDC_SELECT);
		SendMessage(control, CB_SETCURSEL, device->index, 0);
	}

	return(0);
}



/*****************************************************************************
*
*	Function:	select_service
*
*	Purpose:
*
*		Implement the common service entry point for the Device Select
*		services.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		service	The service to perform.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0		No service was called.
*		else	The value from the called service, which may be zero.
*
*****************************************************************************/

LRESULT select_service(
	HWND			dlg,
	device_t*		device,
	service_type_t	service,
	WPARAM			wParam,
	LPARAM			lParam)
{
	static const service_call_t	list[]	=
	{
		/* service				function			*/
		{ SERVICE_INIT,			_service_init		},
		{ SERVICE_REFRESH,		_service_refresh	},
		{ SERVICE_WM_COMMAND,	_service_wm_command	},
		{ SERVICE_END_LIST,		NULL				}
	};

	LRESULT	ret;

	ret	= service_call(dlg, device, list, service, wParam, lParam);
	return(ret);
}


