// HPDI32 - Application - Win32 - Any Compiler
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/SDK/hpdi32/visualhpdi32/win32/service.c $
// $Rev: 2902 $
// $Date: 2009-08-09 17:48:49 -0500 (Sun, 09 Aug 2009) $

/*****************************************************************************
*
*	Implement a common service mechanism. This is intended primarily for
*	control and menu item access.
*
*****************************************************************************/

#include <windows.h>

#include "service.h"



/*****************************************************************************
*
*	Function:	service_call
*
*	Purpose:
*
*		Call the service function from the given list that corresponding to
*		the given service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		list	The list to implemented services.
*
*		service	The service to perform.
*
*		arg1	The "arg1" value to pass to the function.
*
*		arg2	The "arg2" value to pass to the function.
*
*	Returned:
*
*		0		No service was called.
*		else	The value returned from the called service.
*
*****************************************************************************/

LRESULT service_call(
	HWND					dlg,
	device_t*				device,
	const service_call_t*	list,
	service_type_t			service,
	WPARAM					arg1,
	LPARAM					arg2)
{
	int		i;
	LRESULT	ret	= 0;

	for (i = 0;; i++)
	{
		if ((list[i].service == SERVICE_END_LIST) ||
			(list[i].function == NULL))
		{
			break;
		}

		if (service == list[i].service)
		{
			ret	= (list[i].function)(dlg, device, arg1, arg2);
			break;
		}
	}

	return(ret);
}



/*****************************************************************************
*
*	Function:	service_list
*
*	Purpose:
*
*		Call the service function for all items in the given list using the
*		given arguments.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		list	The list to process. The list is terminated by a NULL
*				function.
*
*		service	The service to perform.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void service_list(
	HWND					dlg,
	device_t*				device,
	const service_item_t*	list,
	service_type_t			service,
	WPARAM					wParam,
	LPARAM					lParam)
{
	int	i;

	for (i = 0; list[i].function; i++)
		(list[i].function)(dlg, device, service, wParam, lParam);
}


