// HPDI32 - Application - Win32 - Any Compiler
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/SDK/hpdi32/visualhpdi32/win32/tx_dr.c $
// $Rev: 2902 $
// $Date: 2009-08-09 17:48:49 -0500 (Sun, 09 Aug 2009) $

/*****************************************************************************
*
*	Implement support for the Tx Data Ready configuration options.
*
*****************************************************************************/

#include <windows.h>

#include "tx.h"
#include "align.h"
#include "arrow.h"
#include "resource.h"
#include "static.h"
#include "util_ctrl.h"



/*****************************************************************************
*
*	Function:	_gui_paint
*
*	Purpose:
*
*		Provide additional painting support for the GUI Tx Data Ready control.
*
*	Arguments:
*
*		control		The handle to the control.
*
*		data		The control specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _gui_paint(HWND control, static_t* data)
{
	static const wave_t	down[]	=
	{
		{
		/* flags	*/	WAVE_TOP,
		/* skip		*/	0,
		/* count	*/	1,
		/* w1		*/	1,
		/* w2		*/	6
		},
		{
		/* flags	*/	WAVE_TOP,
		/* skip		*/	0,
		/* count	*/	1,
		/* w1		*/	1,
		/* w2		*/	0
		},
		{ 0 }
	};

	static const wave_t	up[]	=
	{
		{
		/* flags	*/	WAVE_BOTTOM,
		/* skip		*/	0,
		/* count	*/	1,
		/* w1		*/	1,
		/* w2		*/	6
		},
		{
		/* flags	*/	WAVE_BOTTOM,
		/* skip		*/	0,
		/* count	*/	1,
		/* w1		*/	1,
		/* w2		*/	0
		},
		{ 0 }
	};

	int	x;
	int	y;

	x	= data->inside.rect.left
		+ data->arrow.offset
		+ data->arrow.width
		+ MARGIN;
	y	= data->inside.rect.top;

	x	+= static_text_paint(control, data, x, y, " Empty ");
	x	+= static_wave_paint(data, x, y, down);

	x	+= static_text_paint(control, data, x, y, "   Not Empty ");
	static_wave_paint(data, x, y, up);
}



/*****************************************************************************
*
*	Function:	_service_init
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_INIT service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_init(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	static const align_t list[]	=
	{
		{
		/* id		*/	IDC_TX_DR,
		/* ref		*/	IDC_TX_RX_READY,
		/* flags	*/	ALIGN_COPY,
		/* value	*/	0
		},
		{
		/* id		*/	IDC_TX_DR,
		/* ref		*/	IDC_TX_RX_READY,
		/* flags	*/	ALIGN_T_B | ALIGN_H,
		/* value	*/	MARGIN
		},
		{
		/* id		*/	IDC_TX_DR_ARROW,
		/* ref		*/	IDC_TX_CLOCK_ARROW,
		/* flags	*/	ALIGN_L_L | ALIGN_R_R,
		/* value	*/	0
		},
		{
		/* id		*/	IDC_TX_DR_ARROW,
		/* ref		*/	IDC_TX_DR,
		/* flags	*/	ALIGN_T_T | ALIGN_B_B,
		/* value	*/	0
		},

		// The Tx Data Ready source arrow.
		{ IDC_TX_DR_SOURCE_1,		IDC_TX_DR,			ALIGN_R_L | ALIGN_T_T | ALIGN_B_B,	0	},
		{ IDC_TX_DR_SOURCE_1,		IDC_TX_DR_SOURCE_2,	ALIGN_L_C,							0	},
		{ IDC_TX_DR_SOURCE_2,		IDC_TX_FIFO,		ALIGN_T_B,							0	},
		{ IDC_TX_DR_SOURCE_2,		IDC_TX_DR_SOURCE_1,	ALIGN_B_C,							0	},
		{ 0, 0, 0, 0 }
	};

	align_control_list(dlg, list);
	static_init(dlg, device, IDC_TX_DR);
	static_arrow_direction_set(dlg, IDC_TX_DR, +1);
	static_paint_function_set(dlg, IDC_TX_DR, _gui_paint);

	arrow_init(dlg, device, IDC_TX_DR_SOURCE_1);
	arrow_type_set(dlg, IDC_TX_DR_SOURCE_1, ARROW_RIGHT);
	arrow_init(dlg, device, IDC_TX_DR_SOURCE_2);
	arrow_type_set(dlg, IDC_TX_DR_SOURCE_2, ARROW_VERT);
	arrow_init(dlg, device, IDC_TX_DR_ARROW);
	arrow_type_set(dlg, IDC_TX_DR_ARROW, ARROW_RIGHT);
	return(0);
}



/*****************************************************************************
*
*	Function:	_service_refresh
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_REFRESH service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_refresh(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	U32	enable;
	U32	mode;
	U32	status;

	for (;;)	// A convenience loop.
	{
		if (device->handle == NULL)
		{
			// The device is closed.
			ctrl_menu_set(dlg, IDC_TX_DR, CTRL_DISABLE);
			ctrl_menu_set(dlg, IDC_TX_DR_ARROW, CTRL_DISABLE);
			ctrl_menu_set(dlg, IDC_TX_DR_SOURCE_1, CTRL_DISABLE);
			ctrl_menu_set(dlg, IDC_TX_DR_SOURCE_2, CTRL_DISABLE);
			break;
		}

		// The device is open.
		status	= HPDI32_CABLE_COMMAND_MODE__TR_GET(device->handle, &mode);
		error_status(dlg, IDC_INFO_TEXT, status);

		if (mode != HPDI32_CABLE_COMMAND_MODE_FLOW_CONTROL)
		{
			// The signal is configured as GPIO.
			ctrl_menu_set(dlg, IDC_TX_DR, CTRL_ENABLE);
			ctrl_menu_set(dlg, IDC_TX_DR_ARROW, CTRL_DISABLE);
			ctrl_menu_set(dlg, IDC_TX_DR_SOURCE_1, CTRL_ENABLE);
			ctrl_menu_set(dlg, IDC_TX_DR_SOURCE_2, CTRL_ENABLE);
			break;
		}

		// The signal is configured as Flow Control.
		status	= HPDI32_TX_ENABLE__GET(device->handle, &enable);
		error_status(dlg, IDC_INFO_TEXT, status);

		if (enable == HPDI32_TX_ENABLE_YES)
		{
			// The signal is driven by the transmitter.
			ctrl_menu_set(dlg, IDC_TX_DR, CTRL_ENABLE);
			ctrl_menu_set(dlg, IDC_TX_DR_ARROW, CTRL_ENABLE);
			ctrl_menu_set(dlg, IDC_TX_DR_SOURCE_1, CTRL_ENABLE);
			ctrl_menu_set(dlg, IDC_TX_DR_SOURCE_2, CTRL_ENABLE);
			break;
		}

		// The signal is unused by the transmitter at this time.
		ctrl_menu_set(dlg, IDC_TX_DR, CTRL_ENABLE);
		ctrl_menu_set(dlg, IDC_TX_DR_ARROW, CTRL_DISABLE);
		ctrl_menu_set(dlg, IDC_TX_DR_SOURCE_1, CTRL_ENABLE);
		ctrl_menu_set(dlg, IDC_TX_DR_SOURCE_2, CTRL_ENABLE);
		break;
	}

	return(0);
}



/*****************************************************************************
*
*	Function:	tx_data_ready_service
*
*	Purpose:
*
*		Implement the common service entry point for the Tx Data Ready
*		services.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		service	The service to perform.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0		No service was called.
*		else	The value from the called service, which may be zero.
*
*****************************************************************************/

LRESULT tx_data_ready_service(
	HWND			dlg,
	device_t*		device,
	service_type_t	service,
	WPARAM			wParam,
	LPARAM			lParam)
{
	static const service_call_t	list[]	=
	{
		/* service			function			*/
		{ SERVICE_INIT,		_service_init		},
		{ SERVICE_REFRESH,	_service_refresh	},
		{ SERVICE_END_LIST,	NULL				}
	};

	LRESULT	ret;

	ret	= service_call(dlg, device, list, service, wParam, lParam);
	return(ret);
}


