// HPDI32 - Application - Win32 - Any Compiler
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/SDK/hpdi32/visualhpdi32/win32/tx_lv.c $
// $Rev: 2902 $
// $Date: 2009-08-09 17:48:49 -0500 (Sun, 09 Aug 2009) $

/*****************************************************************************
*
*	Implement support for the Tx Line Valid configuration options.
*
*****************************************************************************/

#include <windows.h>
#include <stdio.h>

#include "tx.h"
#include "align.h"
#include "arrow.h"
#include "button.h"
#include "cable.h"
#include "copy.h"
#include "gui.h"
#include "resource.h"
#include "util_ctrl.h"



/* #defines	*****************************************************************/

#undef	SECTION_NAME
#define	SECTION_NAME	"tx line valid"



/* typedefs	*****************************************************************/

typedef struct
{
	U32		defalt;		// MUST BE SET BY CODE WANTING THE DIALOG!
	int		ignore;		// Ignore certain activities.
	U32		setting;	// The device/dialog setting is recorded here.
} _lv_t;

typedef struct
{
	int			changes;	// Are there changes to apply?
	device_t*	device;
	HWND		dlg;
	_lv_t		mirror_sv;
	_lv_t		mode;
	_lv_t		off;
	_lv_t		on;
	int			was_open;	// Was the device already open?
} lv_t;



/* variables	*************************************************************/

static const value_list_entry_t	_status_valid_mirror_value_list[]	=
{
	{
	/* user_text			*/	"Disable (Line Valid stays low when Status"
								" Valid goes high.)",
	/* value + value_text	*/	VAR_STR_MACRO(HPDI32_TX_STATUS_VALID_MIRROR_DISABLE),
	/* suffix				*/	"_DISABLE"
	},
	{
	/* user_text			*/	"Enable (Line Valid goes high when Status"
								" Valid goes high.)",
	/* value + value_text	*/	VAR_STR_MACRO(HPDI32_TX_STATUS_VALID_MIRROR_ENABLE),
	/* suffix				*/	"_ENABLE"
	}
};

static const value_list_t	_status_valid_mirror_values[]	=
{
	{
	/* list	*/	_status_valid_mirror_value_list,
	/* size	*/	ARRAY_ELEMENTS(_status_valid_mirror_value_list)
	}
};

static const value_t	_status_valid_mirror_value	=
{
	/* type	*/	VALUE_TYPE_LIST,
	{
	/* u.vp	*/	_status_valid_mirror_values
	}
};

const parm_entry_t	tx_parm_status_valid_mirror	=
{
	/* title			*/	"Status Valid Mirror",
	/* parm + parm_text	*/	VAR_STR_MACRO(HPDI32_TX_STATUS_VALID_MIRROR),
	/* suffix			*/	"",
	/* which			*/	0,
	/* set_arg			*/	"",
	/* get_arg			*/	"",
	/* values			*/	&_status_valid_mirror_value,
	/* cfg_wiz			*/	NULL,
	/* desc				*/
	"This parameter controls the state of the Line Valid signal while the"
	" Status Valid signal is high. Enabling this parameter causes the Line"
	" Valid signal to go high during those periods. Disabling this parameter"
	" causes Line Valid to remain low during those periods. This parameter"
	" should be configured according to the interface needs of the remote"
	" device."
};

static const value_range_t	_line_valid_off_value_range	=
{
	/* min	*/	0,
	/* max	*/	HPDI32_TX_LINE_VALID_OFF_COUNT_MAX,
	/* k	*/	1024
};

static const value_t	_line_valid_off_value	=
{
	/* type	*/	VALUE_TYPE_RANGE,
	{
	/* u.vp	*/	&_line_valid_off_value_range
	}
};

const parm_entry_t	tx_parm_line_valid_off	=
{
	/* title			*/	"Line Valid Off Count",
	/* parm + parm_text	*/	VAR_STR_MACRO(HPDI32_TX_LINE_VALID_OFF_COUNT),
	/* suffix			*/	"",
	/* which			*/	0,
	/* set_arg			*/	"",
	/* get_arg			*/	"",
	/* values			*/	&_line_valid_off_value,
	/* cfg_wiz			*/	NULL,
	/* desc				*/
	"The parameter controls the length of the Line Valid Off period. The Line"
	" Valid Off period precedes each Line Valid On period. The first Line"
	" Valid Off period begins immediately after the Status Valid period. The"
	" value specified refers to the number of Cable Clock cycles that the"
	" Line Valid signal will remain inactive during each Line Valid Off"
	" period. During this period no data is transferred across the cable."
};

static const value_range_t	_line_valid_on_value_range	=
{
	/* min	*/	0,
	/* max	*/	HPDI32_TX_LINE_VALID_ON_COUNT_MAX,
	/* k	*/	1024
};

static const value_t	_line_valid_on_value	=
{
	/* type	*/	VALUE_TYPE_RANGE,
	{
	/* u.vp	*/	&_line_valid_on_value_range
	}
};

const parm_entry_t	tx_parm_line_valid_on	=
{
	/* title			*/	"Line Valid On Count",
	/* parm + parm_text	*/	VAR_STR_MACRO(HPDI32_TX_LINE_VALID_ON_COUNT),
	/* suffix			*/	"",
	/* which			*/	0,
	/* set_arg			*/	"",
	/* get_arg			*/	"",
	/* values			*/	&_line_valid_on_value,
	/* cfg_wiz			*/	NULL,
	/* desc				*/
	"The parameter controls the length of the Line Valid On period. The Line"
	" Valid On period follows each Line Valid Off period. The value"
	" specified refers to the number of Cable Clock cycles that Line Valid"
	" will remain active during each Line Valid On period. Data is"
	" transferred during the Line Valid On period. If the Tx FIFO goes empty"
	" during the Line Valid On period, then the frame is ended and Line Valid"
	" becomes inactive."
};



/*****************************************************************************
*
*	Function:	_mode_device_read
*
*	Purpose:
*
*		Read the parameter from the device into memory.
*
*	Arguments:
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _mode_device_read(lv_t* data)
{
	U32	status;

	status	= HPDI32_CABLE_COMMAND_MODE__LV_GET(data->device->handle,
												&data->mode.setting);
	return(status);
}



/*****************************************************************************
*
*	Function:	_mode_dialog_write
*
*	Purpose:
*
*		Write the parameter held in memory to the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		data		The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

static U32 _mode_dialog_write(HWND dlg, lv_t* data)
{
	unsigned int	flags;
	const char*		psz;;

	if (data->device->handle)
	{
		flags	= CTRL_ENABLE;

		if (data->mode.setting == HPDI32_CABLE_COMMAND_MODE_FLOW_CONTROL)
			psz	= "Flow Control";
		else
			psz	= "GPIO";
	}
	else
	{
		flags	= CTRL_DISABLE;
		psz		= "";
	}

	ctrl_menu_set(dlg, IDC_LV_MODE_BUTTON, flags);
	ctrl_menu_set(dlg, IDC_LV_MODE_TEXT, flags);
	ctrl_menu_set_text(dlg, IDC_LV_MODE_TEXT, psz);

	return(GSC_SUCCESS);
}



/*****************************************************************************
*
*	Function:	_mirror_sv_device_read
*
*	Purpose:
*
*		Read the parameter from the device into memory.
*
*	Arguments:
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _mirror_sv_device_read(lv_t* data)
{
	U32	status;

	status	= HPDI32_TX_STATUS_VALID_MIRROR__GET(	data->device->handle,
													&data->mirror_sv.setting);
	return(status);
}



/*****************************************************************************
*
*	Function:	_mirror_sv_device_write
*
*	Purpose:
*
*		Write the parameter held in memory to the device.
*
*	Arguments:
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _mirror_sv_device_write(lv_t* data)
{
	U32	status;

	status	= HPDI32_TX_STATUS_VALID_MIRROR__SET(	data->device->handle,
													data->mirror_sv.setting);
	return(status);
}



/*****************************************************************************
*
*	Function:	_mirror_sv_dialog_read
*
*	Purpose:
*
*		Read the parameter from the dialog into memory.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _mirror_sv_dialog_read(HWND dlg, lv_t* data)
{
	HWND	control;
	int		state;

	control	= GetDlgItem(dlg, IDC_LV_MIRROR_SV);
	state	= (int) SendMessage(control, BM_GETCHECK, 0, 0);

	if (state == BST_CHECKED)
		data->mirror_sv.setting	= HPDI32_TX_STATUS_VALID_MIRROR_ENABLE;
	else
		data->mirror_sv.setting	= HPDI32_TX_STATUS_VALID_MIRROR_DISABLE;

	return(GSC_SUCCESS);
}



/*****************************************************************************
*
*	Function:	_mirror_sv_dialog_write
*
*	Purpose:
*
*		Write the parameter held in memory to the dialog.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _mirror_sv_dialog_write(HWND dlg, lv_t* data)
{
	unsigned int	flags;

	if (data->device->handle == NULL)
		flags	= CTRL_DISABLE | CTRL_BM_UNCHECK;
	else if (data->mirror_sv.setting == HPDI32_TX_STATUS_VALID_MIRROR_ENABLE)
		flags	= CTRL_ENABLE | CTRL_BM_CHECK;
	else
		flags	= CTRL_ENABLE | CTRL_BM_UNCHECK;

	ctrl_menu_set(dlg, IDC_LV_MIRROR_SV, flags);
	return(GSC_SUCCESS);
}



/*****************************************************************************
*
*	Function:	_off_count_device_read
*
*	Purpose:
*
*		Read the parameter from the device into memory.
*
*	Arguments:
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _off_count_device_read(lv_t* data)
{
	U32	status;

	status	= HPDI32_TX_LINE_VALID_OFF_COUNT__GET(	data->device->handle,
													&data->off.setting);
	return(status);
}



/*****************************************************************************
*
*	Function:	_off_count_device_write
*
*	Purpose:
*
*		Write the parameter held in memory to the device.
*
*	Arguments:
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _off_count_device_write(lv_t* data)
{
	U32	status;

	status	= HPDI32_TX_LINE_VALID_OFF_COUNT__SET(	data->device->handle,
													data->off.setting);
	return(status);
}



/*****************************************************************************
*
*	Function:	_off_count_dialog_read
*
*	Purpose:
*
*		Read the parameter from the dialog into memory.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _off_count_dialog_read(HWND dlg, lv_t* data)
{
	U32	status;

	status	= ctrl_text_2_u32(	dlg,
								IDC_LV_OFF_COUNT_EDIT,
								_line_valid_off_value_range.k,
								&data->off.setting);
	return(status);
}



/*****************************************************************************
*
*	Function:	_off_count_dialog_write
*
*	Purpose:
*
*		Write the parameter held in memory to the dialog.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _off_count_dialog_write(HWND dlg, lv_t* data)
{
	char			buf[32];
	unsigned int	flags;

	if (data->device->handle)
	{
		flags	= CTRL_ENABLE;
		format_u32(data->off.setting, _line_valid_off_value_range.k, buf);
	}
	else
	{
		flags	= CTRL_DISABLE;
		buf[0]	= 0;
	}

	ctrl_menu_set(dlg, IDC_LV_OFF_COUNT_TITLE, flags);
	ctrl_menu_set(dlg, IDC_LV_OFF_COUNT_EDIT, flags);
	ctrl_menu_set(dlg, IDC_LV_OFF_COUNT_INFO, flags);

	if (data->off.ignore == FALSE)
	{
		data->off.ignore	= TRUE;
		ctrl_menu_set_text(dlg, IDC_LV_OFF_COUNT_EDIT, buf);
		data->off.ignore	= FALSE;
	}

	return(GSC_SUCCESS);
}



/*****************************************************************************
*
*	Function:	_on_count_device_read
*
*	Purpose:
*
*		Read the parameter from the device into memory.
*
*	Arguments:
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _on_count_device_read(lv_t* data)
{
	U32	status;

	status	= HPDI32_TX_LINE_VALID_ON_COUNT__GET(	data->device->handle,
													&data->on.setting);
	return(status);
}



/*****************************************************************************
*
*	Function:	_on_count_device_write
*
*	Purpose:
*
*		Write the parameter held in memory to the device.
*
*	Arguments:
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _on_count_device_write(lv_t* data)
{
	U32	status;

	status	= HPDI32_TX_LINE_VALID_ON_COUNT__SET(	data->device->handle,
													data->on.setting);
	return(status);
}



/*****************************************************************************
*
*	Function:	_on_count_dialog_read
*
*	Purpose:
*
*		Read the parameter from the dialog into memory.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _on_count_dialog_read(HWND dlg, lv_t* data)
{
	U32	status;

	status	= ctrl_text_2_u32(	dlg,
								IDC_LV_ON_COUNT_EDIT,
								_line_valid_on_value_range.k,
								&data->on.setting);
	return(status);
}



/*****************************************************************************
*
*	Function:	_on_count_dialog_write
*
*	Purpose:
*
*		Write the parameter held in memory to the dialog.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _on_count_dialog_write(HWND dlg, lv_t* data)
{
	char			buf[32];
	unsigned int	flags;

	if (data->device->handle)
	{
		flags	= CTRL_ENABLE;
		format_u32(data->on.setting, _line_valid_on_value_range.k, buf);
	}
	else
	{
		flags	= CTRL_DISABLE;
		buf[0]	= 0;
	}

	ctrl_menu_set(dlg, IDC_LV_ON_COUNT_TITLE, flags);
	ctrl_menu_set(dlg, IDC_LV_ON_COUNT_EDIT, flags);
	ctrl_menu_set(dlg, IDC_LV_ON_COUNT_INFO, flags);

	if (data->on.ignore == FALSE)
	{
		data->on.ignore	= TRUE;
		ctrl_menu_set_text(dlg, IDC_LV_ON_COUNT_EDIT, buf);
		data->on.ignore	= FALSE;
	}

	return(GSC_SUCCESS);
}



/*****************************************************************************
*
*	Function:	_device_read
*
*	Purpose:
*
*		Read the parameters from the device into memory.
*
*	Arguments:
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _device_read(lv_t* data)
{
	U32	stat;
	U32	status	= GSC_SUCCESS;

	stat	= _mode_device_read(data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _mirror_sv_device_read(data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _off_count_device_read(data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _on_count_device_read(data);
	status	= (status == GSC_SUCCESS) ? stat : status;

	return(status);
}



/*****************************************************************************
*
*	Function:	_device_write
*
*	Purpose:
*
*		Write the parameters held in memory to the device.
*
*	Arguments:
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _device_write(lv_t* data)
{
	U32	stat;
	U32	status	= GSC_SUCCESS;

	stat	= _mirror_sv_device_write(data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _off_count_device_write(data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _on_count_device_write(data);
	status	= (status == GSC_SUCCESS) ? stat : status;

	return(status);
}



/*****************************************************************************
*
*	Function:	_dialog_read
*
*	Purpose:
*
*		Read the parameters from the dialog into memory.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

static U32 _dialog_read(HWND dlg, lv_t* data)
{
	U32	stat;
	U32	status	= GSC_SUCCESS;

	stat	= _mirror_sv_dialog_read(dlg, data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _off_count_dialog_read(dlg, data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _on_count_dialog_read(dlg, data);
	status	= (status == GSC_SUCCESS) ? stat : status;

	return(status);
}



/*****************************************************************************
*
*	Function:	_dialog_write
*
*	Purpose:
*
*		Write the parameters held in memory to the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		data		The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

static U32 _dialog_write(HWND dlg, lv_t* data)
{
	U32	stat;
	U32	status	= GSC_SUCCESS;

	stat	= _mode_dialog_write(dlg, data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _mirror_sv_dialog_write(dlg, data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _off_count_dialog_write(dlg, data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _on_count_dialog_write(dlg, data);
	status	= (status == GSC_SUCCESS) ? stat : status;

	return(status);
}



/*****************************************************************************
*
*	Function:	_refresh_window
*
*	Purpose:
*
*		Update the state of the window.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		msg		A status message we're to report.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _refresh_window(HWND dlg, device_t* device, const char* msg)
{
	lv_t*			data;
	unsigned int	flags;
	HMENU			menu	= GetMenu(dlg);
	U32				stat;
	U32				status	= GSC_SUCCESS;
	char			title[64];

	data	= (lv_t*) GET_WINDOW_USER_DATA(dlg);

	data->changes	= device->handle ? data->changes : FALSE;
	strcpy(title, "Tx Line Valid");
	strcat(title, data->changes ? " *" : "");
	set_win_title(dlg, data->device, FALSE, TRUE, title);

	if (data->changes)
	{
		ModifyMenu(menu, IDCANCEL, MF_BYCOMMAND | MF_STRING, IDCANCEL, "&Cancel");
		SetDlgItemText(dlg, IDCANCEL, "Cancel");
	}
	else
	{
		ModifyMenu(menu, IDCANCEL, MF_BYCOMMAND | MF_STRING, IDCANCEL, "&Close");
		SetDlgItemText(dlg, IDCANCEL, "Close");
	}

	if (device->handle)
	{
		if ((data->was_open == FALSE) || (data->changes == FALSE))
		{
			// Update everything from the device.
			stat	= _device_read(data);
			status	= (status == GSC_SUCCESS) ? stat : status;
			stat	= _dialog_write(dlg, data);
			status	= (status == GSC_SUCCESS) ? stat : status;
		}

		data->was_open	= TRUE;
		flags			= CTRL_ENABLE;
	}
	else
	{
		data->changes	= FALSE;
		data->was_open	= FALSE;
		flags	= CTRL_DISABLE;
		stat	= _dialog_write(dlg, data);
		status	= (status == GSC_SUCCESS) ? stat : status;
		ctrl_menu_set(dlg, IDCANCEL, CTRL_FOCUS);
	}

	ctrl_menu_set(dlg, IDOK, flags);
	ctrl_menu_set(dlg, IDC_APPLY, flags);
	ctrl_menu_set(dlg, IDC_DEFAULT, flags);
	error_status(dlg, 0, status);
}



/*****************************************************************************
*
*	Function:	_idc_apply
*
*	Purpose:
*
*		Respond to an APPLY request.
*
*	Arguments:
*
*		dlg			The dalog handle.
*
*		data		The window specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idc_apply(HWND dlg, lv_t* data)
{
	U32	stat;
	U32	status	= GSC_SUCCESS;

	data->changes	= FALSE;

	stat	= _dialog_read(dlg, data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _device_write(data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _device_read(data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _dialog_write(dlg, data);
	status	= (status == GSC_SUCCESS) ? stat : status;

	main_refresh(data->device, NULL);
	error_status(dlg, 0, status);
}



/*****************************************************************************
*
*	Function:	_idc_copy
*
*	Purpose:
*
*		Implement a COPY function for the dialog content.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idc_copy(HWND dlg)
{
	static const copy_t	list[]	=
	{
		/* id						flags										*/
		{ -1,						COPY_EOL | COPY_UNDERLINE					},

		{ IDC_LV_MODE_BUTTON,		COPY_WIDTH | 11								},
		{ IDC_LV_MODE_TEXT,			COPY_EOL | COPY_DISABLED					},

		{ IDC_LV_MIRROR_SV,			COPY_EOL | COPY_DISABLED | COPY_BM_GETCHECK	},

		{ IDC_LV_OFF_COUNT_TITLE,	COPY_WIDTH | 11								},
		{ IDC_LV_OFF_COUNT_EDIT,	0											},
		{ IDC_LV_OFF_COUNT_INFO,	COPY_INDENT | COPY_EOL | COPY_DISABLED		},

		{ IDC_LV_ON_COUNT_TITLE,	COPY_WIDTH | 11								},
		{ IDC_LV_ON_COUNT_EDIT,		0											},
		{ IDC_LV_ON_COUNT_INFO,		COPY_INDENT | COPY_EOL | COPY_DISABLED		},

		{ 0,						COPY_UNDERLINE								},
		{ 0,						COPY_EOL									},

		{ IDOK,						COPY_DISABLED | COPY_EOL | COPY_WIDTH | 8	},
		{ IDCANCEL,					COPY_DISABLED | COPY_EOL | COPY_WIDTH | 8	},
		{ IDC_APPLY,				COPY_DISABLED | COPY_EOL | COPY_WIDTH | 8	},
		{ IDC_DEFAULT,				COPY_DISABLED | COPY_EOL | COPY_WIDTH | 8	}
	};

	copy_list_to_clipboard(dlg, list, ARRAY_ELEMENTS(list));
}



/*****************************************************************************
*
*	Function:	_idc_default
*
*	Purpose:
*
*		Respond to a DEFAULT request.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		data		The window specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idc_default(HWND dlg, lv_t* data)
{
	U32	status;

	data->changes	= TRUE;

	data->mirror_sv.setting	= data->mirror_sv.defalt;
	data->off.setting		= data->off.defalt;
	data->on.setting		= data->on.defalt;

	status	= _dialog_write(dlg, data);

	_refresh_window(dlg, data->device, NULL);
	error_status(dlg, 0, status);
}



/*****************************************************************************
*
*	Function:	_idc_lv_mirror_sv
*
*	Purpose:
*
*		Respond to activity on the Line Valid Mirror Status Valid control.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		data	The window specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static LRESULT _idc_lv_mirror_sv(HWND dlg, lv_t* data)
{
	data->changes	= TRUE;
	_refresh_window(dlg, data->device, NULL);
	return(0);
}



/*****************************************************************************
*
*	Function:	_idc_lv_off_count_edit
*
*	Purpose:
*
*		Respond to activity in the Line Valid Off Count edit control.
*
*	Arguments:
*
*		dlg			The dalog handle.
*
*		wParam		The message's WORD sized parameter.
*
*		data		The window specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idc_lv_off_count_edit(HWND dlg, WPARAM wParam, lv_t* data)
{
	int	code	= HIWORD(wParam);

	if ((code == EN_CHANGE) && (data->off.ignore == FALSE))
	{
		data->changes	= TRUE;
		_refresh_window(dlg, data->device, NULL);
	}
}



/*****************************************************************************
*
*	Function:	_idc_lv_on_count_edit
*
*	Purpose:
*
*		Respond to activity in the Line Valid On Count edit control.
*
*	Arguments:
*
*		dlg			The dalog handle.
*
*		wParam		The message's WORD sized parameter.
*
*		data		The window specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idc_lv_on_count_edit(HWND dlg, WPARAM wParam, lv_t* data)
{
	int	code	= HIWORD(wParam);

	if ((code == EN_CHANGE) && (data->on.ignore == FALSE))
	{
		data->changes	= TRUE;
		_refresh_window(dlg, data->device, NULL);
	}
}



/*****************************************************************************
*
*	Function:	_idcancel
*
*	Purpose:
*
*		Respond to a request to close the dialog.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idcancel(HWND dlg)
{
	lv_t*	data;

	main_refresh_add(dlg, NULL);
	data	= (lv_t*) GET_WINDOW_USER_DATA(dlg);

	if (data)
		data->dlg	= NULL;

	win_pos_save(dlg, SECTION_NAME);
	DestroyWindow(dlg);
}



/*****************************************************************************
*
*	Function:	_idok
*
*	Purpose:
*
*		Process clicking of the OK button.
*
*	Arguments:
*
*		dlg			The dalog handle.
*
*		data		The window specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idok(HWND dlg, lv_t* data)
{
	_idc_apply(dlg, data);
	_idcancel(dlg);
}



/*****************************************************************************
*
*	Function:	_wm_command
*
*	Purpose:
*
*		Process WM_COMMAND messages.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		wParam		The message's WORD parameter.
*
*		lParam		The message's LONG parameter.
*
*	Returned:
*
*		TRUE		We processed the message.
*		FALSE		We did not process the message.
*
*****************************************************************************/

static LRESULT _wm_command(HWND dlg, WPARAM wParam, LPARAM lParam)
{
	lv_t*	data	= (lv_t*) GET_WINDOW_USER_DATA(dlg);
	int		id		= LOWORD(wParam);
	LRESULT	ret		= TRUE;

	switch (id)
	{
		default:

			ret	= FALSE;
			break;

		case IDC_LV_MIRROR_SV:

			_idc_lv_mirror_sv(dlg, data);
			break;

		case IDC_LV_MODE_BUTTON:

			dlg	= GetParent(dlg);
			cable_command_1_service(dlg, data->device, SERVICE_WM_COMMAND, wParam, (ULONG) lParam);
			break;

		case IDC_LV_OFF_COUNT_EDIT:

			_idc_lv_off_count_edit(dlg, wParam, data);
			break;

		case IDC_LV_ON_COUNT_EDIT:

			_idc_lv_on_count_edit(dlg, wParam, data);
			break;

		case IDC_APPLY:

			_idc_apply(dlg, data);
			break;

		case IDC_COPY:

			_idc_copy(dlg);
			break;

		case IDC_DEFAULT:

			_idc_default(dlg, data);
			break;

		case IDCANCEL:

			_idcancel(dlg);
			break;

		case IDOK:

			_idok(dlg, data);
			break;
	}

	return(ret);
}



/*****************************************************************************
*
*	Function:	_wm_destroy
*
*	Purpose:
*
*		Process WM_DESTROY messages.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*	Returned:
*
*		0			We processed the message.
*
*****************************************************************************/

static LRESULT _wm_destroy(HWND dlg)
{
	_idcancel(dlg);
	DestroyWindow(dlg);
	return(0);
}



/*****************************************************************************
*
*	Function:	_wm_initdialog
*
*	Purpose:
*
*		Process a WM_INITDIALOG message for the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		lParam		The message's long sized argument.
*
*	Returned:
*
*		TRUE		Windows should set the initial control focus.
*		FALSE		Windows should NOT set the initial control focus.
*
*****************************************************************************/

static LRESULT _wm_initdialog(HWND dlg, LPARAM lParam)
{
	char	buf[32];
	HWND	control;
	lv_t*	data	= (lv_t*) lParam;

	SET_WINDOW_USER_DATA(dlg, lParam);

	SendMessage(dlg, WM_SETICON, ICON_SMALL, (LPARAM) data->device->icon.little);
	SendMessage(dlg, WM_SETICON, ICON_BIG, (LPARAM) data->device->icon.big);

	control	= GetDlgItem(dlg, IDC_LV_OFF_COUNT_EDIT);
	SendMessage(control, EM_LIMITTEXT, 12, 0);

	sprintf(buf, "(1K = %d)", _line_valid_off_value_range.k);
	ctrl_menu_set_text(dlg, IDC_LV_OFF_COUNT_INFO, buf);

	control	= GetDlgItem(dlg, IDC_LV_ON_COUNT_EDIT);
	SendMessage(control, EM_LIMITTEXT, 12, 0);

	sprintf(buf, "(1K = %d)", _line_valid_on_value_range.k);
	ctrl_menu_set_text(dlg, IDC_LV_ON_COUNT_INFO, buf);

	win_pos_restore(dlg, SECTION_NAME, FALSE);
	main_refresh_add(dlg, _refresh_window);
	return(TRUE);
}



/*****************************************************************************
*
*	Function:	_wm_syscommand
*
*	Purpose:
*
*		Process a WM_SYSCOMMAND message for the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		wParam		The message's word sized parameter.
*
*	Returned:
*
*		TRUE		We processed the message.
*		FALSE		We did not process the message.
*
*****************************************************************************/

static LRESULT _wm_syscommand(HWND dlg, WPARAM wParam)
{
	LRESULT	ret;

	switch (wParam)
	{
		default:

			ret	= FALSE;
			break;

		case SC_CLOSE:

			_idcancel(dlg);
			ret	= TRUE;
			break;
	}

	return(ret);
}



/*****************************************************************************
*
*	Function:	_dlg_proc
*
*	Purpose:
*
*		Processes messages for the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		msg			The message to process.
*
*		wParam		The message's word sized parameter.
*
*		lParam		The message's long sized parameter.
*
*	Returned:
*
*		A message specific value.
*
*****************************************************************************/

static INT_PTR CALLBACK _dlg_proc(	HWND	dlg,
									UINT	msg,
									WPARAM	wParam,
									LPARAM	lParam)
{
	LRESULT	ret;

	switch (msg)
	{
		default:

			ret	= FALSE;
			break;

		case WM_COMMAND:

			ret	= _wm_command(dlg, wParam, lParam);
			break;

		case WM_DESTROY:

			ret	= _wm_destroy(dlg);
			break;

		case WM_INITDIALOG:

			ret	= _wm_initdialog(dlg, lParam);
			break;

		case WM_SYSCOMMAND:

			ret	= _wm_syscommand(dlg, wParam);
			break;
	}

	return(ret);
}



/*****************************************************************************
*
*	Function:	_gui_paint
*
*	Purpose:
*
*		Provide additional painting support for the GUI Tx Line Valid control.
*
*	Arguments:
*
*		control		The handle to the control.
*
*		data		The control specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _gui_paint(HWND control, button_t* data)
{
	static const wave_t	all_off[]	=
	{
		{
		/* flags	*/	WAVE_BOTTOM,
		/* skip		*/	0,
		/* count	*/	1,
		/* w1		*/	47,
		/* w2		*/	0
		},
		{ 0 }
	};

	static const wave_t	all_on[]	=
	{
		{
		/* flags	*/	WAVE_BOTTOM,
		/* skip		*/	0,
		/* count	*/	1,
		/* w1		*/	2,
		/* w2		*/	43
		},
		{
		/* flags	*/	WAVE_BOTTOM,
		/* skip		*/	0,
		/* count	*/	1,
		/* w1		*/	2,
		/* w2		*/	0
		},
		{ 0 }
	};

	static const wave_t	lv_on_off[]	=
	{
		{
		/* flags	*/	WAVE_BOTTOM,
		/* skip		*/	0,
		/* count	*/	5,
		/* w1		*/	2,
		/* w2		*/	6
		},
		{
		/* flags	*/	WAVE_BOTTOM,
		/* skip		*/	0,
		/* count	*/	1,
		/* w1		*/	2,
		/* w2		*/	3
		},
		{
		/* flags	*/	WAVE_BOTTOM,
		/* skip		*/	0,
		/* count	*/	1,
		/* w1		*/	2,
		/* w2		*/	0
		},
		{ 0 }
	};

	static const wave_t	sv_off_lv_on[]	=
	{
		{
		/* flags	*/	WAVE_BOTTOM,
		/* skip		*/	0,
		/* count	*/	1,
		/* w1		*/	3,
		/* w2		*/	42
		},
		{
		/* flags	*/	WAVE_BOTTOM,
		/* skip		*/	0,
		/* count	*/	1,
		/* w1		*/	2,
		/* w2		*/	0
		},
		{ 0 }
	};

	static const wave_t	sv_off_lv_off_lv_on[]	=
	{
		{
		/* flags	*/	WAVE_BOTTOM,
		/* skip		*/	0,
		/* count	*/	1,
		/* w1		*/	5,
		/* w2		*/	40
		},
		{
		/* flags	*/	WAVE_BOTTOM,
		/* skip		*/	0,
		/* count	*/	1,
		/* w1		*/	2,
		/* w2		*/	0
		},
		{ 0 }
	};

	static const wave_t	sv_off_lv_off_on[]	=
	{
		{
		/* flags	*/	WAVE_BOTTOM,
		/* skip		*/	0,
		/* count	*/	1,
		/* w1		*/	3,
		/* w2		*/	0
		},
		{
		/* flags	*/	WAVE_BOTTOM,
		/* skip		*/	0,
		/* count	*/	5,
		/* w1		*/	2,
		/* w2		*/	6
		},
		{
		/* flags	*/	WAVE_BOTTOM,
		/* skip		*/	0,
		/* count	*/	1,
		/* w1		*/	4,
		/* w2		*/	0
		},
		{ 0 }
	};

	static const wave_t	sv_on_lv_off_on[]	=
	{
		{
		/* flags	*/	WAVE_BOTTOM,
		/* skip		*/	0,
		/* count	*/	1,
		/* w1		*/	2,
		/* w2		*/	1
		},
		{
		/* flags	*/	WAVE_BOTTOM,
		/* skip		*/	0,
		/* count	*/	5,
		/* w1		*/	2,
		/* w2		*/	6
		},
		{
		/* flags	*/	WAVE_BOTTOM,
		/* skip		*/	0,
		/* count	*/	1,
		/* w1		*/	4,
		/* w2		*/	0
		},
		{ 0 }
	};

	static const wave_t	sv_on_lv_off_lv_on[]	=
	{
		{
		/* flags	*/	WAVE_BOTTOM,
		/* skip		*/	0,
		/* count	*/	1,
		/* w1		*/	2,
		/* w2		*/	1
		},
		{
		/* flags	*/	WAVE_BOTTOM,
		/* skip		*/	0,
		/* count	*/	1,
		/* w1		*/	2,
		/* w2		*/	40
		},
		{
		/* flags	*/	WAVE_BOTTOM,
		/* skip		*/	0,
		/* count	*/	1,
		/* w1		*/	2,
		/* w2		*/	0
		},
		{ 0 }
	};

	static const wave_t*	list[16]	=
	{
		/* sv	mirror	invalid	valid	waveform	*/
		/* 0	no		0		0	*/	all_on,			// lv on
		/* 0	no		0		>=1	*/	all_on,			// lv on
		/* 0	no		>=1		0	*/	all_on,			// lv on
		/* 0	no		>=1		>=1	*/	lv_on_off,		// lv on/off cycle
		/* 0	yes		0		0	*/	all_on,			// lv on
		/* 0	yes		0		>=1	*/	all_on,			// lv on
		/* 0	yes		>=1		0	*/	all_on,			// lv on
		/* 0	yes		>=1		>=1	*/	lv_on_off,		// lv on/off cycle
		/* >=1	no		0		0	*/	sv_off_lv_on,	// sv off, lv on
		/* >=1	no		0		>=1	*/	sv_off_lv_on,	// sv off, lv on
		/* >=1	no		>=1		0	*/	sv_off_lv_off_lv_on,// sv off, lv off, lv on
		/* >=1	no		>=1		>=1	*/	sv_off_lv_off_on,// sv off, lv off/on cycle
		/* >=1	yes		0		0	*/	all_on,			// sv on, lv on
		/* >=1	yes		0		>=1	*/	all_on,			// sv on, lv on
		/* >=1	yes		>=1		0	*/	sv_on_lv_off_lv_on,// sv on, lv off, lv on
		/* >=1	yes		>=1		>=1	*/	sv_on_lv_off_on// sv on, lv off/on cycle
	};

	HWND	dlg;
	int		i;
	U32		mirror;
	U32		off;
	U32		on;
	U32		stat;
	U32		status;
	U32		sv;
	int		x;
	int		y;

	x	= data->inside2.rect.left
		+ data->arrow1.offset
		+ data->arrow1.width
		+ MARGIN
		+ data->offset;
	y	= data->inside2.rect.top
		+ data->offset;

	x	+= button_text_paint(control, data, x, y, " ");

	if (data->device->handle)
	{
		status	= HPDI32_TX_STATUS_VALID_COUNT__GET(data->device->handle, &sv);
		stat	= HPDI32_TX_STATUS_VALID_MIRROR__GET(data->device->handle, &mirror);
		status	= (status == GSC_SUCCESS) ? stat : status;
		stat	= HPDI32_TX_LINE_VALID_OFF_COUNT__GET(data->device->handle, &off);
		status	= (status == GSC_SUCCESS) ? stat : status;
		stat	= HPDI32_TX_LINE_VALID_ON_COUNT__GET(data->device->handle, &on);
		status	= (status == GSC_SUCCESS) ? stat : status;
		dlg	= GetParent(control);
		error_status(dlg, IDC_INFO_TEXT, status);

		i	= 0;
		i	|= sv ? 0x8 : 0;
		i	|= (mirror == HPDI32_TX_STATUS_VALID_MIRROR_ENABLE) ? 0x4 : 0;
		i	|= off ? 0x2 : 0;
		i	|= on ? 0x1 : 0;

		button_wave_paint(data, x, y, list[i]);
	}
	else
	{
		button_wave_paint(data, x, y, all_off);
	}
}



/*****************************************************************************
*
*	Function:	_service_init
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_INIT service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_init(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	static const align_t list[]	=
	{
		{
		/* id		*/	IDC_TX_LV,
		/* ref		*/	IDC_TX_FV,
		/* flags	*/	ALIGN_COPY,
		/* value	*/	0
		},
		{
		/* id		*/	IDC_TX_LV,
		/* ref		*/	IDC_TX_FV,
		/* flags	*/	ALIGN_T_B | ALIGN_H,
		/* value	*/	MARGIN
		},
		{
		/* id		*/	IDC_TX_LV_ARROW,
		/* ref		*/	IDC_TX_CLOCK_ARROW,
		/* flags	*/	ALIGN_L_L | ALIGN_R_R,
		/* value	*/	0
		},
		{
		/* id		*/	IDC_TX_LV_ARROW,
		/* ref		*/	IDC_TX_LV,
		/* flags	*/	ALIGN_T_T | ALIGN_B_B,
		/* value	*/	0
		},
		{ 0, 0, 0, 0 }
	};

	align_control_list(dlg, list);
	button_init(dlg, device, IDC_TX_LV);
	button_arrow_direction_set(dlg, IDC_TX_LV, +1);
	button_paint_function_set(dlg, IDC_TX_LV, _gui_paint);
	arrow_init(dlg, device, IDC_TX_LV_ARROW);
	arrow_type_set(dlg, IDC_TX_LV_ARROW, ARROW_RIGHT);
	return(0);
}



/*****************************************************************************
*
*	Function:	_service_refresh
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_REFRESH service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_refresh(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	HWND	control;
	U32		enable;
	U32		mode;
	U32		status;

	for (;;)	// A convenience loop.
	{
		if (device->handle == NULL)
		{
			// The device is closed.
			ctrl_menu_set(dlg, IDC_TX_LV, CTRL_DISABLE);
			ctrl_menu_set(dlg, IDC_TX_LV_ARROW, CTRL_DISABLE);
			break;
		}

		// The device is open.
		status	= HPDI32_TX_ENABLE__GET(device->handle, &enable);
		error_status(dlg, IDC_INFO_TEXT, status);

		if (enable == HPDI32_TX_ENABLE_NO)
		{
			// The transmitter is disabled.
			ctrl_menu_set(dlg, IDC_TX_LV, CTRL_ENABLE);
			ctrl_menu_set(dlg, IDC_TX_LV_ARROW, CTRL_DISABLE);
			break;
		}

		// The transmitter is enabled.
		status	= HPDI32_CABLE_COMMAND_MODE__LV_GET(device->handle, &mode);
		error_status(dlg, IDC_INFO_TEXT, status);

		if (mode == HPDI32_CABLE_COMMAND_MODE_FLOW_CONTROL)
		{
			// Line Valid is being driven on the signal.
			ctrl_menu_set(dlg, IDC_TX_LV, CTRL_ENABLE);
			ctrl_menu_set(dlg, IDC_TX_LV_ARROW, CTRL_ENABLE);
			break;
		}

		// Line Valid is not in use.
		ctrl_menu_set(dlg, IDC_TX_LV, CTRL_ENABLE);
		ctrl_menu_set(dlg, IDC_TX_LV_ARROW, CTRL_DISABLE);
		break;
	}

	control	= GetDlgItem(dlg, IDC_TX_LV);
	InvalidateRect(control, NULL, TRUE);
	return(0);
}



/*****************************************************************************
*
*	Function:	_service_wm_command
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_WM_COMMAND service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_wm_command(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	static	lv_t	data;

	if (data.dlg)	/*lint !e727 */
	{
		SetFocus(data.dlg);
	}
	else
	{
		data.changes			= FALSE;
		data.device				= device;
		data.off.defalt			= HPDI32_TX_LINE_VALID_OFF_COUNT_DEFAULT;
		data.off.ignore			= TRUE;
		data.on.defalt			= HPDI32_TX_LINE_VALID_ON_COUNT_DEFAULT;
		data.on.ignore			= TRUE;
		data.was_open			= FALSE;
		data.dlg				= CreateDialogParam(device->instance,
													(LPCTSTR) IDD_LV,
													dlg,
													(DLGPROC) _dlg_proc,
													(LPARAM) &data);
		ShowWindow(data.dlg, SW_SHOWNORMAL);
		data.off.ignore	= FALSE;
		data.on.ignore	= FALSE;
		_refresh_window(data.dlg, device, NULL);
	}

	return(0);
}



/*****************************************************************************
*
*	Function:	tx_line_valid_service
*
*	Purpose:
*
*		Implement the common service entry point for the Tx Line Valid
*		services.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		service	The service to perform.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0		No service was called.
*		else	The value from the called service, which may be zero.
*
*****************************************************************************/

LRESULT tx_line_valid_service(
	HWND			dlg,
	device_t*		device,
	service_type_t	service,
	WPARAM			wParam,
	LPARAM			lParam)
{
	static const service_call_t	list[]	=
	{
		/* service				function			*/
		{ SERVICE_INIT,			_service_init		},
		{ SERVICE_REFRESH,		_service_refresh	},
		{ SERVICE_WM_COMMAND,	_service_wm_command	},
		{ SERVICE_END_LIST,		NULL				}
	};

	LRESULT	ret;

	ret	= service_call(dlg, device, list, service, wParam, lParam);
	return(ret);
}


